@@ -1,5 +1,5 @@
 /* Commands for the curses interface to Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -8,7 +8,15 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+extern void update_debugging PARAMS ((void));
+extern void toggle_debugging PARAMS ((int *flagp));
 #include "cconq.h"
+extern int autofinish_start;
+extern int autofinish_count;
+extern int drawlinear;
+extern char linear_char;
+extern char bord_char;
+extern char conn_char;
 
 /* Command table. */
 
@@ -20,6 +28,7 @@ typedef struct cmdtab {
     char *help;                 /* short documentation string */
 } CmdTab;
 
+static void impl_build PARAMS ((Side *side, Unit *unit, int u2, int n));
 static void resize_map PARAMS ((int n));
 static void execute_named_command PARAMS ((char *cmdstr));
 static int execute_command_from_table PARAMS ((CmdTab *cmdtab));
@@ -213,7 +222,7 @@ find_next_and_look()
 void
 do_add_player()
 {
-    cmd_error("can't add new players yet");
+    request_additional_side(cmdargstr);
 }
 
 void
@@ -291,28 +300,65 @@ do_auto()
 void
 do_build()
 {
-    int u2;
+    short possibles[MAXUTYPES];
+    int u, u2, numtypes, ufirst;
 
     REQUIRE_UNIT();
+    u = curunit->type;
     if (!can_build(curunit)) {
 	cmd_error("%s can't build anything!", unit_handle(dside, curunit));
+	return;
+    }
 #if 0
-    } else if (!u_occproduce(curunit->type) && curunit->transport != NULL) {
+    if (!u_occproduce(curunit->type) && curunit->transport != NULL) {
 	cmd_error("%s can't build anything while inside another unit!",
 		  unit_handle(dside, curunit));
+	return;
+    }
 #endif
-    } else {
-	u2 = ask_unit_type("Type to create:", NULL);
+    numtypes = 0;
+    for_all_unit_types(u2) {
+	if (uu_acp_to_create(u, u2) > 0 && type_allowed_on_side(u, dside)) {
+	    possibles[u2] = TRUE;
+	    ++numtypes;
+	    ufirst = u2;
+	} else {
+	    possibles[u2] = FALSE;
+	}
+    }
+    switch (numtypes) {
+      case 0:
+	cmd_error("Nothing to build!");
+	break;
+      case 1:
+	/* Only one type to build - just do it. */
+	impl_build(dside, curunit, ufirst, prefixarg);
+	break;
+      default:
+	/* Player has to choose a type to build. */
+	u2 = ask_unit_type("Type to build:", possibles);
 	if (u2 != NONUTYPE) {
-	    notify(dside, "%s will build %d %s",
-		   unit_handle(dside, curunit), 99, u_type_name(u2));
-	    push_build_task(curunit, u2, 99);
+	    impl_build(dside, curunit, u2, prefixarg);
 	} else {
 	    /* should clear toplines */
 	}
+	break;
     }
 }
 
+static void
+impl_build(side, unit, u2, n)
+Side *side;
+Unit *unit;
+int u2, n;
+{
+    if (n < 0)
+      n = 99;
+    notify(side, "%s will build %d %s",
+	   unit_handle(side, unit), n, u_type_name(u2));
+    push_build_task(unit, u2, n);
+}
+
 void
 do_clear_plan()
 {
@@ -325,7 +371,8 @@ do_clear_plan()
 void
 do_copying()
 {
-    notify(dside, "You may copy freely.  See the file COPYING.");
+    cur_help_node = copying_help_node;
+    do_help();
 }
 
 void
@@ -345,7 +392,18 @@ do_delay()
 void
 do_detach()
 {
-    cmd_error("can't detach units yet");
+    Unit *unit = curunit;
+
+    REQUIRE_UNIT();
+    if (!completed(unit)) {
+	cmd_error("cannot detach from an incomplete unit");
+	return;
+    }
+    if (valid(check_transfer_part_action(unit, unit, unit->hp / 2, NULL))) {
+	prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+    } else {
+	cmd_error("cannot detach right now");
+    }
 }
 
 void
@@ -401,6 +459,38 @@ do_disband()
     }
 }
 
+void
+do_disembark()
+{
+    Unit *transport;
+
+    REQUIRE_UNIT();
+    transport = curunit->transport;
+    if (transport == NULL) {
+	cmd_error("Not in a transport");
+	return;
+    }
+    if (!in_play(transport)) {
+	cmd_error("Transport is nonsensical?");
+	return;
+    }
+    /* Try moving into the transport's transport, if there is one. */
+    if (transport->transport != NULL
+        && can_occupy(curunit, transport->transport)) {
+	prep_enter_action(curunit, curunit, transport->transport);
+	/* (should be able to set up task if can't do action immedly) */
+	return;
+    }
+    /* Try moving into the open in the cell. */
+    if (can_occupy_cell(curunit, curunit->x, curunit->y)
+	|| can_occupy_conn(curunit, curunit->x, curunit->y, curunit->z)) {
+	prep_move_action(curunit, curunit, curunit->x, curunit->y, curunit->z);
+	/* (should be able to set up task if can't do action immedly) */
+	return;
+    }
+    cmd_error("Can't disembark here!");
+}
+
 /* Determine how far away another point is.  */
 
 void
@@ -425,7 +515,7 @@ do_distrust()
 	    return;
 	} 
     }
-    xbeep();
+    cmd_error("no side to distrust");
 }
 
 void
@@ -444,15 +534,18 @@ do_embark()
     Unit *transport;
 
     REQUIRE_UNIT();
+    /* (should only ask if multiple transport possibilities) */
     if (ask_unit("Which unit to board?", &transport)) {
 	if (!in_play(transport)) {
-	    cmd_error("This transport is nonsensical");
-	} else if (!can_occupy(curunit, transport)) {
-	    cmd_error("Can't occupy");
-	} else {
-	    prep_enter_action(curunit, curunit, transport);
-	    /* (should be able to set up task if can't do action immedly) */
+	    cmd_error("This transport is nonsensical?");
+	    return;
 	}
+	if (!can_occupy(curunit, transport)) {
+	    cmd_error("Can't occupy chosen transport");
+	    return;
+	}
+	prep_enter_action(curunit, curunit, transport);
+	/* (should be able to set up task if can't do action immedly) */
     } else {
 	xbeep();
     }
@@ -478,11 +571,12 @@ do_fire()
     sprintf(spbuf, "Fire %s at where?", unit_handle(dside, curunit));
     /* (should have some sort of range feedback) */
     if (ask_position(spbuf, &x, &y)) {
-	for_all_stack(x, y, unit2) {
-	    if (g_see_all() /* or under observation in some way */
-		&& unit2->side != curunit->side) {
-		prep_fire_at_action(curunit, curunit, unit2, -1);
-		return;
+	if (units_visible(dside, x, y)) {
+	    for_all_stack(x, y, unit2) {
+		if (unit2->side != curunit->side) {
+		    prep_fire_at_action(curunit, curunit, unit2, -1);
+		    return;
+		}
 	    }
 	}
 	/* (should say that nothing is visible and verify firing) */
@@ -497,7 +591,15 @@ do_fire()
 void
 do_fire_into()
 {
-    xbeep();
+    int x, y;
+    Unit *unit2;
+
+    REQUIRE_UNIT();
+    sprintf(spbuf, "Fire %s at where?", unit_handle(dside, curunit));
+    /* (should have some sort of range feedback) */
+    if (ask_position(spbuf, &x, &y)) {
+	prep_fire_into_action(curunit, curunit, x, y, 0, -1);
+    }
 }
 
 /* Toggle the action following flag. */
@@ -516,6 +618,7 @@ do_follow_action()
 void
 do_force_global_replan()
 {
+    force_global_replan(dside);
 }
 
 /* Give supplies to a transport.  The argument tells how many to give. */
@@ -583,7 +686,7 @@ do_give_unit()
 void
 do_help()
 {
-    /* Switch to help mode. */
+    /* Switch to help mode, saving current mode first. */
     prevmode = mode;
     mode = HELP;
     show_help();
@@ -636,7 +739,7 @@ do_move_to()
 	    return;
 	}
 #endif /* DESIGNERS */
-	set_moveto_task(curunit, x, y);
+	set_move_to_task(curunit, x, y);
     }
 }
 
@@ -649,9 +752,9 @@ do_name()
 
     REQUIRE_UNIT();
     if (ask_string("New name for unit:", curunit->name, &newname)) {
-      if (strlen(newname) == 0)
-	newname = NULL;
-      set_unit_name(dside, curunit, newname);
+	if (empty_string(newname))
+	  newname = NULL;
+	set_unit_name(dside, curunit, newname);
     }
 }
 
@@ -660,6 +763,9 @@ do_occupant()
 {
     Unit *nextocc;
 
+    if (curunit == NULL) {
+	make_current_at(curx, cury);
+    }
     REQUIRE_UNIT();
     nextocc = find_next_occupant(curunit);
     if (nextocc != curunit)
@@ -675,8 +781,8 @@ do_other()
 	if (empty_string(cmd)) {
 	    cmd_error("No command");
 	} else if (strcmp(cmd, "?") == 0) {
+	    cur_help_node = commands_help_node;
 	    do_help();
-	    /* (should list named commands specifically) */
 	} else {
 	    execute_named_command(cmd);
 	}
@@ -692,7 +798,23 @@ do_print_view()
 void
 do_produce()
 {
-    cmd_error("can't do explicit production yet");
+    int m, n;
+    Unit *unit = curunit;
+
+    REQUIRE_UNIT();
+    if (!can_produce(unit)) {
+	cmd_error("cannot do active production");
+    }
+    n = 9999;
+    if (prefixarg > 0)
+      n = prefixarg;
+    /* Find the first produceable type and set up to produce it. */
+    for_all_material_types(m) {
+	if (um_acp_to_produce(unit->type, m) > 0) {
+	    push_produce_task(unit, m, n);
+	    return;
+	}
+    }
 }
 
 /* Command to get out of a game, one way or another. */
@@ -700,22 +822,36 @@ do_produce()
 void
 do_quit()
 {
-    Side *side2 = NULL;
-
-    if (!dside->ingame) {
-	exit_cconq(dside);
-    } else if (0 /* other players think they can win */) {
-	if (ask_bool("Do you really want to give up?", FALSE)) {
-	    if (numsides > 2) {
-/*		ask_side("Who do you want to surrender to?", NULL); */
-		resign_game(dside, side2);
+    if (endofgame || !dside->ingame) {
+	exit_cconq();
+	return;
+    }
+    /* Confirm the saving of any state. */
+    if (!gamestatesafe) {
+	if (ask_bool("Do you want to save the game?", TRUE)) {
+	    if (all_others_willing_to_save(dside)) {
+		do_save();
+		exit_cconq();
+		return;
 	    } else {
-		resign_game(dside, NULL);
+		set_willing_to_save(dside, TRUE);
+		notify(dside, "Other sides not willing to save.");
 	    }
 	}
-    } else if (ask_bool("Do you really want to quit now?", FALSE)) {
-	/* should force a loss first if necessary. */
-	exit_cconq(dside);
+    }
+    if (all_others_willing_to_quit(dside)) {
+	if (ask_bool("Do you really want to declare a draw?", FALSE)) {
+	    set_willing_to_draw(dside, TRUE);
+	} else {
+	    notify(dside, "Not willing to draw.");
+	}
+	return;
+    } else {
+	if (ask_bool("You must resign to get out; do you want to resign?", FALSE)) {
+	    do_resign();
+	} else {
+	    notify(dside, "Not resigning.");
+	}
     }
 }
 
@@ -772,7 +908,23 @@ do_reserve()
 void
 do_resign()
 {
-    xbeep();
+    Side *side2;
+
+    if (endofgame) {
+	cmd_error("Game is already over.");
+    } else if (!dside->ingame) {
+	cmd_error("You are already out of the game.");
+    } else if (ask_bool("Do you really want to resign?", FALSE)) {
+	side2 = NULL;
+	if (numsides > 2) {
+	    side2 = ask_side("Who do you want to inherit?", NULL);
+	    if (side2 == dside) {
+		cmd_error("You can't inherit your own units! (not giving to anybody)");
+		side2 = NULL;
+	    }
+	}
+	resign_game(dside, side2);
+    }
 }
 
 /* Set unit to return to a good resupply place. */
@@ -781,7 +933,7 @@ void
 do_return()
 {
     REQUIRE_UNIT();
-    set_resupply_task(curunit);
+    set_resupply_task(curunit, NONMTYPE);
 }
 
 /* Stuff game state into a file.  By default, it goes into the current
@@ -797,7 +949,7 @@ do_save()
 
 #ifdef DESIGNERS
     if (dside->designer) {
-	if (ask_string("Data to write?", "everything", NULL)) {
+	if (ask_string("Data to write?", "everything", &rawcontents)) {
 	    /* (should be in a designer_create_module?) */
 	    /* need to be able to get this name from somewhere */
 	    module = create_game_module("random.scn");
@@ -857,11 +1009,6 @@ do_set_formation()
     }
 }
 
-void
-do_set_map_angle()
-{
-}
-
 void
 do_set_rate()
 {
@@ -882,10 +1029,23 @@ do_sleep()
     set_unit_asleep(dside, curunit, TRUE, TRUE);
 }
 
+void
+do_standing_orders()
+{
+    int rslt;
+
+    if (cmdargstr) {
+	rslt = parse_standing_order(dside, cmdargstr);
+	if (rslt < 0)
+	  xbeep();
+    } else
+      xbeep();
+}
+
 void
 do_surrender_to()
 {
-    xbeep();
+    cmd_error("can't surrender to anybody yet");
 }
 
 /* Command to toggle between interaction modes. */
@@ -950,7 +1110,7 @@ do_trust()
 	    return;
 	} 
     }
-    xbeep();
+    cmd_error("no side to trust");
 }
 
 /* Wake *everything* (that's ours) within the given radius.  Two commands
@@ -984,7 +1144,8 @@ do_version()
 void
 do_warranty()
 {
-    notify(dside, "There is no warranty.");
+    cur_help_node = warranty_help_node;
+    do_help();
 }
 
 /* Curses-specific commands. */
@@ -1073,9 +1234,73 @@ do_c_set_info_lines()
 }
 
 void
-do_c_use_both_chars()
+do_c_run()
 {
-    use_both_chars = !use_both_chars;
+    int turns;
+
+    turns = atoi(cmdargstr);
+    notify(dside, "Running free for %d turn%s.",
+	   turns, (turns == 1 ? "" : "s"));
+    if (turns > 0) {
+	set_autofinish(dside, TRUE);
+	autofinish_start = g_turn();
+	autofinish_count = turns;
+    }
+}
+
+/* Set the display of various kinds of data. */
+
+void
+do_c_show()
+{
+    int value;
+    char *str, *str2, tmpbuf[BUFSIZE];
+
+    if (cmdargstr) {
+	str = cmdargstr;
+	while (*str != '\0') {
+	    /* Collect the next whitespace-separated token from the
+               arg string. */
+	    while (*str != '\0' && *str == ' ')
+	      ++str;
+	    str2 = tmpbuf;
+	    while (*str != '\0' && *str != ' ')
+	      *str2++ = *str++;
+	    *str2 = '\0';
+	    str2 = tmpbuf;
+	    value = TRUE;
+	    /* See if it is prefixed with a "-" or "no". */
+	    if (*str2 == '-') {
+		value = FALSE;
+		++str2;
+	    } else if (*str2 == 'n' && *(str2+1) == 'o') {
+		value = FALSE;
+		str2 += 2;
+	    }
+	    if (strcmp(str2, "terrain") == 0 || strcmp(str2, "t") == 0) {
+		drawterrain = value;
+	    } else if (strcmp(str2, "unit") == 0 || strcmp(str2, "u") == 0) {
+		drawunits = value;
+	    } else if (strcmp(str2, "name") == 0 || strcmp(str2, "n") == 0) {
+		drawnames = value;
+	    } else if (strcmp(str2, "people") == 0 || strcmp(str2, "p") == 0) {
+		drawpeople = value;
+	    } else if (strcmp(str2, "one") == 0 || strcmp(str2, "1") == 0) {
+		use_both_chars = !value;
+	    } else if (strcmp(str2, "two") == 0 || strcmp(str2, "2") == 0) {
+		use_both_chars = value;
+	    } else if (strncmp(str2, "lin", 3) == 0) {
+		drawlinear = value;
+		if (value && str2[3] == '=' && str2[4] != '\0') {
+		    linear_char = str2[4];
+		}
+	    } else {
+		cmd_error("\"%s\" not recognized", tmpbuf);
+	    }
+	}
+    } else {
+	notify(dside, "Nothing to do.");
+    }
     show_map();
     refresh();
 }
@@ -1196,7 +1421,7 @@ void
 do_debug()
 {
 #ifndef Debug
-    Debug = !Debug;
+    toggle_debugging(&Debug);
 #endif
 }
 
@@ -1204,7 +1429,7 @@ void
 do_debugg()
 {
 #ifndef DebugG
-    DebugG = !DebugG;
+    toggle_debugging(&DebugG);
 #endif
 }
 
@@ -1212,7 +1437,7 @@ void
 do_debugm()
 {
 #ifndef DebugM
-    DebugM = !DebugM;
+    toggle_debugging(&DebugM);
 #endif
 }
 
@@ -1230,6 +1455,7 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
 #endif
 {
     char tmpnbuf[BUFSIZE];
+
 #ifdef __STDC__
     {
 	va_list ap;
