@@ -1,5 +1,5 @@
 /* The main program of the curses interface to Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -8,14 +8,28 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+extern void low_send PARAMS ((int id, char *buf));
+extern int low_receive PARAMS ((int *id, char *buf, int maxchars,
+				int timeout));
+extern void update_debugging PARAMS ((void));
+extern void toggle_debugging PARAMS ((int *flagp));
+extern char *get_scores PARAMS ((Side *side));
 #include "cmdline.h"
 #include "cconq.h"
+extern void make_generic_image_data PARAMS ((ImageFamily *imf));
 
 #ifdef THINK_C
 /* This is to get the command line reader in Think C on the Mac. */
 #include <console.h>
 #endif /* THINK_C */
 
+#ifdef __MWERKS__
+#define MAC
+/* This is to get the command line reader in Metrowerks on the Mac. */
+#include <console.h>
+#include <sioux.h>
+#endif /* __MWERKS__ */
+
 static void describe_help PARAMS ((int arg, char *key, char *buf));
 
 int announced = FALSE;
@@ -168,8 +182,16 @@ int value;
 int sorton;
 int sortorder;
 
+/* True when the display is up and running. */
+
 int active;
 
+/* Countdown for when multiple turns autofinishing. */
+
+int autofinish_start;
+
+int autofinish_count;
+
 /* The main program just invokes everything else. */
 
 int
@@ -181,12 +203,19 @@ char *argv[];
     /* This is how Think C picks up a command line. */
     argc = ccommand(&argv);
 #endif
+#ifdef __MWERKS__
+    /* This is how Metrowerks C picks up a command line. */
+    argc = ccommand(&argv);
+    /* Configure the SIOUX window. */
+    SIOUXSettings.initializeTB = TRUE;
+    SIOUXSettings.standalone = FALSE;
+    SIOUXSettings.setupmenus = FALSE;
+    SIOUXSettings.autocloseonquit = FALSE;
+    SIOUXSettings.asktosaveonclose = TRUE;
+    SIOUXSettings.showstatusline = TRUE;
+#endif
 #ifdef MAC
-    {
-	extern char *_maccur_pgm_name;
-		
 	_maccur_pgm_name = "Cconq";
-    }
 #endif
     printf("\n              Welcome to Curses Xconq version %s\n\n",
 	   version_string());
@@ -196,6 +225,9 @@ char *argv[];
     clear_game_modules();
     init_data_structures();
     parse_command_line(argc, argv, general_options);
+#ifdef DEBUGGING
+    update_debugging();
+#endif /* DEBUGGING */
     /* (should volunteer to restore saved game if one found) */
     load_all_modules();
     check_game_validity();
@@ -368,13 +400,15 @@ init_interaction()
     /* Set up some cconq-specific help nodes. */
     if (cur_help_node == NULL) {
 	help_help_node =
-	  add_help_node("help", describe_help, 0, firsthelpnode);
+	  add_help_node("help", describe_help, 0, first_help_node);
 	commands_help_node =
-	  add_help_node("commands", describe_commands, 0, firsthelpnode);
+	  add_help_node("commands", describe_commands, 0, first_help_node);
 	topics_help_node =
-	  add_help_node("topics", describe_topics, 0, firsthelpnode);
+	  add_help_node("topics", describe_topics, 0, first_help_node);
 	cur_help_node = topics_help_node;
     }
+    /* By default, require manual `return' to finish each turn. */
+    set_autofinish(dside, FALSE);
 }
 
 /* Windows in curses are just simple bounding boxes. */
@@ -416,9 +450,34 @@ void
 maybe_handle_input(tmout)
 int tmout;
 {
+    Unit *unit;
+
     if (active_display(dside)) {
 	if (mode == MOVE) {
-	    make_current(autonext_unit(dside, curunit));
+	    if (curunit != NULL
+	    	&& in_play(curunit)
+		&& (curunit->act
+		&& curunit->act->acp > 0)  /* should be "above min"? */
+	    	&& (curunit->plan
+		    && !curunit->plan->asleep
+		    && !curunit->plan->reserve
+		    && !curunit->plan->delayed)
+		) {
+		/* should scroll over to unit */
+	    } else {
+		unit = autonext_unit_inbox(dside, curunit, mvp);
+		if (unit
+		    && unit->plan
+		    && !unit->plan->asleep
+		    && !unit->plan->reserve
+		    && !unit->plan->delayed
+		    && unit->plan->waitingfortasks
+		    ) {
+		    make_current(unit);
+		} else {
+		    make_current(autonext_unit(dside, curunit));
+		}
+	    }
 	}
 #ifdef HAVE_SELECT
 please somebody write this code
@@ -491,7 +550,7 @@ int dir, n;
     } else if (curunit == NULL) {
 	return;  /* no beep? */
     } else if (n > 1) {
-	set_movedir_task(curunit, dir, n);
+	set_move_dir_task(curunit, dir, n);
     } else {
 	if (point_in_dir(curx, cury, dir, &nx, &ny)) {
 	    rslt = advance_into_cell(dside, curunit, nx, ny, unit_at(nx, ny));
@@ -514,19 +573,8 @@ void
 move_survey(nx, ny)
 int nx, ny;
 {
-    Unit *unit;
-
     if (inside_area(nx, ny)) {
-	curx = nx;  cury = ny;
-	curunit = NULL;
-	/* Look for a top-level unit to become the new current one. */
-	for_all_stack(nx, ny, unit) {
-	    if (unit->side == dside) {
-		make_current(unit);
-		break;
-	    }
-	}
-	put_on_screen(curx, cury);
+	make_current_at(nx, ny);
     } else {
 	/* Complain if we're trying to move the cursor offworld. */
 	xbeep();
@@ -869,7 +917,8 @@ char *prompt;
 Unit **unitp;
 {
     char ch;
-    int dir, nx, ny;
+    int ndirs, dir, nx, ny;
+    Unit *nextocc;
 
     *unitp = NULL;
     prevmode = mode;
@@ -894,19 +943,20 @@ Unit **unitp;
 	    show_cursor();
 	    return FALSE;
 	}
+	/* Also recognize the "in" ("occupant") command. */
 	if (ch == 'i') {
 	    if (*unitp != NULL)
 	      *unitp = find_next_occupant(*unitp);
+	    make_current(*unitp);
 	    continue;
 	}
-	dir = iindex(ch, dirchars);
-	if (dir >= 0) {
+	ndirs = char_to_dir(ch, &dir, NULL, NULL);
+	if (ndirs > 0) {
 	    point_in_dir(curx, cury, dir, &nx, &ny);
 	    if (inside_area(nx, ny)) {
-		curx = nx;  cury = ny;
-		*unitp = unit_at(curx, cury);
-		put_on_screen(curx, cury);
-		show_cursor();
+		make_current_at(nx, ny);
+		if (curunit != NULL)
+		  *unitp = curunit;
 	    } else {
 		xbeep();
 	    }
@@ -931,6 +981,27 @@ Unit *unit;
     show_cursor();
 }
 
+/* Give a location, make it be current, and pick a unit there
+   to be the current unit. */
+
+void
+make_current_at(x, y)
+int x, y;
+{
+    Unit *unit;
+
+    curx = x;  cury = y;
+    curunit = NULL;
+    /* Look for a top-level unit to become the new current one. */
+    for_all_stack(x, y, unit) {
+	if (unit->side == dside) {
+	    make_current(unit);
+	    break;
+	}
+    }
+    put_on_screen(curx, cury);
+}
+
 /* Add 'P' for printing/saving to file (which?). */
 /* (Need individual screen scrolling also) */
 
@@ -1003,22 +1074,22 @@ announce_lengthy_process(msg)
 char *msg;
 {
     n_seconds_elapsed(0);
-    announcemsg = msg;  /* had better be a static string... */
+    announcemsg = copy_string(msg);
+    if (announcemsg) {
+	printf("%s;", announcemsg);
+	fflush(stdout);
+	announcemsg = NULL;
+	announced = TRUE;
+    }
 }
 
 void
 announce_progress(percentdone)
 int percentdone;
 {
-    if (n_seconds_elapsed(1)) {
-	if (announcemsg) {
-	    printf("%s; ", announcemsg);
-	    announcemsg = NULL;
-	}
-	if (percentdone > 0) {
-	    printf(" %d%%", percentdone);
-	    fflush(stdout);
-	}
+    if (n_seconds_elapsed(2)) {
+	printf(" %d%%,", percentdone);
+	fflush(stdout);
 	announced = TRUE;
     }
 }
@@ -1026,11 +1097,8 @@ int percentdone;
 void
 finish_lengthy_process()
 {
-    if (announced || n_seconds_elapsed(1)) {
-	if (announcemsg) {
-	    printf("%s; ", announcemsg);
-	}
-	printf("; done.\n");
+    if (announced) {
+	printf(" done.\n");
 	announced = FALSE;
     }
 }
@@ -1039,12 +1107,12 @@ finish_lengthy_process()
    game hacking has been taken care of already. */
 
 void
-exit_cconq(side)
-Side *side;
+exit_cconq()
 {
     close_displays();
     /* Leave any commentary on the console. */
-    printf("%s\n", exit_commentary(side));
+    printf("\n%s\n", exit_commentary(dside));
+    printf("\n%s\n", get_scores(dside));
     exit(0);
 }
 
@@ -1137,6 +1205,9 @@ int rightnow;
 	show_game_date();
 	if (rightnow)
 	  refresh();
+	if (g_turn() >= autofinish_start + autofinish_count) {
+	    set_autofinish(dside, FALSE);
+	}
     }
 }
 
@@ -1318,10 +1389,15 @@ char *str;
 }
 
 void
-printlisp(obj)
-Obj *obj;
+print_form(form)
+Obj *form;
+{
+	print_form_and_value(stdout, form);
+}
+
+void
+end_printing_forms()
 {
-    fprintlisp(stdout, obj);
 }
 
 /* Even a curses interface can do simple "movies". */
@@ -1352,3 +1428,24 @@ Side *side;
 
   side->uidata = replace_at_key(side->uidata, "curses", state);
 }
+
+void
+low_send(id, buf)
+int id;
+char *buf;
+{
+}
+
+int
+low_receive(id, buf, maxchars, timeout)
+int *id, maxchars, timeout;
+char *buf;
+{
+  return 0;
+}
+
+void
+make_generic_image_data(imf)
+ImageFamily *imf;
+{
+}
