@@ -38,6 +38,14 @@ int use_both_chars = TRUE;
 
 int use_standout = FALSE;
 
+int drawlinear = TRUE;
+
+char linear_char = '#';
+
+char bord_char = '\0';
+
+char conn_char = '\0';
+
 int listtype = 0;
 
 int firstvisible = 0;
@@ -123,7 +131,8 @@ void
 set_scroll()
 {
     int sx, sy, hexadj = hexagon_adjust(mvp);
- 
+
+    sx = mvp->sx;  sy = mvp->sy; 
     if (mw < (mvp->totsw - hexadj))
       sx = max(sx, hexadj);
     else
@@ -262,11 +271,11 @@ int x0, y0, len;
 	    ch = terrchars[t];
 	    ch2 = (use_both_chars ? ch : ' ');
 	    /* Just provide a hint at presence of aux terrain. */
-	    if (any_aux_terrain_defined()) {
+	    if (drawlinear && any_aux_terrain_defined()) {
 		for_all_terrain_types(t2) {
 		    if (aux_terrain_defined(t2)) {
 			if (aux_terrain_at(x, y0, t2)) {
-			    ch2 = '#';
+			    ch2 = linear_char;
 			    break;
 			}
 		    }
@@ -315,6 +324,8 @@ int x, y;
 	xform(x, y, &sx, &sy);
 	if (cur_at(mapwin, sx, sy)) {
 	    addch(unitchars[u]);
+	}
+	if (cur_at(mapwin, sx + 1, sy)) {
 	    if (between(1, s, 9))
 	      addch(s + '0');
 	    else if (s >= 10)
@@ -407,12 +418,12 @@ char *str;
 	strcpy(extrabuf, str);
 	lastblank = strchr(extrabuf + toplineswin->w - 10, ' '); 
 	if (lastblank) {
-	    sprintf(text2, lastblank + 1);
+	    strcpy(text2, lastblank + 1);
 	    *lastblank = '\0';
 	}
-	sprintf(text1, "%s", extrabuf);
+	strcpy(text1, extrabuf);
     } else {
-	sprintf(text1, "%s", str);
+	strcpy(text1, str);
     }
     DGprintf("%s\n", text1);
     DGprintf("%s\n", text2);
@@ -669,15 +680,11 @@ show_side_list()
 void
 show_game_date()
 {
-#if 0
     clear_window(datewin);
     /* First line of the game state. */
     /* (should cache this date string a la Mac version?) */
-    sprintf(spbuf, "%s", absolute_date_string(g_turn()));
-    draw_text(datewin, 0, 0, spbuf);
+    draw_text(datewin, 1, 0, absolute_date_string(g_turn()));
     /* (should use second line for something) */
-    refresh();
-#endif
 }
 
 /* General list display routine. */
