@@ -1,5 +1,5 @@
 /* Implementations of Xconq actions.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -17,22 +17,30 @@ any later version.  See the file COPYING.  */
    but should never call do_foo_action directly. */
 
 #include "conq.h"
+extern int tmphevtdata1;
+extern int construction_possible PARAMS ((int u2));
+extern void broadcast_next_action PARAMS ((Unit *unit));
+extern void low_send PARAMS ((int id, char *buf));
+extern void add_to_unit_hp PARAMS ((Unit *unit, int hp));
+extern void try_sharing PARAMS ((Unit *from, Unit *to, int m));
 
-#define angle_with(d1, d2)  \
-  min((d1-d2+NUMDIRS)%NUMDIRS, (d2-d1+NUMDIRS)%NUMDIRS)
+extern void change_cell PARAMS ((Unit *unit, int x, int y));
+
+extern int max_detonate_on_approach_range;
+extern int max_u_detonate_effect_range;
+extern int max_t_detonate_effect_range;
+
+int numremotes;
+
+extern int number_member PARAMS ((int x, Obj *lis));
+extern int wind_value PARAMS ((Unit *unit, int angle, int force, Obj *effect, int maxval));
 
 static void try_detonate_on_approach PARAMS ((int x, int y));
-
 static void set_created_unit_props PARAMS ((Unit *unit, int u2, Side *side));
-
 static void play_action_messages PARAMS ((Unit *unit, Action *action));
 
-/* Declare the do_xxx_action functions as static, partly to keep them from
-   conflicting with do_xxx commands, and partly because they shouldn't be
-   visible everywhere. */
-
-/* (Alas, xrefs between actions.c and combat.c preclude the static decl,
-   but still don't declare the do_... functions in a globally visible place.) */
+/* We can't declare all the action functions as static because some of them
+   are in other files, but don't let them be visible to all files. */
 
 #undef  DEF_ACTION
 #define DEF_ACTION(name,code,args,prepfn,DOFN,checkfn,ARGDECL,doc)  \
@@ -71,6 +79,34 @@ init_actions()
 {
 }
 
+void
+broadcast_next_action(unit)
+Unit *unit;
+{
+    int i, j, atype, n;
+    char buf[BUFSIZE], smallbuf[32];
+
+    if (numremotes <= 0)
+      return;
+    atype = unit->act->nextaction.type;
+    sprintf(buf, "A %d", unit->id);
+    if (unit->id != unit->act->nextaction.actee) {
+	sprintf(smallbuf, "/%d", unit->act->nextaction.actee);
+	strcat(buf, smallbuf);
+    }
+    sprintf(smallbuf, " %d", unit->act->nextaction.type);
+    strcat(buf, smallbuf);
+    n = strlen(actiondefns[atype].argtypes);
+    for (j = 0; j < n; ++j) {
+	sprintf(smallbuf, " %d", unit->act->nextaction.args[j]);
+	strcat(buf, smallbuf);
+    }
+    /* Send the formatted description of the action to all the remotes. */
+    for (i = 1; i < numremotes + 1; ++i) {
+	low_send(i, buf);
+    }
+}
+
 /* Just a placeholder action, so not much to do here. */
 
 int
@@ -86,14 +122,14 @@ Unit *unit, *unit2;
     return TRUE;
 }
 
-/* static */ int
+int
 do_none_action(unit, unit2)
 Unit *unit, *unit2;
 {
     return A_ANY_DONE;
 }
 
-/* static */ int
+int
 check_none_action(unit, unit2)
 Unit *unit, *unit2;
 {
@@ -118,6 +154,7 @@ int x, y, z;
     unit->act->nextaction.args[1] = y;
     unit->act->nextaction.args[2] = z;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
@@ -126,7 +163,7 @@ int x, y, z;
    destination cell, will not board, attack, etc - all that is task- and
    plan-level behavior. */
 
-/* static */ int
+int
 do_move_action(unit, unit2, nx, ny, nz)
 Unit *unit, *unit2;
 int nx, ny, nz;
@@ -173,7 +210,7 @@ int nx, ny, nz;
 	speed = unit_speed(unit2, nx, ny);
 	mpcost = move_unit(unit2, nx, ny);
 	/* ZOC move cost is added after action is done. */
-    	mpcost += zoc_move_cost(unit2, ox, oy, oz);
+	mpcost += zoc_move_cost(unit2, ox, oy, oz);
 	rslt = A_ANY_DONE;
     }
     if (alive(unit)) {
@@ -245,7 +282,9 @@ int nx, ny, nz;
     totcost = total_move_cost(u2, u3, ox, oy, oz, nx, ny, nz);
     speed = unit_speed(unit2, nx, ny);
     mpavail = (unit->act->acp * speed) / 100;
-    /* take into account acp-min in computing mpavail; Massimo */ if (u_acp_min(u2)<0)   mpavail = ((unit->act->acp - u_acp_min(u2)) * speed) / 100;
+    /* take into account acp-min in computing mpavail; Massimo */
+    if (u_acp_min(u2) < 0)
+      mpavail = ((unit->act->acp - u_acp_min(u2)) * speed) / 100;
     /* Zero mp always disallows movement, unless intra-cell. */
     if (mpavail <= 0 && !(ox == nx && oy == ny && oz == nz))
       return A_MOVE_NO_MP;
@@ -290,47 +329,12 @@ int nx, ny;
     return FALSE;
 }
 
-/* (should move to unit.c) */
-
-int
-can_occupy_conn(unit, nx, ny, nz)
-Unit *unit;
-int nx, ny, nz;
-{
-    int c, dir, numhere;
-    Unit *unit2;
-
-    /* (this never happens maybe?) */
-    if (numconntypes == 0)
-      return FALSE;
-    c = nz / 2;
-    if (t_is_connection(c)
-	&& aux_terrain_defined(c)
-	&& (dir = closest_dir(nx - unit->x, ny - unit->y)) >= 0
-	&& connection_at(unit->x, unit->y, dir, c)) {
-	numhere = 0;
-	for_all_stack(nx, ny, unit2) {
-	    if (unit->z == nz) {
-		++numhere;
-	    }
-	}
-	if (numhere + 1 <= ut_capacity_x(unit->type, c))
-	  return FALSE;
-	/* (should also calc general fullness) */
-	return FALSE;
-    } else {
-	run_warning("%s is on an invalid connection type %d?",
-		    unit_desig(unit), c);
-    }
-    return TRUE;
-}
-
 int
 unit_speed(unit, nx, ny)
 Unit *unit;
 int nx, ny;
 {
-    int u = unit->type, speed, x = unit->x, y = unit->y /*, angle */;
+    int u = unit->type, speed, x = unit->x, y = unit->y, angle, windval;
     int occeff, totocceff, totoccdenom;
     Unit *occ;
     
@@ -339,14 +343,10 @@ int nx, ny;
 	speed = damaged_value(unit, u_speed_damage_effect(u), speed);
     }
     if (winds_defined() && u_speed_wind_effect(u) != lispnil) {
-	speed *= wind_force_at(x, y);
-    }
-#if 0
-    if (winds_defined() && u_speed_wind_angle_effect(u) != lispnil) {
 	angle = angle_with(closest_dir(nx - x, nx - y), wind_dir_at(x, y));
-	speed += angle * wind_force_at(x, y);
+	windval = wind_value(unit, angle, wind_force_at(x, y), u_speed_wind_effect(u), 10000);
+	speed = (speed * windval) / 100;
     }
-#endif
     if (unit->occupant /* and any occupant speed effects */) {
         totocceff = 100;
         totoccdenom = 100;
@@ -367,7 +367,7 @@ int nx, ny;
     return speed;
 }
 
-/* Compute and return the acp of a damaged unit, using a list of (hp acp) pairs
+/* Compute and return value for a damaged unit, using a list of (hp val) pairs
    and interpolating between them. */
 
 int
@@ -376,9 +376,74 @@ Unit *unit;
 Obj *effect;
 int maxval;
 {
-    return interpolate_in_list(unit->hp, effect, FALSE, maxval);
+    int u, err, rslt;
+
+    u = unit->type;
+    err = interpolate_in_list_ext(unit->hp, effect, 0, 0, 0, 0, u_hp(u), maxval, &rslt);
+    if (err != 0) {
+	run_warning("cannot get damaged speed for %s at hp %d, using 100",
+		    u_type_name(u), unit->hp);
+	rslt = 100;
+    }
+    return rslt;
 }
 
+/* Compute and return the wind's effect on a unit, using a list of lists
+   and interpolating between them. */
+
+int
+wind_value(unit, angle, force, effect, maxval)
+Unit *unit;
+int angle, force, maxval;
+Obj *effect;
+{
+    int err, rslt;
+    Obj *rest, *head, *key, *val;
+
+    for (rest = effect; rest != lispnil; rest = cdr(rest)) {
+	head = car(rest);
+	key = car(head);
+	if ((numberp(key) && angle == c_number(key))
+	    || (symbolp(key))
+	    || (consp(key) && number_member(angle, key))) {
+	    val = cadr(head);
+	    if (numberp(val))
+	      return c_number(val);
+	    else {
+		err = interpolate_in_list(force, val, &rslt);
+		if (err == 0) {
+		    return rslt;
+		} else {
+		    run_warning("no value for wind angle=%d force=%d", angle, force);
+		    return maxval;
+		}
+	    }
+	}
+    }
+    return maxval;
+}
+
+int
+number_member(x, lis)
+int x;
+Obj *lis;
+{
+    Obj *rest;
+
+    if (lis == lispnil) {
+	return FALSE;
+    } else if (!consp(lis)) {
+	/* should probably be an error of some sort */
+	return FALSE;
+    }
+    for (rest = lis; rest != lispnil; rest = cdr(rest)) {
+	if (numberp(car(rest)) && x == c_number(car(rest)))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+
 /* Conduct the actual move (used in both normal moves and some combat).
    Note that the new x,y may be the same as the old; this will happen
    if an occupant is getting off a transport but staying in the same cell. */
@@ -390,13 +455,15 @@ int nx, ny;
 {
     int u = unit->type, u3, ox = unit->x, oy = unit->y, oz = unit->z;
     int nz = oz;
-    extern int max_detonate_on_approach_range;
 
     u3 = (unit->transport ? unit->transport->type : NONUTYPE);
-    /* Disappear from the old location ... */
-    leave_cell(unit);
-    /* ... and appear in the new one! */
-    enter_cell(unit, nx, ny);
+    /* Disappear from the old location and appear at the new one. */
+    if (unit->transport == NULL /* should be unconditional, but bugs still */)
+      change_cell(unit, nx, ny);
+    else {
+      leave_cell(unit);
+      enter_cell(unit, nx, ny);
+    }
     /* Movement may set off other people's alarms. */
     maybe_react_to_move(unit, ox, oy);
     /* Might have auto-detonations in response. */
@@ -441,7 +508,9 @@ int x, y, z;
       return FALSE;
     if (max_zoc_range >= 0) {
 	for_all_stack(x, y, unit2) {
-	    if (unit_blockable_by(unit, unit2)
+	    range = zoc_range(unit2, u);
+	    if (range >= 0
+		&& unit_blockable_by(unit, unit2)
 		&& ut_zoc_into(unit2->type, t)
 		&& ut_zoc_from_terrain(unit2->type, t) > 0)
 	      return TRUE;
@@ -453,8 +522,8 @@ int x, y, z;
 		for_all_stack(x, y, unit2) {
 		    u2 = unit2->type;
 		    range = zoc_range(unit2, u);
-		    if (unit_blockable_by(unit, unit2)
-			&& range > 1
+		    if (range >= 1
+			&& unit_blockable_by(unit, unit2)
 			&& ut_zoc_into(u2, t))
 		      return TRUE;
 		}
@@ -473,7 +542,7 @@ int
 unit_blockable_by(unit, unit2)
 Unit *unit, *unit2;
 {
-    return (unit->side != unit2->side /* should make a better test */
+    return (!trusted_side(unit2->side, unit2->side) /* should make a better test */
 	    && uu_mp_to_enter_zoc(unit->type, unit2->type) < 0);
 }
 
@@ -704,9 +773,6 @@ detonate_on_approach_around(unit)
 Unit *unit;
 {
     int maxrange;
-    extern int max_u_detonate_effect_range;
-    extern int max_t_detonate_effect_range;
-    extern int max_detonate_on_approach_range;
 
     tmpunit = unit;
     apply_to_area(unit->x, unit->y, max_detonate_on_approach_range, try_detonate_on_approach);
@@ -764,10 +830,11 @@ Unit *unit, *unit2, *unit3;
     unit->act->nextaction.type = ACTION_ENTER;
     unit->act->nextaction.args[0] = unit3->id;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_enter_action(unit, unit2, unit3)
 Unit *unit, *unit2, *unit3;
 {
@@ -964,10 +1031,11 @@ int m, n;
     unit->act->nextaction.args[0] = m;
     unit->act->nextaction.args[1] = n;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_produce_action(unit, unit2, m, n)
 Unit *unit, *unit2;
 int m, n;
@@ -1030,10 +1098,11 @@ int m, n;
     unit->act->nextaction.args[1] = n;
     unit->act->nextaction.args[2] = unit3->id;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_transfer_action(unit, unit2, m, n, unit3)
 Unit *unit, *unit2, *unit3;
 int m, n;
@@ -1138,10 +1207,11 @@ int u3;
     unit->act->nextaction.type = ACTION_RESEARCH;
     unit->act->nextaction.args[0] = u3;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_research_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
@@ -1230,10 +1300,11 @@ int u3;
     unit->act->nextaction.type = ACTION_TOOL_UP;
     unit->act->nextaction.args[0] = u3;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_toolup_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
@@ -1328,10 +1399,11 @@ int u3;
     unit->act->nextaction.args[0] = u3;
     unit->act->nextaction.args[1] = dest->id;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_create_in_action(unit, unit2, u3, dest)
 Unit *unit, *unit2, *dest;
 int u3;
@@ -1348,7 +1420,7 @@ int u3;
 	enter_transport(newunit, dest);
 	/* Unit might have started out complete. */
 	if (completed(newunit)) {
-    	    garrison_unit(newunit, unit2);
+	    garrison_unit(unit2, newunit);
 	    make_unit_complete(newunit);
 	} else {
 	    record_event(H_UNIT_CREATED, add_side_to_set(unit2->side, NOSIDES),
@@ -1461,10 +1533,11 @@ int u3, x, y, z;
     unit->act->nextaction.args[2] = y;
     unit->act->nextaction.args[3] = z;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_create_at_action(unit, unit2, u3, x, y, z)
 Unit *unit, *unit2;
 int u3, x, y, z;
@@ -1493,7 +1566,7 @@ int u3, x, y, z;
 	/* and set its altitude? */
 	/* Unit might be complete right away. */
 	if (completed(newunit)) {
-    	    garrison_unit(newunit, unit2);
+    	    garrison_unit(unit2, newunit);
 	    make_unit_complete(newunit);
 	} else {
 	    record_event(H_UNIT_CREATED, add_side_to_set(unit2->side, NOSIDES),
@@ -1539,6 +1612,7 @@ int u3, x, y, z;
 	if (unit->side == NULL)
 	  return A_ANY_ERROR;
 	if (unit->side->tech[u3] < u_tech_to_build(u3))
+	  /* (should have an "insufficient tech" error for this case) */
 	  return A_ANY_ERROR;
     }
     if (distance(unit2->x, unit2->y, x, y) > uu_create_range(u2, u3))
@@ -1546,6 +1620,7 @@ int u3, x, y, z;
     /* Check the tooling. */
     tp = (unit2->tooling ? unit2->tooling[u3] : 0);
     if (tp < uu_tp_to_build(u2, u3))
+      /* (should have an "insufficient tooling" error for this case) */
       return A_ANY_ERROR;
     if (unit2->transport != NULL
         && !uu_occ_can_build(unit2->transport->type, u2))
@@ -1585,10 +1660,11 @@ Unit *unit, *unit2, *newunit;
     unit->act->nextaction.type = ACTION_BUILD;
     unit->act->nextaction.args[0] = newunit->id;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_build_action(unit, unit2, newunit)
 Unit *unit, *unit2, *newunit;
 {
@@ -1602,7 +1678,7 @@ Unit *unit, *unit2, *newunit;
     }
     newunit->cp += uu_cp_per_build(u2, u3);
     if (completed(newunit)) {
-    	garrison_unit(newunit, unit2);
+    	garrison_unit(unit2, newunit);
 	make_unit_complete(newunit);
     }
     update_unit_display(newunit->side, newunit, TRUE);
@@ -1632,33 +1708,46 @@ int u2, u3, m, cp;
 }
 
 void
-garrison_unit(newunit, unit2)
-Unit *newunit, *unit2;
+garrison_unit(unit, unit2)
+Unit *unit, *unit2;
 {
-    int u2 = unit2->type, u3 = newunit->type, x = unit2->x, y = unit2->y;
-    Unit *transport = NULL, *occ;
+    int u = unit->type, u2 = unit2->type, x = unit->x, y = unit->y;
+    Unit *transport = NULL, *occ, *nextocc;
 
     /* Maybe get rid of the building unit if it is to be the garrison. */
-    if (uu_hp_to_garrison(u2, u3) >= unit2->hp) {
+    if (uu_hp_to_garrison(u, u2) >= unit->hp) {
 	/* But first get the about-to-be-killed garrisoning unit
 	   disconnected from everything. */
-	leave_cell(unit2);
-	/* Put new unit in place of the builder, if it was an occupant. */
-	if (newunit->transport == unit2) {
-	    leave_transport(newunit);
+	leave_cell(unit);
+	/* Put new unit in place of the garrisoning one, if it was an occupant. */
+	if (unit2->transport == unit) {
+	    leave_transport(unit2);
 	    if (transport != NULL) { /* some other unit that could be transport? */
-		enter_transport(newunit, transport);
+		enter_transport(unit2, transport);
 	    } else {
-		enter_cell(newunit, x, y);
+		enter_cell(unit2, x, y);
 	    }
 	}
-	/* Move the other occupants anywhere we can find. */
-	for_all_occupants(unit2, occ) {
-	    if (can_occupy(occ, newunit)) {
-		enter_transport(occ, newunit);
+
+	if (unit2->transport != NULL && unit2->transport != unit)
+	  transport = unit2->transport;
+	/* for_all_occupants will not work here, 
+	   since leave_transport changes occ->nexthere */
+	for (occ = unit->occupant; occ != NULL; occ = nextocc) {
+	    nextocc = occ->nexthere;
+	    /* Move the other occupants anywhere we can find. */
+	    if (can_occupy(occ, unit2)) {
+		/* leave_cell won't work here, since "unit" already left cell */
+		leave_transport(occ);
+		update_unit_display(unit->side, unit, TRUE);
+		enter_transport(occ, unit2);
 	    } else if (transport != NULL && can_occupy(occ, transport)) {
+		leave_transport(occ);
+		update_unit_display(unit->side, unit, TRUE);
 		enter_transport(occ, transport);
 	    } else if (can_occupy_cell(occ, x, y)) {
+		leave_transport(occ);
+		update_unit_display(unit->side, unit, TRUE);
 		enter_cell(occ, x, y);
 	    }
 	    /* Otherwise the occupant has to die along with the garrison. */
@@ -1666,11 +1755,14 @@ Unit *newunit, *unit2;
 	}
 	/* Now we can get rid of the garrisoning unit without scrambling
 	   anything else. */
-	kill_unit(unit2, H_UNIT_GARRISONED);
+	tmphevtdata1 = unit2->id;
+	kill_unit(unit, H_UNIT_GARRISONED);
     } else {
-	/* Builder just loses hp, doesn't vanish. */
-	unit2->hp -= uu_hp_to_garrison(u2, u3);
-	unit2->hp2 = unit2->hp;
+	/* Note that if this all happens before damage is reckoned,
+	   hp and hp2 might be different. */
+	unit->hp -= uu_hp_to_garrison(u, u2);
+	unit->hp2 -= uu_hp_to_garrison(u, u2);
+	/* (should record loss of hp as garrison event?) */
     }
 }
 
@@ -1744,17 +1836,23 @@ Unit *unit;
     /* Christen our new unit. Its serial number (if it is a type that has
        one) was assigned just after its creation. */
     make_up_unit_name(unit);
-    /* It also effectively starts viewing its surroundings. */
+    /* It also starts viewing its surroundings. */
     if (unit->transport == NULL
 	|| uu_occ_can_see(unit->type, unit->transport->type)) {
-	cover_area(unit->side, unit, unit->x, unit->y, 1);
+	cover_area(unit->side, unit, -1, -1, unit->x, unit->y);
     }
     /* Set all the supplies up to their unit-just-completed levels. */
     for_all_material_types(m) {
 	unit->supply[m] = max(unit->supply[m], um_completed_supply(u, m));
 	unit->supply[m] = min(unit->supply[m], um_storage_x(u, m));
     }
-    init_unit_actorstate(unit);
+    /* Also see if anybody here is willing to share to make up any
+       deficiencies before the end of the turn. */
+    for_all_material_types(m) {
+	if (unit->transport)
+	  try_sharing(unit->transport, unit, m);
+    }
+    init_unit_actorstate(unit, FALSE);
     init_unit_plan(unit);
     /* Put this unit into action immediately, at full acp. */
     if (unit->act) {
@@ -1762,9 +1860,6 @@ Unit *unit;
 	if (unit->act->initacp > 0) {
 	    side = (unit->side ? unit->side : indepside);
 	    side->actionvector = add_unit_to_vector(side->actionvector, unit, 0);
-	} else {
-	    /* don't give a newly created unit negative acp amd initacp; Massimo */
-	    unit->act->initacp = unit->act->acp = 0;
 	}
     }
     observers = add_side_to_set(unit->side, NOSIDES);
@@ -1789,10 +1884,11 @@ Unit *unit, *unit2, *unit3;
     unit->act->nextaction.type = ACTION_REPAIR;
     unit->act->nextaction.args[0] = unit3->id;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_repair_action(unit, unit2, unit3)
 Unit *unit, *unit2, *unit3;
 {
@@ -1801,8 +1897,7 @@ Unit *unit, *unit2, *unit3;
     u = unit->type;  u2 = unit2->type;  u3 = unit3->type;
     rep = uu_repair(u2, u3);
     /* Add to the repairee's hit points. */
-    unit3->hp += prob_fraction(rep);
-    unit3->hp2 = unit3->hp;
+    add_to_unit_hp(unit3, prob_fraction(rep));
     /* Eat supplies used up by repair. */
     for_all_material_types(m) {
 	unit2->supply[m] -= um_consumption_per_repair(u3, m);
@@ -1861,10 +1956,11 @@ Unit *unit, *unit2;
       return FALSE;
     unit->act->nextaction.type = ACTION_DISBAND;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_disband_action(unit, unit2)
 Unit *unit, *unit2;
 {
@@ -1963,10 +2059,11 @@ int parts;
     unit->act->nextaction.args[0] = parts;
     unit->act->nextaction.args[1] = (unit3 ? unit3->id : 0);
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_transfer_part_action(unit, unit2, parts, unit3)
 Unit *unit, *unit2, *unit3;
 int parts;
@@ -2003,10 +2100,7 @@ int parts;
 	}
     } else {
 	/* Increase the unit3's hp by what's in this unit, and cap it. */
-	unit3->hp += parts * part_hp;
-	/* Should affect morale etc according to proportions of mixing */
-	unit3->hp = min(unit3->hp, u_hp(unit3->type));
-	unit3->hp2 = unit3->hp;
+	add_to_unit_hp(unit3, parts * part_hp);
     }
     /* Need to tweak parts in unit2 also */
     if (parts * part_hp >= unit2->hp) {
@@ -2065,12 +2159,13 @@ int u3;
     unit->act->nextaction.type = ACTION_CHANGE_TYPE;
     unit->act->nextaction.args[0] = u3;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
 /* Actually change the type of a unit. */
 
-/* static */ int
+int
 do_change_type_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
@@ -2135,10 +2230,11 @@ Side *side;
     unit->act->nextaction.type = ACTION_CHANGE_SIDE;
     unit->act->nextaction.args[0] = side_number(side);
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_change_side_action(unit, unit2, side)
 Unit *unit, *unit2;
 Side *side;
@@ -2211,10 +2307,11 @@ int x, y, t;
     unit->act->nextaction.args[1] = y;
     unit->act->nextaction.args[2] = t;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_alter_cell_action(unit, unit2, x, y, t)
 Unit *unit, *unit2;
 int x, y, t;
@@ -2225,15 +2322,7 @@ int x, y, t;
     u = unit->type;  u2 = unit2->type;
     oldt = terrain_at(x, y);
     /* Change the terrain to the new type. */
-    set_terrain_at(x, y, t);
-    /* Let everybody see what has happened. */
-    if (t != oldt) {
-	for_all_sides(side) {
-    	    if (g_see_all() || g_see_terrain_always() || side == unit->side) {
-		update_cell_display(side, x, y, TRUE);
-    	    }
-	}
-    }
+    change_terrain_type(x, y, t);
     /* Note that we still charge acp even if terrain type doesn't change. */
     acpr = ut_acp_to_remove_terrain(u2, oldt);
     acpa = ut_acp_to_add_terrain(u2, t);
@@ -2293,10 +2382,11 @@ int x, y, dir, t;
     unit->act->nextaction.args[2] = dir;
     unit->act->nextaction.args[3] = t;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_add_terrain_action(unit, unit2, x, y, dir, t)
 Unit *unit, *unit2;
 int x, y, dir, t;
@@ -2326,16 +2416,16 @@ int x, y, dir, t;
       	break;
     }
     /* Let everybody see what has happened. */
-    if (newval != oldval) {
-      for_all_sides(side) {
-    	if (g_see_all() || g_see_terrain_always() || side == unit->side) {
-    	    update_cell_display(side, x, y, TRUE);
-    	    if (t_subtype(t) != coatingsubtype) {
-		if (point_in_dir(x, y, dir, &x1, &y1))
+    for_all_sides(side) {
+	if (active_display(side)) {
+	    if (terrain_visible(side, x, y)) {
+		update_cell_display(side, x, y, TRUE);
+	    }
+	    if (t_subtype(t) != coatingsubtype) {
+		if (point_in_dir(x, y, dir, &x1, &y1) && terrain_visible(side, x1, y1))
 		  update_cell_display(side, x1, y1, TRUE);
     	    }
     	}
-      }
     }
     use_up_acp(unit, (newval != oldval ? ut_acp_to_add_terrain(u, t) : 1));
     return A_ANY_DONE;
@@ -2393,10 +2483,11 @@ int x, y, dir, t;
     unit->act->nextaction.args[2] = dir;
     unit->act->nextaction.args[3] = t;
     unit->act->nextaction.actee = unit2->id;
+    broadcast_next_action(unit);
     return TRUE;
 }
 
-/* static */ int
+int
 do_remove_terrain_action(unit, unit2, x, y, dir, t)
 Unit *unit, *unit2;
 int x, y, dir, t;
@@ -2430,15 +2521,17 @@ int x, y, dir, t;
     }
     /* Let everybody see what has happened. */
     for_all_sides(side) {
-    	if (g_see_all() || g_see_terrain_always() || side == unit->side) {
-    	    update_cell_display(side, x, y, TRUE);
-    	    if (t_subtype(t) != coatingsubtype) {
-		if (point_in_dir(x, y, dir, &x1, &y1))
+	if (active_display(side)) {
+	    if (terrain_visible(side, x, y)) {
+		update_cell_display(side, x, y, TRUE);
+	    }
+	    if (t_subtype(t) != coatingsubtype) {
+		if (point_in_dir(x, y, dir, &x1, &y1) && terrain_visible(side, x1, y1))
 		  update_cell_display(side, x1, y1, TRUE);
     	    }
     	}
     }
-    use_up_acp(unit, ut_acp_to_remove_terrain(u, t));
+    use_up_acp(unit, (newval != oldval ? ut_acp_to_remove_terrain(u, t) : 1));
     return A_ANY_DONE;
 }
 
@@ -2610,7 +2703,7 @@ Action *action;
 	   each side to see if they notice.  Note that the type is from
 	   *before* the action, not after (some actions may cause type changes). */
 	if (u_spot_action(u)
-	    && !g_see_all()
+	    && !all_see_all
 	    && !u_see_always(u)
 	    && in_area(unit->x, unit->y)) {
 	    for_all_sides(side2) {
@@ -2645,9 +2738,9 @@ Action *action;
 {
     int found = FALSE;
     char *soundname;
-    Obj *rest, *head, *parms;
+    Obj *rest, *head, *parms, *msgdesc;
 
-    for (rest = g_action_messages(); rest != lispnil; rest = cdr(rest)) {
+    for_all_list(g_action_messages(), rest) {
 	head = car(rest);
 	if (consp(head)
 	    && symbolp(car(head))
@@ -2666,7 +2759,7 @@ Action *action;
 		found = TRUE;
 		break;
 	    }
-	    if (!((symbolp(car(parms))
+	    if (((symbolp(car(parms))
 		   && strcmp(c_string(car(parms)),
 			     u_type_name(unit->type)) == 0)
 		  || match_keyword(car(parms), K_USTAR)
@@ -2679,27 +2772,25 @@ Action *action;
 		              && c_number(symbol_value(car(parms)))
 			      == unit->type)))
 		  )) {
-		continue;
-	    }
-	    if (parms == lispnil) {
 		found = TRUE;
 		break;
 	    }
-	    /* (should be able to match on particular action parmss also) */
+	    /* (should be able to match on particular action parms also) */
 	}
     }
+    /* If we have a match, do something with it. */
     if (found) {
-	if (stringp(cadr(head))) {
-	    notify(unit->side, "%s", c_string(cadr(head)));
-	} else if (consp(cadr(head))
-		   && symbolp(car(cadr(head)))
-		   && strcmp(c_string(car(cadr(head))), "sound") == 0
-		   && stringp(cadr(cadr(head)))) {
-	    soundname = c_string(cadr(cadr(head)));
+	msgdesc = cadr(head);
+	if (stringp(msgdesc)) {
+	    notify(unit->side, "%s", c_string(msgdesc));
+	} else if (consp(msgdesc)
+		   && symbolp(car(msgdesc))
+		   && strcmp(c_string(car(msgdesc)), "sound") == 0
+		   && stringp(cadr(msgdesc))) {
+	    soundname = c_string(cadr(msgdesc));
 	    /* (should not be passing ptrs to schedule_movie) */
 	    schedule_movie(unit->side, movie_extra_0, soundname);
 	    play_movies(add_side_to_set(unit->side, NOSIDES));
-	    return;
 	} else {
 	}
     }
@@ -2727,6 +2818,8 @@ has_enough_acp(unit, acp)
 Unit *unit;
 int acp;
 {
+    if (unit->act == NULL)
+      return FALSE;
     return ((unit->act->acp - acp) >= u_acp_min(unit->type));
 }
 
@@ -2824,7 +2917,8 @@ int u;
     int u2;
 	
     for_all_unit_types(u2) {
-	if (uu_acp_to_create(u, u2) > 0)
+	if (uu_acp_to_create(u, u2) > 0
+	    && uu_tp_max(u, u2) >= uu_tp_to_build(u, u2))
 	  return TRUE;
     }
     return FALSE;
@@ -2844,7 +2938,8 @@ int u;
     int u2;
 	
     for_all_unit_types(u2) {
-	if (uu_acp_to_build(u, u2) > 0)
+	if (uu_acp_to_build(u, u2) > 0
+	    && uu_tp_max(u, u2) >= uu_tp_to_build(u, u2))
 	  return TRUE;
     }
     return FALSE;
@@ -2987,6 +3082,19 @@ any_construction_possible()
     return any_construction;
 }
 
+int
+construction_possible(u2)
+int u2;
+{
+    int u;
+
+    for_all_unit_types(u) {
+	if (uu_acp_to_create(u, u2) > 0
+	    && uu_tp_max(u, u2) >= uu_tp_to_build(u, u2))
+	  return TRUE;
+    }
+}
+
 /* Compose a legible description of a given action. */
 
 char *
