@@ -1,5 +1,5 @@
 /* Functions common to all AIs.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,8 +7,14 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+
+/* The number of AI types defined.  Increment for each new AI type
+   that is compiled in. */
+
 #define numaitypes 2
 
+/* Declarations of all AI types. */
+
 extern AI_ops mplayer_ops;
 
 AI_ops **all_ai_ops;
@@ -16,6 +22,7 @@ AI_ops **all_ai_ops;
 void
 init_ai_types()
 {
+    /* Fill in the table of pointers to AI types. */
     all_ai_ops = (AI_ops **) xmalloc(numaitypes * sizeof(AI_ops *));
     all_ai_ops[0] = (AI_ops *) xmalloc(sizeof(AI_ops));
     all_ai_ops[1] = &mplayer_ops;
@@ -157,15 +164,40 @@ ai_react_to_task_result(side, unit, task, rslt)
 Side *side;
 Unit *unit;
 Task *task;
-int rslt;
+TaskOutcome rslt;
 {
-    void (*fn) PARAMS ((Side *side, Unit *unit, Task *task, int rslt));
+    void (*fn) PARAMS ((Side *side, Unit *unit, Task *task, TaskOutcome rslt));
 
     fn = ((all_ai_ops[side->aitype])->to_react_to_task_result);
     if (fn)
       (*fn)(side, unit, task, rslt);
 }
 
+void
+ai_react_to_new_side(side, side2)
+Side *side, *side2;
+{
+    void (*fn) PARAMS ((Side *side, Side *side2));
+
+    fn = ((all_ai_ops[side->aitype])->to_react_to_new_side);
+    if (fn)
+      (*fn)(side, side2);
+}
+
+int
+ai_planning_to_capture(side, u, x, y)
+Side *side;
+int u, x, y;
+{
+    int (*fn) PARAMS ((Side *side, int u, int x, int y));
+
+    fn = ((all_ai_ops[side->aitype])->planning_to_capture);
+    if (fn)
+      return (*fn)(side, u, x, y);
+    else
+      return FALSE;
+}
+
 int
 ai_guide_explorer(side, unit)
 Side *side;
@@ -241,6 +273,9 @@ Side *side;
     fn = ((all_ai_ops[side->aitype])->to_save_state);
     if (fn)
       state = (*fn)(side);
+    /* Don't bother if there's no AI state to mess with. */
+    if (side->aidata == lispnil && state == lispnil)
+      return;
     side->aidata = replace_at_key(side->aidata, side->player->aitypename, state);
 }
 
@@ -307,7 +342,7 @@ int
 cell_unknown(x, y)
 int x, y;
 {
-    return (!all_see_all && terrain_view(tmpside, x, y) == UNSEEN);
+    return (!all_see_all && !tmpside->see_all && terrain_view(tmpside, x, y) == UNSEEN);
 }
 
 int
@@ -447,13 +482,6 @@ Goal *goal;
 
 /* General collections of numbers used by all machine players. */
 
-#if 0
-int is_base_for PARAMS ((int u1, int u2));
-int is_carrier_for PARAMS ((int u1, int u2));
-#endif
-
-int base_building = FALSE;     /* can some unit build a base */
-
 /* Init used by all machine players.  Precompute useful information
    relating to unit types in general, and that usually gets referenced
    in inner loops. */
@@ -466,15 +494,6 @@ ai_init_shared()
     /* Need 3 scratch layers for routefinding. */
     allocate_area_scratch(3);
 
-    if (!g_see_all() && 1 /* terrain not all known */) {
-	/* Compute approx percentage occurrence of each terrain type */
-	/* in the area not yet seen */
-	/* (not useful if fractal gen not used) */
-	for_all_terrain_types(t) {
-	    set_t_fraction(t, ((t_alt_max(t) - t_alt_min(t)) *
-			       (t_wet_max(t) - t_wet_min(t))) / 100);
-	}
-    }
     /* Recognize unit types that are bases */
     for_all_unit_types(u1) {
 	set_u_is_base(u1, FALSE);
@@ -510,7 +529,6 @@ ai_init_shared()
 		could_create(u1, u2) &&
 		1 /* can be made quickly? */) {
 		set_u_is_base_builder(u1, u2);
-		base_building = TRUE;
 	    }
 	    if (u_speed(u1) > 0 && could_carry(u1, u2)) {
 		set_u_is_transport(u1, TRUE);
@@ -535,7 +553,7 @@ ai_init_shared()
 	    for_all_unit_types(u2) {
 		if (could_carry(u1, u2)) {
 		    for_all_terrain_types(t) {
-			if (could_move(u1, t) && !could_move(u2, t))
+			if (could_be_on(u1, t) && !could_be_on(u2, t))
 			  set_u_is_carrier(u1, FALSE);
 		    }
 		}
@@ -548,6 +566,7 @@ ai_init_shared()
     for_all_unit_types(u) {
 	for_all_unit_types(u2) {
 	    set_uu_bhw(u, u2, basic_hit_worth(u, u2));
+	    set_uu_bfw(u, u2, basic_fire_worth(u, u2));
 	    set_uu_bcw(u, u2, basic_capture_worth(u, u2));
 	    set_uu_btw(u, u2, basic_transport_worth(u, u2));
 	}
@@ -577,6 +596,7 @@ int u;
     for_all_unit_types(u2) {
 	if (could_create(u, u2))
 	  worth += (u_bw(u2) * (50)) / 1 /* uu_make(u, u2) */;
+	/* (should account for shared capacity) */
 	if (could_carry(u, u2))
 	  worth += (1 + u_speed(u)) * uu_capacity_x(u, u2) *
 	    (u_is_base(u) ? 10 : 1) * u_bw(u2) / 30;
@@ -601,38 +621,6 @@ int u;
     return worth;
 }
 
-/* A crude estimate of the worth of having a type of unit for offense. */
-
-int
-offensive_worth(u)
-int u;
-{
-    int worth = 0, u2 /* ,  r, range */;
-
-    if (mobile(u)) {
-    	for_all_unit_types(u2) {
-    		if (basic_hit_worth(u, u2) > 0) worth += basic_hit_worth(u, u2);
-    	}
-    }
-#if 0  
-    worth += u_hp(u) * 10;
-    for_all_unit_types(u2) {
-	if (could_carry(u, u2))
-	  worth += (1 + u_speed(u)) * uu_capacity_x(u, u2) *
-	    (u_is_base(u) ? 10 : 1) * u_bw(u2) / 30;
-    }
-    worth += u_speed(u) * u_hp(u);
-    worth += (range == 12345 ? area.maxdim : range)
-      * u_hp(u) / max(1, 10 - u_speed(u));
-    for_all_unit_types(u2) {
-	worth += (worth * uu_capture(u, u2)) / 150;
-    }
-    worth = isqrt(worth);
-#endif
-    DMprintf("unit type %s worth %d \n ", u_type_name(u), worth);
-    return worth;
-}
-
 /* A basic estimate of the payoff of one unit type attacking another type
    directly.  This is "context-free", does not account for overall goals etc. */
 
@@ -644,21 +632,49 @@ int u, e;
 {
     int dam, numdice, numspots, offset, avgdamage, worth = 0, anti = 0;
 
+    if (uu_acp_to_attack(u, e) < 1)
+      return -9999;
     dam = uu_damage(u, e);
     DICE(dam, numdice, numspots, offset);
     avgdamage = offset + (numdice * numspots) / 2;
-    if (avgdamage > u_hp(e)) avgdamage = u_hp(e);
+    if (avgdamage > u_hp(e))
+      avgdamage = u_hp(e);
     worth = (uu_hit(u, e) * avgdamage) / u_hp(e);
     if (1 /* strength of counterattack */) {
 	dam = uu_damage(e, u);
 	DICE(dam, numdice, numspots, offset);
 	avgdamage = offset + (numdice * numspots) / 2;
-	if (avgdamage > u_hp(u)) avgdamage = u_hp(u);
+	if (avgdamage > u_hp(u))
+	  avgdamage = u_hp(u);
 	anti = (uu_hit(e, u) * avgdamage) / u_hp(u);
     }
     return worth - (anti * 9) / 10;
 }
 
+int
+basic_fire_worth(u, e)
+int u, e;
+{
+    int hitchance, dam, numdice, numspots, offset, avgdamage, worth = 0;
+
+    if (u_acp_to_fire(u) < 1)
+      return -9999;
+    if (uu_fire_hit(u, e) != -1)
+      hitchance = uu_fire_hit(u, e);
+    else
+      hitchance = uu_hit(u, e);
+    if (uu_fire_damage(u, e) != -1)
+      dam = uu_fire_damage(u, e);
+    else
+      dam = uu_damage(u, e);
+    DICE(dam, numdice, numspots, offset);
+    avgdamage = offset + (numdice * numspots) / 2;
+    if (avgdamage > u_hp(e))
+      avgdamage = u_hp(e);
+    worth = (hitchance * avgdamage) / u_hp(e);
+    return worth;
+}
+
 /* A crude estimate of the payoff of one unit type trying to capture. */
 
 int
@@ -667,6 +683,8 @@ int u, e;
 {
     int worth1 = 0, worth2 = 0;
 
+    if (uu_acp_to_capture(u, e) < 1 && uu_acp_to_attack(u, e) < 1)
+      return -9999;
     if (uu_capture(u, e) > 0) {
 	worth1 = uu_capture(u, e) * u_acp(u) /* divided by acp/attack */;
     }
@@ -704,10 +722,6 @@ display_assessment()
 {
     int t, u, u2;
 
-    DMprintf("\n\nEstimated Terrain Percentages:\n");
-    for_all_terrain_types(t) {
-	DMprintf(" %3d%%", t_fraction(t));
-    }
     DMprintf("\nUnit Attributes:\n");
     for_all_unit_types(u) {
 	DMprintf(" %-3.3s : base %d, transport %d, carrier %d, worth %d\n",
@@ -717,23 +731,35 @@ display_assessment()
     }
     DMprintf("\nUnit vs Unit Combat:\n");
     for_all_unit_types(u) {
-	for_all_unit_types(u2) DMprintf("%5d", uu_zz_bhw(u, u2));
+	DMprintf(" %-3.3s:", shortest_unique_name(u));
+	for_all_unit_types(u2)
+	  DMprintf("%5d", uu_zz_bhw(u, u2));
+	DMprintf("\n");
+    }
+    DMprintf("\nUnit vs Unit Fire:\n");
+    for_all_unit_types(u) {
+	DMprintf(" %-3.3s:", shortest_unique_name(u));
+	for_all_unit_types(u2)
+	  DMprintf("%5d", uu_zz_bfw(u, u2));
 	DMprintf("\n");
     }
     DMprintf("\nUnit vs Unit Capture:\n");
     for_all_unit_types(u) {
-	for_all_unit_types(u2) DMprintf(" %4d", uu_zz_bcw(u, u2));
+	DMprintf(" %-3.3s:", shortest_unique_name(u));
+	for_all_unit_types(u2)
+	  DMprintf(" %4d", uu_zz_bcw(u, u2));
 	DMprintf("\n");
     }
     DMprintf("\nUnit vs Unit Transport:\n");
     for_all_unit_types(u) {
-	for_all_unit_types(u2) DMprintf(" %4d", uu_zz_btw(u, u2));
+	DMprintf(" %-3.3s:", shortest_unique_name(u));
+	for_all_unit_types(u2)
+	  DMprintf(" %4d", uu_zz_btw(u, u2));
 	DMprintf("\n");
     }
     DMprintf("\n");
 }
 
-#if 0
 int
 is_base_for(u1, u2)
 int u1, u2;
@@ -751,7 +777,6 @@ int u1, u2;
 	    && (uu_capacity_x(u2, u1) > 0
 		|| (uu_size(u2, u1) <= u_capacity(u1))));
 }
-#endif
 
 /* Since *.def parameters don't have setters usually, we have to supply
    some here.  These are very sensitive to how the parameters are organized,
@@ -764,9 +789,9 @@ void set_u_is_base_builder(u, n) int u, n; {  utypes[u].is_base_builder = n;  }
 void set_u_can_make(u, n) int u, n; {  utypes[u].can_make = n;  }
 void set_u_can_capture(u, n) int u, n; {  utypes[u].can_capture = n;  }
 void set_u_bw(u, n) int u, n; {  utypes[u].bw = n;  }
-void set_t_fraction(t, n) int t, n; {  ttypes[t].fraction = n;  }
 
 int bhwtab = -1;
+int bfwtab = -1;
 int bcwtab = -1;
 int btwtab = -1;
 
@@ -787,6 +812,23 @@ int u1, u2, v;
     (*(tabledefns[bhwtab].table))[numutypes * u1 + u2] = v;
 }
 
+void
+set_uu_bfw(u1, u2, v)
+int u1, u2, v;
+{
+    if (bfwtab < 0) {
+	for (bfwtab = 0; tabledefns[bfwtab].name != NULL; ++bfwtab) {
+	    if (strcmp("zz-basic-fire-worth", tabledefns[bfwtab].name) == 0) {
+		allocate_table(bfwtab, FALSE);
+		break;
+	    }
+	}
+    }
+    if (tabledefns[bfwtab].table == NULL)
+      run_error("no bfw table allocated");
+    (*(tabledefns[bfwtab].table))[numutypes * u1 + u2] = v;
+}
+
 void
 set_uu_bcw(u1, u2, v)
 int u1, u2, v;
