@@ -1,5 +1,5 @@
 /* Command line parsing for Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -13,13 +13,8 @@ any later version.  See the file COPYING.  */
 /* Command lines get parsed in several stages, since for instance
    the choice of game module will decide which variants are available. */
 
-/* Command-line-interpreting programs always have a console. */
-
-#ifndef USE_CONSOLE
-#define USE_CONSOLE
-#endif
-
 #include "conq.h"
+extern void add_library_path PARAMS ((char *path));
 #include "cmdline.h"
 
 static void version_info PARAMS ((void));
@@ -31,7 +26,6 @@ static void parse_world_option PARAMS ((char *str));
 static void parse_realtime_option PARAMS ((char *subopt, char *arg));
 static void parse_variant PARAMS ((char *str));
 static void add_a_raw_player_spec PARAMS ((char *specstr));
-static void parse_player_spec PARAMS ((Player *player, char *spec));
 
 /* The startup-settable options. */
 
@@ -144,7 +138,7 @@ char *argv[];
 		allbedesigners = TRUE;
 #else
 		fprintf(stderr,
-			"No designing available, ignoring this option\n");
+			"No designing available, ignoring option `%s'\n", arg);
 #endif /* DESIGNERS */
 	    } else if (strncmp(arg, "-D", 2) == 0) {
 		GENERAL_OPTION;
@@ -158,7 +152,7 @@ char *argv[];
 		  DebugG = TRUE;
 #else
 		fprintf(stderr,
-			"No debugging available, ignoring this option\n");
+			"No debugging available, ignoring option `%s'\n", arg);
 #endif /* DEBUGGING */
 	    } else if (strncmp(arg, "-e", 2) == 0) {
 		REQUIRE_ONE_ARG;
@@ -202,8 +196,10 @@ char *argv[];
 	    } else if (strcmp(arg, "-L") == 0) {
 		REQUIRE_ONE_ARG;
 		GENERAL_OPTION;
-		/* (should be adding to a search list) */
-		xconqlib = copy_string(argv[i+1]);
+		if (strcmp(argv[i+1], "-") == 0)
+		  add_library_path(NULL);
+		else
+		  add_library_path(argv[i+1]);
 	    } else if (strcmp(arg, "-M") == 0) {
 		REQUIRE_ONE_ARG;
 		VARIANT_OPTION;
@@ -222,7 +218,7 @@ char *argv[];
 		init_xrandom(atoi(argv[i+1]));
 #else
 		fprintf(stderr,
-			"No debugging available, ignoring this option\n");
+			"No debugging available, ignoring option `%s'\n", arg);
 #endif /* DEBUGGING */
 	    } else if (strcmp(arg, "-sm") == 0) {
 		VARIANT_OPTION;
@@ -629,18 +625,19 @@ set_players_from_options()
     Player *player;
     struct raw_spec *spec;
 
+    player = NULL;
     /* Assume that if any players exist already, then this is a restored
        game of some sort, and merge instead of adding new players. */
     mergespecs = (numplayers > 0);
+    if (mergespecs)
+      player = playerlist;
     /* Add the default player. */
     if (raw_player_specs == NULL || option_add_default_player) {
-	if (mergespecs) {
-	    player = playerlist;
-	} else {
-	    player = add_default_player();
-	}
+	if (!mergespecs)
+	  player = add_default_player();
 	parse_player_spec(player, raw_default_player_spec);
 	canonicalize_player(player);
+	player = player->next;
     }
     /* Add the explicitly listed players. */
     for (spec = raw_player_specs; spec != NULL; spec = spec->next) {
@@ -650,62 +647,15 @@ set_players_from_options()
 			spec->spec);
 		continue;
 	    }
-	    player = player->next;
 	} else {
 	    player = add_player();
 	}
 	parse_player_spec(player, spec->spec);
 	canonicalize_player(player);
+	player = player->next;
     }
 }
 
-/* Parse the syntax "[username][,ai][/config][@display][+advantage]". */
-
-static void
-parse_player_spec(player, spec)
-Player *player;
-char *spec;
-{
-    int commapos, slashpos, atpos, pluspos;
-
-    if (spec != NULL && strcmp(spec, "*") != 0) {
-	/* Extract (destructively) a trailing advantage specification. */
-	pluspos = iindex('+', spec);
-	if (pluspos >= 0) {
-	    player->advantage = max(1, atoi(&(spec[pluspos + 1])));
-	    spec[pluspos] = '\0';
-	}
-	/* Extract a displayname if given. */
-	atpos = iindex('@', spec);
-	if (atpos >= 0) {
-	    player->displayname = copy_string(spec + atpos + 1);
-	    spec[atpos] = '\0';
-	}
-	/* Extract a configuration name if given. */
-	slashpos = iindex('/', spec);
-	if (slashpos >= 0) {
-	    player->configname = copy_string(spec + slashpos + 1);
-	    spec[slashpos] = '\0';
-	}
-	/* Extract an AI type if given. */
-	commapos = iindex(',', spec);
-	if (commapos >= 0) {
-	    player->aitypename = copy_string(spec + commapos + 1);
-	    spec[commapos] = '\0';
-	}
-	/* Just a plain old string left. */
-	if (strlen(spec) > 0) {
-	    if (atpos >= 0) {
-		/* Display given separately, so this is a name. */
-		player->name = copy_string(spec);
-	    } else {
-		player->displayname = copy_string(spec);
-	    }
-	}
-    }
-    canonicalize_player(player);
-}
-
 /* This is not, strictly speaking, part of command line processing,
    but command-line programs are also the ones for which stdio is useful. */
 
