@@ -59,6 +59,8 @@ extern int warnings_suppressed;
 extern time_t game_start_in_real_time;
 extern time_t turn_play_start_in_real_time;
 
+extern short *u_possible;
+
 /* Declarations of functions that must be supplied by an interface. */
 
 extern void announce_read_progress PARAMS ((void));
@@ -111,7 +113,8 @@ extern void low_run_warning PARAMS ((char *str));
 extern void run_error PARAMS ((char *str, ...));
 extern void low_run_error PARAMS ((char *str));
 
-extern void printlisp PARAMS ((Obj *));
+extern void print_form PARAMS ((Obj *form));
+extern void end_printing_forms PARAMS ((void));
 
 extern void close_displays PARAMS ((void));
 
@@ -123,19 +126,19 @@ extern void check_game_validity PARAMS ((void));
 extern void calculate_globals PARAMS ((void));
 extern void patch_object_references PARAMS ((void));
 extern void make_trial_assignments PARAMS ((void));
-extern void make_up_a_side PARAMS ((void));
+extern Side *make_up_a_side PARAMS ((void));
 extern void init_side_advantage PARAMS ((Side *side));
 extern int add_side_and_player PARAMS ((void));
 extern int exchange_players PARAMS ((int n, int n2));
 extern int remove_side_and_player PARAMS ((void));
 extern void run_synth_methods PARAMS ((void));
-extern int make_random_date PARAMS ((void));
-extern int make_weather PARAMS ((void));
+extern int make_random_date PARAMS ((int calls, int runs));
+extern int make_weather PARAMS ((int calls, int runs));
 extern void final_init PARAMS ((void));
 extern void configure_sides PARAMS ((void));
 extern void init_all_views PARAMS ((void));
 extern void init_area_views PARAMS ((void));
-extern int make_initial_materials PARAMS ((void));
+extern int make_initial_materials PARAMS ((int calls, int runs));
 extern void init_supply PARAMS ((Unit *unit));
 extern int saved_game PARAMS ((void));
 extern void init_side_balance PARAMS ((void));
@@ -152,8 +155,8 @@ extern void interp_game_module PARAMS ((Obj *form, Module *module));
 extern void do_module_variants PARAMS ((Module *module, Obj *lis));
 extern void fill_in_side PARAMS ((Side *side, Obj *props, int userdata));
 extern void read_warning PARAMS ((char *str, ...));
-
-extern int eval_boolean_expression PARAMS ((Obj *expr, int (*fn)(Obj *), int dflt));
+extern int utype_from_name PARAMS ((char *str));
+extern int utype_from_symbol PARAMS ((Obj *sym));
 
 extern void init_actions PARAMS ((void));
 extern int can_move_via_conn PARAMS ((Unit *unit, int nx, int ny));
@@ -171,7 +174,7 @@ extern int total_entry_cost PARAMS ((int u1, int u3, int x1, int y1, int z1, int
 extern int transfer_supply PARAMS ((Unit *from, Unit *to, int m, int amount));
 extern void adjust_tech_crossover PARAMS ((Side *side, int u));
 extern void adjust_tooling_crossover PARAMS ((Unit *unit, int u2));
-extern void garrison_unit PARAMS ((Unit *newunit, Unit *unit2));
+extern void garrison_unit PARAMS ((Unit *unit, Unit *unit2));
 extern void make_unit_complete PARAMS ((Unit *unit));
 extern void distribute_material PARAMS ((Unit *unit, int m, int amt));
 extern int execute_action PARAMS ((Unit *unit, Action *action));
@@ -214,22 +217,13 @@ extern int type_can_fire PARAMS ((int u));
 extern int type_can_capture PARAMS ((int u));
 extern int can_detonate PARAMS ((Unit *unit));
 
-extern int one_attack PARAMS ((Unit *atker, Unit *defender));
 extern int found_blocking_elevation PARAMS ((int u, int ux, int uy, int uz, int u2, int u2x, int u2y, int u2z));
-extern void fire_on_unit PARAMS ((Unit *atker, Unit *other));
 extern int enough_ammo PARAMS ((Unit *unit, Unit *other));
-extern void attack_unit PARAMS ((Unit *atker, Unit *other));
-extern void maybe_hit_unit PARAMS ((Unit *atker, Unit *other, int fallsoff));
-extern void hit_unit PARAMS ((Unit *unit, int hit, Unit *atker));
 extern void reckon_damage PARAMS ((void));
 extern void reckon_damage_around PARAMS ((int x, int y, int r));
-extern void damage_unit PARAMS ((Unit *unit));
-extern int retreat_unit PARAMS ((Unit *unit, Unit *atker));
-extern int retreat_in_dir PARAMS ((Unit *unit, int dir));
-extern void attempt_to_capture_unit PARAMS ((Unit *atker, Unit *other));
+extern void damage_unit PARAMS ((Unit *unit, enum damage_reasons reason));
 extern int capture_chance PARAMS ((int u, int u2, Side *side2));
 extern void capture_unit PARAMS ((Unit *unit, Unit *pris));
-extern void capture_occupant PARAMS ((Unit *unit, Unit *pris, Unit *occ));
 extern int detonate_unit PARAMS ((Unit *unit, int x, int y, int z));
 
 extern int multiply_dice PARAMS ((int dice, int mult));
@@ -258,26 +252,31 @@ extern int people_surrender_chance PARAMS ((int u, int x, int y));
 extern int unit_priority PARAMS ((Unit *unit));
 extern int num_people_at PARAMS ((int x, int y));
 
-extern int make_fractal_terrain PARAMS ((void));
-extern int make_random_terrain PARAMS ((void));
-extern int make_earthlike_terrain PARAMS ((void));
-extern int make_maze_terrain PARAMS ((void));
-extern int name_geographical_features PARAMS ((void));
+extern int request_additional_side PARAMS ((char *playerspec));
+extern void parse_player_spec PARAMS ((Player *player, char *spec));
+
+extern int get_synth_method_uses PARAMS ((int methkey, int *calls, int *runs));
+
+extern int make_fractal_terrain PARAMS ((int calls, int runs));
+extern int make_random_terrain PARAMS ((int calls, int runs));
+extern int make_earthlike_terrain PARAMS ((int calls, int runs));
+extern int make_maze_terrain PARAMS ((int calls, int runs));
+extern int name_geographical_features PARAMS ((int calls, int runs));
 extern void add_edge_terrain PARAMS ((void));
 
-extern int make_countries PARAMS ((void));
-extern int make_independent_units PARAMS ((void));
+extern int make_countries PARAMS ((int calls, int runs));
+extern int make_independent_units PARAMS ((int calls, int runs));
 
-extern int make_rivers PARAMS ((void));
+extern int make_rivers PARAMS ((int calls, int runs));
 
-extern int make_roads PARAMS ((void));
+extern int make_roads PARAMS ((int calls, int runs));
 
 extern Obj *make_namer PARAMS ((Obj *sym, Obj *meth));
 
 extern void init_namers PARAMS ((void));
 extern void make_up_side_name PARAMS ((Side *side));
 extern int name_in_use PARAMS ((Side *side, char *str));
-extern int name_units_randomly PARAMS ((void));
+extern int name_units_randomly PARAMS ((int calls, int runs));
 extern char *unit_namer PARAMS ((Unit *unit));
 extern char *propose_unit_name PARAMS ((Unit *unit));
 extern void make_up_unit_name PARAMS ((Unit *unit));
@@ -287,6 +286,7 @@ extern char *name_from_grammar PARAMS ((Obj *grammar));
 extern void gen_name PARAMS ((Obj *nonterm, Obj *rules, int depth, char *rslt));
 extern void gen_from_rule PARAMS ((Obj *rule, Obj *rules, int depth, char *rslt));
 
+extern void init_nlang PARAMS ((void));
 extern void pad_out PARAMS ((char *buf, int n));
 extern char *short_side_title PARAMS ((Side *side));
 extern char *shortest_side_title PARAMS ((Side *side2, char *buf));
@@ -321,9 +321,8 @@ extern char *exit_commentary PARAMS ((Side *side));
 extern char *ordinal_suffix PARAMS ((int n));
 extern char *plural_form PARAMS ((char *word));
 extern char *make_text PARAMS ((char *buf, Obj *maker, long a1, long a2, long a3, long a4));
-extern char *absolute_date_string PARAMS ((long date));
-extern char *relative_date_string PARAMS ((long date, long base));
-extern void parse_usual_initial_date PARAMS ((void));
+extern char *absolute_date_string PARAMS ((int date));
+extern char *relative_date_string PARAMS ((int date, int base));
 extern void write_unit_record PARAMS ((FILE *fp, Side *side));
 extern void write_side_results PARAMS ((FILE *fp, Side *side));
 extern void write_combat_results PARAMS ((FILE *fp, Side *side));
@@ -333,6 +332,14 @@ extern char *short_past_unit_handle PARAMS ((PastUnit *past_unit));
 extern void past_name_or_number PARAMS ((PastUnit *past_unit, char *buf));
 extern void record_missing_image PARAMS ((int typtyp, char *str));
 extern int missing_images PARAMS ((char *buf));
+extern void notify_all_of_resignation PARAMS ((Side *side, Side *side2));
+extern int short_side_title_plural_p PARAMS ((Side *side));
+extern char *short_side_title_with_adjective PARAMS ((Side *side, char *adjective));
+
+extern void set_initial_date PARAMS ((char *str));
+extern long parse_date PARAMS ((char *datestr));
+extern int find_event_type PARAMS ((Obj *sym));
+extern char *side_score_desc PARAMS ((char *buf, Side *side, Scorekeeper *sk));
 
 extern void doublecheck_state PARAMS ((Side *side));
 extern long extension_value PARAMS ((Obj *extensions, char *key, long dflt));
@@ -345,6 +352,7 @@ extern Side *coerce_to_side PARAMS ((Obj *x));
 extern int coerce_to_unit_id PARAMS ((Obj *x));
 extern Unit *coerce_to_unit PARAMS ((Obj *x));
 
+extern void init_write PARAMS ((void));
 extern int write_entire_game_state PARAMS ((char *fname));
 extern int write_game_module PARAMS ((Module *module));
 
@@ -360,4 +368,7 @@ extern int n_ms_elapsed PARAMS ((int n));
 extern void record_ms PARAMS ((void));
 extern int all_human_only_sides_finished PARAMS ((void));
 
+extern int get_synth_method_uses PARAMS ((int methkey, int *calls, int *runs));
+extern void set_initial_date PARAMS ((char *str));
+extern int turns_between PARAMS ((char *str1, char *str2));
 extern int visible_to PARAMS ((Unit *unit, Unit *unit2));
