@@ -13,29 +13,46 @@ any later version.  See the file COPYING.  */
 /* This file must also be translated (mostly) for non-English Xconq. */
 
 #include "conq.h"
+extern void notify_instructions PARAMS ((void));
 
+static void describe_help_system PARAMS ((int arg, char *key, char *buf));
+static void describe_synth_run PARAMS ((char *buf, int methkey));
 static int histogram_compare PARAMS ((const void *h1, const void *h2));
 
-/* The first help node in the chain, usually the list of topics. */
+/* The first help node in the chain. */
 
-HelpNode *firsthelpnode = NULL;
+HelpNode *first_help_node;
 
 /* The last help node. */
 
-HelpNode *lasthelpnode = NULL;
+HelpNode *last_help_node;
 
-/* Create the initial help node, which is the copyright info, and link
-   it to itself.  Subsequent nodes will be inserted later, after a game
-   has been loaded. */
+/* The help node with copying and copyright info. */
+
+HelpNode *copying_help_node;
+
+/* The help node with (non-)warranty info. */
+
+HelpNode *warranty_help_node;
+
+HelpNode *default_prev_help_node;
+
+/* Create the initial help node and link it to itself.  Subsequent nodes will be
+   inserted later, after a game has been loaded. */
 
 void
 init_help()
 {
-    firsthelpnode = create_help_node();
-    firsthelpnode->key = "copyright";
-    firsthelpnode->fn = describe_copyright;
-    firsthelpnode->prev = firsthelpnode->next = firsthelpnode;
-    lasthelpnode = firsthelpnode;
+    /* Note that we can't use add_help_node to set up the first help node. */
+    first_help_node = create_help_node();
+    first_help_node->key = "help system";
+    first_help_node->fn = describe_help_system;
+    first_help_node->prev = first_help_node->next = first_help_node;
+    last_help_node = first_help_node;
+    copying_help_node = add_help_node("copyright", describe_copyright, 0, first_help_node);
+    warranty_help_node = add_help_node("warranty", describe_warranty, 0, copying_help_node);
+    /* Set the place for new nodes to appear normally. */
+    default_prev_help_node = copying_help_node;
     add_help_node("news", describe_news, 0, NULL);
 }
 
@@ -73,7 +90,7 @@ create_game_help_nodes()
     }
     add_help_node("general concepts", describe_concepts, 0, NULL);
     /* Invalidate any existing topics node. */
-    firsthelpnode->text = NULL;
+    first_help_node->text = NULL;
 }
 
 /* Create an empty help node. */
@@ -109,18 +126,15 @@ HelpNode *prevnode;
     node->key = key;
     node->fn = fn;
     node->arg = arg;
-    if (prevnode != NULL) {
-	nextnode = prevnode->next;
-    } else {
-	prevnode = lasthelpnode;
-	nextnode = firsthelpnode;
-    }
+    if (prevnode == NULL)
+      prevnode = default_prev_help_node->prev;
+    nextnode = prevnode->next;
     node->prev = prevnode;
     node->next = nextnode;
     prevnode->next = node;
     nextnode->prev = node;
     /* Might need to fix last help node. */
-    lasthelpnode = firsthelpnode->prev;
+    last_help_node = first_help_node->prev;
     return node;
 }
 
@@ -150,20 +164,27 @@ char *
 get_help_text(node)
 HelpNode *node;
 {
+    int allocsize;
+
     if (node != NULL) {
 	/* Maybe calculate the text to display. */
 	if (node->text == NULL) {
 	    if (node->fn != NULL) {
-	    	node->text = xmalloc(5000);
+		/* (should allow for variable-size allocation) */
+		allocsize = 5000;
+		if (node->fn == describe_copyright)
+		  allocsize = 30000;
+	    	node->text = xmalloc(allocsize);
 		if (node->text != NULL) {
-		    /* Make buffer into an empty string. */
+		    /* Make buffer into an empty string, so that the
+		       description function can just start appending. */
 		    (node->text)[0] = '\0';
 		    node->textend = 0;
-		    node->textsize = 5000;
+		    node->textsize = allocsize;
 		    (*(node->fn))(node->arg, node->key, node->text);
 		    node->textend = strlen(node->text);
 		} else {
-		    /* Ran out of memory... */
+		    /* Ran out of memory... (would never get here though!) */
 		}
 	    } else {
 		/* Generate a default message if nothing to compute help. */
@@ -178,12 +199,13 @@ HelpNode *node;
     }
 }
 
-void
-describe_copyright(arg, key, buf)
+static void
+describe_help_system(arg, key, buf)
 int arg;
 char *key, *buf;
 {
-    tprintf(buf, "Xconq is free software.");
+    strcat(buf, "This is the header node of the Xconq help system.\n");
+    strcat(buf, "Go forward or backward from here to see the online help.\n");
 }
 
 /* Create a raw list of help topics by just iterating through all the nodes,
@@ -196,7 +218,7 @@ char *key, *buf;
 {
     HelpNode *topics, *tmp;
 
-    topics = find_help_node(firsthelpnode, "topics");
+    topics = find_help_node(first_help_node, "topics");
     /* Unlikely that we'll call this without the topics node existing
        already, but just in case... */
     if (topics == NULL)
@@ -233,8 +255,11 @@ describe_concepts(arg, key, buf)
 int arg;
 char *key, *buf;
 {
-    tprintf(buf, "Action points (Acp) are what units need to do ");
-    tprintf(buf, "anything at all.");
+    tprintf(buf, "Hit points (hp) represent the overall condition of ");
+    tprintf(buf, "the unit.");
+    tprintf(buf, "\n");
+    tprintf(buf, "Action points (acp) are what a unit needs to be able ");
+    tprintf(buf, "to do anything at all.");
     tprintf(buf, "\n");
 }
 
@@ -433,23 +458,41 @@ describe_utype(u, key, buf)
 int u;
 char *key, *buf;
 {
-    int m;
+    char sidetmpbuf[BUFSIZE];
+    int m, first;
+    Side *side;
 
     append_help_phrase(buf, u_help(u));
     if (u_point_value(u) > 0) {
-    	tprintf(buf, "     (point value %d)\n", u_point_value(u));
+	tprintf(buf, "     (point value %d)\n", u_point_value(u));
     }
     if (u_can_be_self(u)) {
-    	tprintf(buf, "Can be self-unit.\n");
+    	tprintf(buf, "Can be self-unit");
+    	if (u_self_changeable(u))
+	  tprintf(buf, "; side may choose another to be self-unit");
+    	if (u_self_resurrects(u))
+	  tprintf(buf, "; if dies, another becomes self-unit");
+    	tprintf(buf, ".\n");
     }
     if (u_possible_sides(u) != lispnil) {
-    	tprintf(buf, "Some limitations on possible sides.\n");
+	tprintf(buf, "Possible sides (in this game): ");
+	first = TRUE;
+	for_all_sides_plus_indep(side) {
+	    if (type_allowed_on_side(u, side)) {
+		if (first)
+		  first = FALSE;
+		else
+		  tprintf(buf, ", ");
+		tprintf(buf, "%s", shortest_side_title(side, sidetmpbuf));
+	    }
+    	}
+    	tprintf(buf, ".\n");
     }
     if (u_type_in_game_max(u) >= 0) {
-    	tprintf(buf, "At most %d allowed in a game.\n", u_type_in_game_max(u));
+	tprintf(buf, "At most %d allowed in a game.\n", u_type_in_game_max(u));
     }
     if (u_type_per_side_max(u) >= 0) {
-    	tprintf(buf, "At most %d allowed on each side in a game.\n", u_type_per_side_max(u));
+	tprintf(buf, "At most %d allowed on each side in a game.\n", u_type_per_side_max(u));
     }
     if (u_acp(u) > 0) {
 	    tprintf(buf, "Gets %d action points each turn", u_acp(u));
@@ -478,10 +521,7 @@ char *key, *buf;
 		    u_speed_max(u) / 100, u_speed_max(u) % 100);
 	}
 	if (u_speed_wind_effect(u) != lispnil) {
-	    tprintf(buf, "Wind force affects speed.\n");
-	}
-	if (u_speed_wind_angle_effect(u) != lispnil) {
-	    tprintf(buf, "Wind direction affects speed.\n");
+	    tprintf(buf, "Wind affects speed.\n");
 	}
 	if (u_speed_damage_effect(u) != lispnil) {
 	    tprintf(buf, "Damage affects speed.\n");
@@ -662,6 +702,9 @@ char *key, *buf;
 	uu_table_row_desc(spbuf, u, uu_damage, dice_desc);
 	tprintf(buf, "Damages are %s", spbuf);
 	tprintf(buf, ".\n");
+   	if (u_acp_to_fire(u) > 0) {
+	    /* (should describe fire damage if different) */
+   	}
     }
     if (uu_table_row_not_default(u, uu_protection, 100)) {
 	uu_table_row_desc(spbuf, u, uu_protection, NULL);
@@ -672,7 +715,7 @@ char *key, *buf;
 	tprintf(buf, "Chance to retreat from combat is %s.\n", spbuf);
     }
     if (u_wrecked_type(u) != NONUTYPE) {
-    	tprintf(buf, "Becomes a %s when destroyed.\n",
+    	tprintf(buf, "Becomes %s when destroyed.\n",
 		u_type_name(u_wrecked_type(u)));
     }
     if (u_acp(u) > 0
@@ -769,12 +812,7 @@ char *key, *buf;
 	    tprintf(buf, "\n");
 	}
     }
-#if 0
-    if (temperatures_defined() /* not correct, shouldn't look at state of world */) {
-	tprintf(buf, "Temperature survival zone is %d to %d, comfort zone is %d to %d.\n",
-		u_survival_min(u), u_survival_max(u), u_comfort_min(u), u_comfort_max(u));
-    }
-#endif
+    /* (should display weather interaction here) */
     if (u_spy_chance(u) > 0 /* and random event in use */) {
 	tprintf(buf, "%d%% chance to spy, on units up to %d away.",
 		u_spy_chance(u), u_spy_range(u));
@@ -953,7 +991,9 @@ char *key, *buf;
 	    methkey = keyword_code(c_string(method));
 	    switch (methkey) {
 	      case K_MAKE_COUNTRIES:
-		tprintf(buf, "\nCountries:\n");
+		tprintf(buf, "\nCountries:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		tprintf(buf, "  %d cells across, between %d and %d cells apart.\n",
 			2 * g_radius_min() + 1,
 			g_separation_min(), g_separation_max());
@@ -1024,10 +1064,14 @@ char *key, *buf;
 		}
 		break;
 	      case K_MAKE_EARTHLIKE_TERRAIN:
-		tprintf(buf, "\nEarthlike terrain:\n");
+		tprintf(buf, "\nEarthlike terrain:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_MAKE_FRACTAL_PTILE_TERRAIN:
-		tprintf(buf, "\nFractal percentile terrain:\n");
+		tprintf(buf, "\nFractal percentile terrain:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		tprintf(buf, "  Alt blobs density %d, size %d, height %d\n",
 			g_alt_blob_density(), g_alt_blob_size(), g_alt_blob_height());
 		tprintf(buf, "    %d smoothing passes\n", g_alt_smoothing());
@@ -1044,24 +1088,36 @@ char *key, *buf;
 		tprintf(buf, "    Upper percentiles: %s.\n", spbuf);
 		break;
 	      case K_MAKE_INDEPENDENT_UNITS:
-		tprintf(buf, "\nIndependent units:\n");
+		tprintf(buf, "\nIndependent units:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		/* (should show indep density and people) */
 		break;
 	      case K_MAKE_INITIAL_MATERIALS:
-		tprintf(buf, "\nMaterials:\n");
+		tprintf(buf, "\nMaterials:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		/* (should show unit and terrain initial supply) */
 		break;
 	      case K_MAKE_MAZE_TERRAIN:
-		tprintf(buf, "\nMaze terrain:\n");
+		tprintf(buf, "\nMaze terrain:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_MAKE_RANDOM_DATE:
-		tprintf(buf, "\nRandom date\n");
+		tprintf(buf, "\nRandom date:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_MAKE_RANDOM_TERRAIN:
-		tprintf(buf, "\nRandom terrain:\n");
+		tprintf(buf, "\nRandom terrain:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_MAKE_RIVERS:
-		tprintf(buf, "\nRivers:\n");
+		tprintf(buf, "\nRivers:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		if (t_property_not_default(t_river_chance, 0)) {
 		    t_property_desc(spbuf, t_river_chance, NULL);
 		    tprintf(buf, "  Chance to be river source: %s.\n", spbuf);
@@ -1072,7 +1128,9 @@ char *key, *buf;
 		}
 		break;
 	      case K_MAKE_ROADS:
-		tprintf(buf, "\nRoads:\n");
+		tprintf(buf, "\nRoads:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		tprintf(buf, "  Chance to run:\n");
 		for_all_unit_types(u) {
 		    if (uu_table_row_not_default(u, uu_road_chance, 0)) {
@@ -1085,16 +1143,24 @@ char *key, *buf;
 		/* (should include routing terrain preferences) */
 		break;
 	      case K_MAKE_WEATHER:
-		tprintf(buf, "\nWeather:\n");
+		tprintf(buf, "\nWeather:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_NAME_GEOGRAPHICAL_FEATURES:
-		tprintf(buf, "\nNames for geographical features\n");
+		tprintf(buf, "\nNames for geographical features:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      case K_NAME_UNITS_RANDOMLY:
-		tprintf(buf, "\nNames for units\n");
+		tprintf(buf, "\nNames for units:");
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	      default:
-		tprintf(buf, "\n%s:\n", c_string(method));
+		tprintf(buf, "\n%s:", c_string(method));
+		describe_synth_run(buf, methkey);
+		tprintf(buf, "\n");
 		break;
 	    }
 	} else if (consp(method)) {
@@ -1102,6 +1168,31 @@ char *key, *buf;
     }
 }
 
+static void
+describe_synth_run(buf, methkey)
+char *buf;
+int methkey;
+{
+    int calls, runs;
+  
+    if (get_synth_method_uses(methkey, &calls, &runs)) {
+	if (calls > 0) {
+	    if (calls == 1 && runs == 1) {
+		tprintf(buf, " (was run)");
+	    } else if (calls == 1 && runs == 0) {
+		tprintf(buf, " (was not run)");
+	    } else {
+		tprintf(buf, " (was called %d times, was run %d times)",
+			calls, runs);
+	    }
+	} else {
+	    tprintf(buf, " (not attempted)");
+	}
+    } else {
+	tprintf(buf, " (???)");
+    }
+}
+
 /* This describes a command (from cmd.def et al) in a way that all
    interfaces can use. */
 
@@ -1690,10 +1781,10 @@ Obj *notes;
     if (stringp(notes)) {
 	notestr = c_string(notes);
 	if (strlen(notestr) > 0) { 
-	    tprintf(buf, "%s", notestr);
-	    tprintf(buf, " ");
+	    strcat(buf, notestr);
+	    strcat(buf, " ");
 	} else {
-	    tprintf(buf, "\n");
+	    strcat(buf, "\n");
 	}
     } else if (consp(notes)) {
 	for (rest = notes; rest != lispnil; rest = cdr(rest)) {
@@ -1704,7 +1795,29 @@ Obj *notes;
     }
 }
 
-#ifdef USE_CONSOLE
+void
+notify_instructions()
+{
+    Obj *instructions = mainmodule->instructions, *rest;
+
+    if (instructions != lispnil) {
+	if (stringp(instructions)) {
+	    notify_all("%s", c_string(instructions));
+	} else if (consp(instructions)) {
+	    for (rest = instructions; rest != lispnil; rest = cdr(rest)) {
+		if (stringp(car(rest))) {
+		    notify_all("%s", c_string(car(rest)));
+		} else {
+		    /* (should probably warn about this case too) */
+		}
+	    }
+	} else {
+	    run_warning("Instructions are of wrong type");
+	}
+    } else {
+	notify_all("(no instructions supplied)");
+    }
+}
 
 /* Print the news file onto the console if there is anything to print. */
 
@@ -1725,8 +1838,6 @@ print_any_news()
     }
 }
 
-#endif /* USE_CONSOLE */
-
 /* Generate a readable description of the game (design) being played. */
 /* This works by writing out appropriate help nodes, along with some
    indexing material.  This does *not* do interface-specific help,
@@ -1739,7 +1850,7 @@ FILE *fp;
     HelpNode *node;
 
     /* (need to work on which nodes to dump out) */
-    for (node = firsthelpnode; node != firsthelpnode; node = node->next) {
+    for (node = first_help_node; node != first_help_node; node = node->next) {
 	get_help_text(node);
 	if (node->text != NULL) {
 	    fprintf(fp, "\014\n%s\n", node->key);
