@@ -1,5 +1,5 @@
 /* General Xconq initialization.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -18,9 +18,11 @@ any later version.  See the file COPYING.  */
    fatal conflicts. */
 
 #include "conq.h"
-extern void init_write PARAMS ((void));
+extern void add_library_path PARAMS ((char *path));
+extern void prealloc_debug PARAMS ((void));
 #include "version.h"
 
+static void cache_possible_types PARAMS ((void));
 static void trial_assign_player PARAMS ((int i, Player *player));
 static void init_view_2 PARAMS ((Side *side, int terrainset));
 static void init_view_cell PARAMS ((int x, int y));
@@ -47,9 +49,11 @@ short tmpttype;
 
 Assign *assignments = NULL;
 
-/* The pathname to the library directory. */
+/* The list of pathnames to library directories. */
 
-char *xconqlib;
+LibraryPath *xconq_libs;
+
+LibraryPath *last_user_xconq_lib;
 
 /* The name of the default game module. */
 
@@ -84,27 +88,31 @@ int numdesigners = 0;
 
 int max_zoc_range;
 
+short *u_possible;
+
 /* The table of all available synthesis methods. */
 
 struct synthtype {
     int key;
-    int (*fn) PARAMS ((void));
+    int (*fn) PARAMS ((int calls, int runs));
     int dflt;
+    int calls;
+    int runs;
 } synthmethods[] = {
-    { K_MAKE_FRACTAL_PTILE_TERRAIN, make_fractal_terrain, TRUE },
-    { K_MAKE_RANDOM_TERRAIN, make_random_terrain, FALSE },
-    { K_MAKE_EARTHLIKE_TERRAIN, make_earthlike_terrain, FALSE },
-    { K_MAKE_MAZE_TERRAIN, make_maze_terrain, FALSE },
-    { K_MAKE_RIVERS, make_rivers, TRUE },
-    { K_MAKE_COUNTRIES, make_countries, TRUE },
-    { K_MAKE_INDEPENDENT_UNITS, make_independent_units, TRUE },
-    { K_MAKE_INITIAL_MATERIALS, make_initial_materials, TRUE },
-    { K_NAME_UNITS_RANDOMLY, name_units_randomly, TRUE },
-    { K_NAME_GEOGRAPHICAL_FEATURES, name_geographical_features, TRUE },
-    { K_MAKE_ROADS, make_roads, TRUE },
-    { K_MAKE_WEATHER, make_weather, TRUE },
-    { K_MAKE_RANDOM_DATE, make_random_date, FALSE },
-    { -1, NULL, 0 }
+    { K_MAKE_FRACTAL_PTILE_TERRAIN, make_fractal_terrain, TRUE, 0, 0 },
+    { K_MAKE_RANDOM_TERRAIN, make_random_terrain, FALSE, 0, 0 },
+    { K_MAKE_EARTHLIKE_TERRAIN, make_earthlike_terrain, FALSE, 0, 0 },
+    { K_MAKE_MAZE_TERRAIN, make_maze_terrain, FALSE, 0, 0 },
+    { K_MAKE_RIVERS, make_rivers, TRUE, 0, 0 },
+    { K_MAKE_COUNTRIES, make_countries, TRUE, 0, 0 },
+    { K_MAKE_INDEPENDENT_UNITS, make_independent_units, TRUE, 0, 0 },
+    { K_MAKE_INITIAL_MATERIALS, make_initial_materials, TRUE, 0, 0 },
+    { K_NAME_UNITS_RANDOMLY, name_units_randomly, TRUE, 0, 0 },
+    { K_NAME_GEOGRAPHICAL_FEATURES, name_geographical_features, TRUE, 0, 0 },
+    { K_MAKE_ROADS, make_roads, TRUE, 0, 0 },
+    { K_MAKE_WEATHER, make_weather, TRUE, 0, 0 },
+    { K_MAKE_RANDOM_DATE, make_random_date, FALSE, 0, 0 },
+    { -1, NULL, 0, 0, 0 }
 };
 
 /* Allow environment vars to override compiled-in library and game. */
@@ -113,6 +121,8 @@ void
 init_library_path(path)
 char *path;
 {
+    char *xconqlib;
+
     xconqlib = getenv("XCONQLIB");
     if (empty_string(xconqlib)) {
 	if (!empty_string(path)) {
@@ -121,31 +131,45 @@ char *path;
 	    xconqlib = default_library_filename();
 	}
     }
+    xconq_libs = (LibraryPath *) xmalloc(sizeof(LibraryPath));
+    xconq_libs->path = copy_string(xconqlib);
+    last_user_xconq_lib = NULL;
     standard_game_name = getenv("XCONQSTANDARDGAME");
     if (empty_string(standard_game_name))
       standard_game_name = STANDARD_GAME;
 }
 
+void
+add_library_path(path)
+char *path;
+{
+    LibraryPath *lib;
+
+    /* A NULL path indicates that all the existing paths should go away. */
+    if (path == NULL) {
+	xconq_libs = last_user_xconq_lib = NULL;
+	return;
+    }
+    lib = (LibraryPath *) xmalloc(sizeof(LibraryPath));
+    lib->path = copy_string(path);
+    /* Insert this after the previous user-specified library, if there
+       was any, but before the default library places. */
+    if (last_user_xconq_lib != NULL) {
+	lib->next = last_user_xconq_lib->next;
+	last_user_xconq_lib->next = lib;
+    } else {
+	lib->next = xconq_libs;
+	xconq_libs = lib;
+    }
+    last_user_xconq_lib = lib;
+}
+
 /* This is a general init that prepares data structures to be filled in
    by a game definition.  It should run *before* command line parsing. */
 
 void
 init_data_structures()
 {
-#ifdef DEBUGGING
-    /* Map all the debugging outputs to stdout. */
-    /* You might think stderr would be a better choice, but this way
-       the debugging output is sync'ed properly (read: consistently!)
-       with "normal" stdout output. */
-#ifdef USE_CONSOLE
-   	dfp  = stdout;
-    dgfp = stdout;
-    dmfp = stdout;
-#else
-    /* should open files or something */
-#endif /* USE_CONSOLE */
-#endif /* DEBUGGING */
-
     init_xrandom(-1);
     init_lisp();
     init_types();
@@ -156,6 +180,7 @@ init_data_structures()
     init_sides();
     init_agreements();
     init_units();
+    init_nlang();
     init_help();
     init_scorekeepers();
     init_actions();
@@ -285,7 +310,11 @@ check_game_validity()
 	    init_error("unit type %d has nonpositive hp", u1);
 	    failed = TRUE;
 	}
-	if (u_parts(u1) <= 0 || u_hp(u1) % u_parts(u1) != 0) {
+	if (u_parts(u1) <= 0) {
+	    init_error("unit type %d has nonpositive part count.", u1);
+	    failed = TRUE;
+	}
+	if (u_parts(u1) > 0 && u_hp(u1) % u_parts(u1) != 0) {
 	    init_error("unit type %d hp not a multiple of its parts.", u1);
 	    failed = TRUE;
 	}
@@ -340,8 +369,8 @@ check_game_validity()
 void
 calculate_globals()
 {
-	/* The game is now completely defined; no further user-specified changes can
-	   occur. */
+    /* The game is now completely defined; no further user-specified changes can
+       occur. */
     gamedefined = TRUE;
     calculate_world_globals();
     /* This needs to be precalculated instead of as-needed, since range can be
@@ -487,7 +516,7 @@ patch_object_references()
 		unit->number = 0;
 	    }
 	    if (completed(unit)) {
-		init_unit_actorstate(unit);
+		init_unit_actorstate(unit, TRUE);
 		/* Restore acp that wasn't written out because it was the most normal
 		   value. */
 		if (unit->act && unit->act->acp < u_acp_min(unit->type))
@@ -556,7 +585,15 @@ make_trial_assignments()
 	init_warning("too many players (%d) for %d sides, ignoring extra",
 		     numplayers, numsides);
     }
-    /* (should make any prespecified assignments) */
+    /* Make any prespecified assignments. */
+    for (i = 0; i < numsides; ++i) {
+	if (assignments[i].side != NULL
+	    && assignments[i].side->player != NULL
+	    && assignments[i].player == NULL) {
+	    trial_assign_player(i, assignments[i].side->player);
+	    /* (should this assignment be locked?) */
+	}
+    }
     /* Assign any remaining players. */
     player = playerlist;
     for (i = 0; i < numsides; ++i) {
@@ -612,20 +649,24 @@ Player *player;
 
 /* Create a random side with default characteristics. */
 
-void
+Side *
 make_up_a_side()
 {
+	extern Obj *side_defaults;
+
     Side *side = create_side();
 
     if (side == NULL) {
 	run_error("could not create a side");
-	return;
+	return NULL;
     }
+    fill_in_side(side, side_defaults, FALSE);
     make_up_side_name(side);
     init_side_advantage(side);
     /* A newly-created side starts out in the game, can drop out later. */
     side->ingame = TRUE;
     Dprintf("Made up a side %s\n", side_desig(side));
+    return side;
 }
 
 /* If undefined, seed a side's advantage and allowable range from the
@@ -653,8 +694,7 @@ add_side_and_player()
     Side *side;
     Player *player;
 
-    make_up_a_side();
-    side = side_n(numsides);
+    side = make_up_a_side();
     if (side == NULL)
       return FALSE;
     n = numsides - 1;
@@ -717,19 +757,24 @@ remove_side_and_player()
 void
 run_synth_methods()
 {
-    int i, methkey, found = FALSE;
+    int i, methkey, found = FALSE, rslt;
     Obj *synthlist, *methods, *method, *parms;
 
     synthlist = g_synth_methods();
-    Dprintf("Will run syntheses ");  Dprintlisp(synthlist);  Dprintf("\n");
+    Dprintf("Will run syntheses ");
+    Dprintlisp(synthlist);
+    Dprintf("\n");
     for (methods = synthlist; methods != lispnil; methods = cdr(methods)) {
 	method = car(methods);
 	if (symbolp(method)) {
 	    methkey = keyword_code(c_string(method));
 	    for (i = 0; synthmethods[i].key >= 0; ++i) {
 		if (methkey == synthmethods[i].key) {
-		    (*synthmethods[i].fn)();
 		    found = TRUE;
+		    rslt = (*synthmethods[i].fn)(synthmethods[i].calls, synthmethods[i].runs);
+		    ++(synthmethods[i].calls);
+		    if (rslt)
+		      ++(synthmethods[i].runs);
 		    break;
 		}
 	    }
@@ -751,7 +796,24 @@ run_synth_methods()
 }
 
 int
-make_weather()
+get_synth_method_uses(methkey, calls, runs)
+int methkey, *calls, *runs;
+{
+    int i;
+
+    for (i = 0; synthmethods[i].key >= 0; ++i) {
+	if (methkey == synthmethods[i].key) {
+	    *calls = synthmethods[i].calls;
+	    *runs = synthmethods[i].runs;
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
+
+int
+make_weather(calls, runs)
+int calls, runs;
 {
     int x, y, winddir;
     extern int maxwindforce;	
@@ -778,16 +840,27 @@ make_weather()
     return TRUE;
 }
 
-/* Set the starting date/time to a random value. */
+/* Set the starting date/time to a random value within a given range. */
 
 int
-make_random_date()
+make_random_date(calls, runs)
+int calls, runs;
 {
-    extern int baseyear;
+    int n;
 
-    /* (this should tweak a "date offset" rather than curturn,
-       which should be actual turn number) */
-    baseyear = 1900 + xrandom(10);
+    /* Don't bother if the initial date has been set explicitly. */
+    if (!empty_string(g_initial_date()))
+      return FALSE;
+    if (empty_string(g_initial_date_min()))
+      return FALSE;
+    if (empty_string(g_initial_date_max()))
+      return FALSE;
+    /* Use the low end of the date range as a starting place for date
+       calculations. */
+    set_initial_date(g_initial_date_min());
+    n = turns_between(g_initial_date_min(), g_initial_date_max());
+    /* Now bump the starting date to its random place. */
+    set_initial_date(absolute_date_string(xrandom(n) + 1));
     return TRUE;
 }
 
@@ -797,7 +870,6 @@ void
 final_init()
 {
     Side *side;
-    Unit *unit;
 
     /* Fill in any empty doctrines. */
     for_all_sides_plus_indep(side) {
@@ -805,20 +877,13 @@ final_init()
     }
     /* Make sure each side has a self-unit if it needs one. */
     for_all_sides(side) {
-	if ((g_self_required() /* || side prop? */)
-	    && side->self_unit == NULL) {
-	    for_all_side_units(side, unit) {
-		if (u_can_be_self(unit->type)
-		    && in_play(unit)
-		    && completed(unit))
-		  side->self_unit = unit;
-	    }
-	}
+	init_self_unit(side);
     }
     /* At this point we should be ready to roll.  Any inconsistencies
        hereafter will be fatal. */
     check_consistency();
-    /* If any sides' balance sheets were not restored, count the units now. */
+    cache_possible_types();
+    /* Count the units initially present. */
     init_side_balance();
     /* Fix up garbled view data. */
     init_all_views();
@@ -830,11 +895,13 @@ final_init()
     init_scores();
     /* Calculate which random event methods will be run. */
     init_random_events();
-	/* Calculate extent and position of geographical features. */
-	compute_all_feature_centroids();
-	/* Start the recording of history. */
-	start_history();
+    final_init_world();
+    /* Start the recording of history. */
+    start_history();
 #ifdef DEBUGGING
+    /* Make sure that any debugging-related allocation is done. */
+    if (Debug || DebugG || DebugM)
+      prealloc_debug();
     /* Report on memory consumption. */
     Dprintf("One side is %d bytes.\n", sizeof(Side));
     Dprintf("One unit is %d bytes, one plan is %d bytes.\n",
@@ -861,7 +928,7 @@ configure_sides()
 void
 init_all_views()
 {
-    int x, y, i = 0, todo = max(1, numsides * area_cells());
+    int x, y, i = 0, todo = max(1, numsides * area.numcells);
     int terrainset;
     Side *side;
 
@@ -938,7 +1005,7 @@ int x, y;
     /* Guaranteed to see the terrain accurately. */
     set_terrain_view(tmpside, x, y, buildtview(terrain_at(x, y)));
     /* If this cell is under observation, don't need to do anything special. */
-    if (tmpside->coverage && cover(tmpside, x, y) > 0)
+    if (cover(tmpside, x, y) > 0)
       return;
     /* Scan all the units here to see if any are visible. */
     for_all_stack(x, y, unit) {
@@ -947,8 +1014,7 @@ int x, y;
 	    if (u_see_always(u)) {
 	    	see_exact(tmpside, x, y);
 	    	/* or flag unit as spotted? */
-		if (tmpside->coverage)
-		  set_cover(tmpside, x, y, 1);
+		set_cover(tmpside, x, y, 1);
 	    	return;
 	    }
 	    chance = (indep(unit) ? u_already_seen_indep(u)
@@ -961,6 +1027,8 @@ int x, y;
 	    }
 	}
     }
+    /* We get to see the weather conditions here. */
+    set_wind_view(tmpside, x, y, raw_wind_at(x, y));
 }
 
 static void
@@ -1071,7 +1139,8 @@ init_area_views()
 /* Method to give all units and terrain a basic stockpile of supply. */
 
 int
-make_initial_materials()
+make_initial_materials(calls, runs)
+int calls, runs;
 {
     int m, t, amts[MAXTTYPES], doany, x, y;
 
@@ -1127,17 +1196,21 @@ saved_game()
 }
 
 /* Count all the initial units in each side's balance sheet, but only
-   if the balance sheet is blank. */
-
-/* (this is like restoring a history log) */
-
-/* (if no log was found, we're starting over - else re-create log and caches)*/
+   if all the balance sheets are completely blank. */
 
 void
 init_side_balance()
 {
+    int u, anything = FALSE;
     Unit *unit;
+    Side *side;
 
+    for_all_sides(side) {
+	for_all_unit_types(u) {
+	    if (total_gain(side, u) > 0)
+	      return;
+	}
+    }
     for_all_units(unit) {
 	count_gain(unit->side, unit->type, initial_gain);
     }
@@ -1173,6 +1246,33 @@ check_consistency()
     /* Need any kind of unit checks? */
 }
 
+static void
+cache_possible_types()
+{
+    int u, u2, rescan;
+    Unit *unit;
+
+    u_possible = (short *) xmalloc(numutypes * sizeof(short));
+    for_all_units(unit) {
+	u_possible[unit->type] = TRUE;
+    }
+    rescan = TRUE;
+    while (rescan) {
+	rescan = FALSE;
+	for_all_unit_types(u) {
+	    if (u_possible[u]) {
+		for_all_unit_types(u2) {
+		    if (uu_acp_to_create(u, u2) > 0
+			&& !u_possible[u2]) {
+			u_possible[u2] = TRUE;
+			rescan = TRUE;
+		    }
+		}
+	    }
+	}
+    }
+}
+
 /* This does the actual assignment of players to sides, and initializes the
    side structures appropriately. */
 
@@ -1254,6 +1354,23 @@ assign_players_to_sides()
     }
 }
 
+/* Make sure any debugging I/O routines have allocated their space.
+   Usually routines like side_desig allocate their working space on
+   demand, but if the first demand occurs during a game being saved
+   because allocation fails, then we're in big trouble.  So this
+   routine, which need only be be called when debugging is turned
+   on, calls allocation-needing things in such a way to cause them
+   to allocate their working space. */
+
+void
+prealloc_debug()
+{
+    side_desig(NULL);
+    player_desig(NULL);
+    unit_desig(NULL);
+    shortest_unique_name(0);
+}
+
 /* Return the program version. */
 
 char *
