@@ -1,5 +1,5 @@
 /* Support for Lisp-style data.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,6 +13,9 @@ any later version.  See the file COPYING.  */
 #include "misc.h"
 #include "lisp.h"
 
+static Symentry *lookup_string PARAMS ((char *str));
+static int hash_name PARAMS ((char *str));
+
 /* Pointer to "nil", the empty list. */
 
 Obj *lispnil;
@@ -33,17 +36,13 @@ Obj *lispunbound;
 
 int numsymbols = 0;
 
-/* The topend on symbols. */
+/* Pointer to the base of the symbol table itself. */
 
-int maxsymbols = 600;
-
-/* Pointer to the symbol table itself. */
-
-Symentry *symboltable = NULL;
+static Symentry **symboltablebase = NULL;
 
 /* The number of Lisp objects allocated so far. */
 
-long lispmalloc = 0;
+int lispmalloc = 0;
 
 /* This variable is used to track the depth of nested #| |# comments. */
 
@@ -51,6 +50,14 @@ int commentdepth = 0;
 
 int actually_read_lisp = TRUE;
 
+#define BIGBUF 1000
+
+static char *lispstrbuf = NULL;
+
+static int *startlineno;
+static int *endlineno;
+static char linenobuf[50];
+
 /* Allocate a new Lisp object, count it as such. */
 
 static Obj *
@@ -79,7 +86,7 @@ init_lisp()
     /* The "unbound" object is for unbound variables. */
     lispunbound = newobj();
     /* Set up the symbol table. */
-    symboltable = (Symentry *) xmalloc(maxsymbols * sizeof(Symentry));
+    symboltablebase = (Symentry **) xmalloc(256 * sizeof(Symentry *));
     numsymbols = 0;
     init_predefined_symbols();
 }
@@ -115,14 +122,6 @@ Strm *strm;
    first characters, but lots of semantics actions happen while reading, so
    this isn't really a regular expression reader. */
 
-#define BIGBUF 1000
-
-char *lispstrbuf = NULL;
-
-int *startlineno;
-int *endlineno;
-char linenobuf[50];
-
 Obj *
 read_form(fp, p1, p2)
 FILE *fp;
@@ -138,7 +137,7 @@ int *p1, *p2;
     tmpstrm.ptr.fp = fp;
     rslt = read_form_aux(&tmpstrm);
     if (rslt == lispclosingparen) {
-	sprintf_line_numbers(linenobuf, *startlineno, *endlineno);
+	sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
 	init_warning("extra close paren, substituting nil%s", linenobuf);
 	rslt = lispnil;
     }
@@ -160,7 +159,7 @@ int *p1, *p2;
     tmpstrm.ptr.sp = str;
     rslt = read_form_aux(&tmpstrm);
     if (rslt == lispclosingparen) {
-	sprintf_line_numbers(linenobuf, *startlineno, *endlineno);
+	sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
 	init_warning("extra close paren, substituting nil%s", linenobuf);
 	rslt = lispnil;
     }
@@ -168,14 +167,21 @@ int *p1, *p2;
 }
 
 void
-sprintf_line_numbers(buf, start, end)
-char *buf;
-int start, end;
+sprintf_context(buf, n, start, end, context)
+char *buf, *context;
+int n, *start, *end;
 {
-    if (start == end) {
-	sprintf(buf, " (at line %d)", start);
-    } else {
-	sprintf(buf, " (lines %d to %d)", start, end);
+    buf[0] = '\0';
+    if (start != NULL && end != NULL) {
+	if (*start == *end) {
+	    sprintf(buf, " (at line %d)", *start);
+	} else {
+	    sprintf(buf, " (lines %d to %d)", *start, *end);
+	}
+    }
+    if (!empty_string(context)) {
+	strncpy(buf+strlen(buf), context, n - 1);
+	buf[n - 1] = '\0';
     }
 }
 
@@ -210,7 +216,8 @@ Strm *strm;
 			    strmungetc(ch4, strm);
 			}
 		    } else if (ch3 == '\n') {
-			++(*endlineno);
+			if (endlineno != NULL)
+			  ++(*endlineno);
 			announce_read_progress();
 		    }
 		}
@@ -228,14 +235,17 @@ Strm *strm;
 	if (isspace(ch)) {
 	    /* Nothing to do here except count lines. */
 	    if (ch == '\n') {
-		++(*endlineno);
-		++(*startlineno);
+		if (endlineno != NULL)
+		  ++(*endlineno);
+		if (startlineno != NULL)
+		  ++(*startlineno);
 		announce_read_progress();
 	    }
 	} else if (ch == ';') {
 	    /* Discard all from here to the end of this line. */
 	    while ((ch = strmgetc(strm)) != EOF && ch != '\n');
-	    ++(*endlineno);
+	    if (endlineno != NULL)
+	      ++(*endlineno);
 	    announce_read_progress();
 	} else if (ch == '(') {
 	    /* Jump into a list-reading mode. */
@@ -328,27 +338,42 @@ Strm *strm;
     return lispeof;
 }
 
-/* Read a sequence of expressions terminated by a closing paren. */
+/* Read a sequence of expressions terminated by a closing paren.  This works
+   by looping; although recursion is more elegant, if the compiler does not
+   turn tail-recursion into loops, long lists can blow the stack.  (This has
+   happened with real saved games.) */
 
 Obj *
 read_list(strm)
 Strm *strm;
 {
-    Obj *thecar, *thecdr;
+    Obj *thecar, *thenext, *lis, *endlis;
 
     thecar = read_form_aux(strm);
     if (thecar == lispclosingparen) {
 	return lispnil;
     } else if (thecar == lispeof) {
-	sprintf_line_numbers(linenobuf, *startlineno, *endlineno);
-	init_warning("missing a close paren, returning EOF%s", linenobuf);
-	return lispeof;
+	goto at_eof;
     } else {
-	thecdr = read_list(strm);
-	if (thecdr == lispeof)
-	  return lispeof;
-	return (cons(thecar, thecdr));
+	lis = cons(thecar, lispnil);
+	endlis = lis;
+	while (TRUE) {
+	    thenext = read_form_aux(strm);
+	    if (thenext == lispclosingparen) {
+		break;
+	    } else if (thenext == lispeof) {
+		goto at_eof;
+	    } else {
+		set_cdr(endlis, cons(thenext, lispnil));
+		endlis = cdr(endlis);
+	    }
+	}
+	return lis;
     }
+  at_eof:
+    sprintf_context(linenobuf, 50, startlineno, endlineno, NULL);
+    init_warning("missing a close paren, returning EOF%s", linenobuf);
+    return lispeof;
 }
 
 /* Read a quantity of text delimited by a char from the given string,
@@ -577,8 +602,14 @@ char *
 c_string(x)
 Obj *x;
 {
-    /* (should test for appropriate type?) */
-    return (symbolp(x) ? symboltable[x->v.sym.symnum].name : x->v.str);
+    if (x->type == STRING) {
+	return x->v.str;
+    } else if (x->type == SYMBOL) {
+	return x->v.sym.symentry->name;
+    } else {
+     	type_warning("c_string", x, "string/symbol", lispnil);
+	return "";
+   }
 }
 
 /* Return the actual number in a number object. */
@@ -587,8 +618,15 @@ int
 c_number(x)
 Obj *x;
 {
-    /* (should test for appropriate type?) */
-    return x->v.num;
+    if (x->type == NUMBER
+	|| x->type == UTYPE
+	|| x->type == MTYPE
+	|| x->type == TTYPE) {
+	return x->v.num;
+    } else {
+     	type_warning("c_number", x, "number", lispnil);
+	return 0;
+    }
 }
 
 Obj *
@@ -596,40 +634,51 @@ intern_symbol(str)
 char *str;
 {
     int n;
+    Symentry *se;
+    Obj *new1;
 
-    if (numsymbols >= maxsymbols) {
-	run_error("can't intern `%s'; no more symbols allowed!", str);
-    }
-    if ((n = lookup_string(str)) >= 0) {
-	return symboltable[n].symbol;
+    se = lookup_string(str);
+    if (se) {
+	return se->symbol;
     } else {
-	Obj *new = newobj();
-
-	new->type = SYMBOL;
-	new->v.sym.symnum = numsymbols;
+	new1 = newobj();
+	new1->type = SYMBOL;
+	se = (Symentry *) xmalloc(sizeof(Symentry));
+	new1->v.sym.symentry = se;
 	/* Declare a newly created symbol to be unbound. */
-	new->v.sym.value = lispunbound;
-	symboltable[numsymbols].name = copy_string(str);
-	symboltable[numsymbols].symbol = new;
-	symboltable[numsymbols].constantp = FALSE;
-	numsymbols++;
-	return new;
+	new1->v.sym.value = lispunbound;
+	se->name = copy_string(str); 
+	se->symbol = new1;
+	se->constantp = FALSE;
+	n = hash_name(str);
+	/* Push the symbol entry onto the front of its hash bucket. */
+	se->next = symboltablebase[n];
+	symboltablebase[n] = se;
+	++numsymbols;
+	return new1;
     }
 }
 
-/* Given a string, try to find a symbol with that as its name. */
+/* Given a string, try to find a symbol entry with that as its name. */
 
-int
+static Symentry *
 lookup_string(str)
 char *str;
 {
-    int i;
+    Symentry *se;
 
-    for (i = 0; i < numsymbols; ++i) {
-	if (strcmp(symboltable[i].name, str) == 0)
-	  return i;
+    for (se = symboltablebase[hash_name(str)]; se != NULL; se = se->next) {
+	if (strcmp(se->name, str) == 0)
+	  return se;
     }
-    return (-1);
+    return NULL;
+}
+
+static int
+hash_name(str)
+char *str;
+{
+    return str[0];
 }
 
 Obj *
@@ -649,9 +698,8 @@ Obj *
 setq(sym, x)
 Obj *sym, *x;
 {
-    /* what if not a symbol? */
     if (!symbolp(sym)) {
-	run_warning("Can't set a non-symbol");
+	run_warning("Can't set a non-symbol, ignoring attempt");
 	return x;
     }
     if (constantp(sym)) {
@@ -674,14 +722,14 @@ void
 flag_as_constant(sym)
 Obj *sym;
 {
-    symboltable[sym->v.sym.symnum].constantp = TRUE;
+    sym->v.sym.symentry->constantp = TRUE;
 }
 
 int
 constantp(sym)
 Obj *sym;
 {	
-    return (symboltable[sym->v.sym.symnum].constantp);
+    return (sym->v.sym.symentry->constantp);
 }
 
 int
@@ -843,12 +891,12 @@ find_at_key(lis, key)
 Obj *lis;
 char *key;
 {
-    Obj *rest, *bdgs;
+    Obj *rest, *bdgs, *bdg;
 
-    for (rest = lis; rest != lispnil; rest = cdr(rest)) {
+    for_all_list(lis, rest) {
 	bdgs = car(rest);
-	if (stringp(car(bdgs))
-	    && strcmp(key, c_string(car(bdgs))) == 0) {
+	bdg = car(bdgs);
+	if (stringp(bdg) && strcmp(key, c_string(bdg)) == 0) {
 	    return cdr(bdgs);
 	}
     }
@@ -860,12 +908,12 @@ replace_at_key(lis, key, newval)
 Obj *lis, *newval;
 char *key;
 {
-    Obj *rest, *bdgs;
+    Obj *rest, *bdgs, *bdg;
 
-    for (rest = lis; rest != lispnil; rest = cdr(rest)) {
+    for_all_list(lis, rest) {
 	bdgs = car(rest);
-	if (stringp(car(bdgs))
-	    && strcmp(key, c_string(car(bdgs))) == 0) {
+	bdg = car(bdgs);
+	if (stringp(bdg) && strcmp(key, c_string(bdg)) == 0) {
 	    set_cdr(bdgs, newval);
 	    return lis;
 	}
@@ -878,6 +926,14 @@ fprintlisp(fp, obj)
 FILE *fp;
 Obj *obj;
 {
+    int needescape;
+    char *str, *tmp;
+
+    /* Doublecheck, just in case caller is not so careful. */
+    if (obj == NULL) {
+	run_warning("Trying to print NULL as object, skipping");
+	return;
+    }
     switch (obj->type) {
       case NIL:
 	fprintf(fp, "nil");
@@ -886,12 +942,38 @@ Obj *obj;
 	fprintf(fp, "%d", obj->v.num);
 	break;
       case STRING:
-	/* (should print escape chars if needed) */
-	fprintf(fp, "\"%s\"", obj->v.str);
+	if (strchr(obj->v.str, '"')) {
+	    fprintf(fp, "\"");
+	    for (tmp = obj->v.str; *tmp != '\0'; ++tmp) {
+		if (*tmp == '"')
+		  fprintf(fp, "\\");
+		fprintf(fp, "%c", *tmp);
+	    }
+	    fprintf(fp, "\"");
+	} else {
+	    /* Just printf the whole string. */
+	    fprintf(fp, "\"%s\"", obj->v.str);
+	}
 	break;
       case SYMBOL:
-	/* (should print escape chars if needed) */
-	fprintf(fp, "%s", c_string(obj));
+	needescape = FALSE;
+	str = c_string(obj);
+	if (isdigit(str[0])) {
+	    needescape = TRUE;
+	} else {
+	    /* Scan the symbol's name looking for special chars. */
+	    for (tmp = str; *tmp != '\0'; ++tmp) {
+		if (strchr(" ()#\";|", *tmp)) {
+		    needescape = TRUE;
+		    break;
+		}
+	    }
+	}
+	if (needescape) {
+	    fprintf(fp, "|%s|", str);
+	} else {
+	    fprintf(fp, "%s", str);
+	}
 	break;
       case CONS:
 	fprintf(fp, "(");
@@ -925,7 +1007,7 @@ Obj *obj;
 {
     Obj *tmp;
 
-    for (tmp = obj; tmp != lispnil; tmp = cdr(tmp)) {
+    for_all_list(obj, tmp) {
 	fprintf(fp, " ");
 	fprintlisp(fp, car(tmp));
     }	
@@ -997,8 +1079,7 @@ Obj *obj;
 }
 
 #ifdef DEBUGGING
-#ifdef USE_CONSOLE
-/* For calling from debuggers. */
+/* For calling from debuggers, at least that those that support output to stderr. */
 
 void
 dlisp(x)
@@ -1007,9 +1088,25 @@ Obj *x;
     fprintlisp(stderr, x);
     fprintf(stderr, "\n");
 }
-#endif /* USE_CONSOLE */
 #endif /* DEBUGGING */
 
+void
+print_form_and_value(fp, form)
+FILE *fp;
+Obj *form;
+{
+    fprintlisp(fp, form);
+    if (symbolp(form)) {
+	if (boundp(form)) {
+	    fprintf(fp, " -> ");
+	    fprintlisp(fp, symbol_value(form));
+	} else {
+	    fprintf(fp, " <unbound>");
+	}
+    }
+    fprintf(fp, "\n");
+}
+
 Obj *
 append_two_lists(x1, x2)
 Obj *x1, *x2;
@@ -1178,35 +1275,135 @@ Obj *lis;
 }
 
 int
-interpolate_in_list(val, lis, extrap, maxrslt)
-int val, extrap, maxrslt;
-Obj *lis;
+eval_boolean_expression(expr, fn, dflt)
+Obj *expr;
+int (*fn) PARAMS ((Obj *)), dflt;
 {
-    int thisin, thisval, nextin, nextval, rslt;
-    Obj *rest;
+    char *opname;
 
-    for (rest = lis; rest != lispnil; rest = cdr(rest)) {
-	thisin = c_number(car(car(rest)));
-	thisval = c_number(cadr(car(rest)));
-	if (cdr(rest)) {
-	    nextin = c_number(car(cadr(rest)));
-	    nextval = c_number(cadr(cadr(rest)));
-	} else {
-	    nextin = 99;
-	    nextval = maxrslt;
-	}
-	if (val < thisin) {
-	    /* Interpolate between thisin and 0. */
-	    return (thisval * val) / thisin;
-	} else if (between(thisin, val, nextin)) {
-	    rslt = thisval;
-	    if (val != thisin) {
-		/* Add the linear interpolation. */
-	    	rslt += ((nextval - thisval) * (val - thisin)) / (nextin - thisin);
-	    }
-	    return rslt;
+    if (expr == lispnil) {
+	return dflt;
+    } else if (consp(expr) && symbolp(car(expr))) {
+	opname = c_string(car(expr));
+	switch (keyword_code(opname)) {
+	  case K_AND:
+	    return (eval_boolean_expression(cadr(expr), fn, dflt)
+		    && eval_boolean_expression(car(cddr(expr)), fn, dflt));
+	  case K_OR:
+	    return (eval_boolean_expression(cadr(expr), fn, dflt)
+		    || eval_boolean_expression(car(cddr(expr)), fn, dflt));
+	  case K_NOT:
+	    return !eval_boolean_expression(cadr(expr), fn, dflt);
+	  default:
+	    return (*fn)(expr);
 	}
+    } else {
+	return (*fn)(expr);
     }
-    return maxrslt;
+}
+
+int
+interpolate_in_list(val, lis, rslt)
+int val, *rslt;
+Obj *lis;
+{
+    int first, thisin, thisval, nextin, nextval;
+    Obj *rest, *head, *next;
+
+    first = TRUE;
+    for_all_list(lis, rest) {
+	head = car(rest);
+	thisin = c_number(car(head));
+	thisval = c_number(cadr(head));
+	if (cdr(rest) != lispnil) {
+	    next = cadr(rest);
+	    nextin = c_number(car(next));
+	    nextval = c_number(cadr(next));
+	    first = FALSE;
+	} else if (first) {
+	    if (val == thisin) {
+		*rslt = thisval;
+		return 0;
+	    } else if (val < thisin) {
+		return (-1);
+	    } else {
+		return 1;
+	    }
+	} else {
+	    /* We're at the end of a several-item list; the value
+	       must be too high. */
+	    return 1;
+	}
+	if (val < thisin) {
+	    return (-1);
+	} else if (between(thisin, val, nextin)) {
+	    if (val == thisin) {
+		*rslt = thisval;
+	    } else if (val == nextin) {
+		*rslt = nextval;
+	    } else {
+		*rslt = thisval;
+		if (val != thisin && nextin != thisin) {
+		    /* Add the linear interpolation. */
+		    *rslt += ((nextval - thisval) * (val - thisin)) / (nextin - thisin);
+		}
+	    }
+	    return 0;
+	}
+    }
+    return (-1);
+}
+
+int
+interpolate_in_list_ext(val, lis, mindo, minval, minrslt, maxdo, maxval, maxrslt, rslt)
+int val, mindo, minval, minrslt, maxdo, maxval, maxrslt, *rslt;
+Obj *lis;
+{
+    int first, thisin, thisval, nextin, nextval;
+    Obj *rest, *head, *next;
+
+    /* (should use the additional parameters) */
+    first = TRUE;
+    for_all_list(lis, rest) {
+	head = car(rest);
+	thisin = c_number(car(head));
+	thisval = c_number(cadr(head));
+	if (cdr(rest) != lispnil) {
+	    next = cadr(rest);
+	    nextin = c_number(car(next));
+	    nextval = c_number(cadr(next));
+	    first = FALSE;
+	} else if (first) {
+	    if (val == thisin) {
+		*rslt = thisval;
+		return 0;
+	    } else if (val < thisin) {
+		return (-1);
+	    } else {
+		return 1;
+	    }
+	} else {
+	    /* We're at the end of a several-item list; the value
+	       must be too high. */
+	    return 1;
+	}
+	if (val < thisin) {
+	    return (-1);
+	} else if (between(thisin, val, nextin)) {
+	    if (val == thisin) {
+		*rslt = thisval;
+	    } else if (val == nextin) {
+		*rslt = nextval;
+	    } else {
+		*rslt = thisval;
+		if (val != thisin && nextin != thisin) {
+		    /* Add the linear interpolation. */
+		    *rslt += ((nextval - thisval) * (val - thisin)) / (nextin - thisin);
+		}
+	    }
+	    return 0;
+	}
+    }
+    return (-1);
 }
 
