@@ -11,7 +11,8 @@ any later version.  See the file COPYING.  */
 
 static void set_people_on_side PARAMS ((int x, int y));
 static void expand_country PARAMS ((int x, int y));
-static int country_is_complete PARAMS ((Side *side));
+static int country_is_complete PARAMS ((Side *side, int runs));
+static int average_numunits PARAMS ((int u));
 static void count_cells PARAMS ((int x, int y));
 static int good_place PARAMS ((int cx, int cy));
 static void find_a_place PARAMS ((Side *side));
@@ -122,7 +123,7 @@ int x, y;
 		    if (unit->side != NULL) {
 			/* Maybe take over another side's unit. */
 			if (xrandom(10000) < u_unit_takeover(unit->type)
-			    && !at_country_units_max(tmpside, u)
+			    && !at_country_units_max(tmpside, unit->type)
 			    ) {
 			    set_unit_side(unit, tmpside);
 			    set_unit_origside(unit, tmpside);
@@ -135,7 +136,7 @@ int x, y;
 		    } else {
 			/* Maybe take over an independent unit. */
 			if (xrandom(10000) < u_indep_takeover(unit->type)
-			    && !at_country_units_max(tmpside, u)
+			    && !at_country_units_max(tmpside, unit->type)
 			    ) {
 			    set_unit_side(unit, tmpside);
 			    set_unit_origside(unit, tmpside);
@@ -162,7 +163,8 @@ int x, y;
 /* Place all the units belonging to countries. */
 
 int
-make_countries()
+make_countries(calls, runs)
+int calls, runs;
 {
     int x0, y0, u, t, i, tot, x, y;
     int advantage, sideadvantage, favor, x1, y1, dir;
@@ -232,8 +234,8 @@ make_countries()
 	    allocate_area_people_sides();
 	}
     }
-    /* If no radius specified, pick something plausible. */
     if (g_radius_min() <= 0) {
+	/* If no radius specified, pick something plausible. */
 	baseradius = max(1, isqrt(3 * tot) / 2);
     } else {
 	baseradius = g_radius_min();
@@ -245,14 +247,12 @@ make_countries()
 	sideprogress = (100 * numdone++) / numsides;
 	sidedeltahalf = (100 / numsides) / 2;
 	announce_progress(sideprogress);
-	if (!country_is_complete(side)) {
+	if (!country_is_complete(side, runs)) {
 	    sideadvantage = max(1, side->advantage);
 	    advantage = (side->player ? side->player->advantage : sideadvantage);
-	    if (advantage > 1) {
-		tmpradius = (baseradius * isqrt(advantage * 100) + 9) / 10;
-	    } else {
-	    	tmpradius = baseradius;
-	    }
+	    tmpradius = baseradius;
+	    if (advantage > 1)
+	      tmpradius = (baseradius * isqrt(advantage * 100) + 9) / 10;
 	    /* Discover or generate the country's center. */
 	    mungterrain = FALSE;
 	    find_a_place(side);
@@ -260,10 +260,20 @@ make_countries()
 	    Dprintf("%s starts around %d,%d\n", side_desig(side), x0, y0);
 	    announce_progress(sideprogress + sidedeltahalf);
 	    totnumtodo = totnumdone = 0;
+	    /* Calculate how many units of each type we will get, both the ones
+	       that belong to the side, and the independents nearby. */
 	    for_all_unit_types(u) {
 		numtodo[u] = numindeptodo[u] = 0;
 		if (type_allowed_on_side(u, side)) {
-		    numtodo[u] = (u_start_with(u) * advantage) / sideadvantage;
+		    if (runs > 0) {
+			/* If the side is just coming into the game, give it an
+			   average number of each type that the existing sides
+			   started out with, so that the new side has some parity. */
+			if (u_start_with(u) > 0 || u_indep_near_start(u) > 0)
+			  numtodo[u] = (average_numunits(u) * advantage) / sideadvantage;
+		    } else {
+			numtodo[u] = (u_start_with(u) * advantage) / sideadvantage;
+		    }
 		    totnumtodo += numtodo[u];
 		}
 		if (type_allowed_on_side(u, NULL)) {
@@ -312,7 +322,7 @@ make_countries()
 	    }
 	    /* Now do units that have to be occupants.  Note that if occupants
 	       are allowed in independent units, then they might be
-	       placed in one. */
+	       placed in an independent rather than an owned unit. */
 	    for_all_unit_types(u) {
 		if (!canbeinopen[u]) {
 		    for (i = 0; i < numtodo[u]; ++i) {
@@ -436,22 +446,33 @@ make_countries()
    as a country in this game. */
 
 static int
-country_is_complete(side)
+country_is_complete(side, runs)
 Side *side;
+int runs;
 {
-    int u, numunits[MAXUTYPES];
+    int u, totunits, numunits[MAXUTYPES];
     Unit *unit;
     int sideadvantage = max(1, side->advantage);
     int advantage = (side->player ? side->player->advantage : sideadvantage);
 
+    /* If side no longer in game, then we don't want to return country synth on it. */
+    if (!side->ingame)
+      return TRUE;
+    totunits = 0;
     for_all_unit_types(u)
       numunits[u] = 0;
     /* Count up all and only the completed and present units. */
     for_all_side_units(side, unit) {
     	if (in_play(unit)) {
+	    ++totunits;
 	    ++numunits[unit->type];
 	}
     }
+    /* See if we're re-running side generation for the benefit of adding a
+       new side, use the total counts of units to distinguish the new side
+       from the old sides. */ 
+    if (runs > 0)
+      return (totunits > 0);
     for_all_unit_types(u) {
 	if (numunits[u] < ((u_start_with(u) * advantage) / sideadvantage))
 	  return FALSE;
@@ -459,6 +480,29 @@ Side *side;
     return TRUE;
 }
 
+/* Compute the average number of units of a given type that are currently
+   in the game. */
+
+static int
+average_numunits(u)
+int u;
+{
+    int count = 0, nsides = 0;
+    Unit *unit;
+    Side *side;
+
+    for_all_sides(side) {
+	if (side->ingame) {
+	    ++nsides;
+	    for_all_side_units(side, unit) {
+		if (in_play(unit) && unit->type == u)
+		  ++count;
+	    }
+	}
+    }
+    return (count / (nsides > 1 ? nsides - 1 : 1));
+}
+
 /* Count the cell as being of a particular type. */
 
 static void
@@ -518,7 +562,7 @@ static void
 find_a_place(side)
 Side *side;
 {
-    int tries, x, y, maxtries = area_cells() / 5;
+    int tries, x, y, maxtries = area.numcells / 5;
     Unit *unit;
 
     Dprintf("%s country ", side_desig(side));
@@ -562,8 +606,8 @@ Side *side;
 	}
     }
     /* Then search exhaustively, starting from the center of the area. */
-    if (search_around(area.width / 2, area.height / 2,
-		      area.width / 2 + area.height / 2,
+    if (search_around(area.width / 2, area.halfheight,
+		      area.width / 2 + area.halfheight,
 		      good_place, &x, &y, 1)) {
 	side->startx = x;  side->starty = y;
 	Dprintf("placed after search\n");
@@ -703,7 +747,8 @@ int u;
 /* A method that scatters independent units over the world. */
 
 int
-make_independent_units()
+make_independent_units(calls, runs)
+int calls, runs;
 {
     int u, t, x, y;
     int doindeptype[MAXUTYPES], doanytype = FALSE, doindeppeople = FALSE;
