@@ -1,5 +1,5 @@
 /* Interface-independent natural language handling for Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -11,17 +11,17 @@ any later version.  See the file COPYING.  */
 /* (One way to do this would be to call this file "nlang-en.c", then
    symlink this or nlang-fr.c, etc to nlang.c when configuring;
    similarly for help.c.) */
+/* (A better way to would be to run all these through a dispatch
+   vector, then each player could get messages and such in a
+   different language.) */
 
 #include "conq.h"
-extern void notify_all_of_resignation PARAMS ((Side *side, Side *side2));
-
-extern int short_side_title_plural_p PARAMS ((Side *side));
-extern char *short_side_title_with_adjective PARAMS ((Side *side, char *adjective));
+extern void location_desc PARAMS ((char *buf, Side *side, Unit *unit, int u, int x, int y));
+extern int supply_desc PARAMS ((char *buf, Unit *unit, int mrow));
 
+static void init_calendar PARAMS ((void));
 static void maybe_mention_date PARAMS ((Side *side));
 
-static char *usual_date_string PARAMS ((int date));
-
 static int gain_count PARAMS ((Side *side, int u, int r));
 static int loss_count PARAMS ((Side *side, int u, int r));
 static int atkstats PARAMS ((Side *side, int a, int d));
@@ -29,10 +29,9 @@ static int hitstats PARAMS ((Side *side, int a, int d));
 
 static char *tmpnbuf;
 
-int basehour;
-int baseday = -1;
-int basemonth = 0;
-int baseyear = 1900;
+static char *tmpdbuf;
+
+static char *pluralbuf;
 
 /* Short names of directions. */
 
@@ -44,22 +43,55 @@ char *past_unitbuf = NULL;
 
 static char *side_short_title = NULL;
 
-static char *gain_reason_names[] = { "Ini", "Bld", "Cap", "Gft", "Oth" };
+static char *gain_reason_names[] = { "Ini", "Bld", "Cap", "Oth" };
 
-static char *loss_reason_names[] = { "Cbt", "Cap", "Stv", "Acc", "Dis", "Lef", "Gft", "Oth" };
+static char *loss_reason_names[] = { "Cbt", "Cap", "Stv", "Acc", "Dis", "Oth" };
 
-/* (should use an enum for date step names) */
+/* Calendar handling. */
 
-int calendartype = -1;
+typedef enum {
+    cal_unknown,
+    cal_turn,
+    cal_usual
+} CalendarType;
 
-char *datebuf = NULL;
+static CalendarType calendar_type = cal_unknown;
+
+typedef enum {
+    ds_second,
+    ds_minute,
+    ds_hour,
+    ds_day,
+    ds_week,
+    ds_month,
+    ds_season,
+    ds_year
+} UsualDateStepType;
+
+typedef struct a_usualdate {
+    int second;
+    int minute;
+    int hour;
+    int day;
+    int month;
+    int year;
+} UsualDate;
+
+static char *usual_date_string PARAMS ((int date));
+static void parse_usual_date PARAMS ((char *datestr, UsualDate *udate));
+
+static UsualDateStepType datesteptype;
+
+static int datestep;
+
+static char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "???" };
+
+UsualDate *usual_initial;
+
+char *datebuf;
 
 char *turnname;
-char *datestepname;
-int datestep;
-
-char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "???" };
 
 /* This is the number of types to mention by name; any others will
    just be included in the count of missing images. */
@@ -74,6 +106,22 @@ static int missing[4];
 
 static int totlisted = 0;
 
+/* Initialize things.  Note that this happens before any game is loaded, so
+   can't do game-specific init here. */
+
+void
+init_nlang()
+{
+    if (tmpnbuf == NULL)
+      tmpnbuf = xmalloc(BUFSIZE);
+    if (tmpdbuf == NULL)
+      tmpdbuf = xmalloc(BUFSIZE);
+    if (pluralbuf == NULL)
+      pluralbuf = xmalloc(BUFSIZE);
+    if (datebuf == NULL)
+      datebuf = xmalloc(BUFSIZE);
+}
+
 /* Send a message to everybody who's got a screen. */
 
 #ifdef __STDC__
@@ -88,8 +136,6 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
 {
     Side *side;
 
-    if (tmpnbuf == NULL)
-      tmpnbuf = xmalloc(BUFSIZE);
     for_all_sides(side) {
 	if (active_display(side)) {
 	    maybe_mention_date(side);
@@ -125,8 +171,6 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
 {
     if (!active_display(side))
       return;
-    if (tmpnbuf == NULL)
-      tmpnbuf = xmalloc(BUFSIZE);
     maybe_mention_date(side);
 #ifdef __STDC__
     {
@@ -145,7 +189,7 @@ long a1, a2, a3, a4, a5, a6, a7, a8, a9;
     low_notify(side, tmpnbuf);
 }
 
-#ifdef __STDC__
+#if defined(__STDC__) || defined(MPW_C)
 void
 vnotify(side, fmt, ap)
 Side *side;
@@ -154,8 +198,6 @@ va_list ap;
 {
     if (!active_display(side))
       return;
-    if (tmpnbuf == NULL)
-      tmpnbuf = xmalloc(BUFSIZE);
     maybe_mention_date(side);
     vsprintf(tmpnbuf, fmt, ap);
     /* Always capitalize first char of notice. */
@@ -169,8 +211,8 @@ static void
 maybe_mention_date(side)
 Side *side;
 {
-    char tmpdbuf[BUFSIZE];
-
+    /* Note that last_notice_date defaults to 0, so this means that
+       any turn 0 notices will not have a date prepended (which is good). */
     if (g_turn() != side->last_notice_date) {
 	sprintf(tmpdbuf, "%s:", absolute_date_string(g_turn()));
 	low_notify(side, tmpdbuf);
@@ -178,7 +220,8 @@ Side *side;
     }
 }
 
-/* Pad a given buffer with blanks out to the given position. */
+/* Pad a given buffer with blanks out to the given position, and cut off any
+   additional content. */
 
 void
 pad_out(buf, n)
@@ -195,6 +238,34 @@ int n;
     buf[n - 1] = '\0';
 }
 
+/* Get a char string naming the side.  Doesn't have to be pretty. */
+
+/* (should synth complete name/adjective from other parts of speech) */
+
+char *
+side_name(side)
+Side *side;
+{
+    return (side == NULL ? "independent" :
+      (side->name ? side->name :
+	    (side->adjective ? side->adjective :
+	     (side->pluralnoun ? side->pluralnoun :
+	      (side->noun ? side->noun :
+	       "")))));
+}
+
+char *
+side_adjective(side)
+Side *side;
+{
+    return (side == NULL ? "independent" :
+       (side->adjective ? side->adjective :
+	    (side->noun ? side->noun :
+	     (side->pluralnoun ? side->pluralnoun :
+	      (side->name ? side->name :
+	       "")))));
+}
+
 char *
 short_side_title(side)
 Side *side;
@@ -336,6 +407,29 @@ Side *side, *side2;
     return "???";
 }
 
+char *
+side_score_desc(buf, side, sk)
+char *buf;
+Side *side;
+Scorekeeper *sk;
+{
+    if (symbolp(sk->body)
+		&& match_keyword(sk->body, K_LAST_SIDE_WINS)) {
+	sprintf(buf, "Point Value: %d", side_point_value(side));
+    } else {
+	/* Compose the generic scorekeeper status display. */
+	if (sk->title != NULL) {
+	    sprintf(buf, "%s", sk->title);
+	} else {
+	    sprintf(buf, "SK #%d", sk->id);
+	}
+	if (sk->scorenum >= 0) {
+	    tprintf(buf, ": %d", side->scores[sk->scorenum]);
+	}
+    }
+    return buf;
+}
+
 char *
 long_player_title(buf, player, thisdisplayname)
 char *buf, *thisdisplayname;
@@ -347,13 +441,6 @@ Player *player;
     } else if (player->displayname != NULL) {
 	if (player->name != NULL) {
 	    strcat(buf, player->name);
-	}
-	if (player->aitypename != NULL) {
-	    strcat(buf, "(& AI ");
-	    strcat(buf, player->aitypename);
-	    strcat(buf, ")");
-	}
-	if (player->name != NULL) {
 	    strcat(buf, "@");
 	}
 	if (thisdisplayname != NULL
@@ -362,6 +449,11 @@ Player *player;
 	} else {
 	    strcat(buf, player->displayname);
 	}
+	if (player->aitypename != NULL) {
+	    strcat(buf, "(& AI ");
+	    strcat(buf, player->aitypename);
+	    strcat(buf, ")");
+	}
     } else if (player->aitypename != NULL) {
 	strcat(buf, "AI ");
 	strcat(buf, player->aitypename);
@@ -377,28 +469,27 @@ char *buf, *thisdisplayname;
 Player *player;
 {
     buf[0] = '\0';
-    if (player == NULL) {
-	/* Do nothing */
-    } else if (player->displayname != NULL) {
-	if (player->name != NULL) {
-	    strcat(buf, player->name);
-	}
-	if (player->aitypename != NULL) {
-	    strcat(buf, ",");
-	    strcat(buf, player->aitypename);
-	}
-	if (player->name != NULL) {
-	    strcat(buf, "@");
-	}
-	if (thisdisplayname != NULL
-	    && strcmp(player->displayname, thisdisplayname) == 0) {
-	    strcat(buf, "You");
-	} else {
-	    strcat(buf, player->displayname);
-	}
-    } else if (player->aitypename != NULL) {
+    if (player == NULL)
+      return buf;
+    if (player->name != NULL) {
+	strcat(buf, player->name);
+    }
+    if (player->aitypename != NULL) {
+	strcat(buf, ",");
 	strcat(buf, player->aitypename);
-    } else {
+    }
+    if ((player->name != NULL || player->aitypename != NULL)
+	&& player->displayname != NULL) {
+	strcat(buf, "@");
+    }
+    if (thisdisplayname != NULL
+	&& player->displayname != NULL
+	&& strcmp(player->displayname, thisdisplayname) == 0) {
+	strcat(buf, "You");
+    } else if (player->displayname != NULL) {
+	strcat(buf, player->displayname);
+    }
+    if (strlen(buf) == 0) {
 	strcat(buf, "-");
     }
     return buf;
@@ -434,7 +525,7 @@ Side *side;
 Unit *unit;
 {
     char *utypename;
-    Side *side2;
+    Side *side2, *side3;
 
     if (unitbuf == NULL)
       unitbuf = xmalloc(BUFSIZE);
@@ -453,8 +544,14 @@ Unit *unit;
     if (u_name_format(unit->type) == 2 && unit->name)
       return unit->name;
 #endif
-    /* Decide how to identify the side. */
+    /* Decide how to identify the side.  If the unit's original side
+       is not its current side, list both of them. */
     side2 = unit->side;
+    side3 = NULL;
+    if (unit->origside != NULL && side2 != unit->origside) {
+	side2 = unit->origside;
+	side3 = unit->side;
+    }
     if (side2 == NULL) {
 	sprintf(unitbuf, "the ");
     } else if (side2 == side) {
@@ -464,6 +561,15 @@ Unit *unit;
 		(side2->adjective ? side2->adjective :
 		 (side2->noun ? side2->noun : side_desig(side2))));
     }
+    if (side3 != NULL) {
+	if (side3 == side) {
+	    tprintf(unitbuf, "(your) ");
+	} else {
+	    tprintf(unitbuf, "(%s-held) ",
+		    (side3->adjective ? side3->adjective :
+		     (side3->noun ? side3->noun : side_desig(side3))));
+	}
+    }
     /* Now add the unit's unique description. */
     /* (Should this ever use short name?) */
     utypename = u_type_name(unit->type);
@@ -603,7 +709,7 @@ char *buf;
 
 /* Given a unit and optional type u, summarize construction status
    and timing. */
-
+/* (should reindent) */
 void
 construction_desc(buf, unit, u)
 char *buf;
@@ -627,7 +733,7 @@ int u;
     }
     name_or_number(unit, tmpbuf);
     sprintf(buf, "%s%s %s", ubuf, u_type_name(unit->type), tmpbuf);
-    pad_out(buf, 20);
+    pad_out(buf, 25);
     if (unit->plan
 	&& unit->plan->tasks) {
       task = unit->plan->tasks;
@@ -658,6 +764,7 @@ Unit *unit;
 {
     char estbuf[20];
     char techbuf[50];
+    char typenamebuf[50];
     int est, tp, num;
 
     if (unit != NULL) {
@@ -679,7 +786,13 @@ Unit *unit;
     } else {
 	techbuf[0] = '\0';
     }
-    sprintf(buf, "%s%s%-16.16s", estbuf, techbuf, u_type_name(u));
+    strcpy(typenamebuf, u_type_name(u));
+    /* If the single char for the type is different from the first character
+       of its type name, mention the char. */
+    if (!empty_string(u_uchar(u)) && (u_uchar(u))[0] != typenamebuf[0]) {
+	tprintf(typenamebuf, "(%c)", (u_uchar(u))[0]);
+    }
+    sprintf(buf, "%s%s%-16.16s", estbuf, techbuf, typenamebuf);
     num = num_units_in_play(side, u);
     if (num > 0) {
 	tprintf(buf, "  %3d", num);
@@ -695,9 +808,20 @@ est_completion_time(unit, u2)
 Unit *unit;
 int u2;
 {
-    if (uu_acp_to_create(unit->type, u2) < 1)
+    int u, tooluptime, tp;
+
+    u = unit->type;
+    if (uu_acp_to_create(u, u2) < 1)
       return (-1);
-    return normal_completion_time(unit->type, u2);
+    tooluptime = 0;
+    tp = (unit->tooling ? unit->tooling[u2] : 0);
+    if (tp < uu_tp_to_build(u, u2)) {
+	if (uu_acp_to_toolup(u, u2) < 1 || uu_tp_per_toolup(u, u2) <= 0 || u_acp(u) <= 0)
+	  return (-1);
+	tooluptime = ((uu_tp_to_build(u, u2) - tp) * uu_acp_to_toolup(u, u2))
+	 / (uu_tp_per_toolup(u, u2) * u_acp(u));
+    }
+    return tooluptime + normal_completion_time(unit->type, u2);
 }
 
 void
@@ -708,8 +832,6 @@ char *buf;
     sprintf(buf, "%d: ", hevt->startdate);
 }
 
-int find_event_type PARAMS ((Obj *sym));
-
 int
 find_event_type(sym)
 Obj *sym;
@@ -731,29 +853,28 @@ char *buf;
 {
     int data0 = hevt->data[0];
     int data1 = hevt->data[1];
-    Obj *rest, *pattern, *text;
+    Obj *rest, *head, *pattern, *text;
     Unit *unit;
     PastUnit *pastunit, *pastunit2;
     Side *side2;
     
-    if (g_event_messages() != lispnil) {
-	for (rest = g_event_messages(); rest != lispnil; rest = cdr(rest)) {
-	    if (consp(car(rest))) {
-		pattern = car(car(rest));
-		if (symbolp(pattern)
-		    && find_event_type(pattern) == hevt->type) {
-		    text = cadr(car(rest));
-		    sprintf(buf, c_string(text));
-		    return;
-		} else if (consp(pattern)
-			   && symbolp(car(pattern))
-			   && find_event_type(car(pattern)) == hevt->type
-			   /* && args match pattern args */
-			   ) {
-		    text = cadr(car(rest));
-		    sprintf(buf, c_string(text));
-		    return;
-		}
+    for_all_list(g_event_messages(), rest) {
+	head = car(rest);
+	if (consp(head)) {
+	    pattern = car(head);
+	    if (symbolp(pattern)
+		&& find_event_type(pattern) == hevt->type) {
+		text = cadr(head);
+		sprintf(buf, c_string(text));
+		return;
+	    } else if (consp(pattern)
+		       && symbolp(car(pattern))
+		       && find_event_type(car(pattern)) == hevt->type
+		       /* && args match pattern args */
+		       ) {
+		text = cadr(head);
+		sprintf(buf, c_string(text));
+		return;
 	    }
 	}
     }
@@ -816,7 +937,7 @@ char *buf;
 		(side == side2 ? "you" : side_name(side2)));
 	unit = find_unit(data1);
 	if (unit != NULL) {
-	  tprintf(buf, "%s", unit_handle(side, unit));
+	    tprintf(buf, "%s", unit_handle(side, unit));
 	} else {
 	    pastunit = find_past_unit(data1);
 	    if (pastunit != NULL) {
@@ -832,7 +953,7 @@ char *buf;
 		(side == side2 ? "you" : side_name(side2)));
 	unit = find_unit(data1);
 	if (unit != NULL) {
-	  tprintf(buf, "%s", unit_handle(side, unit));
+	    tprintf(buf, "%s", unit_handle(side, unit));
 	} else {
 	    pastunit = find_past_unit(data1);
 	    if (pastunit != NULL) {
@@ -930,6 +1051,8 @@ char *buf;
 	}
 	break;
       case H_UNIT_KILLED:
+      case H_UNIT_DIED_IN_ACCIDENT:
+      case H_UNIT_DIED_FROM_TEMPERATURE:
 	/* Obviously, the unit mentioned here can only be a past unit. */
 	pastunit = find_past_unit(data0);
 	if (pastunit != NULL) {
@@ -937,16 +1060,25 @@ char *buf;
 	} else {
 	    sprintf(buf, "%d??", data0);
 	}
-	tprintf(buf, " was destroyed");
+	if (hevt->type == H_UNIT_KILLED)
+	  tprintf(buf, " was destroyed");
+	else if (hevt->type == H_UNIT_DIED_IN_ACCIDENT)
+	  tprintf(buf, " died in an accident");
+	else
+	  tprintf(buf, " died from excessive temperature");
 	break;
       case H_UNIT_WRECKED:
+      case H_UNIT_WRECKED_IN_ACCIDENT:
 	pastunit = find_past_unit(data0);
 	if (pastunit != NULL) {
 	    sprintf(buf, "%s", past_unit_handle(side, pastunit));
 	} else {
 	    sprintf(buf, "%d??", data0);
 	}
-	tprintf(buf, " was wrecked");
+	if (hevt->type == H_UNIT_WRECKED)
+	  tprintf(buf, " was wrecked");
+	else
+	  tprintf(buf, " was wrecked in an accident");
 	break;
       case H_UNIT_VANISHED:
 	pastunit = find_past_unit(data0);
@@ -966,6 +1098,27 @@ char *buf;
 	}
 	tprintf(buf, " was disbanded");
 	break;
+      case H_UNIT_GARRISONED:
+	pastunit = find_past_unit(data0);
+	if (pastunit != NULL) {
+	    sprintf(buf, "%s", past_unit_handle(side, pastunit));
+	} else {
+	    sprintf(buf, "%d??", data0);
+	}
+	tprintf(buf, " was used to garrison ");
+	unit = find_unit(data1);
+	if (unit != NULL) {
+	    tprintf(buf, "%s", unit_handle(side, unit));
+	} else {
+	    pastunit2 = find_past_unit(data1);
+	    if (pastunit2 != NULL) {
+		tprintf(buf, "%s", past_unit_handle(side, pastunit2));
+	    } else {
+		/* Should never happen, but don't choke if it does. */
+		tprintf(buf, "?????");
+	    }
+	}
+	break;
       case H_UNIT_STARVED:
 	pastunit = find_past_unit(data0);
 	if (pastunit != NULL) {
@@ -984,7 +1137,7 @@ char *buf;
 	}
 	tprintf(buf, " left the world");
 	break;
-       case H_UNIT_NAME_CHANGED:
+      case H_UNIT_NAME_CHANGED:
 	pastunit = find_past_unit(data0);
 	if (pastunit != NULL) {
 	    sprintf(buf, "%s", past_unit_handle(side, pastunit));
@@ -998,19 +1151,21 @@ char *buf;
 	    else
 	      tprintf(buf, " became anonymous");
 	} else {
-	pastunit2 = find_past_unit(data1);
- 	  if (pastunit2 != NULL) {
-	    if (pastunit2->name != NULL)
-	      tprintf(buf, " changed name to \"%s\"", pastunit2->name);
-	    else
-	      tprintf(buf, " became anonymous");
-	  }
+	    pastunit2 = find_past_unit(data1);
+	    if (pastunit2 != NULL) {
+		if (pastunit2->name != NULL)
+		  tprintf(buf, " changed name to \"%s\"", pastunit2->name);
+		else
+		  tprintf(buf, " became anonymous");
+	    }
 	}
 	break;
       default:
-	/* Don't warn, will cause serious problems for windows that are lists of
-	   events, but make sure the non-understood event is obvious. */
-	sprintf(buf, "?????????? \"%s\" ??????????", hevtdefns[hevt->type].name);
+	/* Don't warn, will cause serious problems for windows that
+	   are lists of events, but make sure the non-understood event
+	   is obvious. */
+	sprintf(buf, "?????????? \"%s\" ??????????",
+		hevtdefns[hevt->type].name);
 	break;
     }
 }
@@ -1020,7 +1175,7 @@ action_result_desc(rslt)
 int rslt;
 {
     char *str;
-
+    
     switch (rslt) {
       case A_ANY_OK:
 	str = "OK";
@@ -1314,6 +1469,65 @@ int label;
     }
 }
 
+int
+supply_desc(buf, unit, mrow)
+char *buf;
+Unit *unit;
+int mrow;
+{
+    int u = unit->type, m, mm, tmprow;
+
+    tmprow = 0;
+    buf[0] = '\0';
+    mm = 0;
+    for_all_material_types(m) {
+	if (um_storage_x(u, m) > 0) {
+	    if (mm > 0 && mm % 3 == 0)
+	      ++tmprow;
+	    if (tmprow == mrow) {
+		tprintf(buf, "%s %d/%d  ",
+			m_type_name(m), unit->supply[m], um_storage_x(u, m));
+	    }
+	    ++mm;
+	}
+    }
+    return (strlen(buf) > 0);
+}
+
+void
+location_desc(buf, side, unit, u, x, y)
+char *buf;
+Side *side;
+Unit *unit;
+int u, x, y;
+{
+    int t = terrain_at(x, y);
+    char *featurename;
+
+    if (unit != NULL && unit->transport != NULL) {
+	sprintf(buf, "In %s", short_unit_handle(unit->transport));
+    } else if (unit != NULL || u != NONUTYPE) {
+	sprintf(buf, "In %s", t_type_name(t));
+    } else if (terrain_visible(side, x, y)) {
+	sprintf(buf, "Empty %s", t_type_name(t));
+    } else {
+	sprintf(buf, "Unknown");
+    }
+    if (terrain_visible(side, x, y)) {
+	linear_desc(buf, x, y);
+	/* (should use feature desc?) */
+	featurename = feature_name_at(x, y);
+	if (!empty_string(featurename))
+	  tprintf(buf, " (%s)", featurename);
+	if (temperatures_defined())
+	  tprintf(buf, " (T %d)", temperature_at(x, y));
+	if (elevations_defined())
+	  tprintf(buf, " (El %d)", elev_at(x, y));
+	/* (should list local weather also) */
+    }
+    tprintf(buf, " at %d,%d", x, y);
+}
+
 void
 plan_desc(buf, unit)
 char *buf;
@@ -1348,7 +1562,7 @@ Unit *unit;
       strcat(buf, " <formation>");
     if (plan->tasks) {
 	i = 0;
-	for (task = plan->tasks; task != NULL; task = task->next)
+	for_all_tasks(plan, task)
 	  ++i;
 	tprintf(buf, " %d task", i);
 	if (i > 1)
@@ -1384,7 +1598,7 @@ Task *task;
       	tprintf(buf, " at %d,%d (type %d side %d)",
       		task->args[0], task->args[1], task->args[2], task->args[3]);
 	break;
-      case TASK_MOVETO:
+      case TASK_MOVE_TO:
         if (task->args[2] == 0) {
 	    tprintf(buf, " %d,%d", task->args[0], task->args[1]);
         } else if (task->args[2] == 1) {
@@ -1555,16 +1769,12 @@ int n;
 
 /* There should probably be a test for when to add "es" instead of "s". */
 
-char *pluralbuf = NULL;
-
 char *
 plural_form(word)
 char *word;
 {
     char endch = ' ', nextend = ' ';
 
-    if (pluralbuf == NULL)
-      pluralbuf = xmalloc(BUFSIZE);
     if (word == NULL) {
 	run_warning("plural_form given NULL string");
 	pluralbuf[0] = '\0';
@@ -1601,107 +1811,233 @@ long a1, a2, a3, a4;
 
 /* Date/time parsing and formatting. */
 
+long
+parse_date(str)
+char *str;
+{
+    return 1;
+}
+
 /* Compose a readable form of the given date. */
 
 char *
 absolute_date_string(date)
-long date;
+int date;
 {
-    int caltypeunknown = FALSE;
-    Obj *cal, *caltype, *stepname, *step;
-
-    if (datebuf == NULL)
-      datebuf = xmalloc(BUFSIZE);
     /* The first time we ask for a date, interpret the calendar. */
-    if (calendartype < 0) {
-    	cal = g_calendar();
-    	if (consp(cal)) {
-    	    caltype = car(cal);
-    	    if ((symbolp(caltype) || stringp(caltype))
-    	    	&& strcmp("usual", c_string(caltype)) == 0) {
-    	    	calendartype = 1;
-    		stepname = cadr(cal);
-    		datestepname = (stringp(stepname) ? c_string(stepname) : "day");
-    		step = car(cddr(cal));
-    		datestep = (numberp(step) ? c_number(step) : 1);
-    	    } else {
-    	    	caltypeunknown = TRUE;
-    	    }
-    	} else {
-    	    calendartype = 0;
-    	    turnname = "Turn";
-    	    if (stringp(cal)) {
-    	    	turnname = c_string(cal);
-    	    } else if (cal != lispnil) {
-    	    	caltypeunknown = TRUE;
-    	    }
-	}
-	if (caltypeunknown) {
-	    init_warning("Unknown calendar type");
-	}
-    }
-    switch (calendartype) {
-      case 0:
+    if (calendar_type == cal_unknown)
+      init_calendar();
+    switch (calendar_type) {
+      case cal_turn:
 	sprintf(datebuf, "%s%4ld", turnname, date);
 	return datebuf;
-      case 1:
+      case cal_usual:
 	return usual_date_string(date);
       default:
-	case_panic("calendar type", calendartype);
+	case_panic("calendar type", calendar_type);
     }
     return "!?!";
 }
 
+/* Interpret the calendar definition. */
+
+static void
+init_calendar()
+{
+    int caltypeunknown = FALSE;
+    char *stepnamestr;
+    Obj *cal, *caltype, *stepname, *step;
+
+    cal = g_calendar();
+    /* Turn-based calendar is the default. */
+    calendar_type = cal_turn;
+    turnname = "Turn";
+    if (cal == lispnil) {
+	/* Default is fine. */
+    } else if (consp(cal)) {
+	caltype = car(cal);
+	if ((symbolp(caltype) || stringp(caltype))
+	     && strcmp("usual", c_string(caltype)) == 0) {
+	    calendar_type = cal_usual;
+	    stepname = cadr(cal);
+	    if (symbolp(stepname) || stringp(stepname)) {
+		stepnamestr = c_string(stepname);
+		    if (strcmp(stepnamestr, "second") == 0) {
+			datesteptype = ds_second;
+		    } else if (strcmp(stepnamestr, "minute") == 0) {
+			datesteptype = ds_minute;
+		    } else if (strcmp(stepnamestr, "hour") == 0) {
+			datesteptype = ds_hour;
+		    } else if (strcmp(stepnamestr, "day") == 0) {
+			datesteptype = ds_day;
+		    } else if (strcmp(stepnamestr, "week") == 0) {
+			datesteptype = ds_week;
+		    } else if (strcmp(stepnamestr, "month") == 0) {
+			datesteptype = ds_month;
+		    } else if (strcmp(stepnamestr, "season") == 0) {
+			datesteptype = ds_season;
+		    } else if (strcmp(stepnamestr, "year") == 0) {
+			datesteptype = ds_year;
+		    } else {
+			init_warning("\"%s\" not a known date step name", stepnamestr);
+		    }
+	    } else {
+		init_warning("No name for date step type, substituting `day'");
+		datesteptype = ds_day;
+	    }
+	    /* Collect an optional multiple. */
+	    step = car(cddr(cal));
+	    datestep = (numberp(step) ? c_number(step) : 1);
+	    if (!empty_string(g_initial_date())) {
+		set_initial_date(g_initial_date());
+	    }
+	} else {
+	    caltypeunknown = TRUE;
+	}
+    } else if (stringp(cal)) {
+	turnname = c_string(cal);
+    } else {
+	caltypeunknown = TRUE;
+    }
+    if (caltypeunknown)
+      init_warning("Unknown calendar type");
+}
+
 /* Compose a date, omitting components supplied by the base date. */
 
 char *
 relative_date_string(date, base)
-long date, base;
+int date, base;
 {
-    if (datebuf == NULL)
-      datebuf = xmalloc(BUFSIZE);
     /* should do this for real eventually */
     sprintf(datebuf, "%ld(%ld)", date, base);
     return datebuf;
 }
 
-void
-parse_usual_initial_date()
-{
-    char *date = g_base_date();
-    char monthname[BUFSIZE];
-    int i;
+extern int turns_between PARAMS ((char *str1, char *str2));
 
-    if (baseday < 0) {
-	if (!empty_string(date)) {
-	    /* Assume it's in a standard date format. */
-	    if (strcmp(datestepname, "hour") == 0) {
-		sscanf(date, "%d %d %s %d",
-		       &basehour, &baseday, monthname, &baseyear);
-		--baseday;
-	    } else if (strcmp(datestepname, "day") == 0) {
-		sscanf(date, "%d %s %d", &baseday, monthname, &baseyear);
-		--baseday;
-	    } else if (strcmp(datestepname, "week") == 0) {
-		sscanf(date, "%d %s %d", &baseday, monthname, &baseyear);
-		--baseday;
-	    } else if (strcmp(datestepname, "month") == 0) {
-		sscanf(date, "%s %d", monthname, &baseyear);
-	    } else if (strcmp(datestepname, "year") == 0) {
-		sscanf(date, "%d", &baseyear);
-	    }
-	    for (i = 0; i < 12; ++i) {
-		if (strcmp(monthname, months[i]) == 0) {
-		    basemonth = i;
-		    return;
-		}
-	    }
-	} else {
-	    baseday = 0;
+int
+turns_between(datestr1, datestr2)
+char *datestr1, *datestr2;
+{
+    int rslt;
+    UsualDate date1, date2;
+
+    if (calendar_type == cal_unknown)
+      init_calendar();
+    switch (calendar_type) {
+      case cal_usual:
+	parse_usual_date(datestr1, &date1);
+	parse_usual_date(datestr2, &date2);
+	rslt = date2.year - date1.year;
+	if (datesteptype == ds_year)
+	  return (rslt / datestep);
+	if (datesteptype < ds_year) {
+	    rslt = (12 * rslt) - (date2.month - date1.month);
 	}
+	if (datesteptype == ds_month)
+	  return (rslt / datestep);
+	if (datesteptype < ds_month) {
+	    rslt = (30 * rslt) - (date2.day - date1.day);
+	}
+	if (datesteptype == ds_week)
+	  return (((rslt + 6) / 7) / datestep);
+	if (datesteptype == ds_day)
+	  return (rslt / datestep);
+	if (datesteptype < ds_day) {
+	    rslt = (24 * rslt) - (date2.hour - date1.hour);
+	}
+	if (datesteptype == ds_hour)
+	  return (rslt / datestep);
+	return (rslt / datestep); /* semi-bogus */
+      default:
+	return 1;
     }
 }
 
+void
+set_initial_date(str)
+char *str;
+{
+    if (calendar_type == cal_unknown)
+      init_calendar();
+    switch (calendar_type) {
+      case cal_usual:
+	if (usual_initial == NULL)
+	  usual_initial = (UsualDate *) xmalloc(sizeof(UsualDate));
+	parse_usual_date(str, usual_initial);
+	break;
+      default:
+	break;
+    }
+}
+
+static void
+parse_usual_date(datestr, udate)
+char *datestr;
+UsualDate *udate;
+{
+    char monthname[BUFSIZE];
+    int i, cnt;
+    
+    udate->second = udate->minute = udate->hour = udate->day = udate->month = udate->year = 0;
+    monthname[0] = '\0';
+    if (!empty_string(datestr)) {
+	/* Assume it's in a standard date format. */
+	switch (datesteptype) {
+	  case ds_hour:
+	    cnt = sscanf(datestr, "%d:00 %d %s %d",
+			 &(udate->hour), &(udate->day), monthname,
+			 &(udate->year));
+	    if (cnt != 4)
+	      cnt = sscanf(datestr, "%d %d %s %d",
+			   &(udate->hour), &(udate->day), monthname,
+			   &(udate->year));
+	    if (cnt != 4)
+	      goto bad_format;
+	    --(udate->day);
+	    break;
+	  case ds_day:
+	    cnt = sscanf(datestr, "%d %s %d", &(udate->day), monthname,
+			 &(udate->year));
+	    if (cnt != 3)
+	      goto bad_format;
+	    --(udate->day);
+	    break;
+	  case ds_week:
+	    cnt = sscanf(datestr, "%d %s %d", &(udate->day), monthname,
+			 &(udate->year));
+	    if (cnt != 3)
+	      goto bad_format;
+	    --(udate->day);
+	    break;
+	  case ds_month:
+	    cnt = sscanf(datestr, "%s %d", monthname, &(udate->year));
+	    if (cnt != 2)
+	      goto bad_format;
+	    break;
+	  case ds_year:
+	    cnt = sscanf(datestr, "%d", &(udate->year));
+	    if (cnt != 2)
+	      goto bad_format;
+	    return;
+	  default:
+	    init_warning("%d not a known date step type", datesteptype);
+	    break;
+	}
+	for (i = 0; i < 12; ++i) {
+	    if (strcmp(monthname, months[i]) == 0) {
+		udate->month = i;
+		return;
+	    }
+	}
+	init_warning("\"%s\" not a recognized month name", monthname);
+    }
+    return;
+  bad_format:
+    init_warning("\"%s\" is a badly formatted date", datestr);
+}
+
 
 /* Given a numeric data, convert it into something understandable.  Depends
    on the length of a turn. */
@@ -1717,61 +2053,75 @@ int date;
        routine may be called before the game really starts, so return
        something that will be distinctive if it's ever displayed. */
     if (date <= 0)
-      return "?date<=0?";
+      return "pregame";
     /* First displayed date is normally turn 1; be zero-based for
        the benefit of calculation. */
     --date;
     date *= datestep;
-    parse_usual_initial_date();
-    if (strcmp(datestepname, "second") == 0) {
+    if (usual_initial == NULL) {
+	usual_initial = (UsualDate *) xmalloc(sizeof(UsualDate));
+	if (!empty_string(g_initial_date()))
+	  parse_usual_date(g_initial_date(), usual_initial);
+    }
+    switch (datesteptype) {
+      case ds_second:
 	second = date % 60;
 	minute = date / 60;
 	sprintf(datebuf, "%d:%d", minute, second);
-    } else if (strcmp(datestepname, "minute") == 0) {
+	break;
+      case ds_minute:
 	minute = date % 60;
-	hour = date / 60 + basehour;
+	hour = date / 60 + usual_initial->hour;
 	sprintf(datebuf, "%d:%d", hour, minute);
-    } else if (strcmp(datestepname, "hour") == 0) {
- 	hour = (date + basehour) % 24;
+	break;
+      case ds_hour:
+ 	hour = (date + usual_initial->hour) % 24;
 	/* Convert to days, then proceed as for days. */
- 	date = (date + basehour) / 24;
-	date += 30 * basemonth;
+ 	date = (date + usual_initial->hour) / 24;
+	date += 30 * usual_initial->month;
 	day = date % 365;
 	month = day / 30;
-	day = (day + baseday) % 30 + 1;
-	year = date / 365 + baseyear;
+	day = (day + usual_initial->day) % 30 + 1;
+	year = date / 365 + usual_initial->year;
 	sprintf(datebuf, "%d:00 %2d %s %d", hour, day, months[month], ABS(year));
-   } else if (strcmp(datestepname, "day") == 0) {
+	break;
+      case ds_day:
 	/* Should do this more accurately... */
-	date += 30 * basemonth;
+	date += 30 * usual_initial->month;
 	day = date % 365;
 	month = day / 30;
-	day = (day + baseday) % 30 + 1;
-	year = date / 365 + baseyear;
+	day = (day + usual_initial->day) % 30 + 1;
+	year = date / 365 + usual_initial->year;
 	sprintf(datebuf, "%2d %s %d", day, months[month], ABS(year));
-    } else if (strcmp(datestepname, "week") == 0) {
+	break;
+      case ds_week:
 	/* Convert to days, then proceed as for days. */
 	date *= 7;
-	date += 30 * basemonth;
+	date += 30 * usual_initial->month;
 	day = date % 365;
 	month = day / 30;
-	day = (day + baseday) % 30 + 1;
-	year = date / 365 + baseyear;
+	day = (day + usual_initial->day) % 30 + 1;
+	year = date / 365 + usual_initial->year;
 	sprintf(datebuf, "%2d %s %d", day, months[month], ABS(year));
-    } else if (strcmp(datestepname, "month") == 0) {
-	date += basemonth;
+	break;
+      case ds_month:
+	date += usual_initial->month;
     	month = date % 12;
-	year = date / 12 + baseyear;
+	year = date / 12 + usual_initial->year;
 	sprintf(datebuf, "%s %d", months[month], ABS(year));
-    } else if (strcmp(datestepname, "season") == 0) {
-    	season = date % 4;
-	year = date / 4 + baseyear;
+	break;
+      case ds_season:
+     	season = date % 4;
+	year = date / 4 + usual_initial->year;
 	sprintf(datebuf, "%d %d", season, ABS(year));
-    } else if (strcmp(datestepname, "year") == 0) {
-	year = date + baseyear;
+	break;
+      case ds_year:
+	year = date + usual_initial->year;
 	sprintf(datebuf, "%d", ABS(year));
-    } else {
-	sprintf(datebuf, "?what's a %s?", datestepname);
+	break;
+      default:
+	sprintf(datebuf, "?what's a %d?", datesteptype);
+	break;
     }
     if (year < 0) {
 	strcat(datebuf, " BC");
@@ -1786,16 +2136,21 @@ write_side_results(fp, side)
 FILE *fp;
 Side *side;
 {
+    int i;
+
     if (side == NULL) {
 	fprintf(fp, "Results for game as a whole:\n\n");
     } else {
-	fprintf(fp, "Results for %s%s, played by %s:\n\n",
+	fprintf(fp, "Results for %s%s",
 		short_side_title(side),
-		(side->status == 1 ? " (WINNER)" :
-		 (side->status == -1 ? " (LOSER)" :
-		  "")),
+		(side_won(side) ? " (WINNER)" :
+		 (side_lost(side) ? " (LOSER)" :
+		  "")));
+	for (i = 0; i < numscores; ++i) {
+	    fprintf(fp, " %d", side->scores[i]);
+	}
+	fprintf(fp, ", played by %s:\n\n",
 		long_player_title(spbuf, side->player, NULL));
-	/* (should mention numerical scores) */
     }
 }
 
@@ -1821,7 +2176,7 @@ Side *side;
     fprintf(fp, " Loss |");
     fprintf(fp, " Total\n");
     for_all_unit_types(u) {
-	if (1 /* type was at least potentially in game */) {
+	if (u_possible[u]) {
 	    totgain = 0;
 	    fprintf(fp, " %9s ", utype_name_n(u, 9));
 	    for (gainreason = 0; gainreason < num_gain_reasons; ++gainreason) {
@@ -1897,16 +2252,16 @@ Side *side;
 	    "Unit Combat Results (average damage over # attacks against enemy, by type)\n");
     fprintf(fp, " A  D->");
     for_all_unit_types(d) {
-	if (1 /* part of game */) {
+	if (u_possible[d]) {
 	    fprintf(fp, " %4s ", utype_name_n(d, 4));
 	}
     }
     fprintf(fp, "\n");
     for_all_unit_types(a) {
-	if (1 /* part of game */) {
+	if (u_possible[a]) {
 	    fprintf(fp, " %4s ", utype_name_n(a, 4));
 	    for_all_unit_types(d) {
-		if (1 /* part of game */) {
+		if (u_possible[d]) {
 		    atk = atkstats(side, a, d);
 		    if (atk > 0) {
 			fprintf(fp, " %5.2f",
@@ -1918,7 +2273,7 @@ Side *side;
 	    }
 	    fprintf(fp, "\n     ");
 	    for_all_unit_types(d) {
-		if (1 /* part of game */) {
+		if (u_possible[d]) {
 		    atk = atkstats(side, a, d);
 		    if (atk > 0) {
 			fprintf(fp, " %4d ", atk);
