@@ -1,5 +1,5 @@
-/* Unit plan execution for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+/* Unit plan handling for Xconq.
+   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,6 +10,8 @@ any later version.  See the file COPYING.  */
 
 static void wake_at PARAMS ((int x, int y));
 
+#define CLEAR_AGENDA 99
+
 /* (should have a generic struct for all plan type attrs) */
 
 char *plantypenames[] = {
@@ -22,6 +24,128 @@ char *plantypenames[] = {
     NULL
 };
 
+/* Every unit that can act needs a plan object, types that can't act
+   should have it cleared out. */
+
+void
+init_unit_plan(unit)
+Unit *unit;
+{
+    if (u_acp(unit->type) > 0 && unit->cp > 0) {
+	/* Might already have a plan, so don't always realloc. */
+	if (unit->plan == NULL) {
+	    unit->plan = create_plan();
+	}
+	/* Put the plan into a default state, side will work it up later. */
+	unit->plan->type = PLAN_NONE;
+	unit->plan->creation_turn = g_turn();
+	unit->plan->asleep = FALSE;
+	unit->plan->reserve = FALSE;
+	unit->plan->delayed = FALSE;
+	clear_task_agenda(unit->plan);
+	/* Allow AIs to make this unit do things. */
+	unit->plan->aicontrol = TRUE;
+	/* Enable supply alarms by default. */
+	unit->plan->supply_alarm = TRUE;
+    } else {
+	/* Brainless units don't need anything, can free up plan. */
+	if (unit->plan != NULL) {
+	    free_plan(unit->plan);
+	}
+	unit->plan = NULL;
+    }
+}
+
+void
+set_unit_plan_type(side, unit, type)
+Side *side;
+Unit *unit;
+int type;
+{
+    int oldtype;
+
+    if (unit->plan) {
+	oldtype = unit->plan->type;
+	if (type != oldtype) {
+	    unit->plan->type = type;
+	    if (side != NULL)
+	      update_unit_display(side, unit, TRUE);
+	}
+    }
+}
+
+void
+set_unit_asleep(side, unit, flag, recurse)
+Side *side;
+Unit *unit;
+int flag, recurse;
+{
+    int oldflag;
+    Unit *occ;
+
+    if (unit->plan) {
+	oldflag = unit->plan->asleep;
+	if (flag != oldflag) {
+	    unit->plan->asleep = flag;
+	    if (side != NULL)
+	      update_unit_display(side, unit, TRUE);
+	}
+    }
+    if (recurse) {
+    	for_all_occupants(unit, occ) {
+	    set_unit_asleep(side, occ, flag, recurse);
+    	}
+    }
+}
+
+void
+set_unit_reserve(side, unit, flag, recurse)
+Side *side;
+Unit *unit;
+int flag, recurse;
+{
+    int oldflag;
+    Unit *occ;
+
+    if (unit->plan) {
+	oldflag = unit->plan->reserve;
+	if (flag != oldflag) {
+	    unit->plan->reserve = flag;
+	    if (side != NULL)
+	      update_unit_display(side, unit, TRUE);
+	}
+    }
+    if (recurse) {
+    	for_all_occupants(unit, occ) {
+	    set_unit_reserve(side, occ, flag, recurse);
+    	}
+    }
+}
+
+void
+set_unit_ai_control(side, unit, flag, recurse)
+Side *side;
+Unit *unit;
+int flag, recurse;
+{
+    int oldflag;
+    Unit *occ;
+
+    if (unit->plan) {
+	oldflag = unit->plan->aicontrol;
+	if (flag != oldflag) {
+	    unit->plan->aicontrol = flag;
+	    if (side != NULL)
+	      update_unit_display(side, unit, TRUE);
+	}
+    }
+    if (recurse) {
+    	for_all_occupants(unit, occ) {
+	    set_unit_ai_control(side, occ, flag, recurse);
+    	}
+    }
+}
+
 /* Execute the plan. */
 
 void
@@ -48,6 +172,12 @@ int try;
 	plan->reserve = TRUE;
 	return;
     }
+    if (plan->execs_this_turn > 1000) {
+	DMprintf("%s executed plan 1000 times this turn, going into reserve\n",
+		 unit_desig(unit));
+	plan->reserve = TRUE;
+	return;
+    }
     /* Unit actually has a plan, dispatch on its type. */
     switch (plan->type) {
       case PLAN_NONE:
@@ -68,10 +198,10 @@ int try;
 	if (plan->supply_is_low && plan->supply_alarm) {
 	    plan->supply_alarm = FALSE;
 	    if (0 /* auto resupply */) {
-		set_resupply_task(unit);
+		set_resupply_task(unit, NONMTYPE);
 	    } else if (plan->tasks
 	    	       && (plan->tasks->type == TASK_RESUPPLY
-	    	           || (plan->tasks->type == TASK_MOVETO
+	    	           || (plan->tasks->type == TASK_MOVE_TO
 	    	               && plan->tasks->next
 	    	               && plan->tasks->next->type == TASK_RESUPPLY))) {
 		/* do nothing */
@@ -83,6 +213,8 @@ int try;
 	if (plan->tasks) {
 	    /* (should check that doctrine being followed correctly) */
 	    execute_task(unit);
+	} else if (unit->side && unit->side->orders && execute_standing_order(unit, TRUE)) {
+	    execute_task(unit);
 	} else if (plan->formation && move_into_formation(unit)) {
 	    execute_task(unit);
 	} else if (unit->side && side_has_ai(unit->side)) {
@@ -115,6 +247,7 @@ int try;
 	case_panic("plan type", plan->type);
 	break;
     }
+    ++(plan->execs_this_turn);
 }
 
 /* See if we're too far away from an assigned position, set a task
@@ -137,13 +270,105 @@ Unit *unit;
 	dist = goal->args[3];
 	if (distance(unit->x, unit->y, nx, ny) > dist) {
 	    /* (should perhaps insert after current task?) */
-	    set_movenear_task(unit, nx, ny, dist);
+	    set_move_near_task(unit, nx, ny, dist);
 	    return TRUE;
 	}
     }
     return FALSE;
 }
 
+int task_is_in_agenda PARAMS ((Plan *plan, Task *task));
+
+/* See if there are any standing orders that currently apply to the given unit,
+   and schedule a task if so.  Return TRUE if a task was added. */
+
+int
+execute_standing_order(unit, addtask)
+Unit *unit;
+int addtask;
+{
+    Unit *transport;
+    Side *side;
+    StandingOrder *sorder;
+
+    side = unit->side;
+    if (side == NULL)
+      return FALSE;
+    for (sorder = side->orders; sorder != NULL; sorder = sorder->next) {
+	if (sorder->types[unit->type] && unit->plan) {
+	    switch (sorder->etype) {
+	      case 1:
+		if (unit->x == sorder->a1 && unit->y == sorder->a2) {
+		    /* If the task is already in the plan, don't do
+		       anything. */
+		    if (task_is_in_agenda(unit->plan, sorder->task))
+		      return FALSE;
+		    if (addtask)
+		      add_task(unit, 0, clone_task(sorder->task));
+		    return TRUE;
+		}
+		break;
+	      case 2:
+		transport = unit->transport;
+		if (transport != NULL && transport->id == sorder->a1) {
+		    /* If the task is already in the plan, don't do
+		       anything. */
+		    if (task_is_in_agenda(unit->plan, sorder->task))
+		      return FALSE;
+		    if (addtask)
+		      add_task(unit, 0, clone_task(sorder->task));
+		    return TRUE;
+		}
+		break;
+	      case 3:
+		if (distance(unit->x, unit->y, sorder->a1, sorder->a2) <= sorder->a3) {
+		    /* If the task is already in the plan, don't do
+		       anything. */
+		    if (task_is_in_agenda(unit->plan, sorder->task))
+		      return FALSE;
+		    if (addtask)
+		      add_task(unit, 0, clone_task(sorder->task));
+		    return TRUE;
+		}
+		break;
+	      default:
+		break;
+	    }
+	}
+    }
+    return FALSE;
+}
+
+int tasks_match PARAMS ((Task *task1, Task *task2));
+
+int
+task_is_in_agenda(plan, task)
+Plan *plan;
+Task *task;
+{
+    Task *task2;
+
+    for (task2 = plan->tasks; task2 != NULL; task2 = task2->next) {
+	if (tasks_match(task, task2))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+tasks_match(task1, task2)
+Task *task1, *task2;
+{
+    int i;
+
+    if (task1->type != task2->type)
+      return FALSE;
+    for (i = 0; i < MAXTASKARGS; ++i)
+      if (task1->args[i] != task2->args[i])
+	return FALSE;
+    return TRUE;
+}
+
 /* A unit operating offensively advances and attacks when possible. */
 
 void
@@ -154,10 +379,16 @@ Unit *unit;
     int x, y, w, h, range, x1, y1;
     Plan *plan = unit->plan;
 
-    if (resupply_if_low(unit))
-      return;
-    if (rearm_if_low(unit))
-      return;
+    if (resupply_if_low(unit)) {
+	if (plan->tasks)
+	  execute_task(unit);
+	return;
+    }
+    if (rearm_if_low(unit)) {
+	if (plan->tasks)
+	  execute_task(unit);
+	return;
+    }
     if (plan->tasks) {
     	execute_task(unit);
     	return;
@@ -174,11 +405,11 @@ Unit *unit;
 	    	}
 		DMprintf("%s to go on offensive to %d,%d\n",
 			 unit_desig(unit), x, y);
-		push_movenear_task(unit, x, y, max(w, h) / 2);
+		set_move_near_task(unit, x, y, max(w, h) / 2);
 		if (unit->transport
 		    && mobile(unit->transport->type)
 		    && unit->transport->plan) {
-		    push_movenear_task(unit->transport, x, y, max(w, h) / 2);
+		    set_move_near_task(unit->transport, x, y, max(w, h) / 2);
 		}
 	    } else {
 		range = max(w, h);
@@ -187,9 +418,11 @@ Unit *unit;
 		if (probability(50))
 		  range *= 2;
 		if (do_for_occupants(unit)) {
+		    /* Occupants have decided for us, fall through. */
 		} else if (go_after_victim(unit, range)) {
+		    /* Found a victim to go after, fall through. */
 		} else if (probability(20) && self_build_base_for_self(unit)) {
-		} else if (!g_see_all()) {
+		} else if (!all_see_all) {
 		    DMprintf("%s will explore instead\n", unit_desig(unit));
 		    plan_exploration(unit); /* or patrol */
 		    /* Running under exploration rules now. */
@@ -208,7 +441,7 @@ Unit *unit;
 	if (do_for_occupants(unit)) {
 	} else if (go_after_victim(unit, range)) {
 	    /* No special goal, but found something to fight with. */
-	} else if (!g_see_all()) {
+	} else if (!all_see_all) {
 	    DMprintf("%s will explore instead\n", unit_desig(unit));
 	    plan_exploration(unit); /* or patrol */
 	    /* Running under exploration rules now. */
@@ -236,6 +469,9 @@ Unit *unit;
     }
 }
 
+/* Look through list of occupants to see if an occupant needs the transport to do
+   something. */
+
 int
 do_for_occupants(unit)
 Unit *unit;
@@ -252,15 +488,15 @@ Unit *unit;
 		&& goal->type == GOAL_VICINITY_HELD
 		&& distance(goal->args[0], goal->args[1], unit->x, unit->y)
 		> goal->args[2]) {
-		set_movenear_task(unit, goal->args[0], goal->args[1],
+		set_move_near_task(unit, goal->args[0], goal->args[1],
 				  max(goal->args[2] / 2, 1));
 		DMprintf("%s will go where occupant %s wants to go (goal %s)\n",
 			 unit_desig(unit), unit_desig(occ), goal_desig(goal));
 		return TRUE;
 	    }
 	    /* If the unit does not have a goal, see if it has a task. */
-	    for (task = occ->plan->tasks; task != NULL; task = task->next) {
-		if ((task->type == TASK_MOVETO
+	    for_all_tasks(occ->plan, task) {
+		if ((task->type == TASK_MOVE_TO
 		     || task->type == TASK_HIT_UNIT)
 		    && (task->args[0] != unit->x
 			|| task->args[1] != unit->y)
@@ -268,7 +504,7 @@ Unit *unit;
 			) {
 		    /* Note that we assume the transport is mobile, which is OK currently
 		       because of where this routine is called from. */
-		    set_movenear_task(unit, task->args[0], task->args[1], 1);
+		    set_move_near_task(unit, task->args[0], task->args[1], 1);
 		    DMprintf("%s will go where occupant %s wants to go (task %s)\n",
 			     unit_desig(unit), unit_desig(occ), task_desig(task));
 		    return TRUE;
@@ -365,10 +601,16 @@ void
 plan_defense(unit)
 Unit *unit;
 {
-    if (resupply_if_low(unit))
-      return;
-    if (rearm_if_low(unit))
-      return;
+    if (resupply_if_low(unit)) {
+	if (unit->plan->tasks)
+	  execute_task(unit);
+	return;
+    }
+    if (rearm_if_low(unit)) {
+	if (unit->plan->tasks)
+	  execute_task(unit);
+	return;
+    }
     if (unit->plan->tasks) {
     	/* (should analyze and maybe decide to change task) */
     	execute_task(unit);
@@ -402,12 +644,15 @@ Unit *unit;
     Side *us = unit->side;
 
     /* If the world has no secrets, exploration is sort of pointless. */
-    if (g_see_all()) {
+    if (all_see_all) {
     	plan->reserve = TRUE;
     	return;
     }
-    if (resupply_if_low(unit))
-      return;
+    if (resupply_if_low(unit)) {
+	if (plan->tasks)
+	  execute_task(unit);
+	return;
+    }
     if (capture_indep_if_nearby(unit))
       return;
     if (capture_useful_if_nearby(unit))
@@ -430,7 +675,7 @@ Unit *unit;
 	    	    }
 		    DMprintf("%s to explore towards %d,%d\n",
 			     unit_desig(unit), x, y);
-		    push_movenear_task(unit, x, y, max(w, h) / 2);
+		    set_move_near_task(unit, x, y, max(w, h) / 2);
 		} else {
 		    if (explore_reachable_cell(unit, max(w, h) + 2)) {
 		    } else if (us != NULL && side_has_ai(us) && ai_guide_explorer(us, unit)) {
@@ -542,8 +787,7 @@ int x, y;
     Unit *unit2;
     Side *side = tmpunit->side, *oside = NULL;
 
-    if (g_see_all()
-	|| cover(side, x, y) > 0 /* should be "guaranteed to see" */) {
+    if (units_visible(side, x, y)) {
 	if ((unit2 = unit_at(x, y)) != NULL) {
 	    u2 = unit2->type;
 	    oside = unit2->side;
@@ -565,7 +809,14 @@ int x, y;
 	        || uu_capture(tmpunit->type, u2) > 0))
 	    || uu_capture(tmpunit->type, u2) > 0)
 	) {
-	rating = uu_zz_bhw(tmpunit->type, u2);
+	/* If our unit can't capture the prospective victim, but somebody
+	   else is planning to do so, leave it alone and keep looking for
+	   a victim. */
+	if ((uu_acp_to_attack(tmpunit->type, u2) < 1
+	     || capture_chance(tmpunit->type, u2, oside) <= 0)
+	    && ai_planning_to_capture(side, u2, x, y)) {
+	    return FALSE;
+	}
 	if (capture_chance(tmpunit->type, u2, oside) > 0) {
 	    return TRUE;
 	}
@@ -576,6 +827,7 @@ int x, y;
 	    ) {
 	    return FALSE;
 	}
+	rating = uu_zz_bhw(tmpunit->type, u2);
 	/* Further-away units are less interesting than closer ones. */
 	dist = distance(tmpunit->x, tmpunit->y, x, y);
 	if (dist > u_acp(tmpunit->type)) {
@@ -631,7 +883,7 @@ int range;
 	if (unit->transport != NULL
 	    && mobile(unit->transport->type)
 	    && unit->transport->plan) {
-	    set_movenear_task(unit->transport, x, y, 1);
+	    set_move_near_task(unit->transport, x, y, 1);
 	}
     } else if (victimrating > -9999) {
 	DMprintf("one (rated %d) at %d,%d\n", victimrating, victimx, victimy);
@@ -641,7 +893,7 @@ int range;
 	if (unit->transport != NULL
 	    && mobile(unit->transport->type)
 	    && unit->transport->plan) {
-	    set_movenear_task(unit->transport, victimx, victimy, 1);
+	    set_move_near_task(unit->transport, victimx, victimy, 1);
 	}
     } else {
 	DMprintf("nothing\n");
@@ -659,8 +911,7 @@ int x, y;
     Unit *unit2;
     Side *side = tmpunit->side, *oside = NULL;
 
-    if (g_see_all()
-	|| cover(side, x, y) > 0 /* should be "guaranteed to see" */) {
+    if (units_visible(side, x, y)) {
 	if ((unit2 = unit_at(x, y)) != NULL) {
 	    u2 = unit2->type;
 	    oside = unit2->side;
@@ -681,11 +932,13 @@ int x, y;
 	&& could_hit(tmpunit->type, u2)
 	&& uu_damage(tmpunit->type, u2) > 0
 	/* and have correct ammo */
+	&& !ai_planning_to_capture(side, u2, x, y)
 	) {
-        rating = uu_hit(tmpunit->type, u2);
+        rating = uu_zz_bfw(tmpunit->type, u2);
 	/* Further-away units are less interesting than closer ones. */
 	dist = distance(tmpunit->x, tmpunit->y, x, y);
-	/* (effect should be quality of fire) */
+	if (dist > 0)
+	  rating /= dist;
 	if (rating > targetrating || (rating == targetrating && flip_coin())) {
 	    targetx = x;  targety = y;
 	    targetrating = rating;
@@ -727,65 +980,88 @@ int
 resupply_if_low(unit)
 Unit *unit;
 {
-    int u = unit->type, m, lowm = NONMTYPE;
+    int u = unit->type, lowm;
     Task *curtask = unit->plan->tasks;
 
     if (!mobile(u))
       return FALSE;
-    for_all_material_types(m) {
-	if ((um_base_consumption(u, m) > 0 || um_consumption_per_move(u, m) > 0)
-	    && 2 * unit->supply[m] < um_storage_x(u, m)) {
-	    lowm = m;
-	    break;
-	}
-    }
+    lowm = low_on_supplies_one(unit);
     if (lowm != NONMTYPE) {
+	/* See if we're already moving to a supply source. */
 	if (curtask != NULL
-	    && curtask->type == TASK_MOVETO
+	    && curtask->type == TASK_MOVE_TO /* or other types? */
 	    && supplies_here(unit, curtask->args[0], curtask->args[1], lowm))
 	  /* Let the movement task execute. */
 	  return FALSE;
 	/* Otherwise set up a task. */
 	DMprintf("%s low on %s, looking for source\n",
-		 unit_desig(unit), m_type_name(m));
-	set_resupply_task(unit);
+		 unit_desig(unit), m_type_name(lowm));
+	set_resupply_task(unit, lowm);
 	return (execute_task(unit) != TASK_FAILED);
     }
     return FALSE;
 }
 
+/* Return a type of essential material that the unit is running out of. */
+
+int
+low_on_supplies_one(unit)
+Unit *unit;
+{
+    int u = unit->type, m;
+
+    for_all_material_types(m) {
+	if ((um_base_consumption(u, m) > 0 || um_consumption_per_move(u, m) > 0)
+	    && 2 * unit->supply[m] <= um_storage_x(u, m)) {
+	    return m;
+	}
+    }
+    return NONMTYPE;
+}
+
 int
 rearm_if_low(unit)
 Unit *unit;
 {
-    int u = unit->type, m, lowm = NONMTYPE;
+    int u = unit->type, lowm;
     Task *curtask = unit->plan->tasks;
 
     if (!mobile(u))
       return FALSE;
-    for_all_material_types(m) {
-	if (um_consumption_per_attack(u, m) > 0
-	    && unit->supply[m] == 0
-	    && um_storage_x(u, m) > 0) {
-	    lowm = m;
-	    break;
-	}
-    }
+    lowm = low_on_ammo_one(unit);
     if (lowm != NONMTYPE) {
 	if (curtask != NULL
-	    && curtask->type == TASK_MOVETO
+	    && curtask->type == TASK_MOVE_TO /* or other types? */
 	    && supplies_here(unit, curtask->args[0], curtask->args[1], lowm))
 	  /* Let the movement task execute. */
 	  return FALSE;
 	/* Otherwise set up a task. */
 	DMprintf("%s low on %s, looking for source\n",
-		 unit_desig(unit), m_type_name(m));
-	set_resupply_task(unit);
+		 unit_desig(unit), m_type_name(lowm));
+	set_resupply_task(unit, lowm);
 	return (execute_task(unit) != TASK_FAILED);
     }
     return FALSE;
 }
 
+/* Return a type of material that we want to use in combat. */
+
+int
+low_on_ammo_one(unit)
+Unit *unit;
+{
+    int u = unit->type, m;
+
+    for_all_material_types(m) {
+	if (um_consumption_per_attack(u, m) > 0
+	    && unit->supply[m] == 0
+	    && um_storage_x(u, m) > 0) {
+	    return m;
+	}
+    }
+    return NONMTYPE;
+}
+
 int
 supplies_here(unit, x, y, m)
 Unit *unit;
@@ -794,8 +1070,10 @@ int x, y, m;
     Unit *unit2;
 
     for_all_stack(x, y, unit2) {
-	if (unit_trusts_unit(unit2, unit)
-	    && unit2->supply[m] > 0) {
+	if (unit2 != unit
+	    && unit_trusts_unit(unit2, unit)
+	    && unit2->supply[m] > 0
+	    && um_outlength(unit2->type, m) >= 0) {
 	    return TRUE;
 	}
     }
@@ -810,8 +1088,7 @@ int x, y;
     Unit *unit2;
     Side *side = tmpunit->side, *side2 = NULL;
 
-    if (g_see_all()
-	|| cover(side, x, y) > 0 /* should be "guaranteed to see" */) {
+    if (units_visible(side, x, y)) {
 	if ((unit2 = unit_at(x, y)) != NULL) {
 	    u2 = unit2->type;
 	    side2 = unit2->side;
@@ -827,7 +1104,7 @@ int x, y;
     }
     return (is_unit_type(u2)
     	    && side2 == NULL
-	    && capture_chance(tmpunit->type, u2, side2) > 0);
+	    && capture_chance(tmpunit->type, u2, side2) > 10);
 }
 
 /*  */
@@ -855,7 +1132,7 @@ Unit *unit;
 	if (unit->transport
 	    && mobile(unit->transport->type)
 	    && unit->transport->plan) {
-	    push_movenear_task(unit->transport, x, y, 1);
+	    set_move_near_task(unit->transport, x, y, 1);
 	}
 	return (execute_task(unit) != TASK_FAILED);
     } else {
@@ -872,28 +1149,31 @@ int x, y;
     Unit *unit2;
     Side *side = tmpunit->side, *oside = NULL;
 
-    if (g_see_all()
-	|| cover(side, x, y) > 0 /* should be "guaranteed to see" */) {
+    if (units_visible(side, x, y)) {
 	for_all_stack(x, y, unit2) {
 	    u2 = unit2->type;
 	    oside = unit2->side;
             if (is_unit_type(u2)
-    	    && !trusted_side(side, oside)
-	    && capture_chance(tmpunit->type, u2, oside) > 0
-	    && useful_type(side, u2)
-	    )
-	  return TRUE;
+		&& !trusted_side(side, oside)
+		&& capture_chance(tmpunit->type, u2, oside) > 0
+		&& useful_type(side, u2)
+		) {
+		tmputype = u2;
+		return TRUE;
+	    }
 	}
     } else if (terrain_view(side, x, y) != UNSEEN
 	       && (uview = unit_view(side, x, y)) != EMPTY) {
 	u2 = vtype(uview);
 	oside = side_n(vside(uview));
-        return (is_unit_type(u2)
+        if (is_unit_type(u2)
     	    && !trusted_side(side, oside)
 	    && capture_chance(tmpunit->type, u2, oside) > 0
 	    && useful_type(side, u2)
-	    );
-    } else {
+	    ) {
+	    tmputype = u2;
+	    return TRUE;
+	}
     }
     return FALSE;
 }
@@ -936,7 +1216,7 @@ Unit *unit;
 	if (unit->transport
 	    && mobile(unit->transport->type)
 	    && unit->transport->plan) {
-	    push_movenear_task(unit->transport, x, y, 1);
+	    set_move_near_task(unit->transport, x, y, 1);
 	}
 	return (execute_task(unit) != TASK_FAILED);
     } else {
@@ -964,12 +1244,17 @@ int u;
    system should function correctly no matter what this thing comes up with.
    Piles of warnings are likely, but that's OK. */
 
+/* (should actually have two modes for this - testing, which allows bad data,
+   and normal usage with no garbage generated, which is used with unintelligent
+   AIs) */
+
 void
 plan_random(unit)
 Unit *unit;
 {
-    int dir, x1, y1;
+    int dir, x1, y1, n;
     TaskType tasktype;
+    Task *task;
     Action action;
     char *argtypestr;
 
@@ -982,50 +1267,67 @@ Unit *unit;
 	tasktype = xrandom((int) NUMTASKTYPES);
 	switch (tasktype) {
 	  case TASK_NONE:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_BUILD:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_build_task(xrandom(numutypes), xrandom(99));
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_RESEARCH:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
-	  case TASK_CAPTURE_UNIT:
+	  case TASK_CAPTURE:
 	    dir = random_dir();
 	    point_in_dir(unit->x, unit->y, dir, &x1, &y1);
 	    set_capture_task(unit, x1, y1);
 	    break;
 	  case TASK_DO_ACTION:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_HIT_POSITION:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_HIT_UNIT:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
-	  case TASK_MOVEDIR:
-	    unit->plan->tasks = create_task (tasktype);
+	  case TASK_MOVE_DIR:
+	    dir = random_dir();
+	    n = xrandom(10);
+	    set_move_dir_task(unit, dir, n);
 	    break;
-	  case TASK_MOVETO:
+	  case TASK_MOVE_TO:
 	    dir = random_dir();
 	    point_in_dir(unit->x, unit->y, dir, &x1, &y1);
-	    set_moveto_task(unit, x1, y1);
+	    set_move_to_task(unit, x1, y1);
 	    break;
 	  case TASK_OCCUPY:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_PICKUP:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
+	    break;
+	  case TASK_PRODUCE:
+	    if (nummtypes > 0) {
+		task = create_produce_task(xrandom(nummtypes), xrandom(100));
+		add_task(unit, CLEAR_AGENDA, task);
+	    }
 	    break;
 	  case TASK_REPAIR:
-	    unit->plan->tasks = create_task (tasktype);
+	    task = create_task(tasktype);
+	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_DISBAND:
 	    set_disband_task(unit);
 	    break;
 	  case TASK_RESUPPLY:
-	    set_resupply_task(unit);
+	    set_resupply_task(unit, (flip_coin() ? xrandom(nummtypes) : NONMTYPE));
 	    break;
 	  case TASK_SENTRY:
 	    set_sentry_task(unit, xrandom(5));
@@ -1065,8 +1367,7 @@ short *args;
 Unit *unit;
 {
     char argch;
-    int	slen;
-    long arg;
+    int	slen, arg;
 
     slen = strlen(argtypestr);
     while (i < slen && i < 10) {
@@ -1196,7 +1497,7 @@ Unit *unit;
 	}
 	dir = random_dir();
     }
-    set_moveto_task(unit, x1, y1);
+    set_move_to_task(unit, x1, y1);
 }
 
 /* Put a unit in reserve. */
@@ -1304,10 +1605,10 @@ int flag;
 }
 
 
-/* Return the distance that we can go by shortest path before running out */
-/* of important supplies.  Will return at least 1, since we can *always* */
-/* move one cell to safety.  This is a worst-case routine, too complicated */
-/* to worry about units getting refreshed by terrain or whatever. */
+/* Return the distance that we can go by shortest path before running out
+   of important supplies.  Will return at least 1, since we can *always*
+   move one cell to safety.  This is a worst-case routine, too complicated
+   to worry about units getting refreshed by terrain or whatever. */
 
 int
 range_left(unit)
@@ -1320,6 +1621,8 @@ Unit *unit;
 	    least = min(least, unit->supply[m] / um_consumption_per_move(u, m));
 	}
 #if 0
+	/* This code is too pessimistic if no account taken of supply line or
+	   production, so leave out for now. */
 	if (um_base_consumption(u, m) > 0) {
 	    tmp = (u_speed(u) * unit->supply[m]) / um_base_consumption(u, m);
 	    least = min(least, tmp);
@@ -1384,42 +1687,6 @@ int u, x0, y0;
     return thr;
 }
 
-#if 0
-/* Heuristics for units to build "bases". */
-
-int
-should_build_base(unit)
-Unit *unit;
-{
-    DMprintf(
-        "%s should build base survive %d base near %d wakeup %d - ",
-		       unit_desig(unit),
-		       survive_to_build_base(unit), base_nearby(unit,1),
-		       unit->plan->wakeup_reason);
-    if ((exact_survive_to_build_base(unit)) && 0 &&
-	unit->transport == NULL &&
-	!any_base_nearby(unit,1) &&
-	probability(90) &&
-	unit->plan->wakeup_reason != WAKEENEMY) {
-	push_build_task(unit, machine_product(unit));
-	DMprintf("going to build a %d\n", unit->product);
-	return TRUE;
-    }
-    if ((survive_to_build_base(unit) && 0 &&  probability(20)) &&
-	unit->transport == NULL &&
-	!any_base_nearby(unit,1) &&
-	probability(90) &&
-	!neutral_base_nearby(unit, 5) &&
-	unit->plan->wakeup_reason != WAKEENEMY) {
-	push_build_task(unit, machine_product(unit));
-	DMprintf("going to build a %d\n", unit->product);
-	return TRUE;
-    }
-    DMprintf("decided not to.\n");
-    return FALSE;
-}
-#endif
-
 void
 pop_task(plan)
 Plan *plan;
@@ -1475,7 +1742,7 @@ Unit *unit;
 int prod;
 {
     int schedule = 1 /* uu_make(unit->type, prod) */;
-    int u, research_delay;
+    int u, research_delay = 0;
 
     /* Add penalty (or unpenalty!) for first unit of a type. */
     /* is "counts" a reliable way to test? */
@@ -1495,23 +1762,23 @@ int prod;
     return schedule;
 }
 
-void
+int
 clear_task_agenda(plan)
 Plan *plan;
 {
-    int numcleared = 0;
+    int numcleared;
     Task *oldtask;
 
     if (plan == NULL || plan->tasks == NULL)
-      return;
+      return 0;
+    numcleared = 0;
     while (plan->tasks != NULL) {
     	oldtask = plan->tasks;
     	plan->tasks = plan->tasks->next;
     	free_task(oldtask);
     	++numcleared;
     }
-    if (numcleared > 0)
-      Dprintf("Cleared %d tasks from %s\n", numcleared, plan_desig(plan));
+    return numcleared;
 }
 
 Plan *
@@ -1551,7 +1818,7 @@ Plan *plan;
     } else {
 	if (plan->tasks) {
 	    tmpbuf[0] = '\0';
-	    for (task = plan->tasks; task != NULL; task = task->next) {
+	    for_all_tasks(plan, task) {
 		if (strlen(tmpbuf) < 100) {
 		    strcat(tmpbuf, " ");
 		    strcat(tmpbuf, task_desig(task));
@@ -1774,15 +2041,44 @@ int
 can_build(unit)
 Unit *unit;
 {
-    int p;
+    int u2;
 
-    for_all_unit_types(p) {
-	if (could_create(unit->type, p))
+    for_all_unit_types(u2) {
+	if (could_create(unit->type, u2))
 	  return TRUE;
     }
     return FALSE;
 }
 
+int
+can_build_or_help(unit)
+Unit *unit;
+{
+    int u2;
+
+    for_all_unit_types(u2) {
+	if (could_create(unit->type, u2)
+	    || uu_acp_to_build(unit->type, u2) > 0
+	    || uu_acp_to_research(unit->type, u2) > 0)
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+int
+can_produce(unit)
+Unit *unit;
+{
+    int m;
+
+    for_all_material_types(m) {
+	if (um_acp_to_produce(unit->type, m))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
+
 /* Test if unit can move out into adjacent cells. */
 
 int
@@ -1792,11 +2088,11 @@ Unit *unit;
     int d, x, y;
 
     for_all_directions(d) {
-      if (interior_point_in_dir(unit->x, unit->y, d, &x, &y)) {
-        /* (should account for world-leaving options?) */
-	if (could_move(unit->type, terrain_at(x, y)))
-	  return TRUE;
-      }
+	if (interior_point_in_dir(unit->x, unit->y, d, &x, &y)) {
+	    /* (should account for world-leaving options?) */
+	    if (could_live_on(unit->type, terrain_at(x, y)))
+	      return TRUE;
+	}
     }
     return FALSE;
 }
@@ -1851,7 +2147,7 @@ int x, y;
     return (((view == EMPTY) || view == UNSEEN ||
 	    (allied_side(side_n(vside(view)),unit->side) &&
 	     could_carry(vtype(view), u))) &&
-	       could_move(u, terrain_at(x, y)));
+	       could_live_on(u, terrain_at(x, y)));
 }
 
 Task *explorechain;
@@ -1913,13 +2209,13 @@ int range;
 {
     int x, y;
 
-    if (g_see_all() || g_terrain_seen())
+    if (all_see_all || g_terrain_seen())
       return FALSE;
     tmpunit = unit;
     tmpside = unit->side;
     DMprintf("%s searching within %d for cell to explore -", unit_desig(unit), range);
     if (search_around(unit->x, unit->y, range, reachable_unknown, &x, &y, 1)) {
-	push_movenear_task(unit, x, y, 1);
+	set_move_near_task(unit, x, y, 1);
 	DMprintf("found one at %d,%d\n", x, y);
 	return TRUE;
     }
@@ -2049,18 +2345,30 @@ operating_range_best(u)
 int u;
 {
     int m, t, prod, range, tbestrange, tbest = 0, bestrange = 0;
+    int moves, consump;
 
     for_all_terrain_types(t) {
 	if (!terrain_always_impassable(u, t)) {
 	    tbestrange = area.maxdim;
 	    for_all_material_types(m) {
+		consump = 0;
+		moves = (u_acp(u) * u_speed(u)) / 100;
+		if (um_consumption_per_move(u, m) > 0) {
+		    consump = moves * um_consumption_per_move(u, m);
+		}
+		if (moves <= 0)
+		  moves = 1;
 		if (um_base_consumption(u, m) > 0) {
-	    	    prod = (um_base_production(u, m) * ut_productivity(u, t)) / 100;
-	    	    if (prod < um_base_consumption(u, m)) {
-			range = um_storage_x(u, m) / (um_base_consumption(u, m) - prod);
-			if (range < tbestrange)
-			  tbestrange = range;
-		    }
+		    consump = max(consump, um_base_consumption(u, m));
+		}
+	    	prod = (um_base_production(u, m) * ut_productivity(u, t)) / 100;
+	    	if (prod > 0) {
+		    consump = max(0, consump - prod);
+		}
+		if (consump > 0) {
+		    range = (um_storage_x(u, m) * moves) / consump;
+		    if (range < tbestrange)
+		      tbestrange = range;
 		}
 	    }
 	    if (tbestrange > bestrange) {
@@ -2084,3 +2392,14 @@ int u, t;
       return TRUE;
     return FALSE;
 }
+
+void
+dispose_of_plan(unit)
+Unit *unit;
+{
+    /* Might not be anything to dispose of. */
+    if (unit->plan == NULL)
+      return;
+    free_plan(unit->plan);
+    unit->plan = NULL;
+}
