@@ -1,5 +1,5 @@
 /* The main simulation-running code in Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -10,12 +10,15 @@ any later version.  See the file COPYING.  */
 /* This is the main simulation-running code. */
 
 #include "conq.h"
-extern void notify_all_of_resignation PARAMS ((Side *side, Side *side2));
-extern void change_terrain_type PARAMS ((int x, int y, int t2));
+extern void all_see_cell_weather PARAMS ((int x, int y));
 extern int stop_apply;
+extern int execute_standing_order PARAMS ((Unit *unit, int addtask));
+extern void add_to_unit_hp PARAMS ((Unit *unit, int hp));
+extern void try_sharing PARAMS ((Unit *from, Unit *to, int m));
 
 static void run_turn_start PARAMS ((void));
 static void run_restored_turn_start PARAMS ((void));
+static void add_new_sides_to_game PARAMS ((void));
 static void init_movement PARAMS ((void));
 static void init_actionvectors PARAMS ((void));
 static void run_tech_leakage PARAMS ((void));
@@ -34,9 +37,6 @@ static void compute_sun PARAMS ((void));
 static void run_sun PARAMS ((void));
 static void compute_season PARAMS ((void));
 static void run_environment PARAMS ((void));
-#if 0
-static void damage_unit_with_temperature PARAMS ((Unit *unit, int n));
-#endif
 static void mix_winds PARAMS ((void));
 static void run_spies PARAMS ((void));
 static void run_random_events PARAMS ((void));
@@ -58,7 +58,6 @@ static void try_sharing_with_cell PARAMS ((int x, int y));
 static void run_economy PARAMS ((void));
 static int base_production PARAMS ((Unit *unit, int m));
 static void try_transfer PARAMS ((Unit *from, Unit *to, int r));
-static void try_sharing PARAMS ((Unit *from, Unit *to, int m));
 static void try_transfer_aux PARAMS ((Unit *from, Unit *to, int r));
 static int can_satisfy_need PARAMS ((Unit *unit, int r, int need));
 static void run_turn_end PARAMS ((void));
@@ -68,6 +67,8 @@ static void run_unit_base_consumption PARAMS ((void));
 static void unit_consumes PARAMS ((Unit *unit));
 static int in_supply PARAMS ((Unit *unit, int m));
 static void run_self_builds PARAMS ((void));
+static void run_environment_effects PARAMS ((void));
+static void damage_unit_with_temperature PARAMS ((Unit *unit, int n));
 static void run_people_side_changes PARAMS ((void));
 static void update_cell_display_all_sides PARAMS ((int x, int y, int rightnow));
 static void run_appearances PARAMS ((void));
@@ -153,6 +154,12 @@ int gamestatesafe = TRUE;
 
 int compromised = FALSE;
 
+/* The number of new sides that have been requested to be added. */
+
+int new_sides_requested;
+
+char **players_requested;
+
 /* True whenever the game has both day and night. */
 
 int daynight = FALSE;
@@ -435,6 +442,8 @@ all_sides_finished()
     return TRUE;
 }
 
+/* This is true when AIs should move more slowly. */
+
 int
 all_human_only_sides_finished()
 {
@@ -535,6 +544,7 @@ run_turn_start()
 	/* The game is over, don't bother with the other calcs. */
 	return;
     }
+    add_new_sides_to_game();
     time(&turncalcstart);
     update_all_progress_displays("turn start calcs", -1);
     compute_season();
@@ -607,6 +617,161 @@ init_actionvectors()
     }
 }
 
+/* Interfaces should call this to have another side added to the current
+   game.  The actual addition happens during turn setup, so as not to
+   risk confusing list traversals or AI calculations. */
+
+/* (should take a player spec as arg?) */
+
+int
+request_additional_side(playerspec)
+char *playerspec;
+{
+    if (numsides + new_sides_requested + 1 <= g_sides_max()) {
+	if (players_requested == NULL)
+	  players_requested = (char **) xmalloc(MAXSIDES * sizeof(char *));
+	if (empty_string(playerspec))
+	  playerspec = ",ai";
+	players_requested[new_sides_requested++] = copy_string(playerspec);
+	notify_all("Will add a new side (player \"%s\") at the start of the next turn",
+		   playerspec);
+	return TRUE;
+    } else {
+	notify_all("Additional side requested, but not possible to add");
+	return FALSE;
+    }
+}
+
+/* Add the requested number of new sides into an ongoing game. */
+
+static void
+add_new_sides_to_game()
+{
+    int i;
+    Side *side, *side2;
+    Player *player;
+    Unit *unit;
+
+    if (new_sides_requested > 0) {
+	for (i = 0; i < new_sides_requested; ++i) {
+	    if (numsides >= g_sides_max())
+	      break;
+	    /* Grow side-referencing objects. */
+	    for_all_units(unit) {
+		if (unit->opinions != NULL) {
+		    init_unit_opinions(unit, numsides + 1);
+		}
+	    }
+	    side = make_up_a_side();
+	    player = add_player();
+	    parse_player_spec(player, players_requested[i]);
+	    player->side = side;
+	    side->player = player;
+	    /* Set the player's advantage to be the side's advantage, if not
+	       already set. */
+	    if (player->advantage == 0) {
+		player->advantage = side->advantage;
+	    }
+	    run_synth_methods();
+	    init_doctrine(side);
+	    init_self_unit(side);
+	    if (g_use_side_priority()) {
+		int maxpri = 0;
+
+		for_all_sides(side2) {
+		    if (side2->priority > maxpri)
+		      maxpri = side2->priority;
+		}
+		side->priority = maxpri + 1;
+		/* If the indepside's priority was set automatically to be
+		   one up from regular sides', bump it up to be past the new
+		   side's priority also. */
+		if (indepside->priority == side->priority)
+		  ++(indepside->priority);
+	    }
+	    /* Count all of the new side's units as initial gain. */
+	    for_all_side_units(side, unit) {
+		count_gain(side, unit->type, initial_gain);
+	    }
+	    if (side_wants_ai(side)) {
+		init_ai(side);
+	    }
+	    for_all_sides(side2) {
+		/* Give all other AIs a chance to re-evaluate the situation. */
+		if (side_has_ai(side2) && side2 != side) {
+		    ai_react_to_new_side(side2, side);
+		}
+		/* Add the new side to displays. */
+		update_side_display(side2, side, TRUE);
+		notify(side2, "A new side %s (played by %s) is in the game.",
+		       side_desig(side), player_desig(player));
+	    }
+	}
+	if (i > 0) {
+	    /* Recalculate all view info for all sides; simpler than
+	       trying to figure out what's really changed. */
+	    reset_coverage();
+	    reset_all_views();
+	    compute_all_feature_centroids();
+	    /* Redraw everything that's displayed. */
+	    update_everything();
+	}
+	if (i < new_sides_requested) {
+	    notify_all("Cannot create %d of the requested new sides",
+		       new_sides_requested - i);
+	}
+    }
+    /* We've handled everything we're going to, reset the counter. */
+    new_sides_requested = 0;
+}
+
+/* Parse the syntax "[username][,ai][/config][@display][+advantage]". */
+
+void
+parse_player_spec(player, spec)
+Player *player;
+char *spec;
+{
+    int commapos, slashpos, atpos, pluspos;
+
+    if (spec != NULL && strcmp(spec, "*") != 0) {
+	/* Extract (destructively) a trailing advantage specification. */
+	pluspos = iindex('+', spec);
+	if (pluspos >= 0) {
+	    player->advantage = max(1, atoi(&(spec[pluspos + 1])));
+	    spec[pluspos] = '\0';
+	}
+	/* Extract a displayname if given. */
+	atpos = iindex('@', spec);
+	if (atpos >= 0) {
+	    player->displayname = copy_string(spec + atpos + 1);
+	    spec[atpos] = '\0';
+	}
+	/* Extract a configuration name if given. */
+	slashpos = iindex('/', spec);
+	if (slashpos >= 0) {
+	    player->configname = copy_string(spec + slashpos + 1);
+	    spec[slashpos] = '\0';
+	}
+	/* Extract an AI type if given. */
+	commapos = iindex(',', spec);
+	if (commapos >= 0) {
+	    player->aitypename = copy_string(spec + commapos + 1);
+	    spec[commapos] = '\0';
+	}
+	/* Just a plain old string left. */
+	if (strlen(spec) > 0) {
+	    if (atpos >= 0) {
+		/* Display given separately, so this is a name. */
+		player->name = copy_string(spec);
+	    } else {
+		player->displayname = copy_string(spec);
+	    }
+	}
+    }
+    canonicalize_player(player);
+}
+
 static void
 init_movement()
 {
@@ -621,6 +786,8 @@ init_movement()
     indepside->finishedturn = TRUE;
     for_all_sides(side) {
 	if (side->ingame) {
+	    /* Record that this side was active during at least one turn. */
+	    side->everingame = TRUE;
 	    /* No units are waiting for orders initially. */
 	    side->numwaiting = 0;
 	}
@@ -809,6 +976,9 @@ compose_actionvectors()
 		      unit->plan->delayed = FALSE;
 		}
 	    }
+	    if (unit->plan) {
+		unit->plan->execs_this_turn = 0;
+	    }
 	}
 	/* (should sort vector by unit priority) */
 	Dprintf("Action vector for %s has %d units, at priority %d\n",
@@ -830,7 +1000,7 @@ void
 compute_acp(unit)
 Unit *unit;
 {
-    int u = unit->type, t, acp, maxacp, minacp;
+    int u = unit->type, t, acp, tempeff, maxacp, minacp, err;
     Unit *occ;
 
     /* Units still under construction or off-area can't do anything. */
@@ -858,6 +1028,15 @@ Unit *unit;
 	t = terrain_at(unit->x, unit->y);
     	acp = (acp * ut_acp_night_effect(u, t)) / 100;
     }
+    /* Adjust for temperature. */
+    if (temperatures_defined() && u_acp_temp_effect(u) != lispnil) {
+	err = interpolate_in_list(temperature_at(unit->x, unit->y), u_acp_temp_effect(u), &tempeff);
+	if (err == 0) {
+	    acp = (acp * tempeff) / 100;
+	} else {
+	    /* (should complain?) */
+	}
+    }
     /* Adjust for season. */
     if (u_acp_season_effect(u) != lispnil) {
     	acp = (acp * season_effect(u)) / 100;
@@ -887,7 +1066,16 @@ damaged_acp(unit, effect)
 Unit *unit;
 Obj *effect;
 {
-    return interpolate_in_list(unit->hp, effect, FALSE, u_acp(unit->type));
+    int u, err, rslt;
+
+    u = unit->type;
+    err = interpolate_in_list_ext(unit->hp, effect, 0, 0, 0, 0, u_hp(u), u_acp(u), &rslt);
+    if (err != 0) {
+	run_warning("cannot get damaged acp for %s at hp %d, using %d",
+		    u_type_name(u), u_acp(u));
+	rslt = u_acp(u);
+    }
+    return rslt;
 }
 
 static void
@@ -920,9 +1108,15 @@ static int
 season_effect(u)
 int u;
 {
+    int err, rslt;
+
     if (curyearpart < 0)
       compute_season();
-    return interpolate_in_list(curyearpart, u_acp_season_effect(u), FALSE, 100);
+    err = interpolate_in_list(curyearpart, u_acp_season_effect(u), &rslt);
+    if (err != 0) {
+	rslt = 100;
+    }
+    return rslt;
 }
 
 /* Do some number of actions. */
@@ -966,7 +1160,9 @@ int lim;
 	    continue;
 	}
 	current_play_rate = slow_play_rate;
-	if (all_human_only_sides_finished()) {
+	/* AIs should play as fast as possible if turns are sequential or if the
+	   human players are all done. */
+	if (g_use_side_priority() || all_human_only_sides_finished()) {
 	    current_play_rate = fast_play_rate;
 	}
 	/* If the unit is keeping formation, then give it a chance to
@@ -979,6 +1175,17 @@ int lim;
 	    num += move_one_unit_multiple(unit, lim - num);
 	    foundanytomove = TRUE;
 	}
+	if (unit->side
+	    && unit->side->orders
+	    && unit->plan
+	    && unit->plan->tasks == NULL
+	    && execute_standing_order(unit, FALSE)) {
+	    /* We're not waiting because standing order execution will
+	       shortly be allowed to fill in a task for real. */
+	    unit->plan->waitingfortasks = FALSE;
+	    num += move_one_unit_multiple(unit, lim - num);
+	    foundanytomove = TRUE;
+	}
 	if (unit_still_acting(unit, TRUE)) {
 	    num += move_one_unit_multiple(unit, lim - num);
 	    foundanytomove = TRUE;
@@ -1058,6 +1265,14 @@ int lim;
 	    gamestatesafe = FALSE;
 	    ++num;
 	} else if (unit->plan) {
+	    if (unit->side
+		&& unit->side->orders
+		&& unit->plan->tasks == NULL
+		&& execute_standing_order(unit, TRUE)) {
+		execute_plan(unit, 1);
+		gamestatesafe = FALSE;
+		++buzz;
+	    }
 	    if (unit->plan->formation && move_into_formation(unit)) {
 		execute_plan(unit, 1);
 		gamestatesafe = FALSE;
@@ -1165,8 +1380,6 @@ finish_movement()
     Side *side, *side2;
 
     for_all_sides (side) {
-	/* Remove dead units that have been dead more than a turn. (?) */
-	flush_side_dead(side);
 	if (Debug) {
 	    lostacp = 0;
 	    for_all_side_units(side, unit) {
@@ -1209,14 +1422,14 @@ test_agreements()
 static void
 compute_sun()
 {
-    int curtime;
+    int curtime, highest;
 
     switch (world.daylength) {
       case 0:
 	/* Sun is at a fixed position. */
 	daynight = TRUE;
 	/* (should be possible to set explicitly somehow? implicitly from lat/long?) */
-	sunx = area.width / 2;  suny = area.height / 2;
+	sunx = area.width / 2;  suny = area.halfheight;
 	break;
       case 1:
 	/* No sun effects at all, every place uniformly lit. */
@@ -1229,10 +1442,16 @@ compute_sun()
 	   it at a regular pace. */
 	if (world.circumference >= area.width) {
 	    lastsunx = sunx;  lastsuny = suny;
-	    curtime = (g_turn() + g_initial_day_part()) % world.daylength;
-	    sunx = (curtime * world.circumference) / world.daylength + area.width / 2;
-	    /* (should adjust suny for nonzero latitudes and axial tilt) */
-	    suny = area.height / 2;
+	    curtime = (g_turn() * 100 + g_initial_day_part()) % (world.daylength * 100);
+	    sunx = (curtime * world.circumference) / (world.daylength * 100) + area.width / 2;
+	    /* Adjust for the world's axialtilt.  The approximation is basically a triangle
+	       wave instead of a more correct sinusoid. */
+	    highest = ((world.circumference / 4) * world.axialtilt) / 90;
+	    if (curtime < ((world.daylength * 100) / 2))
+	      suny = area.halfheight - highest + (curtime * 2 * highest) / ((world.daylength * 100) / 2);
+	    else
+	      suny = area.halfheight + highest - ((curtime - ((world.daylength * 100) / 2)) * 2 * highest) / ((world.daylength * 100) / 2);
+	    suny -= area.latitude;
 	}
 	break;
     }
@@ -1284,7 +1503,7 @@ void
 calc_key_point_temps(yearpart)
 int yearpart;
 {
-    int i;
+    int i, err, rslt;
     Obj *lis, *item, *loc;
 
     num_key_points = length(g_temp_year());
@@ -1300,7 +1519,12 @@ int yearpart;
 	    key_points[i].x = c_number(car(loc));
 	    key_points[i].y = c_number(car(cdr(loc)));
 	}
-	key_points[i].temp = interpolate_in_list(yearpart, cdr(item), TRUE, 0);
+	err = interpolate_in_list(yearpart, cdr(item), &rslt);
+	if (err != 0) {
+	    /* (should complain) */
+	    rslt = 0;
+	}
+	key_points[i].temp = rslt;
 	++i;
     }
 }
@@ -1345,21 +1569,19 @@ int x, y;
     }
 }
 
-/* Seasonal change moves the average temperature up and down gradually,
-   modifying it according to the terrain underneath. */
+/* Compute environment changes. */
 
 static void
 run_environment()
 {
-    int yrlen = world.yearlength, x, y, dir, t, celltemp, u, temp;
-    Unit *unit;
-    Side *side;
+    int yrlen = world.yearlength, x, y, dir, t, celltemp;
+    int anychanges = FALSE;
 
     if (mintemp == maxtemp && !any_wind_variation_in_layer)
       return;
-    if (!temperatures_defined()) {
+    if (mintemp != maxtemp && !temperatures_defined()) {
         allocate_area_temperatures();
-        allocate_area_scratch(2);
+        allocate_area_scratch(3);
     }
     if (any_annual_temp_change < 0) {
 	any_annual_temp_change = FALSE;
@@ -1370,97 +1592,85 @@ run_environment()
     if (any_annual_temp_change) {
 	calc_key_point_temps(curyearpart);
     }
-    /* Compute the average temperature at each point in the world. */
-    for (y = area.height - 1; y >= 0; --y) {
-	for (x = 0; x < area.width; ++x) {
-	    if (!in_area(x, y))
-	      continue;
-	    /* Save the prev temp. */
-	    set_tmp1_at(x, y, temperature_at(x, y));
-	    t = terrain_at(x, y);
-	    if (any_annual_temp_change)
-	      celltemp = interpolate_temperature(x, y);
-	    else
-	      celltemp = t_temp_avg(t);
-	    /* Add in a random variation if specified. */
-	    if (t_temp_variability(t) > 0) {
-		celltemp += (xrandom(t_temp_variability(t))
-			    - t_temp_variability(t)/2);
-	    }
-	    /* Higher elevations can be much colder. */
-	    /* (In this pos, will influence lowlands via moderation - realistic?) */
-	    if (elevations_defined()
-	        && g_temp_floor_elev() != 0
-	        && elev_at(x, y) < g_temp_floor_elev()) {
-		celltemp -=
-		    ((celltemp - g_temp_floor()) * elev_at(x, y))
-		    / g_temp_floor_elev();
-	    }
-	    /* Record the (unmoderated) temperature of the cell. */
-	    set_temperature_at(x, y, celltemp);
-	}
+    if (any_wind_variation_in_layer && !winds_defined()) {
+        allocate_area_winds();
+        allocate_area_scratch(2);
     }
-    /* Sometimes the scale of the world is such that neighboring cells
-       influence each other's temperatures. */
-    if (g_temp_mod_range() > 0) {
-	/* only doing a range of 1... */
-	for_all_interior_cells(x, y) {
-	    set_tmp2_at(x, y, temperature_at(x, y));
-	    for_all_directions(dir) {
-		set_tmp2_at(x, y,
-			    (tmp2_at(x, y)
-			     + temperature_at(x+dirx[dir], y+diry[dir])));
+    /* The tmp1 record will record where any changes occur. */
+    for_all_cells(x, y) {
+	set_tmp1_at(x, y, FALSE);
+    }
+    if (mintemp != maxtemp) {
+	/* Compute the average temperature at each point in the world. */
+	for (y = area.height - 1; y >= 0; --y) {
+	    for (x = 0; x < area.width; ++x) {
+		if (!in_area(x, y))
+		  continue;
+		/* Save the prev temp. */
+		set_tmp2_at(x, y, temperature_at(x, y));
+		t = terrain_at(x, y);
+		if (any_annual_temp_change)
+		  celltemp = interpolate_temperature(x, y);
+		else
+		  celltemp = t_temp_avg(t);
+		/* Add in a random variation if specified. */
+		if (t_temp_variability(t) > 0) {
+		    celltemp += (xrandom(t_temp_variability(t))
+				 - t_temp_variability(t)/2);
+		}
+		/* Higher elevations can be much colder. */
+		/* (In this pos, will influence lowlands via moderation -
+		   realistic?) */
+		if (elevations_defined()
+		    && g_temp_floor_elev() != 0
+		    && elev_at(x, y) < g_temp_floor_elev()) {
+		    celltemp -=
+		      ((celltemp - g_temp_floor()) * elev_at(x, y))
+			/ g_temp_floor_elev();
+		}
+		/* Clip to terrain type's limits. */
+		if (celltemp < t_temp_min(t))
+		  celltemp = t_temp_min(t);
+		if (celltemp > t_temp_max(t))
+		  celltemp = t_temp_max(t);
+		/* Record the (unmoderated) temperature of the cell. */
+		set_temperature_at(x, y, celltemp);
 	    }
 	}
-	for_all_interior_cells(x, y) {
-	    set_temperature_at(x, y, (tmp2_at(x, y) / (NUMDIRS + 1)));
-	}
-    }
-    /* See if any displays should change and report if so. */
-    for_all_sides(side) {
-	if (side_has_display(side)) {
-            for_all_cells(x, y) {
-	        if (temperature_at(x, y) != tmp1_at(x, y) &&
-		    terrain_visible(side, x, y)) {
-		    update_cell_display(side, x, y, 34);
+	/* Sometimes the scale of the world is such that neighboring cells
+	   influence each other's temperatures. */
+	if (g_temp_mod_range() > 0) {
+	    /* only doing a range of 1... */
+	    for_all_interior_cells(x, y) {
+		set_tmp3_at(x, y, temperature_at(x, y));
+		for_all_directions(dir) {
+		    set_tmp3_at(x, y,
+				(tmp3_at(x, y)
+				 + temperature_at(x+dirx[dir], y+diry[dir])));
 		}
 	    }
-	    flush_display_buffers(side);
-	}
-    }
-    /* Affect any temperature-sensitive units. */
-    for_all_units(unit) {
-	if (in_play(unit) &&
-	    !(unit->transport != NULL &&
-	      uu_temp_protect(unit->transport->type, unit->type))) {
-	    u = unit->type;
-	    temp = temperature_at(unit->x, unit->y);
-#if 0
-	    if (between(u_survival_min(u), temp, u_survival_max(u))) {
-		if (temp < (comfmin = u_comfort_min(u))) {
-		    range = comfmin - u_survival_min(u);
-		    if (probability((100 * (comfmin - temp)) / range)) {
-			damage_unit_with_temperature(unit, 1);
-		    }
-		} else if (temp > (comfmax = u_comfort_max(u))) {
-		    range = u_survival_max(u) - comfmax;
-		    if (probability((100 * (temp - comfmax)) / range)) {
-			damage_unit_with_temperature(unit, 1);
-		    }
-		} else {
-		    /* Unit gets off scot-free - this time... */
-		}
-	    } else {
-		damage_unit_with_temperature(unit, unit->hp);
+	    for_all_interior_cells(x, y) {
+		celltemp = tmp3_at(x, y) / (NUMDIRS + 1);
+		t = terrain_at(x, y);
+		/* Clip to terrain type's limits. */
+		if (celltemp < t_temp_min(t))
+		  celltemp = t_temp_min(t);
+		if (celltemp > t_temp_max(t))
+		  celltemp = t_temp_max(t);
+		set_temperature_at(x, y, celltemp);
 	    }
-#endif
+	}
+ 	for_all_cells(x, y) {
+	    if (temperature_at(x, y) != tmp2_at(x, y))
+	      set_tmp1_at(x, y, TRUE);
+	    anychanges = TRUE;
 	}
     }
     /* Do wind changes. */
     if (any_wind_variation_in_layer) {
 	/* Save the previous state. */
 	for_all_interior_cells(x, y) {
-	    set_tmp1_at(x, y, raw_wind_at(x, y));
+	    set_tmp2_at(x, y, raw_wind_at(x, y));
 	}
 	for_all_interior_cells(x, y) {
 	    int winddir = wind_dir_at(x, y);
@@ -1479,43 +1689,29 @@ run_environment()
 		windforce = min(windforce, t_wind_force_max(t));
 		anychange = TRUE;
 	    }
-	    if (anychange)
-	      set_wind_at(x, y, winddir, windforce);
+	    if (anychange) {
+		set_wind_at(x, y, winddir, windforce);
+	    }
 	}
 	if (g_wind_mix_range() > 0) {
 	    mix_winds();
 	}
-	/* See if any displays should change and report if so. */
-	for_all_sides(side) {
-	    if (side_has_display(side)) {
-		for_all_cells(x, y) {
-		    if (raw_wind_at(x, y) != tmp1_at(x, y)
-			&& (g_see_all() || terrain_view(side, x, y) != UNSEEN)) {
-			update_cell_display(side, x, y, 35);
-		    }
-		}
-		flush_display_buffers(side);
+	for_all_cells(x, y) {
+	    if (raw_wind_at(x, y) != tmp2_at(x, y))
+	      set_tmp1_at(x, y, TRUE);
+	    anychanges = TRUE;
+	}
+    }
+    /* See if any displays should change and report if so. */
+    if (anychanges) {
+	for_all_cells(x, y) {
+	    if (tmp1_at(x, y)) {
+		all_see_cell_weather(x, y);
 	    }
 	}
     }
 }
 
-#if 0
-static void
-damage_unit_with_temperature(unit, n)
-Unit *unit;
-int n;
-{
-    if (n >= unit->hp) {
-	Dprintf("%s dies from excessive temps\n", unit_desig(unit));
-	kill_unit(unit, H_UNIT_KILLED);
-    } else {
-	Dprintf("%s damaged by excessive temps\n", unit_desig(unit));
-	unit->hp -= n;
-    }
-}
-#endif
-
 static void
 mix_winds()
 {
@@ -1549,6 +1745,35 @@ mix_winds()
     }
 }
 
+/* Update all sides with all weather changes that have happened at
+   the given location. */
+
+void
+all_see_cell_weather(x, y)
+int x, y;
+{
+    int oldview;
+    Side *side;
+
+    for_all_sides(side) {
+	if (g_see_weather_always()
+	    ? (terrain_view(side, x, y) != UNSEEN)
+	    : (cover(side, x, y) > 0)) {
+	    oldview = temperature_view(side, x, y);
+	    if (oldview != temperature_at(x, y)) {
+		set_temperature_view(side, x, y, temperature_at(x, y));
+		update_cell_display(side, x, y, 34);
+	    }
+	    /* (should update cloud view here too) */
+	    oldview = wind_view(side, x, y);
+	    if (oldview != raw_wind_at(x, y)) {
+		set_wind_view(side, x, y, raw_wind_at(x, y));
+		update_cell_display(side, x, y, 35);
+	    }
+	}
+    }
+}
+
 /* Given that the spying unit is going to get info about other units at this
    location, figure out just what it is going to see. */
 
@@ -1592,7 +1817,7 @@ run_spies()
 		break;
 	    }
 	}
-	if (g_see_all())
+	if (all_see_all)
 	  any_spying = FALSE;  /* override */
     }
     if (!any_spying)
@@ -1694,7 +1919,7 @@ run_accidents()
 		/* Damage the unit. */
 		unit->hp2 -= ut_accident_damage(unit->type, t);
 		/* (should be able to pass reason to damage_unit) */
-		damage_unit(unit);
+		damage_unit(unit, accident_dmg);
 	    }
 	}
     }
@@ -1731,7 +1956,7 @@ run_attrition()
 	    /* This is like hit_unit but doesn't have other effects. */
 	    unit->hp2 -= dmg;
 	    /* (should be able to pass reason to damage_unit) */
-	    damage_unit(unit);
+	    damage_unit(unit, accident_dmg);
 	}
     }
 }
@@ -1887,7 +2112,7 @@ Unit *unit, *unit2;
 {
     int uview;
 
-    if (g_see_all()) {
+    if (all_see_all) {
 	return TRUE;
     } else if (unit->side != NULL) {
     	uview = unit_view(unit->side, unit2->x, unit2->y);
@@ -2219,7 +2444,9 @@ int m;
     tmpexcess -= (oldsupply - from->supply[m]);
 }
 
-static void
+/* Note that this may be called on newly-completed units during a turn. */
+
+void
 try_sharing(from, to, m)
 Unit *from, *to;
 int m;
@@ -2298,8 +2525,8 @@ int m, need;
     int supp = unit->supply[m];
     int stor = um_storage_x(unit->type, m);
 
-    return (((2 * supp > stor) && (need < ((supp * 4) / 5))) ||
-	    (need < stor / 5));
+    return (((2 * supp > stor) && (need < ((supp * 4) / 5)))
+	    || (need < stor / 5));
 }
 
 /* Do everything associated with the end of a turn. */
@@ -2311,6 +2538,7 @@ run_turn_end()
     run_people_consumption();
     run_cell_consumption();
     run_unit_base_consumption();
+    run_environment_effects();
     run_people_side_changes();
     /* This should come after other people-related computations,
        since this only constrains generic overcrowding. */
@@ -2482,30 +2710,40 @@ static void
 unit_consumes(unit)
 Unit *unit;
 {
-    int u = unit->type, m, usedup, consump, checkstarve = FALSE;
+    int u = unit->type, m, usedup, consump, tempeff, checkstarve = FALSE;
+    Obj *effect;
 
     if (alive(unit)) {    
-      for_all_material_types(m) {
-	if (um_base_consumption(u, m) > 0 &&
-	    !(unit->transport != NULL && um_consumption_as_occupant(u, m) == 0)) {
-	    /* Calculate what was already consumed by movement. */
-	    usedup = 0;
-	    if (unit->act != NULL)
-	      usedup = unit->act->actualmoves * um_consumption_per_move(u, m);
-	    consump = um_base_consumption(u, m);
-	    /* If being transported, modify the base consumption. */
-	    if (unit->transport != NULL)
-	      consump = (consump * um_consumption_as_occupant(u, m)) / 100;
-	    /* Subtract consumption that was not already used up in movement. */
-	    if (usedup < consump)
-	      unit->supply[m] -= (consump - usedup);
-	    /* Don't let supply go below zero. */
-	    if (unit->supply[m] <= 0) {
-		unit->supply[m] = 0;
-		checkstarve = TRUE;
+	for_all_material_types(m) {
+	    if (um_base_consumption(u, m) > 0 &&
+		!(unit->transport != NULL
+		  && um_consumption_as_occupant(u, m) == 0)) {
+		/* Calculate what was already consumed by movement. */
+		usedup = 0;
+		if (unit->act != NULL)
+		  usedup = unit->act->actualmoves * um_consumption_per_move(u, m);
+		consump = um_base_consumption(u, m);
+		/* If being transported, modify the base consumption. */
+		if (unit->transport != NULL)
+		  consump = (consump * um_consumption_as_occupant(u, m)) / 100;
+		/* Modify consumption if temperature effects. */
+		effect = u_consume_temp_effect(u);
+#if 0		/* (should think about this code a bit more) */
+		if (temperatures_defined() && effect != lispnil) {
+		    tempeff = interpolate_in_list(temperature_at(unit->x, unit->y), effect, FALSE, 999);
+		    consump = (consump * tempeff) / 100;
+		}
+#endif
+		/* Subtract consumption that was not already used up in movement. */
+		if (usedup < consump)
+		  unit->supply[m] -= (consump - usedup);
+		/* Don't let supply go below zero. */
+		if (unit->supply[m] <= 0) {
+		    unit->supply[m] = 0;
+		    checkstarve = TRUE;
+		}
 	    }
 	}
-      }
     }
     if (checkstarve)
       maybe_starve(unit, TRUE);
@@ -2531,23 +2769,24 @@ int partial;
     int u = unit->type, m, starv, oneloss, hploss = 0;
 
     for_all_material_types(m) {
-      if (unit->supply[m] <= 0 && !in_supply(unit, m)) {
-	starv = um_hp_per_starve(u, m);
-	if (starv > 0) {
-	    oneloss = prob_fraction(starv);
-	    hploss = max(hploss, oneloss);
+	if (unit->supply[m] <= 0 && !in_supply(unit, m)) {
+	    starv = um_hp_per_starve(u, m);
+	    if (starv > 0) {
+		oneloss = prob_fraction(starv);
+		hploss = max(hploss, oneloss);
+	    }
 	}
-      }
     }
     if (hploss > 0) {
-      if (hploss >= unit->hp) {
-	/* (should let occupants try to escape first) */
-	kill_unit(unit, H_UNIT_STARVED);
-      } else if (partial) {
-    	unit->hp -= hploss;
-    	/* (should do other hp loss consequences) */
-	/* (use generic damage routine?) */
-      }
+	if (hploss >= unit->hp) {
+	    /* (should let occupants try to escape first) */
+	    kill_unit(unit, H_UNIT_STARVED);
+	} else if (partial) {
+	    unit->hp -= hploss;
+	    unit->hp2 -= hploss;
+	    /* (should do other hp loss consequences) */
+	    /* (use generic damage routine?) */
+	}
     }
 }
 
@@ -2606,6 +2845,54 @@ run_self_builds()
     }
 }
 
+static void
+run_environment_effects()
+{
+    int err, dmg;
+    Unit *unit;
+    Obj *attrition;
+
+    if (!temperatures_defined())
+      return;
+    /* (should check if temperature attrition ever non-nil) */
+    Dprintf("Running environmental effects\n");
+    for_all_units(unit) {
+	if (is_active(unit)) {
+	    attrition = u_temp_attrition(unit->type);
+	    if (attrition != lispnil) {
+		err = interpolate_in_list(temperature_at(unit->x, unit->y), attrition, &dmg);
+		if (err != 0) {
+		    dmg = 0;
+		}
+		damage_unit_with_temperature(unit, dmg);
+	    }
+	    /* (should check for storm damage here?) */
+	}
+    }
+}
+
+static void
+damage_unit_with_temperature(unit, dmg)
+Unit *unit;
+int dmg;
+{
+    int n;
+
+    n = prob_fraction(dmg);
+    if (n >= unit->hp) {
+	notify(unit->side, "%s dies from excessive temperatures",
+	       unit_handle(unit->side, unit));
+	/* should have a different hevt type - this will be recorded as a combat death */
+	kill_unit(unit, H_UNIT_DIED_FROM_TEMPERATURE);
+    } else if (n > 0) {
+	notify(unit->side, "%s loses %d HP due to excessive temperatures",
+		unit_handle(unit->side, unit), n);
+	unit->hp -= n;
+	unit->hp2 -= n;
+	update_unit_display(unit->side, unit, TRUE);
+    }
+}
+
 static void
 run_people_side_changes()
 {
@@ -2688,10 +2975,13 @@ Side *side;
 	    }
 	}
 	/* Previous side loses its free coverage. */
-	if (oldside)
-	  add_cover(oldside, x, y, -1);
+	if (oldside) {
+	    add_cover(oldside, x, y, -1);
+	    /* Update coverage display. */
+	    update_cell_display(oldside, x, y, 36);
+	}
     }
-    /* (should be able to do adjacent cells also) */
+    /* (should add ability to change adjacent cells also) */
 }
 
 /* See if the numbers of individuals in a cell exceeds the max, and migrate or
@@ -2764,7 +3054,9 @@ int x, y, rightnow;
     Side *side;
 
     for_all_sides(side) {
-	if (side->ingame && (g_see_all() || cover(side, x, y) > 0)) {
+	/* (Testing for unit visibility is not quite right, but close
+	   enough for now.) */
+	if (side->ingame && units_visible(side, x, y)) {
 	    update_cell_display(side, x, y, rightnow);
 	}
     }
@@ -2776,18 +3068,14 @@ static void
 run_appearances()
 {
     int curturn, nx, ny, nw, nh, nx1, ny1;
-    Obj *appear;
     Unit *unit, *transport;
 
     if (any_appearances < 0) {
     	any_appearances = FALSE;
 	for_all_units(unit) {
-	    if (unit->cp < 0 && unit->hook != lispnil) {
-		appear = get_x_property(unit, K_APPEAR);
-		if (appear != lispnil) {
-		    any_appearances = TRUE;
-		    break;
-		}
+	    if (unit->cp < 0 && unit_appear_turn(unit) >= 0) {
+		any_appearances = TRUE;
+		break;
 	    }
 	}
     }
@@ -2797,21 +3085,15 @@ run_appearances()
     curturn = g_turn();
     for_all_units(unit) {
     	/* See if now time for a unit to appear. */
-    	if (unit->cp < 0 && unit->hook != lispnil) {
-	    appear = get_x_property(unit, K_APPEAR);
-	    if (appear != lispnil && c_number(car(appear)) <= curturn) {
+    	if (unit->cp < 0 && unit_appear_turn(unit) >= 0) {
+	    if (unit_appear_turn(unit) <= curturn) {
 		/* Set the unit to its correct cp. */
 		unit->cp = (- unit->cp);
 		/* Get the base location at which it will appear. */
 		nx = (- unit->prevx);  ny = (- unit->prevy);
-		if (cdr(appear) != lispnil && numberp(cadr(appear))) {
+		if (unit_appear_x(unit) >= 0 && unit_appear_y(unit) >= 0) {
 		    /* Appear at a random location around nx,ny. */
-		    nw = c_number(cadr(appear));
-		    if (cddr(appear) != lispnil && numberp(caddr(appear))) {
-			nh = c_number(caddr(appear));
-		    } else {
-			nh = nw;
-		    }
+		    nw = unit_appear_x(unit);  nh = unit_appear_y(unit);
 		    if (random_point_in_area(nx, ny, nw, nh, &nx1, &ny1)) {
 			nx = nx1;  ny = ny1;
 		    }
@@ -2830,14 +3112,14 @@ run_appearances()
 				break;
 			    }
 			}
-			/* We got a problem, make the unit wait for next turn;
-			   will just try again. */
+			/* We've got a problem, make the unit wait for next turn;
+			   since the appearance turn test is "<=", will just try again. */
 			unit->cp = (- unit->cp);
 		    }
 		} else {
 		    /* loc of reinforcement is messed up */
 		}
-		init_unit_actorstate(unit);
+		init_unit_actorstate(unit, FALSE);
 		init_unit_plan(unit);
 	    }
     	}
@@ -2848,18 +3130,14 @@ static void
 run_disappearances()
 {
     int curturn;
-    Obj *disappear;
     Unit *unit;
 
     if (any_disappearances < 0) {
     	any_disappearances = FALSE;
 	for_all_units(unit) {
-	    if (unit->hook != lispnil) {
-		disappear = get_x_property(unit, K_DISAPPEAR);
-		if (disappear != lispnil) {
-		    any_disappearances = TRUE;
-		    break;
-		}
+	    if (unit_disappear_turn(unit) >= 0) {
+		any_disappearances = TRUE;
+		break;
 	    }
 	}
     }
@@ -2869,11 +3147,10 @@ run_disappearances()
     curturn = g_turn();
     for_all_units(unit) {
     	/* See if now time for a unit to disappear. */
-    	if (in_play(unit) && unit->hook != lispnil) {
-    	    disappear = get_x_property(unit, K_DISAPPEAR);
-    	    if (disappear != lispnil && c_number(car(disappear)) <= curturn) {
-	    /* (should eject occupants first if possible) */
-		kill_unit(unit, H_UNIT_KILLED);
+    	if (in_play(unit) && unit_disappear_turn(unit) >= 0) {
+	    if (unit_disappear_turn(unit) <= curturn) {
+		/* (should eject occupants first if possible) */
+		kill_unit(unit, H_UNIT_LEFT_WORLD);
 	    }
 	}
     }
@@ -2907,9 +3184,7 @@ run_hp_recovery()
 	    /* (should only do for one part of multi-part unit?) */
 	    if (hprecovery > 0 && unit->hp < hpmax) {
 		oldhp = unit->hp;
-		unit->hp += prob_fraction(hprecovery);
-		if (unit->hp > hpmax)
-		  unit->hp = hpmax;
+		add_to_unit_hp(unit, prob_fraction(hprecovery));
 		/* Inform the player if the unit's hp changed. */
 		if (unit->hp != oldhp) {
 		    update_unit_display(unit->side, unit, TRUE);
@@ -3020,7 +3295,7 @@ static void
 auto_repair_unit(unit, unit2)
 Unit *unit, *unit2;
 {
-    int u = unit->type, u2 = unit2->type, m, repair, hpmax;
+    int u = unit->type, u2 = unit2->type, m, repair;
 
     /* Check the basic restrictions on repair. */
     if (unit->hp < uu_hp_to_repair(u, u2))
@@ -3030,10 +3305,23 @@ Unit *unit, *unit2;
 	  return;
     }
     repair = uu_auto_repair(u, u2);
-    unit2->hp += prob_fraction(repair);
-    hpmax = u_hp(u2);
-    if (unit2->hp > hpmax)
+    add_to_unit_hp(unit2, prob_fraction(repair));
+}
+
+void
+add_to_unit_hp(unit, hp)
+Unit *unit;
+int hp;
+{
+    int hpmax;
+
+    unit->hp += hp;
+    hpmax = u_hp(unit->type);
+    if (unit->hp > hpmax)
       unit->hp = hpmax;
+    unit->hp2 += hp;
+    if (unit->hp2 > hpmax)
+      unit->hp2 = hpmax;
 }
 
 static void
@@ -3149,6 +3437,10 @@ end_the_game()
     record_event(H_GAME_ENDED, ALLSIDES);
     /* Set the global that indicates the game is over for everybody. */
     endofgame = TRUE;
+    end_history();
+    record_into_scorefile();
+    dump_statistics();
+    /* Done with internal state change, now echo onto displays. */
     for_all_sides(side) {
     	/* (should there be any other effects on sides, like final scoring?) */
     	if (side_has_display(side)) {
@@ -3156,6 +3448,4 @@ end_the_game()
     	    update_side_display(side, side, TRUE);
     	}
     }
-    end_history();
-    dump_statistics();
 }
