@@ -1,5 +1,5 @@
 /* Scores and scoring in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -19,6 +19,27 @@ any later version.  See the file COPYING.  */
    or whatever.  */
 
 #include "conq.h"
+extern FILE *open_scorefile_for_reading PARAMS ((char *name));
+extern FILE *open_scorefile_for_writing PARAMS ((char *name));
+
+int read_scorefile PARAMS ((void));
+int interp_score_record PARAMS ((Obj *form));
+
+typedef struct a_score_record {
+    char *gamename;
+    Obj *sides;
+    Obj *raw;
+    struct a_score_record *next;
+} ScoreRecord;
+
+/* Iteration over all recorded scores. */
+
+#define for_all_score_records(sr)  \
+  for ((sr) = records;  (sr) != NULL;  (sr) = (sr)->next)
+
+ScoreRecord *records;
+
+ScoreRecord *last_record;
 
 /* This is true when the given side should be tested against the given scorekeeper. */
 
@@ -240,19 +261,20 @@ check_post_turn_scores()
 	for_all_scorekeepers(sk) {
 	    Dprintf("Checking post-turn scorekeeper %d\n", sk->id);
 	    if (sk->when == lispnil || match_keyword(sk->when, K_AFTER_TURN)) {
-		/* Decide whether the scorekeeper applies to one side or to the whole game. */
+		/* Decide whether the scorekeeper applies to one side
+                   or to the whole game. */
 	    	if (symbolp(sk->body)
 	    	    && match_keyword(sk->body, K_LAST_SIDE_WINS)) {
-			eval_sk_last_side_wins(sk);
+		    eval_sk_last_side_wins(sk);
 	    	} else {
-			for_all_sides(side) {
-			    if (scorekeeper_applicable(side, sk)) {
-				run_scorekeeper(side, sk); 
-			    } else {
-			    	Dprintf("sk %d not applicable to %s\n",
-			    		sk->id, side_desig(side));
-			    }
+		    for_all_sides(side) {
+			if (scorekeeper_applicable(side, sk)) {
+			    run_scorekeeper(side, sk); 
+			} else {
+			    Dprintf("sk %d not applicable to %s\n",
+				    sk->id, side_desig(side));
 			}
+		    }
 		}
 	    }
 	}
@@ -425,9 +447,11 @@ Obj *form;
     
     if (typelist != lispnil) {
     	/* (should decide overall type of objects first) */
-	for (i = 0; i < 200; ++i) typevec[i] = FALSE;
+	for (i = 0; i < 200; ++i)
+	  typevec[i] = FALSE;
     } else {
-	for (i = 0; i < 200; ++i) typevec[i] = TRUE;
+	for (i = 0; i < 200; ++i)
+	  typevec[i] = TRUE;
     }
     if (consp(form)) {
 	prop = car(form);
@@ -444,7 +468,7 @@ Obj *form;
 		    if (prop != lispnil) {
 			if (symbolp(prop)
 			    && strcmp(c_string(prop), "point-value") == 0) {
-			    sum += unit_point_value(unit);
+			    sum += point_value(unit);
 			} else {
 			}
 		    } else {
@@ -458,28 +482,41 @@ Obj *form;
 }
 
 int
-unit_point_value(unit)
+point_value(unit)
 Unit *unit;
 {
     Obj *val;
 
     /* Incomplete units are always worthless. */
-    if (!completed(unit))
+    if (unit->cp > 0 && !completed(unit))
       return 0;
-    /* Look for an extension property and use it as point value. */
-    if (unit->hook != lispnil) {
-	val = get_x_property_by_name(unit, "point-value");
-	if (val != lispnil) {
-	    val = eval(car(val));
-	    if (numberp(val))
-	      return c_number(val);
-	    run_warning("non-numeric point value for %s", unit_desig(unit));
-	    /* Just fall through and use point value for type. */
-	}
-    }
+    if (unit_point_value(unit) >= 0)
+      return unit_point_value(unit);
     return u_point_value(unit->type);
 }
 
+int
+side_point_value(side)
+Side *side;
+{
+    int points;
+    Unit *unit;
+
+    if (!side->point_value_valid) {
+	side->point_value_cache = 0;
+	for_all_side_units(side, unit) {
+	    if ((in_play(unit) && completed(unit))
+		|| (alive(unit) && unit->cp < 0)) {
+		side->point_value_cache += point_value(unit);
+	    }
+	}
+	side->point_value_valid = TRUE;
+    }
+    return side->point_value_cache;
+}
+
+/* Evaluate a given form with respect to the given scorekeeper and side. */
+
 int
 eval_sk_test(side, sk, form)
 Side *side;
@@ -491,17 +528,21 @@ Obj *form;
 
     if (consp(form)) {
 	formtype = c_string(car(form));
-	if (consp(cdr(form)))  arg1 = cadr(form);
-	if (consp(cddr(form))) arg2 = car(cddr(form));
+	if (consp(cdr(form)))
+	  arg1 = cadr(form);
+	if (consp(cddr(form)))
+	  arg2 = car(cddr(form));
 	switch (keyword_code(formtype)) {
 	  case K_AND:
 	    for (rest = cdr(form); rest != lispnil; rest = cdr(rest)) {
-		if (!eval_sk_form(side, sk, car(rest))) return FALSE;
+		if (!eval_sk_form(side, sk, car(rest)))
+		  return FALSE;
 	    }
 	    return TRUE;
 	  case K_OR:
 	    for (rest = cdr(form); rest != lispnil; rest = cdr(rest)) {
-		if (eval_sk_form(side, sk, car(rest))) return TRUE;
+		if (eval_sk_form(side, sk, car(rest)))
+		  return TRUE;
 	    }
 	    return FALSE;
 	  case K_NOT:
@@ -509,6 +550,9 @@ Obj *form;
 	  case K_EQ:
 	    return (eval_sk_form(side, sk, arg1) ==
 		    eval_sk_form(side, sk, arg2));
+	  case K_NE:
+	    return (eval_sk_form(side, sk, arg1) !=
+		    eval_sk_form(side, sk, arg2));
 	  case K_LT:
 	    return (eval_sk_form(side, sk, arg1) <
 		    eval_sk_form(side, sk, arg2));
@@ -536,19 +580,13 @@ eval_sk_last_side_wins(sk)
 Scorekeeper *sk;
 {
     Side *side2, *winner = NULL;
-    Unit *unit;
     int numleft = 0, points;
 
     /* This is only meaningful in games with at least two sides. */
     if (num_sides_originally >= 2) {
 	for_all_sides(side2) {
 	    if (side2->ingame) {
-		points = 0;
-		for_all_side_units(side2, unit) {
-		    if (in_play(unit) && completed(unit)) {
-			points += unit_point_value(unit);
-		    }
-		}
+		points = side_point_value(side2);
 		Dprintf("%s has %d points worth of units\n",
 			side_desig(side2), points);
 		if (points == 0) {
@@ -579,7 +617,7 @@ int why;
 }
 
 /* Implement the effects of a side losing. */
-/* (should reindent) */
+
 void
 side_loses(side, side2, why)
 Side *side, *side2;
@@ -588,14 +626,20 @@ int why;
     int x, y, s, s2, ux, uy;
     Unit *unit;
 
-    if (side == side2)
-      run_error("losing to self");
-    if (side2 != NULL && !side2->ingame)
-      run_error("losing to side not in game");
+    /* These should not happen, but a stupid AI might try, so just
+       and clear the mistaken side. */
+    if (side == side2) {
+	run_warning("losing to self, ignoring side");
+	side2 = NULL;
+    }
+    if (side2 != NULL && !side2->ingame) {
+	run_warning("losing to side not in game, ignoring side");
+	side2 = NULL;
+    }
     if (side2 != NULL) {
-      /* Dispose of all of a side's units. */
-      for_all_units(unit) {
-	if (unit->side == side) {
+	/* Dispose of all of a side's units. */
+	for_all_units(unit) {
+	    if (unit->side == side) {
 		if (in_play(unit)) {
 		    ux = unit->x;  uy = unit->y;
 		    change_unit_side(unit, side2, H_SIDE_LOST, NULL);
@@ -605,25 +649,25 @@ int why;
 		    /* Even out-of-play units need to have their side set. */
 		    set_unit_side(unit, side2);
 		}
+	    }
 	}
-      }
-      /* The people also change sides. */
-      if (people_sides_defined()) {
-	s = side_number(side);
-	s2 = side_number(side2);
-	for_all_cells(x, y) {
-	    if (people_side_at(x, y) == s) {
-		set_people_side_at(x, y, s2);
-		all_see_cell(x, y);
-	    } 
+	/* The people also change sides. */
+	if (people_sides_defined()) {
+	    s = side_number(side);
+	    s2 = side_number(side2);
+	    for_all_cells(x, y) {
+		if (people_side_at(x, y) == s) {
+		    set_people_side_at(x, y, s2);
+		    all_see_cell(x, y);
+		}
+	    }
 	}
-      }
-      /* Reset view coverage everywhere. */
-      if (!all_see_all) {
-	for_all_cells(x, y) {
-	    set_cover(side, x, y, 0);
+	/* Reset view coverage everywhere. */
+	if (!all_see_all) {
+	    for_all_cells(x, y) {
+		set_cover(side, x, y, 0);
+	    }
 	}
-      }
     }
     /* Add the mark of shame itself. */
     side->status = -1;
@@ -645,31 +689,257 @@ all_sides_draw()
     }
 }
 
-/* Add scorefile handling here. */
-
-/* (should only be done if some scorekeeper says how?) */
+/* Record the outcome of the game into the scorefile. */
 
 void
 record_into_scorefile()
 {
-    char *filename = SCOREFILE;
+    int i;
+    int any_advantage_variation = FALSE, adv, adv2;
+    char *filename, *mversion;
+    Variant *variants, *var;
     FILE *fp;
+    Side *side;
 
+    /* (should make following code into a separate routine) */
+    adv = max(1, sidelist->advantage);
+    adv = (sidelist->player ? sidelist->player->advantage : adv);
+    for_all_sides(side) {
+	adv2 = max(1, sidelist->advantage);
+	adv2 = (sidelist->player ? sidelist->player->advantage : adv2);
+	if (adv != adv2) {
+	    any_advantage_variation = TRUE;
+	    break;
+	}
+    }
+    filename = SCOREFILE;
     if (!empty_string(g_scorefile_name()))
       filename = g_scorefile_name();
-    /* (should assemble a path to specifiable scorefile dir) */
-    fp = fopen(filename, "a");
+    fp = open_scorefile_for_writing(filename);
     if (fp != NULL) {
 	fprintf(fp, "(game");
-	fprintf(fp, " \"%s\"", mainmodule->name);
-	/* (should write players, scores, game facts, etc) */
+	fprintf(fp, " \"%s\"",
+		/* (should record this for comparison when displaying scores) */
+		(mainmodule->origmodulename
+		 ? mainmodule->origmodulename
+		 : mainmodule->name));
+	/* Record the module's version if defined. */
+	mversion = (mainmodule->origversion
+		    ? mainmodule->origversion
+		    : mainmodule->version);
+	if (!empty_string(mversion))
+	  fprintf(fp, " (version \"%s\")", mversion);
+	/* Record all the choices of variant. */
+	variants = (mainmodule->origvariants
+		    ? mainmodule->origvariants
+		    : mainmodule->variants);
+	if (variants) {
+	    fprintf(fp, " (variants");
+	    for (i = 0; variants[i].id != lispnil; ++i) {
+		var = &(variants[i]);
+		if (symbolp(var->id)) {
+		    fprintf(fp, " (%s", c_string(var->id));
+		} else {
+		    fprintf(fp, " (%d", i);
+		}
+		if (var->hasintvalue) {
+		    fprintf(fp, " %d", var->intvalue);
+		}
+		fprintf(fp, ")");
+	    }
+	    fprintf(fp, ")");
+	}
+	fprintf(fp, "\n");
+	/* Record all the participants and how they fared. */
+	fprintf(fp, "  (sides");
+	for_all_sides(side) {
+	    fprintf(fp, " (%d", side_number(side));
+	    if (side->everingame) {
+		fprintf(fp, " %s",
+			(side_won(side)
+			 ? "won"
+			 : (side_lost(side) ? "lost" : "drew")));
+		short_player_title(spbuf, side->player, NULL);
+		fprintf(fp, " \"%s\"", spbuf);
+		if (any_advantage_variation) {
+		    /* (should have cached as "actual advantage"?) */
+		    int advantage = max(1, side->advantage);
+		    
+		    advantage =
+		      (side->player ? side->player->advantage : advantage);
+		    fprintf(fp, " (+ %d)", advantage); 
+		}
+		if (numscores > 0) {
+		    fprintf(fp, " (scores");
+		    for (i = 0; i < numscores; ++i) {
+			fprintf(fp, " %d", side->scores[i]);
+		    }
+		    fprintf(fp, ")");
+		}
+	    }
+	    fprintf(fp, ")");
+	}
+	fprintf(fp, ")");
+	/* (should record other useful info about game, such as date(s) played) */
 	fprintf(fp, ")\n");
 	fclose(fp);
     }
 }
 
+int
+read_scorefile()
+{
+    int startlineno = 1, endlineno = 1;
+    int numrecs;
+    char *filename;
+    Obj *form;
+    FILE *fp;
+    ScoreRecord *sr;
+
+    filename = SCOREFILE;
+    if (!empty_string(g_scorefile_name()))
+      filename = g_scorefile_name();
+    fp = open_scorefile_for_reading(filename);
+    if (fp != NULL) {
+	numrecs = 0;
+	/* Read everything in the file. */
+	while ((form = read_form(fp, &startlineno, &endlineno)) != lispeof) {
+	    if (interp_score_record(form)) {
+		++numrecs;
+	    }
+	}
+	fclose(fp);
+	Dprintf("%d score records read.\n", numrecs);
+	for_all_score_records(sr) {
+	    Dprintf("%s\n", sr->gamename);
+	}
+	return TRUE;
+    }
+    return FALSE;
+}
+
+int
+interp_score_record(form)
+Obj *form;
+{
+    char *propname;
+    Obj *props, *prop;
+    ScoreRecord *sr;
+
+    if (consp(form)
+	&& symbolp(car(form))
+	/* (should) and symbol is 'game' */
+	&& stringp(cadr(form))) {
+	sr = (ScoreRecord *) xmalloc(sizeof(ScoreRecord));
+	sr->gamename = c_string(cadr(form));
+	for_all_list(cddr(form), props) {
+	    prop = car(props);
+	    if (symbolp(car(prop))) {
+		propname = c_string(car(prop));
+		switch (keyword_code(propname)) {
+		  case K_VERSION:
+		    break;
+		  case K_VARIANTS:
+		    break;
+		  default:
+		    /* This is for keywords that may appear in a
+		       score record but are not actually part of GDL. */
+		    if (strcmp(propname, "sides") == 0) {
+			sr->sides = cdr(prop);
+		    } else {
+			/* should complain? */
+		    }
+		    break;
+		}
+	    }
+	}
+	sr->raw = form;
+	/* Add the record to the end of the list. */
+	if (records != NULL) {
+	    last_record->next = sr;
+	    last_record = sr;
+	} else {
+	    records = last_record = sr;
+	}
+	return TRUE;
+    }
+    return FALSE;
+}
+
+char *get_scores PARAMS ((Side *side));
+
+char *
+get_scores(side)
+Side *side;
+{
+    char *buf = xmalloc(5000);
+    char buf2[BUFSIZE];
+    char *thisgame, *playername;
+    Obj *sds, *sd;
+    ScoreRecord *sr;
+
+    thisgame = (mainmodule->origmodulename
+		 ? mainmodule->origmodulename
+		 : mainmodule->name);
+    if (thisgame == NULL)
+      return "???";
+    sprintf(buf, "Scores for \"%s\":\n", thisgame);
+    read_scorefile();
+    playername = NULL;
+    if (side != NULL) {
+	short_player_title(buf2, side->player, NULL);
+	playername = buf2;
+    }
+    if (1 /* summarize */) {
+	int wins, losses, plays;
+
+	wins = losses = plays = 0;
+	for_all_score_records(sr) {
+	    if (!empty_string(sr->gamename)
+		&& strcmp(thisgame, sr->gamename) == 0) {
+		if (playername != NULL && sr->sides != NULL) {
+		    for_all_list(sr->sides, sds) {
+			sd = car(sds);
+			if (strcmp(playername, c_string(car(cddr(sd)))) == 0) {
+			    if (strcmp("won", c_string(car(cdr(sd)))) == 0) {
+				++wins;
+			    } else if (strcmp("lost", c_string(car(cdr(sd)))) == 0) {
+				++losses;
+			    }
+			}
+		    }
+		}
+		++plays;
+	    }
+	}
+	tprintf(buf, "%s won %d and lost %d of %d games played.\n",
+		"You", wins, losses, plays);
+	return buf;
+    }
+    /* List all the games explicitly. */
+    for_all_score_records(sr) {
+	if (!empty_string(sr->gamename)
+	    && strcmp(thisgame, sr->gamename) == 0) {
+	    if (playername == NULL || 1 /* playername != NULL */) {
+		if (sr->sides != NULL) {
+		    for_all_list(sr->sides, sds) {
+			sd = car(sds);
+			tprintf(buf, "  %s %s",
+				c_string(car(cddr(sd))), c_string(car(cdr(sd))));
+		    }
+		}
+	    } else {
+	    }
+	    tprintf(buf, "\n");
+	    if (strlen(buf) > 4500)
+	      break;
+	}
+    }
+    return buf;
+}
+
 /* This is a general test of whether the given side should be trying
-   to win or should just be goofing off. */
+   to win somehow, or if it can just goof off. */
 
 int
 should_try_to_win(side)
@@ -678,7 +948,7 @@ Side *side;
     Scorekeeper *sk;
 
     for_all_scorekeepers(sk) {
-	if (1 /* sk applies to this side? */) {
+	if (scorekeeper_applicable(side, sk)) {
 	    return TRUE;
 	}
     }
