@@ -1,5 +1,5 @@
 /* Sides in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -10,12 +10,14 @@ any later version.  See the file COPYING.  */
 /* This file implements sides and functionality relating to sides in
    general. */
 
-/* (should sides and players be listed in arrays a la assignments array? */
-
 #include "conq.h"
 
+#define checked_elev_at(x, y) (elevations_defined() ? elev_at(x, y) : 0)
+
 static void init_visible_elevation PARAMS ((int x, int y));
+static void init_visible_elevation_2 PARAMS ((int x, int y));
 static void calc_visible_elevation PARAMS ((int x, int y));
+static void calc_visible_elevation_2 PARAMS ((int x, int y));
 
 /* Head of the list of all sides. */
 
@@ -46,6 +48,8 @@ int nextsideid;
 
 int all_see_all;
 
+int any_los;
+
 /* Pointer to buffer used for readable side description (for debugging). */
 
 char *sidedesigbuf = NULL;
@@ -100,6 +104,7 @@ init_sides()
     indepside->id = 0;
     /* Put valid Lisp data into slots that need it. */
     indepside->symbol = lispnil;
+    indepside->instructions = lispnil;
     indepside->rawscores = lispnil;
     indepside->aidata = lispnil;
     indepside->uidata = lispnil;
@@ -136,7 +141,9 @@ create_side()
     newside->id = nextsideid++;
     /* Always start sides IN the game. */
     newside->ingame = TRUE;
+    /* Note that "everingame" is only set at the beginning of a turn. */
     /* Set up the relationships with other sides. */
+    newside->knows_about = ALLSIDES; /* for now */
     newside->trusts = (short *) xmalloc((g_sides_max() + 1) * sizeof(short));
     newside->trades = (short *) xmalloc((g_sides_max() + 1) * sizeof(short));
     /* Set up per-unit-type slots. */
@@ -155,6 +162,7 @@ create_side()
     newside->willingtosave = TRUE;
     /* Put valid Lisp data into slots that need it. */
     newside->symbol = lispnil;
+    newside->instructions = lispnil;
     newside->rawscores = lispnil;
     newside->aidata = lispnil;
     newside->uidata = lispnil;
@@ -180,7 +188,7 @@ init_side_unithead(side)
 Side *side;
 {
     if (side->unithead == NULL) {
-	side->unithead = (Unit *) create_bare_unit(0);
+	side->unithead = create_bare_unit(0);
 	side->unithead->next = side->unithead;
 	side->unithead->prev = side->unithead;
     }
@@ -209,6 +217,8 @@ Side *side;
     if (side->default_doctrine == NULL) {
 	side->default_doctrine = new_doctrine(0);
 	side->default_doctrine->everaskside = TRUE;
+	side->default_doctrine->construction_run =
+	  (short *) xmalloc (numutypes * sizeof(short));
     }
     if (side->udoctrine == NULL) {
 	/* Make each individual unit doctrine point to the generic doctrine by default. */
@@ -220,6 +230,23 @@ Side *side;
     }
 }
 
+void
+init_self_unit(side)
+Side *side;
+{
+    Unit *unit;
+
+    if ((g_self_required() /* || side prop? */)
+	&& side->self_unit == NULL) {
+	for_all_side_units(side, unit) {
+	    if (u_can_be_self(unit->type)
+		&& in_play(unit)
+		&& completed(unit))
+	      side->self_unit = unit;
+	}
+    }
+}
+
 /* Initialize basic viewing structures for a side.  This happens when the
    side is created (during/after reading but before synthesis). */
 
@@ -227,12 +254,32 @@ int
 init_view(side)
 Side *side;
 {
-    int terrainset = FALSE;
+    int terrainset = FALSE, t, u;
 
-    /* Allocate space for views. */
     /* (should check world validity/compatibility first?) */
     if (side->terrview == NULL) {
-	side->terrview = malloc_area_layer(short);
+	side->terrview = malloc_area_layer(char);
+	if (!g_see_terrain_always()) {
+	    side->terrviewdate = malloc_area_layer(short);
+	    /* The terrview also holds the "have we seen it" flag, so it
+	       must always be allocated, but the aux terrain view need not be. */
+	    if (numcelltypes < numttypes) {
+		side->auxterrview = (char **) xmalloc(numttypes * sizeof(short *));
+		for_all_terrain_types(t) {
+		    if (!t_is_cell(t)) {
+			side->auxterrview[t] = malloc_area_layer(char);
+		    }
+		}
+		if (0 /* aux terrain seen separately from main terrain */) {
+		    side->auxterrviewdate = (short **) xmalloc(numttypes * sizeof(short *));
+		    for_all_terrain_types(t) {
+			if (!t_is_cell(t)) {
+			    side->auxterrviewdate[t] = malloc_area_layer(short);
+			}
+		    }
+		}
+	    }
+	}
     } else {
 	terrainset = TRUE;
     }
@@ -243,10 +290,53 @@ Side *side;
 	side->viewhistory[MAXVIEWHISTORY] = malloc_area_layer(long);
 #endif
     }
+    /* Allocate weather view layers. */
+    if (any_temp_variation && !g_see_weather_always()) {
+	if (side->tempview == NULL) {
+	    side->tempview = malloc_area_layer(short);
+	}
+	if (side->tempviewdate == NULL) {
+	    side->tempviewdate = malloc_area_layer(short);
+	}
+    }
+    if (any_clouds && !g_see_weather_always()) {
+	if (side->cloudview == NULL) {
+	    side->cloudview = malloc_area_layer(short);
+	}
+	if (side->cloudbottomview == NULL) {
+	    side->cloudbottomview = malloc_area_layer(short);
+	}
+	if (side->cloudheightview == NULL) {
+	    side->cloudheightview = malloc_area_layer(short);
+	}
+	if (side->cloudviewdate == NULL) {
+	    side->cloudviewdate = malloc_area_layer(short);
+	}
+    }
+    if (any_wind_variation && !g_see_weather_always()) {
+	if (side->windview == NULL) {
+	    side->windview = malloc_area_layer(short);
+	}
+	if (side->windviewdate == NULL) {
+	    side->windviewdate = malloc_area_layer(short);
+	}
+    }
     /* Allocate the vision coverage cache if needed. */
     if (side->coverage == NULL) {
 	side->coverage = malloc_area_layer(short);
     }
+    /* (should put this calc elsewhere?) */
+    any_los = FALSE;
+    for_all_unit_types(u) {
+	if (u_vision_bend(u) != 100) {
+	    any_los = TRUE;
+	    break;
+	}
+    }
+    /* Allocate vision altitude coverage if needed */
+    if (side->alt_coverage == NULL && any_los) {
+	side->alt_coverage = malloc_area_layer(short);
+    }
     return terrainset;
 }
 
@@ -270,35 +360,6 @@ Side *side;
     }
 }
 
-/* Get a char string naming the side.  Doesn't have to be pretty. */
-/* (should be in nlang.c?) */
-
-/* (should synth complete name/adjective from other parts of speech) */
-
-char *
-side_name(side)
-Side *side;
-{
-    return (side == NULL ? "independent" :
-      (side->name ? side->name :
-	    (side->adjective ? side->adjective :
-	     (side->pluralnoun ? side->pluralnoun :
-	      (side->noun ? side->noun :
-	       "")))));
-}
-
-char *
-side_adjective(side)
-Side *side;
-{
-    return (side == NULL ? "independent" :
-       (side->adjective ? side->adjective :
-	    (side->noun ? side->noun :
-	     (side->pluralnoun ? side->pluralnoun :
-	      (side->name ? side->name :
-	       "")))));
-}
-
 /* Given a side, get its "number", which is same as its "id". */
 
 int
@@ -495,7 +556,7 @@ Unit *unit;
     if (side->designer)
       return TRUE;
 #endif /* DESIGNERS */
-    if (g_see_all())
+    if (all_see_all)
       return TRUE;
     if (side_controls_side(side, unit->side))
       return TRUE;
@@ -551,16 +612,7 @@ Unit *prevunit;
     if (side != NULL) {
 	if (prevunit == NULL)
 	  prevunit = side->unithead;
-#ifdef USE_CONSOLE
-	printf("Find_next_unit starting with %s\n", unit_desig(prevunit));
-#endif
 	for (unit = prevunit->next; unit != prevunit; unit = unit->next) {
-#ifdef USE_CONSOLE
-	    printf("Looking at %s", unit_desig(unit));
-	    printf(", prev is %s", unit_desig(unit->prev));
-	    printf(", next is %s", unit_desig(unit->next));
-	    printf("\n");
-#endif
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)) {
@@ -620,7 +672,8 @@ Unit *nextunit;
     Unit *unit = NULL;
 
     if (side != NULL) {
-	if (nextunit == NULL) nextunit = side->unithead;
+	if (nextunit == NULL)
+	  nextunit = side->unithead;
 	for (unit = nextunit->prev; unit != nextunit; unit = unit->prev) {
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
@@ -633,27 +686,26 @@ Unit *nextunit;
     return NULL;
 }
 
-/* (should move these to kernel) */
-
 Unit *
 find_next_mover(side, prevunit)
 Side *side;
 Unit *prevunit;
 {
-	Unit *unit = NULL;
+    Unit *unit = NULL;
 
-	if (side != NULL) {
-		if (prevunit == NULL) prevunit = side->unithead;
-		for (unit = prevunit->next; unit != prevunit; unit = unit->next) {
-			if (is_unit(unit) && unit->id > 0
-				&& alive(unit)
-				&& inside_area(unit->x, unit->y)
-				&& (unit->act && unit->act->acp > 0)) {
-				return unit;
-			}
-		}
+    if (side != NULL) {
+	if (prevunit == NULL)
+	  prevunit = side->unithead;
+	for (unit = prevunit->next; unit != prevunit; unit = unit->next) {
+	    if (is_unit(unit) && unit->id > 0
+		&& alive(unit)
+		&& inside_area(unit->x, unit->y)
+		&& (unit->act && unit->act->acp > 0)) {
+		return unit;
+	    }
 	}
-	return NULL;
+    }
+    return NULL;
 }
 
 Unit *
@@ -664,7 +716,8 @@ Unit *nextunit;
     Unit *unit = NULL;
 
     if (side != NULL) {
-	if (nextunit == NULL) nextunit = side->unithead;
+	if (nextunit == NULL)
+	  nextunit = side->unithead;
 	for (unit = nextunit->prev; unit != nextunit; unit = unit->prev) {
 	    if (is_unit(unit) && unit->id > 0
 		&& alive(unit)
@@ -693,7 +746,10 @@ Unit *prevunit;
 		&& alive(unit)
 		&& inside_area(unit->x, unit->y)
 		&& (unit->act && unit->act->acp > 0)
-		&& (unit->plan && !unit->plan->asleep && !unit->plan->reserve && !unit->plan->delayed)) {
+		&& (unit->plan
+		    && !unit->plan->asleep
+		    && !unit->plan->reserve
+		    && !unit->plan->delayed)) {
 		return unit;
 	    }
 	}
@@ -855,7 +911,8 @@ num_displayed_sides()
     Side *side;
 
     for_all_sides(side) {
-	if (side_has_display(side)) ++n;
+	if (side_has_display(side))
+	  ++n;
     }
     return n;
 }
@@ -878,9 +935,10 @@ Side *side;
 {
     Side *side2;
 
-    if (!side->designer)
-      ++numdesigners;
+    if (side->designer)
+      return;
     side->designer = TRUE;
+    ++numdesigners;
     /* Designers have godlike power in the game, so mark it (permanently)
        as no longer a normal game. */
     compromised = TRUE;
@@ -899,10 +957,10 @@ Side *side;
 {
     Side *side2;
 
-    if (side->designer)
-      --numdesigners;
+    if (!side->designer)
+      return;
     side->designer = FALSE;
-    /* (should just recalc from scratch) */
+    --numdesigners;
     side->may_set_see_all = FALSE;
     side->see_all = all_see_all;
     update_everything();
@@ -1090,7 +1148,7 @@ Unit *unit;
 void
 send_message(side, sidemask, str)
 Side *side;
-long sidemask;
+SideMask sidemask;
 char *str;
 {
     Side *side2;
@@ -1135,7 +1193,7 @@ int *types;
     int x, y;
     Unit *unit;
 
-    if (g_see_all())
+    if (all_see_all)
       return;
     if (!g_terrain_seen()) {
 	for_all_cells(x, y) {
@@ -1191,7 +1249,8 @@ int inopen;
 	    see_cell(side, x, y);
 	} else {
 	    if (always && terrain_view(side, x, y) != UNSEEN) {
-		if (side->coverage) add_cover(side, x, y, 1);
+		add_cover(side, x, y, 1);
+		set_alt_cover(side, x, y, 0);
 	    }
 	    if (inopen || always) {
 		see_cell(side, x, y);
@@ -1248,15 +1307,17 @@ int inopen;
 	} else {
 	    if (always && terrain_view(side, x, y) != UNSEEN) {
 		see_cell(side, x, y);
-		if (side->coverage && cover(side, x, y) > 0)
+		if (cover(side, x, y) > 0)
 		  add_cover(side, x, y, -1);
+		if (side->alt_coverage)
+		  /* (should recalc alt coverage, since was 0) */;
 	    }
 	    /* Won't be called twice, because view coverage is 0 now. */
 	    if (inopen) {
 		see_cell(side, x, y);
 	    }
 	    /* special hack to flush images we *know* are garbage */
-	    if (side->coverage && cover(side, x, y) < 1) {
+	    if (cover(side, x, y) < 1) {
 	    	olduview = unit_view(side, x, y);
 	    	if (vside(olduview) == side_number(side)) {
 	    	    set_unit_view(side, x, y, EMPTY);
@@ -1269,16 +1330,27 @@ int inopen;
 }
 
 static int tmpx0, tmpy0, tmpz0;
+static int tmpnx, tmpny, tmpnz;
 
 static void
 init_visible_elevation(x, y)
 int x, y;
 {
-    int elev = (elevations_defined() ? elev_at(x, y) : 0);
+    int elev = checked_elev_at(x, y);
 
-    set_tmp1_at(x, y, elev - tmpz0);
+    set_tmp2_at(x, y, elev - tmpz0);
 }
 
+static void
+init_visible_elevation_2(x, y)
+int x, y;
+{
+    int elev = checked_elev_at(x, y);
+
+    set_tmp3_at(x, y, elev - tmpnz);
+}
+
+/* (should reindent) */
 static void
 calc_visible_elevation(x, y)
 int x, y;
@@ -1286,110 +1358,280 @@ int x, y;
     int dir, x1, y1, elev, tmp, tmpa, tmpb, adjelev = 9999, viselev, dist, dist1;
     int cellwid = area.cellwidth;
 
-    elev = (elevations_defined() ? elev_at(x, y) : 0);
+    elev = checked_elev_at(x, y);
     dist = distance(x, y, tmpx0, tmpy0);
-    if (cellwid <= 0) cellwid = 1;
+    if (cellwid <= 0)
+      cellwid = 1;
     for_all_directions(dir) {
 	if (point_in_dir(x, y, dir, &x1, &y1)) {
 	  dist1 = distance(x1, y1, tmpx0, tmpy0);
 	  if (dist1 < dist) {
-	    tmpa = tmp1_at(x1, y1);
+	    tmpa = tmp2_at(x1, y1);
 	    /* Account for the screening effect of the elevation difference. */
 	    /* (dist1 will never be zero) */
 	    tmpa = (tmpa * dist * cellwid) / (dist1 * cellwid);
-	    tmpb = (elevations_defined() ? elev_at(x1, y1) : 0)
+	    tmpb = checked_elev_at(x1, y1)
 	           + t_thickness(terrain_at(x1, y1))
 	           - tmpz0;
 	    tmpb = (tmpb * dist * cellwid) / (dist1 * cellwid);
 	    tmp = max(tmpa, tmpb);
-	    adjelev = min(adjelev, tmp);
+	    adjelev = min(adjelev, tmp + tmpz0);
 	  }
 	}
     }
     viselev = max(adjelev, elev);
-    set_tmp1_at(x, y, viselev - tmpz0);
+    set_tmp2_at(x, y, viselev - tmpz0);
 }
 
-/* Unit's beady eyes are now covering (or not, depending on sign of onoff)
-   the immediate area.
+/* (should reindent) */
+static void
+calc_visible_elevation_2(x, y)
+int x, y;
+{
+    int dir, x1, y1, elev, tmp, tmpa, tmpb, adjelev = 9999, viselev, dist, dist1;
+    int cellwid = area.cellwidth;
+
+    elev = checked_elev_at(x, y);
+    dist = distance(x, y, tmpnx, tmpny);
+    if (cellwid <= 0)
+      cellwid = 1;
+    for_all_directions(dir) {
+	if (point_in_dir(x, y, dir, &x1, &y1)) {
+	  dist1 = distance(x1, y1, tmpnx, tmpny);
+	  if (dist1 < dist) {
+	    tmpa = tmp3_at(x1, y1);
+	    /* Account for the screening effect of the elevation difference. */
+	    /* (dist1 will never be zero) */
+	    tmpa = (tmpa * dist * cellwid) / (dist1 * cellwid);
+	    tmpb = checked_elev_at(x1, y1)
+	           + t_thickness(terrain_at(x1, y1))
+	           - tmpnz;
+	    tmpb = (tmpb * dist * cellwid) / (dist1 * cellwid);
+	    tmp = max(tmpa, tmpb);
+	    adjelev = min(adjelev, tmp + tmpnz);
+	  }
+	}
+    }
+    viselev = max(adjelev, elev);
+    set_tmp3_at(x, y, viselev - tmpnz);
+}
+
+/* Unit's beady eyes are shifting from one location to another.
    Since new things may be coming into view, we have to check and maybe
    draw lots of cells (but only need the one output flush, fortunately). */
 
 /* (LOS comes in here, to make irregular coverage areas) */
 
 void
-cover_area(side, unit, x0, y0, onoff)
+cover_area(side, unit, x0, y0, nx, ny)
 Side *side;
 Unit *unit;
-int x0, y0, onoff;
+int x0, y0, nx, ny;
 {
-    int u = unit->type, range, x, y, x1, y1, x2, y2, y1c, y2c, xw, cov, los, r;
+    int u = unit->type, range0, nrange, range, x, y, x1, y1, x2, y2, y1c, y2c, xw, cov, los, r;
+    int xmin, ymin, xmax, ymax, oldcov, newcov, anychanges;
     extern int daynight, sunx, suny;
 
     if (side == NULL
 	|| side->coverage == NULL
     	|| indep(unit)
 	|| all_see_all
-	|| !in_area(unit->x, unit->y)
 	|| !completed(unit)
 	)
       return;
-    range = u_vision_range(u);
+    range0 = nrange = u_vision_range(u);
     /* Adjust for the effects of nighttime on vision range. */
-    if (night_at(x0, y0)) {
-    	range = (range * ut_vision_night_effect(u, terrain_at(x0, y0))) / 100;
+    if (in_area(x0, y0)) {
+	if (night_at(x0, y0)) {
+	    range0 =
+	      (range0 * ut_vision_night_effect(u, terrain_at(x0, y0))) / 100;
+	}
+    } else {
+	range0 = 0;
+    }
+    if (in_area(nx, ny)) {
+	if (night_at(nx, ny)) {
+	    nrange =
+	      (nrange * ut_vision_night_effect(u, terrain_at(nx, ny))) / 100;
+	}
+    } else {
+	nrange = 0;
+    }
+    range = max(range0, nrange);
+    allocate_area_scratch(1);
+    anychanges = FALSE;
+    /* First, set the union of the from and to areas to the existing
+       coverage. */
+    /* These may be outside the area - necessary since units may be able
+       to see farther in x than the height of the area. */
+    /* Compute the maximum bounds that may be affected. */
+    if (y0 >= 0) {
+	if (ny >= 0) {
+	    ymin = min(y0, ny);
+	    ymax = max(y0, ny);
+	} else {
+	    ymin = ymax = y0;
+	}
+    } else if (ny >= 0) {
+	ymin = ymax = ny;
+    }
+    if (x0 >= 0) {
+	if (nx >= 0) {
+	    xmin = min(x0, nx);
+	    xmax = max(x0, nx);
+	} else {
+	    xmin = xmax = x0;
+	}
+    } else if (nx >= 0) {
+	xmin = xmax = nx;
+    }
+    if (any_los) {
+	/* Need extra scratch layers, will be used for visible
+           elevation cache. */
+	allocate_area_scratch(3);
     }
     los = FALSE;
     /* (should also adjust for effect of clouds here) */
     if (u_vision_bend(u) != 100) {
 	los = TRUE;
-	/* Need one scratch layer, will be used for visible elevation cache. */
-	allocate_area_scratch(1);
 	/* Compute the minimum elevation for visibility at each cell. */
-	apply_to_area(x0, y0, range, init_visible_elevation);
-	/* Leave own and adj cells alone, will always be visible. */
-	tmpx0 = x0;  tmpy0 = y0;
-	tmpz0 = (elevations_defined() ? elev_at(x0, y0) : 0)
-	  + unit_alt(unit)
-	    * ut_eye_height(unit->type, terrain_at(x0, y0));
-	for (r = 2; r <= range; ++r) {
-	    apply_to_ring(x0, y0, r, r, calc_visible_elevation);
+	if (in_area(x0, y0)) {
+	    tmpx0 = x0;  tmpy0 = y0;
+	    tmpz0 = checked_elev_at(x0, y0)
+		+ unit_alt(unit)
+		+ ut_eye_height(u, terrain_at(x0, y0));
+	    apply_to_area(x0, y0, range0, init_visible_elevation);
+	    /* Leave own and adj cells alone, they will always be visible. */
+	    for (r = 2; r <= range0; ++r) {
+		apply_to_ring(x0, y0, r, r, calc_visible_elevation);
+	    }
+	    /* We now have a layer indicating how high things must be
+               to be visible. */
+	}
+	if (in_area(nx, ny)) {
+	    tmpnx = nx;  tmpny = ny;
+	    tmpnz = checked_elev_at(nx, ny)
+		+ unit_alt(unit)
+		+ ut_eye_height(u, terrain_at(nx, ny));
+	    apply_to_area(nx, ny, nrange, init_visible_elevation_2);
+	    /* Leave own and adj cells alone, they will always be visible. */
+	    for (r = 2; r <= nrange; ++r) {
+		apply_to_ring(nx, ny, r, r, calc_visible_elevation_2);
+	    }
+	    /* We now have another layer, indicating how high things must be
+               to be visible from the new location. */
 	}
-	/* We now have a layer indicating how high things must be to be visible. */
     }
-    /* These may be outside the area - necessary since units may be able
-       to see farther in x than the height of the area. */
-    y1 = y1c = y0 - range;
-    y2 = y2c = y0 + range;
-    /* Clip the iteration bounds though. */
-    if (y1c < 0) y1c = 0;
-    if (y2c > area.height - 1) y2c = area.height - 1;
+    /* Copy the current coverage into the tmp layer. */
+    y1 = y1c = ymin - range;
+    y2 = y2c = ymax + range;
+    /* Clip the iteration bounds. */
+    if (y1c < 0)
+      y1c = 0;
+    if (y2c > area.height - 1)
+      y2c = area.height - 1;
     for (y = y1c; y <= y2c; ++y) {
-	x1 = x0 - (y < y0 ? (y - y1) : range);
-	x2 = x0 + (y > y0 ? (y2 - y) : range);
+	x1 = xmin - range;
+	x2 = xmax + range;
 	for (x = x1; x <= x2; ++x) {
 	    if (in_area(x, y)) {
-		if (!los
-		    || (tmp1_at(x, y)
-			<= ((elevations_defined()
-			     ? elev_at(x, y) : 0)
-			    + t_thickness(terrain_at(x, y))))) {
+		xw = wrapx(x);
+		set_tmp1_at(xw, y, cover(side, xw, y));
+	    }
+	}
+    }
+    /* Decrement coverage around the old location. */
+    if (in_area(x0, y0)) {
+	y1 = y1c = y0 - range0;
+	y2 = y2c = y0 + range0;
+	/* Clip the iteration bounds. */
+	if (y1c < 0)
+	  y1c = 0;
+	if (y2c > area.height - 1)
+	  y2c = area.height - 1;
+	for (y = y1c; y <= y2c; ++y) {
+	    x1 = x0 - (y < y0 ? (y - y1) : range0);
+	    x2 = x0 + (y > y0 ? (y2 - y) : range0);
+	    for (x = x1; x <= x2; ++x) {
+		if (in_area(x, y)) {
 		    xw = wrapx(x);
-		    cov = onoff + cover(side, xw, y);
-		    if (cov < 0) {
-			Dprintf("Negative coverage for %s at %d,%d\n",
-				side_desig(side), xw, y);
+		    if (!los
+			|| ((tmp2_at(xw, y) + tmpz0)
+			    <= (checked_elev_at(xw, y)
+				+ t_thickness(terrain_at(xw, y))))) {
+			cov = tmp1_at(xw, y) - 1;
+			/* Should never go negative, detect if so. */
+			if (cov < 0) {
+			    Dprintf("Negative coverage for %s at %d,%d\n",
+				    side_desig(side), xw, y);
+			}
+			set_tmp1_at(xw, y, cov);
 		    }
-		    set_cover(side, xw, y, cov);
-		    /* View the cell and maybe get excited. */
-		    if (onoff > 0 && see_cell(side, xw, y))
+		    if (los && (alt_cover(side, xw, y) == (tmp2_at(xw, y) + tmpz0)))
+		      /* this unit set the min, should recalc alt coverage now */;
+		}
+	    }
+	}
+    }
+    /* Increment coverage around the new location. */
+    if (in_area(nx, ny)) {
+	y1 = y1c = ny - nrange;
+	y2 = y2c = ny + nrange;
+	/* Clip the iteration bounds. */
+	if (y1c < 0)
+	  y1c = 0;
+	if (y2c > area.height - 1)
+	  y2c = area.height - 1;
+	for (y = y1c; y <= y2c; ++y) {
+	    x1 = nx - (y < ny ? (y - y1) : nrange);
+	    x2 = nx + (y > ny ? (y2 - y) : nrange);
+	    for (x = x1; x <= x2; ++x) {
+		if (in_area(x, y)) {
+		    xw = wrapx(x);
+		    if (!los
+			|| ((tmp3_at(xw, y) + tmpnz)
+			    <= (checked_elev_at(xw, y)
+				+ t_thickness(terrain_at(xw, y))))) {
+			cov = tmp1_at(xw, y) + 1;
+			set_tmp1_at(xw, y, cov);
+		    }
+		    if (los)
+		      set_alt_cover(side, xw, y, min(alt_cover(side, xw, y), (tmp3_at(xw, y) + tmpnz)));
+		}
+	    }
+	}
+    }
+    /* Now update the actual coverage.  Do this over an area that includes
+       both the decrement and increment changes. */
+    y1 = y1c = ymin - range;
+    y2 = y2c = ymax + range;
+    /* Clip the iteration bounds. */
+    if (y1c < 0)
+      y1c = 0;
+    if (y2c > area.height - 1)
+      y2c = area.height - 1;
+    for (y = y1c; y <= y2c; ++y) {
+	x1 = xmin - range;
+	x2 = xmax + range;
+	for (x = x1; x <= x2; ++x) {
+	    if (in_area(x, y)) {
+		xw = wrapx(x);
+		oldcov = cover(side, xw, y);
+		newcov = tmp1_at(xw, y);
+		if (newcov != oldcov) {
+		    set_cover(side, xw, y, newcov);
+		    if (newcov > oldcov && see_cell(side, xw, y))
 		      react_to_seen_unit(side, unit, xw, y);
+		    if ((newcov > 0 && oldcov == 0)
+			|| (newcov == 0 && oldcov > 0))
+		      update_cell_display(side, xw, y, 36);
+		    anychanges = TRUE;
 		}
 	    }
 	}
     }
     /* If we're seeing new things, make sure they're on the display. */
-    if (onoff > 0)
+    if (anychanges)
       flush_display_buffers(side);
 }
 
@@ -1400,7 +1642,7 @@ reset_coverage()
 {
     Side *side;
 
-    if (g_see_all())
+    if (all_see_all)
       return;
     for_all_sides(side)
       calc_coverage(side);
@@ -1432,10 +1674,15 @@ Side *side;
 	    set_cover(side, x, y, 0);
 	}
     }
-    /* Add coverage by the units already in place. */
+    /* Add coverage by, and of, the units already in place. */
     for_all_units(unit) {
-	if (in_play(unit) && trusted_side(unit->side, side)) {
-	    cover_area(side, unit, unit->x, unit->y, 1);
+	if (in_play(unit)) {
+	    if (unit->side != NULL && trusted_side(unit->side, side)) {
+		cover_area(side, unit, -1, -1, unit->x, unit->y);
+	    } else if (u_see_always(unit->type)
+		       && terrain_view(side, unit->x, unit->y) != UNSEEN) {
+		add_cover(side, unit->x, unit->y, 1);
+	    }
 	}
     }
 }
@@ -1445,7 +1692,8 @@ reset_all_views()
 {
     Side *side;
 
-    if (g_see_all()) return;
+    if (all_see_all)
+      return;
     for_all_sides(side) {
 	reset_view(side);
     }
@@ -1476,7 +1724,7 @@ int x, y;
     Unit *eunit;
     Side *es;
 
-    if (g_see_all() /* see real unit */) {
+    if (all_see_all /* see real unit */) {
     	/* (should look at all of stack if can be mixed) */
 	if ((eunit = unit_at(x, y)) != NULL) {
 	    if (react_to_enemies(unit) && !allied_side(eunit->side, side)) {
@@ -1585,12 +1833,14 @@ int x, y;
 /* Look at the given position, possibly not seeing anything.  Return true if
    a unit was spotted. */
 
+static int see_weather PARAMS ((Side *side, int x, int y));
+
 int
 see_cell(side, x, y)
 Side *side;
 int x, y;
 {
-    int update, chance, t, curtview, curuview, newuview, x1, y1, m;
+    int update, chance, t, curview, curuview, newuview, x1, y1, m, wupdate;
     Unit *unit, *unitseen;
 
     if (side == NULL || side == indepside || !in_area(x, y))
@@ -1603,11 +1853,18 @@ int x, y;
     	unitseen = unit_at(x, y);
     } else if (cover(side, x, y) > 0) {
     	/* Always update our knowledge of the terrain. */
-    	curtview = terrain_view(side, x, y);
- 	if (curtview == UNSEEN) {
+    	curview = terrain_view(side, x, y);
+ 	if (curview == UNSEEN || !g_see_terrain_always()) {
 	    set_terrain_view(side, x, y, buildtview(terrain_at(x, y)));
+	    if (!g_see_terrain_always())
+	      set_terrain_view_date(side, x, y, g_turn());
 	    update = TRUE;
 	}
+	if (temperatures_defined() || clouds_defined() || winds_defined()) {
+	    wupdate = see_weather(side, x, y);
+	    if (wupdate)
+	      update = TRUE;
+	}
 	if (any_see_chances < 0)
 	  compute_see_chances();
    	curuview = unit_view(side, x, y);
@@ -1689,12 +1946,12 @@ see_exact(side, x, y)
 Side *side;
 int x, y;
 {
-    register int olduview, oldtview, newtview, newuview, update;
+    register int olduview, oldtview, newtview, newuview, update, wupdate;
     register Unit *unit;
 
     if (side == NULL || side == indepside || !in_area(x, y))
       return;
-    if (g_see_all()) {
+    if (all_see_all) {
 	/* It may not really be necessary to do anything to the display, but
 	   the kernel doesn't know if the interface is drawing all the units
 	   that are visible, or is only drawing "interesting" ones, or whatever.
@@ -1704,9 +1961,16 @@ int x, y;
 	   what's already been drawn on the screen. */
     	update = TRUE;
     } else {
+	update = FALSE;
     	oldtview = terrain_view(side, x, y);
     	newtview = buildtview(terrain_at(x, y));
     	set_terrain_view(side, x, y, newtview);
+	set_terrain_view_date(side, x, y, g_turn());
+	if (oldtview != newtview)
+	  update = TRUE;
+	wupdate = see_weather(side, x, y);
+	if (wupdate)
+	  update = TRUE;
     	olduview = unit_view(side, x, y);
     	newuview = EMPTY;
     	unit = unit_at(x, y);
@@ -1714,7 +1978,8 @@ int x, y;
 	  newuview = builduview(side_number(unit->side), unit->type);
 	set_unit_view(side, x, y, newuview);
 	set_unit_view_date(side, x, y, g_turn());
-	update = (oldtview != newtview || olduview != newuview);
+	if (olduview != newuview)
+	  update = TRUE;
     }
     /* If there was any change in what was visible, tell the display. */
     if (update) {
@@ -1722,6 +1987,62 @@ int x, y;
     }
 }
 
+static int
+see_weather(side, x, y)
+Side *side;
+int x, y;
+{
+    int curview, update;
+
+    update = FALSE;
+    if (temperatures_defined()) {
+	see_weather(side, x, y);
+	curview = temperature_view(side, x, y);
+	if (curview != temperature_at(x, y)) {
+	    set_temperature_view(side, x, y, temperature_at(x, y));
+	    update = TRUE;
+	}
+	/* Even if the data didn't change, our information is
+	   now up-to-date. */
+	if (!g_see_weather_always())
+	  set_temperature_view_date(side, x, y, g_turn());
+    }
+    if (clouds_defined()) {
+	curview = cloud_view(side, x, y);
+	if (curview != raw_cloud_at(x, y)) {
+	    set_cloud_view(side, x, y, raw_cloud_at(x, y));
+	    update = TRUE;
+	}
+	curview = cloud_bottom_view(side, x, y);
+	if (curview != raw_cloud_bottom_at(x, y)) {
+	    set_cloud_bottom_view(side, x, y, raw_cloud_bottom_at(x, y));
+	    update = TRUE;
+	}
+	curview = cloud_height_view(side, x, y);
+	if (curview != raw_cloud_height_at(x, y)) {
+	    set_cloud_height_view(side, x, y, raw_cloud_height_at(x, y));
+	    update = TRUE;
+	}
+	/* Even if the data didn't change, our information is
+	   now up-to-date. */
+	if (!g_see_weather_always())
+	  set_cloud_view_date(side, x, y, g_turn());
+	/* Only need one date for the three layers of view data. */
+    }
+    if (winds_defined()) {
+	curview = wind_view(side, x, y);
+	if (curview != raw_wind_at(x, y)) {
+	    set_wind_view(side, x, y, raw_wind_at(x, y));
+	    update = TRUE;
+	}
+	/* Even if the data didn't change, our information is
+	   now up-to-date. */
+	if (!g_see_weather_always())
+	  set_wind_view_date(side, x, y, g_turn());
+    }
+    return update;
+}
+
 /* A border has been seen if the cell on either side has been seen. */
 
 int
@@ -1806,6 +2127,9 @@ Player *player;
       player->displayname = NULL;
     if (empty_string(player->aitypename))
       player->aitypename = NULL;
+    /* This seems like a logical place to canonicalize an AI type
+       of "ai" into a specific type, but we don't necessarily know
+       the best default until the game is closer to starting. */
 }
 
 /* Make a printable identification of the given player. */
@@ -1890,10 +2214,201 @@ Doctrine *olddoc;
     tmpnext = newdoc->next;
     memcpy(newdoc, olddoc, sizeof(Doctrine));
     newdoc->id = tmpid;
+    if (newdoc->construction_run) {
+	newdoc->construction_run =
+	  (short *) xmalloc (numutypes * sizeof(short));
+	memcpy(newdoc->construction_run, olddoc->construction_run, numutypes * sizeof(short));
+    }
     newdoc->next = tmpnext;
     return newdoc;
 }
 
+/* Standing order handling. */
+
+StandingOrder *
+new_standing_order()
+{
+    StandingOrder *sorder;
+
+    sorder = (StandingOrder *) xmalloc(sizeof(StandingOrder));
+    sorder->types = xmalloc(numutypes);
+    return sorder;
+}
+
+int order_conds_match PARAMS ((StandingOrder *sorder, StandingOrder *sorder2));
+
+void
+add_standing_order(side, sorder, pos)
+Side *side;
+StandingOrder *sorder;
+int pos;
+{
+    StandingOrder *sorder2;
+
+    if (sorder->task == NULL) {
+	if (side->orders == NULL) {
+	    /* No orders, so nothing to do. */
+	    return;
+	} else if (order_conds_match(sorder, side->orders)) {
+	    /* Delete the first order in the list. */
+	    if (side->last_order == side->orders)
+	      side->last_order = side->orders->next;
+	    side->orders = side->orders->next;
+	} else {
+	    for (sorder2 = side->orders; sorder2->next != NULL; sorder2 = sorder2->next) {
+		if (order_conds_match(sorder, sorder2->next)) {
+		    if (side->last_order == sorder2->next)
+		      side->last_order = sorder2->next->next;
+		    sorder2->next = sorder2->next->next;
+		    return;
+		}
+	    }
+	    /* If we're here, no match; might be user error, but can't be sure,
+	       so don't say anything. */
+	}
+    } else if (pos == 0) {
+	sorder->next = side->orders;
+	side->orders = sorder;
+	if (side->last_order == NULL)
+	  side->last_order = sorder;
+    } else if (side->last_order != NULL) {
+	side->last_order->next = sorder;
+	side->last_order = sorder;
+    } else {
+	side->orders = side->last_order = sorder;
+    }
+}
+
+int
+order_conds_match(sorder, sorder2)
+StandingOrder *sorder, *sorder2;
+{
+    return (sorder->etype == sorder2->etype
+	    && sorder->a1 == sorder2->a1
+	    && sorder->a2 == sorder2->a2
+	    && sorder->a3 == sorder2->a3);
+}
+
+int
+parse_standing_order(side, cmdstr)
+Side *side;
+char *cmdstr;
+{
+    StandingOrder *sorder;
+
+    sorder = new_standing_order();
+    cmdstr = parse_unit_types(side, cmdstr, sorder->types);
+    if (cmdstr == NULL)
+      return (-1);
+    cmdstr = parse_order_cond(side, cmdstr, sorder);
+    if (cmdstr == NULL)
+      return (-2);
+    cmdstr = parse_task(side, cmdstr, &(sorder->task));
+    if (cmdstr == NULL)
+      return (-3);
+    add_standing_order(side, sorder, 0);
+    return 0;
+}
+
+char *
+parse_unit_types(side, str, utypevec)
+Side *side;
+char *str, *utypevec;
+{
+    char *arg, substr[BUFSIZE], *rest;
+    int u;
+
+    rest = get_next_arg(str, substr, &arg);
+    u = utype_from_name(arg);
+    if (u != NONUTYPE) {
+	utypevec[u] = 1;
+    } else if (strcmp(arg, "all") == 0) {
+	for_all_unit_types(u)
+	  utypevec[u] = 1;
+    } else {
+	notify(side, "Unit type \"%s\" not recognized", arg);
+	return NULL;
+    }
+    return rest;
+}
+
+char *
+parse_order_cond(side, str, sorder)
+Side *side;
+char *str;
+StandingOrder *sorder;
+{
+    int x = 0, y = 0, dist = 0;
+    char *arg, *arg2, substr[BUFSIZE], *rest;
+    Unit *unit;
+
+    rest = get_next_arg(str, substr, &arg);
+    if (strcmp(arg, "at") == 0 || strcmp(arg, "@") == 0) {
+	sorder->etype = 1;
+    } else if (strcmp(arg, "in") == 0) {
+	sorder->etype = 2;
+    } else if (strcmp(arg, "within") == 0 || strcmp(arg, "near") == 0) {
+	sorder->etype = 3;
+    } else {
+	notify(side, "Condition type \"%s\" not recognized", arg);
+	return NULL;
+    }
+    if (sorder->etype == 3) {
+	rest = get_next_arg(rest, substr, &arg);
+	dist = strtol(arg, &arg2, 10);
+	sorder->a3 = dist;
+    }
+    rest = get_next_arg(rest, substr, &arg);
+    x = strtol(arg, &arg2, 10);
+    if (arg != arg2 && *arg2 == ',') {
+	y = strtol(arg2 + 1, &arg, 10);
+	sorder->a1 = x;  sorder->a2 = y;
+	return rest;
+    } else if ((unit = find_unit_by_name(arg)) != NULL) {
+	if (sorder->etype == 1 || sorder->etype == 3) {
+	    sorder->a1 = x;  sorder->a2 = y;
+	} else if (sorder->etype == 2) {
+	    sorder->a1 = unit->id;
+	} else {
+	    return NULL;
+	}
+	return rest;
+    } else {
+	notify(side, "Condition argument \"%s\" not recognized", arg);
+	return NULL;
+    }
+}
+
+/* Collect the next whitespace-separated argument. */
+
+char *
+get_next_arg(str, buf, rsltp)
+char *str, *buf, **rsltp;
+{
+    char *p;
+
+    strcpy(buf, str);
+    p = buf;
+    /* Skip past any leading whitespace. */
+    while (isspace(*p))
+      ++p;
+    if (*p == '"') {
+	++p;
+	*rsltp = p;
+	while (*p != '"' && *p != '\0')
+	  ++p;
+	*p = '\0';
+	/* Increment so later scanning looks past the closing quote. */ 
+	++p;
+    } else {
+	*rsltp = p;
+	while (!isspace(*p) && *p != '\0')
+	  ++p;
+	*p = '\0';
+    }
+    return str + (p - buf);
+}
+
 /* Agreement handling here for now. */
 
 Agreement *agreementlist = NULL;
@@ -1902,6 +2417,8 @@ Agreement *lastagreement;
 
 int numagreements = 0;
 
+char *agreementdesigbuf;
+
 void
 init_agreements()
 {
@@ -1927,12 +2444,12 @@ int id;
     return ag;
 }
 
-char agreementdesigbuf[BUFSIZE];
-
 char *
 agreement_desig(ag)
 Agreement *ag;
 {
+    if (agreementdesigbuf == NULL)
+      agreementdesigbuf = xmalloc(BUFSIZE);
     sprintf(agreementdesigbuf, "<ag %s %s %s>",
 	    (ag->typename ? ag->typename : "(null)"),
 	    (ag->name ? ag->name : "(null)"),
@@ -1942,8 +2459,23 @@ Agreement *ag;
 
 #ifdef DESIGNERS
 
-static int tmpint;
-static int tmpint2;
+static void paint_view_1 PARAMS ((int x, int y));
+
+static int tmptview;
+static int tmpuview;
+
+/* Paint the side's view with given values. */
+
+void
+paint_view(side, x, y, r, tview, uview)
+Side *side;
+int x, y, r, tview, uview;
+{
+    tmpside = side;
+    tmptview = tview;
+    tmpuview = uview;
+    apply_to_area_plus_edge(x, y, r, paint_view_1);
+}
 
 static void
 paint_view_1(x, y)
@@ -1952,22 +2484,11 @@ int x, y;
     int oldtview = terrain_view(tmpside, x, y);
     int olduview = unit_view(tmpside, x, y);
 
-    if (oldtview != tmpint || olduview != tmpint2) {
-	set_terrain_view(tmpside, x, y, tmpint);
-	set_unit_view(tmpside, x, y, tmpint2);
+    if (oldtview != tmptview || olduview != tmpuview) {
+	set_terrain_view(tmpside, x, y, tmptview);
+	set_unit_view(tmpside, x, y, tmpuview);
 	see_exact(tmpside, x, y);
     }
 }
 
-void
-paint_view(side, x, y, r, tview, uview)
-Side *side;
-int x, y, r, tview, uview;
-{
-    tmpside = side;
-    tmpint = tview;
-    tmpint2 = uview;
-    apply_to_area_plus_edge(x, y, r, paint_view_1);
-}
-
 #endif /* DESIGNERS */
