@@ -15,11 +15,10 @@ any later version.  See the file COPYING.  */
    since it has simple or default implementations of the routines that any
    interface has to support. */
 
-#ifndef USE_CONSOLE
-#define USE_CONSOLE
-#endif
-
 #include "conq.h"
+extern void low_send PARAMS ((int id, char *buf));
+extern int low_receive PARAMS ((int *id, char *buf, int maxchars,
+				int timeout));
 #include "print.h"
 #include "cmdline.h"
 
@@ -69,7 +68,7 @@ static void describe_commands PARAMS ((int, char *, char *));
 #endif /* THINK_C */
 
 /* This structure maintains state that is local to a side's display.
-   At the very least, it should track when the display is open and closed. */
+   At the very least, it must track when the display is open and closed. */
 
 typedef struct a_ui {
     int active;
@@ -144,7 +143,7 @@ char *argv[];
     while (1) {
 	if (freerunturns > 0) {
 	    if (probability(10)) {
-	    	printf("No apparent progress, forcing the turn to finish.\n");
+	    	Dprintf("No apparent progress, forcing the turn to finish.\n");
 	    	do_finish_all(NULL, lispnil, lispnil);
 	    }
 	    if (endofgame)
@@ -152,6 +151,7 @@ char *argv[];
 	} else {
 	    get_input();
 	}
+	/* We stay in here until nothing more to do. */
 	while (run_game(99) > 0)
 	  ;
     }
@@ -278,7 +278,7 @@ Obj *origcmd;
 	interpret_help(side, str+1);
     } else {
 	printf("Command ");
-	printlisp(origcmd);
+	fprintlisp(stdout, origcmd);
 	printf(" not understood, ignoring it\n");
     }
 }
@@ -985,8 +985,8 @@ char *topic;
 	return;
     }
     if (curhelpnode == NULL) {
-	add_help_node("commands", describe_commands, 0, firsthelpnode);
-	curhelpnode = firsthelpnode;
+	add_help_node("commands", describe_commands, 0, first_help_node);
+	curhelpnode = first_help_node;
     }
     switch (topic[0]) {
       case 'n':
@@ -996,14 +996,14 @@ char *topic;
 	curhelpnode = curhelpnode->prev;
 	break;
       case 'a':
-	for (node = firsthelpnode->next; node != firsthelpnode; node = node->next) {
+	for (node = first_help_node->next; node != first_help_node; node = node->next) {
 	    show_help(side, node);
 	}
 	return; /* don't show cur help node too */
       case '\0':
 	/* Note that no topic char equals '\0', so will come here. */
       default:
-	curhelpnode = firsthelpnode;
+	curhelpnode = first_help_node;
 	break;
     }
     show_help(side, curhelpnode);
@@ -1154,8 +1154,36 @@ char *str;
 }
 
 void
-printlisp(obj)
-Obj *obj;
+print_form(form)
+Obj *form;
+{
+    print_form_and_value(stdout, form);
+}
+
+void
+end_printing_forms()
+{
+}
+
+void
+low_send(id, buf)
+int id;
+char *buf;
+{
+}
+
+int
+low_receive(id, buf, maxchars, timeout)
+int *id, maxchars, timeout;
+char *buf;
+{
+  return 0;
+}
+
+#include "imf.h"
+
+void
+make_generic_image_data(imf)
+ImageFamily *imf;
 {
-    fprintlisp(stdout, obj);
 }
