@@ -1,5 +1,5 @@
 /* Unit task execution and general task functions.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,9 +17,6 @@ any later version.  See the file COPYING.  */
 
 #define CLEAR_AGENDA 99
 
-#define can_see_actual_units(side, x, y)  \
-  (g_see_all() || cover((side), (x), (y)) > 0)
-
 static int compare_directions PARAMS ((const void *a0, const void *a1));
 static int test_for_buildable PARAMS ((int x, int y));
 
@@ -104,6 +101,22 @@ TaskType type;
     return task;
 }
 
+Task *
+clone_task(oldtask)
+Task *oldtask;
+{
+    int i;
+    Task *newtask;
+
+    newtask = create_task(oldtask->type);
+    /* Probably not a good idea to copy exec/retry counts, skip them. */
+    for (i = 0; i < MAXTASKARGS; ++i)
+      newtask->args[i] = oldtask->args[i];
+    return newtask;
+}
+
+/* The empty task always succeeds immediately. */
+
 static TaskOutcome
 do_none_task(unit, task)
 Unit *unit;
@@ -138,10 +151,10 @@ do_build_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int u = unit->type, dir, nx, ny, tp, range;
+    int u = unit->type, dir, nx, ny, tp;
     int x = unit->x, y = unit->y;
     int u2 = task->args[0], run = task->args[3];
-    Unit *unit2 = NULL, *occ;
+    Unit *unit2 = NULL;
     Side *us = unit->side;
 
     /* First see if we've already built all the units requested. */
@@ -192,49 +205,13 @@ Task *task;
 		if (task->args[2] >= run) {
 		    return TASK_IS_COMPLETE;
 		}
-		unit2 = NULL;
-	    } else {
-		/* press on */
-	    }
-	} else {
-	    unit2 = NULL;
-	}
-    }
-    /* Maybe search for any appropriate incomplete occupants. */
-    if (unit2 == NULL) {
-	for_all_occupants(unit, occ) {
-	    if (in_play(occ)
-		&& !fullsized(occ)
-		&& occ->type == u2
-		&& occ->side == unit->side) {
-		unit2 = occ;
-		break;
-	    }
-	}
-    }
-    /* Or else search for any appropriate incomplete units in this cell. */
-    if (unit2 == NULL) {
-    	for_all_stack(x, y, occ) {
-	    if (in_play(occ)
-		&& !fullsized(occ)
-		&& occ->type == u2
-		&& occ->side == unit->side) {
-		unit2 = occ;
-		break;
-	    }
-    	}
-    }
-    /* Or else search nearby area. */
-    if (is_unit_type(u2) && unit2 == NULL) {
-	range = uu_build_range(u, u2);
-	if (range > 0) {
-	    tmpbuilder = unit;
-	    tmpbuildutype = u2;
-	    if (search_around(x, y, range, test_for_buildable, &nx, &ny, 1)) {
-	    	unit2 = tmpbuildunit;
 	    }
 	}
     }
+
+    /* Find something to work on */
+    unit2 = find_unit_to_complete(unit, task);
+
     /* No incomplete unit found, so try to create one. */
     if (unit2 == NULL) {
 	if (valid(check_create_in_action(unit, unit, u2, unit))) {
@@ -268,6 +245,54 @@ Task *task;
     }
 }
 
+Unit *
+find_unit_to_complete(unit, task)
+Unit *unit;
+Task *task;
+{
+    Unit *occ;
+    int u = unit->type, nx, ny, range;
+    int u2 = task->args[0];
+    int x = unit->x, y = unit->y;
+
+    /* Check out the unit supposedly in progress. */
+    if (task->args[1] != 0) {
+	occ = find_unit(task->args[1]);
+	if (in_play(occ) && occ->type == u2 && !fullsized(occ)) 
+	  return occ;
+    }
+    /* Maybe search for any appropriate incomplete occupants. */
+    for_all_occupants(unit, occ) {
+	if (in_play(occ)
+	    && !fullsized(occ)
+	    && occ->type == u2
+	    && occ->side == unit->side) 
+	  return(occ);
+    }
+    /* Or else search for any appropriate incomplete units in this cell. */
+    for_all_stack(x, y, occ) {
+	if (in_play(occ)
+	    && !fullsized(occ)
+	    && occ->type == u2
+	    && occ->side == unit->side) {
+	    return (occ);
+    	}
+    }
+    /* Or else search nearby area. */
+    if (is_unit_type(u2)) {
+	range = uu_build_range(u, u2);
+	if (range > 0) {
+	    tmpbuilder = unit;
+	    tmpbuildutype = u2;
+	    if (search_around(x, y, range, test_for_buildable, &nx, &ny, 1)) {
+	    	return(tmpbuildunit);
+	    }
+	}
+    }
+    /* nothing found */
+    return NULL;
+}
+
 /* This is a "pure research" task, with the sole objective of increasing
    technology. */
 
@@ -298,26 +323,30 @@ Task *task;
     }
 }
 
+/* This is to capture a given type/side of unit at a given place. */
+
 static TaskOutcome
-do_capture_unit_task(unit, task)
+do_capture_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int u = unit->type, tx, ty, dist, movedist;
+    int u = unit->type, tx, ty, tu2, ts2, dist, movedist, rslt;
     Unit *unit2;
     Side *us = unit->side;
 
-    /* This is to capture a given type/side of unit at a given place. */
-    /* (need to be able to say how hard to try) */
+    /* (should be able to say how hard to try) */
     tx = task->args[0];  ty = task->args[1];
+    tu2 = task->args[2];
+    ts2 = task->args[3];
     dist = distance(tx, ty, unit->x, unit->y);
     switch (dist) {
       case 0:
-	/* huh? */
-	return TASK_FAILED;
       case 1:
-	if ((unit2 = unit_at(tx, ty)) != NULL) {
-	    if (unit2->side != us /* should be "not a friendly side" */) {
+	for_all_stack(tx, ty, unit2) {
+	    if ((ts2 >= 0 ?
+		 (side_number(unit2->side) == ts2) :
+		 (unit2->side != us /* should be "not a friendly side" */))
+		&& (tu2 == NONUTYPE || tu2 == unit->type)) {
 		if (valid(check_capture_action(unit, unit, unit2))) {
 		    prep_capture_action(unit, unit, unit2);
 		    return TASK_PREPPED_ACTION;
@@ -329,34 +358,31 @@ Task *task;
 		    unit->plan->reserve = TRUE;
 		    return (task->execnum < 5 ? TASK_IS_INCOMPLETE : TASK_FAILED);
 		}
-	    } else {
-		/* Our victim apparently saw the light already. */
-		return TASK_IS_COMPLETE;
 	    }
-	} else {
-	    /* Odd, nothing was here to capture. */
-	    return TASK_FAILED;
 	}
+	/* Nothing was here to capture. */
+	return TASK_IS_COMPLETE;
       default:
-	    /* If on mobile transport, let it handle things. */
-	    if (unit->transport != NULL
-	        && mobile(unit->transport->type)
-		/* and the transport is not blocked */
-	        && flip_coin()) {
-	        return TASK_IS_INCOMPLETE;
+	/* If on mobile transport, let it handle things. */
+	if (unit->transport != NULL
+	    && mobile(unit->transport->type)
+	    /* and the transport is not blocked */
+	    && flip_coin()) {
+	    return TASK_IS_INCOMPLETE;
+	}
+	/* If out of range and can move, push a task to get closer (maybe). */
+	if (mobile(u) && flip_coin()) {
+	    movedist = max(1 /* attack range */, u_range(u));
+	    if (dist > movedist + u_acp(u) /* or dist that could be covered in 1-2 turns */) {
+		movedist = dist - max(1, (dist - movedist) / 4);
+		/* We're too far away to capture directly, add a move-to
+		   task. */
+		push_move_near_task(unit, tx, ty, movedist);
+		return TASK_IS_INCOMPLETE;
 	    }
-	    /* If out of range and can move, push a task to get closer (maybe). */
-	    if (mobile(u) && flip_coin()) {
-		movedist = max(1 /* attack range */, u_range(u));
-		if (dist > movedist + u_acp(u) /* or dist that could be covered in 1-2 turns */) {
-		    movedist = dist - max(1, (dist - movedist) / 4);
-			/* We're too far away to capture directly, add a moveto task. */
-			push_movenear_task(unit, tx, ty, movedist);
-			return TASK_IS_INCOMPLETE;
-		}
-	    }
-	    return TASK_FAILED;
-    } 
+	}
+	return TASK_FAILED;
+    }
 }
 
 /* The disband task's purpose is to make the unit disappear, so just keep doing
@@ -416,8 +442,8 @@ Task *task;
 	prep_fire_into_action(unit, unit, tx, ty, 0, -1);
 	return TASK_PREPPED_ACTION;
     } else if (mobile(u) && flip_coin()) {
-	/* We're too far away to shoot directly, add a moveto task. */
-	push_movenear_task(unit, tx, ty, max(1 /* attack range */, u_range(u)));
+	/* We're too far away to shoot directly, add a move-to task. */
+	push_move_near_task(unit, tx, ty, max(1 /* attack range */, u_range(u)));
 	return TASK_IS_INCOMPLETE;
     }
     return TASK_FAILED;
@@ -457,7 +483,7 @@ Task *task;
 	return TASK_FAILED;
     }
     if (dist < u_range(u)) {
-	if (can_see_actual_units(us, tx, ty)) {
+	if (units_visible(us, tx, ty)) {
 	  for_all_stack(tx, ty, unit2) {
 	    if (!trusted_side(us, unit2->side)
 	    	&& (tu == NONUTYPE || unit2->type == tu)
@@ -473,7 +499,7 @@ Task *task;
 	}
 	return TASK_FAILED;	    
     }
-    if (can_see_actual_units(us, tx, ty)) {
+    if (units_visible(us, tx, ty)) {
 	enemythere = FALSE;
 	for_all_stack(tx, ty, unit2) {
 	    if (!trusted_side(us, unit2->side)
@@ -517,7 +543,7 @@ Task *task;
 	if (dist > movedist + u_acp(u) /* or dist that could be covered in 1-2 turns */) {
 	    movedist = dist - max(1, (dist - movedist) / 4);
 	}
-	push_movenear_task(unit, tx, ty, movedist);
+	push_move_near_task(unit, tx, ty, movedist);
 	return TASK_IS_INCOMPLETE;
     }
     return TASK_FAILED;
@@ -542,7 +568,7 @@ Unit *unit, *unit2;
 }
 
 static TaskOutcome
-do_movedir_task(unit, task)
+do_move_dir_task(unit, task)
 Unit *unit;
 Task *task;
 {
@@ -577,18 +603,19 @@ enum choicestate {
     rightonly
 };
 
+static TaskOutcome do_approach_subtask PARAMS ((Unit *unit, Task *task, int tx, int ty, short *statep));
+
 /* The move-to task is the main way for units to get from point A to point B.
    In addition to the destination, the task has a required distance, so it will
    succeed if the unit is within that distance to the nominal destination. */
 
 static TaskOutcome
-do_moveto_task(unit, task)
+do_move_to_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int dist, tx, ty, nx, ny;
-    int dirs[NUMDIRS], numdirs, i, numdirs2;
-    Unit *unit2;
+    int dist, tx, ty;
+    Unit *unit2, *occ;
 
     /* This task is to get to a designated location somehow. */
     tx = task->args[0];  ty = task->args[1];
@@ -602,7 +629,6 @@ Task *task;
 	return TASK_IS_COMPLETE;
       case 1:
 	/* Adjacent cell, do a single move. */
-	/* (similar to multi-move case, merge code?) */
 	if (unit_at(tx, ty)) {
 	    for_all_stack(tx, ty, unit2) {
 		if (can_occupy(unit, unit2)) {
@@ -621,6 +647,16 @@ Task *task;
 		    } else {
 			continue;
 		    }
+		} else {
+		    /* We could find room in an occupant... */
+		    for_all_occupants(unit2, occ) {
+			if (can_occupy(unit, occ)) {
+			    if (valid(check_enter_action(unit, unit, occ))) {
+				prep_enter_action(unit, unit, occ);
+				return TASK_PREPPED_ACTION;
+			    }
+			}
+		    }
 		}
 	    }
 	    return TASK_FAILED;
@@ -635,154 +671,180 @@ Task *task;
 	break;
       default:
 	/* Still some distance away, pick a way to go. */
-	    /* If on mobile transport, let it handle things. */
-	    if (unit->transport != NULL
-	        && mobile(unit->transport->type)
-	        /* and the transport is not blocked */
-	        && flip_coin()) {
-	        unit->plan->reserve = TRUE;
-	        return TASK_IS_INCOMPLETE;
-	    }
-	numdirs = choose_move_dirs(unit, tx, ty, TRUE,
-				   plausible_move_dir, sort_directions, dirs);
-	for (i = 0; i < numdirs; ++i) {
-	    point_in_dir(unit->x, unit->y, dirs[i], &nx, &ny);
-	    for_all_stack(nx, ny, unit2) {
-		if (can_occupy(unit, unit2)) {
-		    if (valid(check_enter_action(unit, unit, unit2))) {
-			prep_enter_action(unit, unit, unit2);
-			/* We (probably) made forward progress, so reopen choice of dirs. */
-			task->args[3] = eitherway;
-			return TASK_PREPPED_ACTION;
-		    } else {
-			continue;
-		    }
-		} else if (!trusted_side(unit->side, unit2->side)) {
-		    if (unit->occupant) {
-		    	/* More important to find a way through. */
-		    	continue;
-		    } else {
-		    	/* This will encourage some re-evaluation. */
-		    	return TASK_FAILED;
-		    }
-#if 0 /* the following is rarely a good idea */
-		    if (valid(check_attack_action(unit, unit, unit2, 100))) {
-			prep_attack_action(unit, unit, unit2, 100);
-			/* We (probably) made forward progress, so reopen choice of dirs. */
-			task->args[3] = eitherway;
-			return TASK_PREPPED_ACTION;
-		    } else {
-			continue;
-		    }
-#endif
+	return do_approach_subtask(unit, task, tx, ty, &(task->args[4]));
+    }
+    return TASK_FAILED;
+}
+
+static TaskOutcome
+do_approach_subtask(unit, task, tx, ty, statep)
+Unit *unit;
+Task *task;
+int tx, ty;
+short *statep;
+{
+    int nx, ny, dirs[NUMDIRS], numdirs, i, numdirs2;
+    Unit *unit2;
+
+    /* If on mobile transport, let it handle things. */
+    if (unit->transport != NULL
+	&& mobile(unit->transport->type)
+	/* and the transport is not blocked */
+	&& flip_coin()) {
+	unit->plan->reserve = TRUE;
+	return TASK_IS_INCOMPLETE;
+    }
+    numdirs = choose_move_dirs(unit, tx, ty, TRUE,
+			       plausible_move_dir, sort_directions, dirs);
+    for (i = 0; i < numdirs; ++i) {
+	point_in_dir(unit->x, unit->y, dirs[i], &nx, &ny);
+	for_all_stack(nx, ny, unit2) {
+	    if (can_occupy(unit, unit2)) {
+		if (valid(check_enter_action(unit, unit, unit2))) {
+		    prep_enter_action(unit, unit, unit2);
+		    /* We (probably) made forward progress, so reopen choice of dirs. */
+		    *statep = eitherway;
+		    return TASK_PREPPED_ACTION;
+		} else {
+		    continue;
 		}
-	    }
-	    if (valid(check_move_action(unit, unit, nx, ny, unit->z))) {
-		prep_move_action(unit, unit, nx, ny, unit->z);
-		/* We (probably) made forward progress, so reopen choice of dirs. */
-		task->args[3] = eitherway;
-		return TASK_PREPPED_ACTION;
+	    } else if (!trusted_side(unit->side, unit2->side)) {
+		if (unit->occupant) {
+		    /* More important to find a way through. */
+		    continue;
+		} else {
+		    /* This will encourage some re-evaluation. */
+		    return TASK_FAILED;
+		}
+#if 0 /* the following is rarely a good idea */
+		if (valid(check_attack_action(unit, unit, unit2, 100))) {
+		    prep_attack_action(unit, unit, unit2, 100);
+		    /* We (probably) made forward progress, so reopen choice of dirs. */
+		    *statep = eitherway;
+		    return TASK_PREPPED_ACTION;
+		} else {
+		    continue;
+		}
+#endif
 	    }
 	}
-	/* Get both right and left non-decreasing dirs. */
-	numdirs  = choose_move_dirs(unit, tx, ty, TRUE, NULL, NULL, dirs);
-	numdirs2 = choose_move_dirs(unit, tx, ty, FALSE, NULL, NULL, dirs);
-	for (i = numdirs; i < numdirs2; ++i) {
-	    if (plausible_move_dir(unit, dirs[i])) {
-		switch (task->args[3]) {
-		  case eitherway:
-		    if (i == numdirs)
-		      task->args[3] = leftonly /* leftthenright */;
-		    if (i == numdirs+1)
-		      task->args[3] = rightonly /* rightthenleft */;
-		    break;
-		  case leftthenright:
-		    if (i == numdirs)
-		      task->args[3] = rightonly;
-		    if (i == numdirs+1)
-		      task->args[3] = rightonly;
-		    continue;
-		    break;
-		  case rightthenleft:
-		    if (i == numdirs+1)
-		      task->args[3] = leftonly;
-		    continue;
-		    break;
-		  case leftonly:
-		    if (i == numdirs+1)
-		      continue;
-		    break;
-		  case rightonly:
-		    if (i == numdirs)
-		      continue;
-		    break;
-		}
-	    } else {
-		switch (task->args[3]) {
-		  case eitherway:
-		    if (i == numdirs)
-		      task->args[3] = rightonly;
-	    	    if (i == numdirs+1)
-	    	      task->args[3] = leftonly;
-	    	    continue;
-		    break;
-		  case leftthenright:
-		    if (i == numdirs)
-		      task->args[3] = rightonly;
-	    	    if (i == numdirs+1)
-	    	      task->args[3] = rightonly;
-	    	    continue;
-		    break;
-		  case rightthenleft:
-	    	    if (i == numdirs+1)
-	    	      task->args[3] = leftonly;
-	    	    continue;
-		    break;
-		  case leftonly:
-		    if (i == numdirs)
-		      return TASK_FAILED;
-		    if (i == numdirs+1)
-		      continue;
-		    break;
-		  case rightonly:
-		    if (i == numdirs)
-		      continue;
-		    if (i == numdirs+1)
-		      return TASK_FAILED;
-		    break;
-		}
-	    }
-	    point_in_dir(unit->x, unit->y, dirs[i], &nx, &ny);
-	    for_all_stack(nx, ny, unit2) {
-		if (can_occupy(unit, unit2)) {
-		    if (valid(check_enter_action(unit, unit, unit2))) {
-			prep_enter_action(unit, unit, unit2);
-			return TASK_PREPPED_ACTION;
-		    } else {
-			continue;
-		    }
-		} else if (!trusted_side(unit->side, unit2->side)) {
-		    if (unit->occupant) {
-		    	/* More important to find a way through. */
-		    	continue;
-		    } else {
-		    	/* This will encourage some re-evaluation. */
-		    	return TASK_FAILED;
-		    }
-#if 0				/* the following is rarely a good idea */
-		    if (valid(check_attack_action(unit, unit, unit2, 100))) {
-			prep_attack_action(unit, unit, unit2, 100);
-			return TASK_PREPPED_ACTION;
-		    } else {
-			continue;
-		    }
+	if (valid(check_move_action(unit, unit, nx, ny, unit->z))) {
+	    prep_move_action(unit, unit, nx, ny, unit->z);
+	    /* We (probably) made forward progress, so reopen choice of dirs. */
+	    *statep = eitherway;
+	    return TASK_PREPPED_ACTION;
+	}
+    }
+    /* Get both right and left non-decreasing dirs. */
+    numdirs  = choose_move_dirs(unit, tx, ty, TRUE, NULL, NULL, dirs);
+    numdirs2 = choose_move_dirs(unit, tx, ty, FALSE, NULL, NULL, dirs);
+    for (i = numdirs; i < numdirs2; ++i) {
+	if (plausible_move_dir(unit, dirs[i])) {
+	    switch (*statep) {
+	      case eitherway:
+		if (i == numdirs)
+		  *statep = leftonly /* leftthenright */;
+		if (i == numdirs+1)
+		  *statep = rightonly /* rightthenleft */;
+		break;
+#if 0
+	      case leftthenright:
+		if (i == numdirs)
+		  *statep = rightonly;
+		if (i == numdirs+1)
+		  *statep = rightonly;
+		continue;
+		break;
+	      case rightthenleft:
+		if (i == numdirs+1)
+		  *statep = leftonly;
+		continue;
+		break;
 #endif
+	      case leftonly:
+		if (i == numdirs+1)
+		  continue;
+		break;
+	      case rightonly:
+		if (i == numdirs)
+		  continue;
+		break;
+	      default:
+		run_warning("Weird right/left state %d", *statep);
+		*statep = leftonly;
+		break;
+	    }
+	} else {
+	    switch (*statep) {
+	      case eitherway:
+		if (i == numdirs)
+		  *statep = rightonly;
+		if (i == numdirs+1)
+		  *statep = leftonly;
+		continue;
+		break;
+#if 0
+	      case leftthenright:
+		if (i == numdirs)
+		  *statep = rightonly;
+		if (i == numdirs+1)
+		  *statep = rightonly;
+		continue;
+		break;
+	      case rightthenleft:
+		if (i == numdirs+1)
+		  *statep = leftonly;
+		continue;
+		break;
+#endif
+	      case leftonly:
+		if (i == numdirs)
+		  return TASK_FAILED;
+		if (i == numdirs+1)
+		  continue;
+		break;
+	      case rightonly:
+		if (i == numdirs)
+		  continue;
+		if (i == numdirs+1)
+		  return TASK_FAILED;
+		break;
+	      default:
+		run_warning("Weird right/left state %d", *statep);
+		*statep = leftonly;
+		break;
+	    }
+	}
+	point_in_dir(unit->x, unit->y, dirs[i], &nx, &ny);
+	for_all_stack(nx, ny, unit2) {
+	    if (can_occupy(unit, unit2)) {
+		if (valid(check_enter_action(unit, unit, unit2))) {
+		    prep_enter_action(unit, unit, unit2);
+		    return TASK_PREPPED_ACTION;
+		} else {
+		    continue;
 		}
+	    } else if (!trusted_side(unit->side, unit2->side)) {
+		if (unit->occupant) {
+		    /* More important to find a way through. */
+		    continue;
+		} else {
+		    /* This will encourage some re-evaluation. */
+		    return TASK_FAILED;
+		}
+#if 0				/* the following is rarely a good idea */
+		if (valid(check_attack_action(unit, unit, unit2, 100))) {
+		    prep_attack_action(unit, unit, unit2, 100);
+		    return TASK_PREPPED_ACTION;
+		} else {
+		    continue;
+		}
+#endif
 	    }
-	    if (valid(check_move_action(unit, unit, nx, ny, unit->z))) {
-		prep_move_action(unit, unit, nx, ny, unit->z);
-		return TASK_PREPPED_ACTION;
-	    }
+	}
+	if (valid(check_move_action(unit, unit, nx, ny, unit->z))) {
+	    prep_move_action(unit, unit, nx, ny, unit->z);
+	    return TASK_PREPPED_ACTION;
 	}
     }
     return TASK_FAILED;
@@ -790,17 +852,31 @@ Task *task;
 
 static TaskOutcome
 do_occupy_task(unit, task)
-Unit *unit;
-Task *task;
+     Unit *unit;
+     Task *task;
 {
+    int dist;
     Unit *transport = find_unit(task->args[0]);
 
     if (!in_play(transport))
       return TASK_FAILED;
+    /* (should also fail if we don't know where transport is anymore) */
     if (unit->transport == transport) {
 	return TASK_IS_COMPLETE;
     }
-    return TASK_FAILED;
+    dist = distance(unit->x, unit->y, transport->x, transport->y);
+    if (dist <= 1) {
+	if (valid(check_enter_action(unit, unit, transport))) {
+	    prep_enter_action(unit, unit, transport);
+	    return TASK_PREPPED_ACTION;
+	} else {
+	    /* Try a couple times, then fail if not working. */
+	    return (task->execnum < 3 ? TASK_IS_INCOMPLETE : TASK_FAILED);
+	}
+    } else {
+	/* Still some distance away, pick a way to go. */
+	return do_approach_subtask(unit, task, transport->x, transport->y, &(task->args[1]));
+    }
 }
 
 /* Wait around for a particular unit.  Give up if the unit is not forthcoming. */
@@ -835,6 +911,26 @@ Task *task;
     }
 }
 
+static TaskOutcome
+do_produce_task(unit, task)
+Unit *unit;
+Task *task;
+{
+    int m, tot, sofar, amt;
+
+    m = task->args[0];
+    tot = task->args[1];
+    sofar = task->args[2];
+    if (sofar >= tot)
+      return TASK_IS_COMPLETE;
+    amt = um_material_per_production(unit->type, m);
+    if (valid(check_produce_action(unit, unit, m, amt))) {
+	prep_produce_action(unit, unit, m, amt);
+	return TASK_PREPPED_ACTION;
+    }
+    return TASK_FAILED;
+}
+
 Unit *
 repair_here(x, y)
 int x, y;
@@ -889,7 +985,7 @@ Task *task;
 	return TASK_PREPPED_ACTION;
     } else if (search_around(ux, uy, range, repair_test, &x, &y, 1)) {
     	/* (should collect actual unit and chase it directly) */
-	push_moveto_task(unit, x, y);
+	push_move_to_task(unit, x, y);
 	return TASK_IS_INCOMPLETE;
     } else {
     	/* (should be able to signal interface usefully somehow) */
@@ -897,7 +993,7 @@ Task *task;
     }
 }
 
-int *lowm = NULL, numlow; 
+static int *lowm = NULL, numlow; 
 
 Unit *
 aux_resupply_here(unit)
@@ -907,13 +1003,12 @@ Unit *unit;
     Unit *occ;
 
     /* what about allies? */
-    if (unit->side == tmpside
+    if (unit_trusts_unit(unit, tmpunit)
 	&& can_carry(unit, tmpunit)) {
 	for (i = 0; i < numlow; ++i) {
 	    if (unit->supply[lowm[i]] == 0)
 	      enough = FALSE;
 	}
-	/* should test that unit has desired supply too... */
 	/* this should be controlled by doctrine? */
 	/* shouldn't wake up, should get a new task to "wait up"
 	   or even approach if possible */
@@ -963,7 +1058,7 @@ do_resupply_task(unit, task)
 Unit *unit;
 Task *task;
 {
-    int x, y, u = unit->type, m, range = area.maxdim;
+    int x, y, u = unit->type, m, range;
     int ux = unit->x, uy = unit->y;
     Unit *unit2;
 
@@ -972,13 +1067,16 @@ Task *task;
     if (lowm == NULL)
       lowm = (int *) xmalloc(nummtypes * sizeof(int));
     numlow = 0;
-    for_all_material_types(m) {
-    	if (unit->supply[m] < um_storage_x(u, m)) {
-    	    lowm[numlow++] = m;
-    	    /* (doesn't account for shared hold) */
-    	}
-	if (um_consumption_per_move(u, m) > 0) {
-	    range = min(range, unit->supply[m] / um_consumption_per_move(u, m));
+    if (task->args[0] == NONMTYPE) {
+	for_all_material_types(m) {
+	    if (unit->supply[m] < um_storage_x(u, m)) {
+		lowm[numlow++] = m;
+	    }
+	}
+    } else {
+	m = task->args[0];
+	if (unit->supply[m] < um_storage_x(u, m)) {
+	    lowm[numlow++] = m;
 	}
     }
     /* We're all full up, must be OK. */
@@ -988,6 +1086,7 @@ Task *task;
 	set_unit_reserve(unit->side, unit, TRUE, FALSE);
     	return (probability(10) ? TASK_FAILED : TASK_IS_INCOMPLETE);
     } else if (unit->transport != NULL) {
+	/* (should fix this test, transport is not necessarily of any help) */
     	/* (could attempt to resupply via direct action) */
 	set_unit_reserve(unit->side, unit, TRUE, FALSE);
     	return (probability(10) ? TASK_FAILED : TASK_IS_INCOMPLETE);
@@ -995,22 +1094,29 @@ Task *task;
     	&& unit2 != unit->transport) {	
     	prep_enter_action(unit, unit, unit2);
 	return TASK_PREPPED_ACTION;
-    } else if (search_around(ux, uy, range, resupply_test, &x, &y, 1)) {
-    	/* (should collect actual unit and chase it directly) */
-    	/* (only need to get within outlength of needed supplies) */
-	push_moveto_task(unit, x, y);
-	return TASK_IS_INCOMPLETE;
     } else {
-	/* Failure - sometimes just sit, but usually try something else. */
-	if (probability(10))
-	  set_unit_reserve(unit->side, unit, TRUE, FALSE);
-    	/* (should be able to signal interface usefully somehow) */
-	return TASK_FAILED;
+	/* Compute how far out to look for a resupply point; be a little
+	   optimistic. */
+	range = operating_range_best(u);
+	/* (should reduce range if materials are really low) */
+	if (search_around(ux, uy, range, resupply_test, &x, &y, 1)) {
+    	    /* (should collect actual unit and chase it directly) */
+    	    /* (should only need to get within outlength of needed supplies) */
+	    push_move_to_task(unit, x, y);
+	    return TASK_IS_INCOMPLETE;
+	} else {
+	    /* Failure - sometimes just sit, but usually try something else. */
+	    if (probability(10))
+	      set_unit_reserve(unit->side, unit, TRUE, FALSE);
+    	    /* (should be able to signal interface usefully somehow) */
+	    return TASK_FAILED;
+	}
     }
 }
 
+
 /* Return true if our own automatic material production is *greater*
-   than our consumption. */
+   than our consumption, for the given list of materials. */
 
 int
 can_auto_resupply_self(unit, materials, numtypes)
@@ -1020,7 +1126,7 @@ int *materials, numtypes;
     int u = unit->type, i, m, rslt = TRUE, t = terrain_at(unit->x, unit->y);
 
     for (i = 0; i < numtypes; ++i) {
-    	m = materials[i];
+	m = materials[i];
 	if (um_base_production(u, m) * ut_productivity(u, t) <= um_base_consumption(u, m))
 	  rslt = FALSE;
     }
@@ -1130,8 +1236,8 @@ Task *task;
 	return TASK_IS_COMPLETE;
       case TASK_BUILD:
 	return do_build_task(unit, task);
-      case TASK_CAPTURE_UNIT:
-	return do_capture_unit_task(unit, task);
+      case TASK_CAPTURE:
+	return do_capture_task(unit, task);
       case TASK_DISBAND:
 	return do_disband_task(unit, task);
       case TASK_DO_ACTION:
@@ -1140,14 +1246,16 @@ Task *task;
 	return do_hit_position_task(unit, task);
       case TASK_HIT_UNIT:
 	return do_hit_unit_task(unit, task);
-      case TASK_MOVEDIR:
-	return do_movedir_task(unit, task);
-      case TASK_MOVETO:
-	return do_moveto_task(unit, task);
+      case TASK_MOVE_DIR:
+	return do_move_dir_task(unit, task);
+      case TASK_MOVE_TO:
+	return do_move_to_task(unit, task);
       case TASK_OCCUPY:
         return do_occupy_task(unit, task);
       case TASK_PICKUP:
         return do_pickup_task(unit, task);
+      case TASK_PRODUCE:
+        return do_produce_task(unit, task);
       case TASK_REPAIR:
         return do_repair_task(unit, task);
       case TASK_RESEARCH:
@@ -1385,14 +1493,20 @@ Unit *unit;
 int pos;
 Task *task;
 {
+    int numcleared;
+
     if (!in_play(unit) || unit->plan == NULL) {
 	run_warning("Trying to do %s task with bad %s",
 		    task_desig(task), unit_desig(unit));
 	return;
     }
+    Dprintf("To %s task agenda, add %s",
+	    unit_desig(unit), task_desig(task));
     if (pos == CLEAR_AGENDA) {
-	clear_task_agenda(unit->plan);
+	numcleared = clear_task_agenda(unit->plan);
+	Dprintf(" (cleared %d existing tasks)", numcleared);
     }
+    Dprintf("\n");
     switch (pos) {
       case 0:
       case CLEAR_AGENDA:
@@ -1417,17 +1531,20 @@ Task *task;
 }
 
 Task *
-create_moveto_task(x, y)
+create_move_to_task(x, y)
 int x, y;
 {
-    Task *task = create_task(TASK_MOVETO);
+    Task *task = create_task(TASK_MOVE_TO);
 
     task->args[0] = x;  task->args[1] = y;
+    task->args[2] = 0;
+    task->args[3] = 0;
+    task->args[4] = eitherway;
     return task;
 }
 
 void
-push_moveto_task(unit, x, y)
+push_move_to_task(unit, x, y)
 Unit *unit;
 int x, y;
 {
@@ -1435,53 +1552,55 @@ int x, y;
     	run_warning("Trying to move %s to %d,%d", unit_desig(unit), x, y);
     	return;
     }
-    add_task(unit, 0, create_moveto_task(x, y));
+    add_task(unit, 0, create_move_to_task(x, y));
 }
 
 /* Give the unit a task to move to a given place, erasing every other task. */
 
 void
-set_moveto_task(unit, x, y)
+set_move_to_task(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    add_task(unit, CLEAR_AGENDA, create_moveto_task(x, y));
+    add_task(unit, CLEAR_AGENDA, create_move_to_task(x, y));
 }
 
 Task *
-create_movenear_task(x, y, dist)
+create_move_near_task(x, y, dist)
 int x, y, dist;
 {
-    Task *task = create_task(TASK_MOVETO);
+    Task *task = create_task(TASK_MOVE_TO);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = dist;
+    task->args[2] = 0;
+    task->args[3] = dist;
+    task->args[4] = eitherway;
     return task;
 }
 
 void
-set_movenear_task(unit, x, y, dist)
+set_move_near_task(unit, x, y, dist)
 Unit *unit;
 int x, y, dist;
 {
-    add_task(unit, CLEAR_AGENDA, create_movenear_task(x, y, dist));
+    add_task(unit, CLEAR_AGENDA, create_move_near_task(x, y, dist));
 }
 
 void
-push_movenear_task(unit, x, y, dist)
+push_move_near_task(unit, x, y, dist)
 Unit *unit;
 int x, y, dist;
 {
-    add_task(unit, 0, create_movenear_task(x, y, dist));
+    add_task(unit, 0, create_move_near_task(x, y, dist));
 }
 
 /* Create a task to move in a given direction for a given distance. */
 
 Task *
-create_movedir_task(dir, n)
+create_move_dir_task(dir, n)
 int dir, n;
 {
-    Task *task = create_task(TASK_MOVEDIR);
+    Task *task = create_task(TASK_MOVE_DIR);
 
     task->args[0] = dir;
     task->args[1] = n;
@@ -1491,11 +1610,11 @@ int dir, n;
 /* Fill in the given unit with direction-moving orders. */
 
 void
-set_movedir_task(unit, dir, n)
+set_move_dir_task(unit, dir, n)
 Unit *unit;
 int dir, n;
 {
-    add_task(unit, CLEAR_AGENDA, create_movedir_task(dir, n));
+    add_task(unit, CLEAR_AGENDA, create_move_dir_task(dir, n));
 }
 
 /* This routine sets up a task to build a unit of the given type. */
@@ -1548,7 +1667,8 @@ int x, y;
     Task *task = create_task(TASK_HIT_UNIT);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = NONUTYPE;  task->args[3] = -1;
+    task->args[2] = NONUTYPE;
+    task->args[3] = -1;
     add_task(unit, CLEAR_AGENDA, task);
 }
 
@@ -1560,7 +1680,8 @@ int x, y, u, s;
     Task *task = create_task(TASK_HIT_UNIT);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = u;  task->args[3] = s;
+    task->args[2] = u;
+    task->args[3] = s;
     add_task(unit, 0, task);
 }
 
@@ -1572,7 +1693,8 @@ int x, y, u, s;
     Task *task = create_task(TASK_HIT_UNIT);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = u;  task->args[3] = s;
+    task->args[2] = u;
+    task->args[3] = s;
     add_task(unit, CLEAR_AGENDA, task);
 }
 
@@ -1584,20 +1706,29 @@ int x, y;
     Task *task = create_task(TASK_HIT_UNIT);
 
     task->args[0] = x;  task->args[1] = y;
-    task->args[2] = NONUTYPE;  task->args[3] = ALLSIDES;
+    task->args[2] = NONUTYPE;
+    task->args[3] = -1;
     add_task(unit, 0, task);
 }
 
+Task *
+create_capture_task(x, y)
+int x, y;
+{
+    Task *task = create_task(TASK_CAPTURE);
+
+    task->args[0] = x;  task->args[1] = y;
+    task->args[2] = NONUTYPE;
+    task->args[3] = -1;
+    return task;
+}
+
 void
 set_capture_task(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    Task *task;
-
-    task = create_task(TASK_CAPTURE_UNIT);
-    task->args[0] = x;  task->args[1] = y;
-    add_task(unit, CLEAR_AGENDA, task);
+    add_task(unit, CLEAR_AGENDA, create_capture_task(x, y));
 }
 
 void
@@ -1605,11 +1736,7 @@ push_capture_task(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    Task *task;
-
-    task = create_task(TASK_CAPTURE_UNIT);
-    task->args[0] = x;  task->args[1] = y;
-    add_task(unit, 0, task);
+    add_task(unit, 0, create_capture_task(x, y));
 }
 
 void
@@ -1619,11 +1746,22 @@ Unit *unit;
     add_task(unit, CLEAR_AGENDA, create_task(TASK_DISBAND));
 }
 
-void
-set_resupply_task(unit)
-Unit *unit;
+Task *
+create_resupply_task(m)
+int m;
 {
-    add_task(unit, CLEAR_AGENDA, create_task(TASK_RESUPPLY));
+    Task *task = create_task(TASK_RESUPPLY);
+
+    task->args[0] = m;
+    return task;
+}
+
+void
+set_resupply_task(unit, m)
+Unit *unit;
+int m;
+{
+    add_task(unit, CLEAR_AGENDA, create_resupply_task(m));
 }
 
 Task *
@@ -1633,15 +1771,16 @@ Unit *transport;
     Task *task = create_task(TASK_OCCUPY);
 
     task->args[0] = transport->id;
+    task->args[1] = eitherway;
     /* add a waiting period also? */
     return task;
 }
 
 void
-push_occupy_task(unit, transp)
-Unit *unit, *transp;
+push_occupy_task(unit, transport)
+Unit *unit, *transport;
 {
-    add_task(unit, 0, create_occupy_task(transp));
+    add_task(unit, 0, create_occupy_task(transport));
 }
 
 Task *
@@ -1662,6 +1801,26 @@ Unit *unit, *occ;
     add_task(unit, 0, create_pickup_task(occ));
 }
 
+Task *
+create_produce_task(m, n)
+int m, n;
+{
+    Task *task = create_task(TASK_PRODUCE);
+
+    task->args[0] = m;
+    task->args[1] = n;
+    /* Third arg is amount produced, which starts at 0. */
+    return task;
+}
+
+void
+push_produce_task(unit, m, n)
+Unit *unit;
+int m, n;
+{
+    add_task(unit, 0, create_produce_task(m, n));
+}
+
 Task *
 create_sentry_task(n)
 int n;
@@ -1688,6 +1847,183 @@ int n;
     add_task(unit, 0, create_sentry_task(n));
 }
 
+extern int parse_location PARAMS ((Side *side, char *arg, int *xp, int *yp));
+
+extern Unit *parse_unit PARAMS ((Side *side, char *arg));
+
+/* Find a unit with the given name, either alive or dead. */
+
+Unit *
+parse_unit(side, nm)
+Side *side;
+char *nm;
+{
+    Unit *unit;
+
+    if (empty_string(nm))
+      return NULL;
+    for_all_side_units(side, unit) {
+	if (alive(unit) && unit->name != NULL && strcmp(unit->name, nm) == 0)
+	  return unit;
+    }
+    /* Under some circumstances, we can refer to other sides' units by name. */
+    for_all_units(unit) {
+	if (alive(unit)
+	    && unit->side != side
+	    && unit->name != NULL
+	    && strcmp(unit->name, nm) == 0
+	    && (side->see_all
+		|| side_sees_image(side, unit)))
+	  return unit;
+    }
+    return NULL;
+}
+
+/* Given a textual description of a location, compute an x,y for it
+   if possible. */
+
+int
+parse_location(side, arg, xp, yp)
+Side *side;
+char *arg;
+int *xp, *yp;
+{
+    char *arg2;
+    Unit *unit;
+
+    *xp = strtol(arg, &arg2, 10);
+    if (arg != arg2 && *arg2 == ',') {
+	*yp = strtol(arg2 + 1, &arg, 10);
+	if (arg2 + 1 != arg) {
+	    return TRUE;
+	} else if ((unit = parse_unit(side, arg)) != NULL) {
+	    *xp = unit->x;  *yp = unit->y;
+	    return TRUE;
+	}
+    }
+    notify(side, "location \"%s\" not recognized", arg);
+    return FALSE;
+}
+
+/* Given a string describing a task that has been entered in
+   by a player, generate a task object and return the rest
+   of the string, if NULL if failure. */
+
+char *
+parse_task(side, str, taskp)
+Side *side;
+char *str;
+Task **taskp;
+{
+    int tasktype, i, x, y, n, dir, u, taskargs[MAXTASKARGS], numargs, rslt;
+    char *arg, *arg2, substr[BUFSIZE], *rest, *argtypes;
+    Unit *unit;
+
+    rest = get_next_arg(str, substr, &arg);
+    /* Recognize special cases of task types first. */
+    if (strcmp(arg, "nil") == 0 || strcmp(arg, "nothing") == 0) {
+	/* NULL task but non-NULL return indicates order cancellation. */
+	*taskp = NULL;
+	return rest;
+    } else if (strcmp(arg, "move-near") == 0) {
+	rest = get_next_arg(rest, substr, &arg);
+	if (parse_location(side, arg, &x, &y)) {
+	    rest = get_next_arg(rest, substr, &arg);
+	    n = strtol(arg, &arg2, 10);
+	    *taskp = create_move_near_task(x, y, n);
+	    return rest;
+	}
+    }
+    tasktype = lookup_task_type(arg);
+    if (tasktype < 0) {
+	notify(side, "task type \"%s\" not recognized", arg);
+	return NULL;
+    }
+    switch (tasktype) {
+      case TASK_MOVE_TO:
+	rest = get_next_arg(rest, substr, &arg);
+	if (parse_location(side, arg, &x, &y)) {
+	    *taskp = create_move_to_task(x, y);
+	    return rest;
+	} else {
+	    return NULL;
+	}
+	break;
+      default:
+	argtypes = taskdefns[tasktype].argtypes;
+	numargs = strlen(argtypes);
+	for (i = 0; i < numargs; ++i)
+	  taskargs[i] = 0;
+	rest = get_next_arg(rest, substr, &arg);
+	for (i = 0; i < numargs; ++i) {
+	    if (argtypes[i] == 'x' && argtypes[i+1] == 'y') {
+		/* If there are two arguments that are together a position,
+		   interpret both together. */
+		if (parse_location(side, arg, &x, &y)) {
+		    taskargs[i] = x;  taskargs[i + 1] = y;
+		    ++i;
+		} else {
+		    return NULL;
+		}
+	    } else if (argtypes[i] == 'd') {
+		char *mydirchars = "ulnbhy"; /* (a local copy of ui.c thing) */
+		/* Match on names or chars for directions. */
+		for_all_directions(dir) {
+		    if (strcmp(arg, dirnames[dir]) == 0) {
+			taskargs[i] = dir;
+			goto nextarg;
+		    }
+		    if (strlen(arg) == 1 && arg[0] == mydirchars[dir]) {
+			taskargs[i] = dir;
+			goto nextarg;
+		    }
+		}
+		notify(side, "direction \"%s\" not recognized", arg);
+	    } else if (argtypes[i] == 'u') {
+		u = utype_from_name(arg);
+		if (u != NONUTYPE) {
+		    taskargs[i] = u;
+		} else {
+		    notify(side, "unit type \"%s\" not recognized", arg);
+		}
+	    } else if (argtypes[i] == 'U') {
+		unit = parse_unit(side, arg);
+		if (unit != NULL) {
+		    taskargs[i] = unit->id;
+		} else {
+		    notify(side, "unit called \"%s\" not recognized", arg);
+		}
+	    } else {
+		/* Just collect an integer and stuff it. */
+		taskargs[i] = strtol(arg, &arg2, 10);
+		if (arg == arg2) {
+		    notify(side, "argument \"%s\" not recognized", arg);
+		}
+	    }
+	  nextarg:
+	    rest = get_next_arg(str, substr, &arg);
+	    /* (should check for end of command or not?) */
+	}
+	*taskp = create_task(tasktype);
+	for (i = 0; i < numargs; ++i) {
+	    (*taskp)->args[i] = taskargs[i];
+	}
+	return rest;
+    }
+}
+
+int
+lookup_task_type(name)
+char *name;
+{
+    int i;
+
+    for (i = 0; taskdefns[i].name != NULL; ++i)
+      if (strcmp(name, taskdefns[i].name) == 0)
+	return i; /* should get real enum? */
+    return -1;
+}
+
 /* Describe a task succinctly - use for debugging only. */
 
 char *
