@@ -1,5 +1,5 @@
 /* Graphics support not specific to any Xconq interface.
-   Copyright (C) 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,14 +17,16 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 extern int disband_unit_directly PARAMS ((Side *side, Unit *unit));
 #include "imf.h"
+extern void make_generic_image_data PARAMS ((ImageFamily *imf));
 #include "ui.h"
 
 #ifdef MAC /* temporary */
-#include <Values.h>
 #include <Types.h>
 #include <Resources.h>
 #endif
 
+static void compute_q PARAMS ((void));
+
 static int blocking_utype PARAMS ((int u, int block));
 
 static ImageFamily *get_generic_images PARAMS ((Side *side, char *name, 
@@ -41,28 +43,28 @@ char *second_game_name = "standard";
 
 /* This is the basic cell size. */
 
-short mags[] = { 1, 2, 4, 8, 16, 32, 64, 128 };
+short mags[NUMPOWERS] = { 1, 2, 4, 8, 16, 32, 64, 128 };
 
 /* These give the total dimensions of each hex cell, plus the vertical
    distance center-to-center.  This is all carefully calculated to make
    the cells overlap perfectly at each different magnification, assuming
    that the icons have the right shape and size. */
 
-short hws[] = { 1, 2, 4, 8, 24, 44, 88, 174 };
-short hhs[] = { 1, 2, 4, 8, 26, 48, 96, 192 };
-short hcs[] = { 1, 2, 4, 8, 20, 37, 75, 148 };
+short hws[NUMPOWERS] = { 1, 2, 4, 8, 24, 44, 88, 174 };
+short hhs[NUMPOWERS] = { 1, 2, 4, 8, 26, 48, 96, 192 };
+short hcs[NUMPOWERS] = { 1, 2, 4, 8, 20, 37, 75, 148 };
 
 /* The sizes of the unit subcells.  This is available drawing area, exact
    unit icon sizes depends on what's available. */
 
-short uhs[] = { 1, 2, 3, 8, 16, 32, 64, 128 };
-short uws[] = { 1, 2, 3, 8, 16, 32, 64, 128 };
+short uhs[NUMPOWERS] = { 1, 2, 3, 8, 16, 32, 64, 128 };
+short uws[NUMPOWERS] = { 1, 2, 3, 8, 16, 32, 64, 128 };
 
 /* Widths of borders and connections (0 implies don't draw at all). */
 
 /* Full border width. */
 
-short bwid[NUMPOWERS]  = { 0, 0, 1, 1, 3, 5, 7, 9 };
+short bwid[NUMPOWERS] = { 0, 0, 1, 1, 3, 5, 7, 9 };
 
 /* Half-width, for narrower inset borders. */
 
@@ -121,6 +123,8 @@ short lsy[NUMPOWERS][6] = {
 
 short qx[NUMPOWERS][7], qy[NUMPOWERS][7];
 
+static int q_computed;
+
 /* The traditional direction characters. */
 
 char *dirchars = "ulnbhy";
@@ -134,6 +138,12 @@ char *terrchars = NULL;
 char unseen_char_1;
 char unseen_char_2;
 
+ImageFamily **recorded_imfs;
+
+int num_recorded_imfs;
+
+int max_recorded_imfs;
+
 /* This routine finds and lists all the games that should be listed as choices for
    the user. */
 
@@ -334,7 +344,7 @@ int *xp, *yp;
     if (side->startx < 0 || side->starty < 0)
       calc_start_xy(side);
     if (side->startx < 0 || side->starty < 0) {
-	*xp = area.width / 2 - area.height / 4;  *yp = area.height / 2;
+	*xp = area.width / 2 - area.height / 4;  *yp = area.halfheight;
     } else {
 	tmpx = side->startx;  tmpy = side->starty;
 	/* Rescan the units to find a closest one. */
@@ -390,11 +400,14 @@ char *cmdstr, **namep, **argp, *buf;
 	return;
     }
     strcpy(buf, cmdstr);
+    /* Look for the first nonwhite char, make it start of command name. */
     cmdname = buf;
     while ((*cmdname == ' ' || *cmdname == '\t') && *cmdname != '\0')
       ++cmdname;
-    for (j = 0; cmdname[j] != ' ' && *cmdname != '\t' && cmdname[j] != '\0'; ++j)
+    /* Scan over command name, which is delimited by whitespace or end of line. */
+    for (j = 0; cmdname[j] != ' ' && cmdname[j] != '\t' && cmdname[j] != '\0'; ++j)
       ;
+    /* If there's more than just the command name, extract an argument. */
     if (cmdname[j] != '\0') {
 	cmdarg = cmdname + j + 1;
 	while ((*cmdarg == ' ' || *cmdname == '\t') && *cmdarg != '\0')
@@ -404,8 +417,14 @@ char *cmdstr, **namep, **argp, *buf;
     } else {
 	cmdarg = NULL;
     }
-    /* (should remove trailing whitespace also) */
+    /* Terminate the command name. */
     cmdname[j] = '\0';
+    /* Remove trailing whitespace from the argument. */
+    if (!empty_string(cmdarg)) {
+	for (j = strlen(cmdarg) - 1; j >= 0 && (cmdarg[j] == ' ' || cmdarg[j] == '\t'); --j)
+	  ;
+	cmdarg[j + 1] = '\0';
+    }
     DGprintf("Command is \"%s\", argument is \"%s\"\n",
 	     (cmdname != NULL ? cmdname : "<null>"),
 	     (cmdarg != NULL ? cmdarg : "<null>"));
@@ -413,60 +432,54 @@ char *cmdstr, **namep, **argp, *buf;
     *argp = cmdarg;
 }
 
-int cellwidthguess = -1;
-int avgelev;
-int vertexagg = 1;
+/* Compute positions at each hex corner, slightly inset. */ 
 
-void
-guess_elev_stuff()
-{
-    int x, y, el;
-    long count = 0, sum = 0, maxelev = -9999;
-
-    for_all_cells(x, y) {
-	++count;
-	el = elev_at(x, y);
-	sum += el;
-	if (el > maxelev)
-	  maxelev = el;
-    }
-    avgelev = sum / count;
-    cellwidthguess = (area.cellwidth > 1 ? area.cellwidth : max(2, (maxelev - avgelev) * 2));
-}
-
-static q_computed = FALSE;
-
-void compute_q PARAMS ((void));
-
-void compute_q()
+static void
+compute_q()
 {
     int d, p, w;
 
-    if (q_computed)
-      return;
     for (p = 0; p < NUMPOWERS; ++p) {
-	    if (p < 2)
-	      continue;
-	    w = bwid[p] + 1;
-	    for_all_directions(d) {
-		qx[p][d] = bsx[p][d] + ((hws[p] - 2 * bsx[p][d]) * w) / (2 * mags[p]);
-		qy[p][d] = bsy[p][d] + ((hhs[p] - 2 * bsy[p][d]) * w) / (2 * mags[p]);
-	    }
-	    qx[p][NUMDIRS] = qx[p][0];
-	    qy[p][NUMDIRS] = qy[p][0];
+	if (p < 2)
+	  continue;
+	w = bwid[p] + 1;
+	for_all_directions(d) {
+	    qx[p][d] = bsx[p][d] + ((hws[p] - 2 * bsx[p][d]) * w) / (2 * mags[p]);
+	    qy[p][d] = bsy[p][d] + ((hhs[p] - 2 * bsy[p][d]) * w) / (2 * mags[p]);
+	}
+	qx[p][NUMDIRS] = qx[p][0];
+	qy[p][NUMDIRS] = qy[p][0];
     }
-    q_computed = TRUE;
 }
 
 VP *
 new_vp()
 {
+    int t, thickest;
     VP *vp;
 
-    compute_q();  /* a hack */
+    if (!q_computed) {
+	compute_q();
+	q_computed = TRUE;
+    }
     vp = (VP *) xmalloc(sizeof(VP));
     /* View at a 90 degree angle by default. */
     vp->angle = 90;
+    /* No vertical exaggeration by default. */
+    vp->vertscale = 1;
+    vp->cellwidth = area.cellwidth;
+    /* If the cellwidth is not reasonable for drawing elevations, use an
+       approximation based on the range of elevations and terrain thicknesses. */
+    if (vp->cellwidth <= 1) {
+	thickest = 0;
+	for_all_terrain_types(t) {
+	    if (t_thickness(t) > thickest)
+	      thickest = t_thickness(t);
+	}
+	vp->cellwidth = ((area.maxelev + thickest) - area.avgelev) / 2;
+    }
+    if (vp->cellwidth < 1)
+      vp->cellwidth = 1;
     return vp;
 }
 
@@ -477,23 +490,23 @@ xform_cell(vp, x, y, sxp, syp)
 VP *vp;
 int x, y, *sxp, *syp;
 {
-    extern int cellwidthguess, avgelev, vertexagg;
+    int xnw, elev, offset;
 
     if (in_area(x, y)) {
-	*sxp = x * vp->hw + (y * vp->hw) / 2 - vp->sx;
-#if 0 /* if two possibilities for sx, let caller make the choice and do the work */
-	if (area.xwrap && *sxp < (- vp->hw))
-	  *sxp += area.width * vp->hw;
-#endif
+	/* If x is in the upper right part of the area, wrap it around to the left,
+	   thus making it negative.  The transformed value will be positive, however. */
+	xnw = x;
+	if (area.xwrap && xnw >= (area.width - y / 2))
+	  xnw -= area.width;
+	*sxp = xnw * vp->hw + (y * vp->hw) / 2 - vp->sx;
 	*syp = (vp->totsh - (vp->hh + y * vp->hch)) - vp->sy;
-	if (vp->angle != 90 && elevations_defined()) {
-	    int elev, offset;
-	    if (cellwidthguess < 0) {
-		guess_elev_stuff();
-	    }
-	    elev = elev_at(x, y) - avgelev + t_thickness(terrain_at(x, y));
-	    elev *= vertexagg;
-	    offset = (elev * vp->hh) / cellwidthguess;
+	if (vp->angle != 90) {
+	    elev = (elevations_defined() ? elev_at(x, y) : 0) - area.avgelev;
+	    /* We see the top of the terrain in the cell, not the bottom. */
+	    elev += t_thickness(terrain_at(x, y));
+	    /* Exaggerate the vertical scale if requested. */
+	    elev *= vp->vertscale;
+	    offset = (elev * vp->hh) / vp->cellwidth;
 	    *syp -= offset;
 	}
     } else {
@@ -523,8 +536,12 @@ int *sxp, *syp, *swp, *shp;
 	      n = num;
 	    ++num;
 	}
-	if (n < 0)
-	  run_error("xform_unit weirdness");
+	if (n < 0) {
+	    run_warning("xform_unit weirdness with %s", unit_desig(unit));
+	    *sxp = *syp = 0;
+	    *swp = *shp = 1;
+	    return;
+	}
 	if (num <= 1) {
 	    sq = 1;
 	} else if (num <= 4) {
@@ -631,7 +648,13 @@ int sx, sy, *xp, *yp;
     /* Flip the raw y and then scale to hex coords. */
     *yp = (vp->totsh - (vp->sy + sy)) / vp->hch;
     /* Scale adjusted x to hex coord. */
-    *xp = (sx + vp->sx - (*yp * vp->hw) / 2) / vp->hw;
+    *xp = (sx + vp->sx - (*yp * vp->hw) / 2);
+    /* The division by hw below might round towards 0, so wrap negative numbers
+       around to positive values.  This should only ever happens for cylinder
+       areas, but doesn't hurt to just adjust all negative values. */
+    if (*xp < 0)
+      *xp += (2 * area.width * vp->hw);
+    *xp = *xp / vp->hw;
     /* If the magnification of the map is large enough that the top and bottom
        edges of a hex are visibly sloping, then we have to take those edges
        int account, and accurately. */
@@ -767,8 +790,8 @@ int x, y;
 {
     int sx, sy, insetx1, insety1, insetx2, insety2;
     
- 	if (!in_area(x, y))
- 	  return FALSE;   
+    if (!in_area(x, y))
+      return FALSE;   
     xform_cell(vp, x, y, &sx, &sy);
     /* Adjust to be the center of the cell, more reasonable if large. */
     sx += vp->hw / 2;  sy += vp->hh / 2;
@@ -801,10 +824,10 @@ VP *vp;
 	vp->vcx = wrapx(vp->vcx);
     } else {
 	vp->vcx = max(1, min(vp->vcx, area.width - 2));
-	if (vp->vcx + vp->vcy < area.height / 2 + 1)
-	  vp->vcx = area.height / 2 + 1;
-	if (vp->vcx + vp->vcy > area.width + area.height / 2 - 1)
-	  vp->vcx = area.width + area.height / 2 - 1;
+	if (vp->vcx + vp->vcy < area.halfheight + 1)
+	  vp->vcx = area.halfheight + 1;
+	if (vp->vcx + vp->vcy > area.width + area.halfheight - 1)
+	  vp->vcx = area.width + area.halfheight - 1;
     }
 }
 
@@ -812,8 +835,12 @@ void
 center_on_focus(vp)
 VP *vp;
 {
+    int xnw = vp->vcx;
+    if (area.xwrap && xnw >= (area.width - vp->vcy / 2))
+      xnw -= area.width;
+
     /* Scale, add hex offset adjustment, translate to get left edge. */
-    vp->sx = vp->vcx * vp->hw + (vp->vcy * vp->hw) / 2 - vp->pxw / 2 + vp->hw / 2;
+    vp->sx = xnw * vp->hw + (vp->vcy * vp->hw) / 2 - vp->pxw / 2 + vp->hw / 2;
     /* Scale, translate to top edge, flip. */
     vp->sy = vp->totsh - (vp->vcy * vp->hch + vp->pxh / 2 + vp->hh / 2);
     /* Weird vcx,vcy might make sx,sy nonsensical, so clip to rational
@@ -925,6 +952,14 @@ int dir;
     return TRUE;
 }
 
+void
+compute_fire_line_segment(sx1, sy1, sx2, sy2, i, n, xx, yy, dx, dy)
+int sx1, sy1, sx2, sy2, i, n, *xx, *yy, *dx, *dy;
+{
+    /* Position one segment of a line between the locations. */
+    *dx = (sx2 - sx1) / n;  *dy = (sy2 - sy1) / n;
+    *xx = sx1 + ((i / 2) % n) * *dx;  *yy = sy1 + ((i / 2) % n) * *dy;
+}
 
 /* This routine can be used by the interface to place legends */
 
@@ -1117,7 +1152,7 @@ VP *vp;
     if (unit == NULL) {
 	u = 0;
 	if (!nearest_cell(vp, vp->sx + vp->pxw / 2, vp->sy + vp->pxh / 2, &mx, &my)) {
-	    mx = area.width / 2;  my = area.height / 2;
+	    mx = area.width / 2;  my = area.halfheight;
 	}
     } else {
 	u = unit->type;
@@ -1187,6 +1222,59 @@ Unit *unit;
     return unit;
 }
 
+/* Given a character, compute the direction(s) that it represents.
+   Return the number of directions. */
+
+int
+char_to_dir(ch, dir1p, dir2p, modp)
+int ch, *dir1p, *dir2p, *modp;
+{
+    char basech, *rawdir;
+    int ndirs = 0;
+
+    if (isupper(ch)) {
+	basech = tolower(ch);
+	if (modp)
+	 *modp = 1;
+    } else if (ch < ' ') {
+	basech = ch + 0x60;
+	if (modp)
+	  *modp = 2;
+    } else {
+	basech = ch;
+	if (modp)
+	  *modp = 0;
+    }
+    rawdir = strchr(dirchars, basech);
+    if (rawdir) {
+	*dir1p = rawdir - dirchars;
+	ndirs = 1;
+    } else if (basech == 'k') {
+	if (flip_coin()) {
+	    *dir1p = NORTHEAST;
+	    if (dir2p)
+	      *dir2p = NORTHWEST;
+	} else {
+	    *dir1p = NORTHWEST;
+	    if (dir2p)
+	      *dir2p = NORTHEAST;
+	}
+	ndirs = 2;
+    } else if (basech == 'j') {
+	if (flip_coin()) {
+	    *dir1p = SOUTHEAST;
+	    if (dir2p)
+	      *dir2p = SOUTHWEST;
+	} else {
+	    *dir1p = SOUTHWEST;
+	    if (dir2p)
+	      *dir2p = SOUTHEAST;
+	}
+	ndirs = 2;
+    }
+    return ndirs;
+}
+
 /* Given that the player desires to move the given unit into the given
    cell/other unit, prepare a "most appropriate" action. */
 /* (should share diff cell and same cell interaction code) */
@@ -1232,10 +1320,10 @@ int x, y;
 			       after other unit is actually inside, in
 			       case it fills dest) */
 			    prep_enter_action(other, other, unit);
-			    set_moveto_task(unit, x, y);
+			    set_move_to_task(unit, x, y);
 			} else {
 			    prep_enter_action(unit, other, unit);
-			    set_moveto_task(unit, x, y);
+			    set_move_to_task(unit, x, y);
 			}
 		    } else if (other->transport != NULL
 			       && can_occupy(unit, other->transport)) {
@@ -1282,7 +1370,7 @@ int x, y;
 		}
 	    } else {
 		/* We're not adjacent to the destination, set up a move task. */
-		set_moveto_task(unit, x, y);
+		set_move_to_task(unit, x, y);
 	    }
 	} else {
 	    /* ??? can't act ??? */
@@ -1423,6 +1511,27 @@ Unit *unit;
     }
 }
 
+/* Return the type to build that dialogs should highlight initially. */
+
+int
+favored_type(unit)
+Unit *unit;
+{
+	int u;
+
+	if (unit == NULL)
+	  return NONUTYPE;
+	if (unit->plan
+		&& unit->plan->tasks
+		&& unit->plan->tasks->type == TASK_BUILD)
+	  return unit->plan->tasks->args[0];
+	for_all_unit_types(u) {
+		if (uu_acp_to_create(unit->type, u) > 0)
+		  return u;
+	}
+	return NONUTYPE;
+}
+
 /* Generic image setup. */
 
 static ImageFamily *
@@ -1467,7 +1576,8 @@ void (*load_hook) PARAMS ((ImageFamily *imf));
 	  return NULL;
 	if (imf->location != NULL) {
 	    /* Load data filling in the family. */
-	    make_pathname(xconqlib, imf->location->name, "", spbuf);
+	    /* (should use library path list) */
+	    make_pathname(xconq_libs->path, imf->location->name, "", spbuf);
 	    if (load_imf_file(spbuf, NULL)) {
 	    } else if (load_imf_file(imf->location->name, NULL)) {
 	    } else {
@@ -1503,6 +1613,7 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 	imf->ersatz = TRUE;
 	(*default_hook)(imf, u, u_type_name(u));
     }
+    record_imf_get(imf);
     return imf;
 }
 
@@ -1526,6 +1637,7 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
 	imf->ersatz = TRUE;
 	(*default_hook)(imf, t, t_type_name(t));
     }
+    record_imf_get(imf);
     return imf;
 }
 
@@ -1541,10 +1653,11 @@ void (*default_hook) PARAMS ((ImageFamily *imf, char *name));
 	if (unseen_image != NULL && unseen_image->numsizes == 0) {
 	    /* Appears to have failed - clear the unseen image then. */
 	    unseen_image = NULL;
-	    /* Note that we shouldn't try to free the imf, because it may be use
+	    /* Note that we shouldn't try to free the imf, because it may be in use
 	       elsewhere. */
 	}
     }
+    record_imf_get(unseen_image);
     return unseen_image;
 }
 
@@ -1569,9 +1682,46 @@ void (*default_hook) PARAMS ((ImageFamily *imf, int n, char *name));
     if (imf != NULL && imf->numsizes == 0 && default_hook != NULL && strcmp(name, "none") != 0) {
 	(*default_hook)(imf, s2, name);
     }
+    record_imf_get(imf);
     return imf;
 }
 
+void
+record_imf_get(imf)
+ImageFamily *imf;
+{
+    int i;
+    ImageFamily **new_record;
+
+    if (imf == NULL)
+      return;
+    /* Estimate and allocate the usual amount of space needed. */
+    if (max_recorded_imfs == 0)
+      max_recorded_imfs = numutypes + numttypes + (MAXSIDES + 1) + 1;
+    if (recorded_imfs == NULL) {
+	recorded_imfs = (ImageFamily **) xmalloc(max_recorded_imfs * sizeof(ImageFamily *));
+    }
+    /* Allocate more space if needed. */
+    if (num_recorded_imfs >= max_recorded_imfs) {
+	max_recorded_imfs += max_recorded_imfs / 2;
+	new_record = (ImageFamily **) xmalloc(max_recorded_imfs * sizeof(ImageFamily *));
+	for (i = 0; i < num_recorded_imfs; ++i) {
+	    new_record[i] = recorded_imfs[i];
+	}
+	recorded_imfs = new_record;
+    }
+    for (i = 0; i < num_recorded_imfs; ++i) {
+	if (strcmp(imf->name, recorded_imfs[i]->name) == 0)
+	  return;
+    }
+    recorded_imfs[num_recorded_imfs++] = imf;
+    /* Expand any interface-specific data into its all-interface form, so that
+       saved games will include it.  This needs to be done now, because game
+       saving may occur in a low-memory situation and there may not be enough
+       memory available then. */
+    make_generic_image_data(imf);
+}
+
 /* Compute and cache single-char representations for things. */
 
 void
@@ -1692,7 +1842,7 @@ int x, y;
 {
     if (in_area(x, y)
 	&& (side == NULL
-	    || g_see_all()
+	    || all_see_all
 #ifdef DESIGNERS
 	    || side->designer
 #endif /* DESIGNERS */
@@ -1711,7 +1861,7 @@ int x, y;
     if (!in_area(x, y))
       return NULL;
     if (side == NULL
-	|| g_see_all()
+	|| all_see_all
 #ifdef DESIGNERS
 	|| side->designer
 #endif /* DESIGNERS */
