@@ -1,5 +1,5 @@
 /* Units in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -8,7 +8,12 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int disband_unit_directly PARAMS ((Side *side, Unit *unit));
+extern int change_cell PARAMS ((Unit *unit, int x, int y));
+
+extern void set_unit_position PARAMS ((Unit *unit, int x, int y, int z));
+extern void add_unit_to_stack PARAMS ((Unit *unit, int x, int y));
+extern void remove_unit_from_stack PARAMS ((Unit *unit));
+extern void glimpse_adjacent_terrain PARAMS ((Unit *unit));
 
 /* This is not a limit, just sets initial allocation of unit objects
    and how many additional to get if more are needed.  This can be
@@ -19,10 +24,12 @@ extern int disband_unit_directly PARAMS ((Side *side, Unit *unit));
 #define INITMAXUNITS 200
 #endif
 
+static void allocate_unit_block PARAMS ((void));
 static int compare_units PARAMS ((Unit *unit1, Unit *unit2));
-
 static int compare_units_by_keys PARAMS ((const void *e1, const void *e2));
 
+static void change_cell_aux PARAMS ((Unit *unit, int x0, int y0, int x, int y));
+
 /* The list of available units. */
 
 Unit *freeunits;
@@ -67,21 +74,29 @@ int *numlivebytype;
 
 int nextid = 1;
 
+/* A cache of completenesses. */
+
+short *completenesses;
+
 /* Have units died since dead unit lists made. */
 
 int recent_dead_flushed = TRUE;
 
+/* Local tmp var. */
+
+static Side *tmpside_classtest;
+
 /* Grab a block of unit objects to work with. */
 
-void
+static void
 allocate_unit_block()
 {
     int i;
     Unit *unitblock = (Unit *) xmalloc(INITMAXUNITS * sizeof(Unit));
 
     for (i = 0; i < INITMAXUNITS; ++i) {
-	unitblock[i].id = -1;
-	unitblock[i].next = &unitblock[i+1];
+        unitblock[i].id = -1;
+        unitblock[i].next = &unitblock[i+1];
     }
     unitblock[INITMAXUNITS-1].next = NULL;
     freeunits = unitblock;
@@ -97,6 +112,7 @@ init_units()
     unitlist = NULL;
     allocate_unit_block();
     init_side_unithead(indepside);
+    completenesses = (short *) xmalloc(MAXUTYPES * sizeof(short));
 }
 
 /* The primitive unit creator, called by regular creator and also used to
@@ -135,6 +151,12 @@ int type, makebrains;
     if (numlivebytype == NULL) {
     	numlivebytype = (int *) xmalloc(MAXUTYPES * sizeof(int));
     }
+    /* Fill this in. */
+    if (completenesses[0] == 0) {
+	int i;
+	for_all_unit_types(i)
+	  completenesses[i] = u_cp(i) / u_parts(i);
+    }
     /* Test whether we've hit any designer-specified limits. */
     if ((u_type_in_game_max(type) >= 0
 	 && numlivebytype[type] >= u_type_in_game_max(type))
@@ -163,20 +185,11 @@ int type, makebrains;
     newunit->cp = u_cp(type);
     /* Not in a transport. */
     newunit->transport = NULL;
-#ifndef MSDOS /* (should not be a separate case?) */
-#if (MAXMTYPES > 2)
     /* Note that the space never needs to be freed. */
     if (newunit->supply == NULL && nummtypes > 0) {
-	newunit->supply = (short *) xmalloc(nummtypes * sizeof(short));
+        newunit->supply = (short *) xmalloc(nummtypes * sizeof(short));
     }
-#endif
-#else /* MSDOS */
-    /* Note that the space never needs to be freed. */
-    if (newunit->supply == NULL && nummtypes > 0) {
-	newunit->supply = (short *) xmalloc(nummtypes * sizeof(short));
-    }
-#endif /* MSDOS */
-    /* In any case, zero out all the supply values. */
+    /* Always zero out all the supply values. */
     for_all_material_types(m)
       newunit->supply[m] = 0;
     /* Will allocate tooling state when actually needed. */
@@ -184,12 +197,14 @@ int type, makebrains;
     /* Will allocate opinions when actually needed. */
     newunit->opinions = NULL;
     if (makebrains) {
-	init_unit_actorstate(newunit);
+	init_unit_actorstate(newunit, TRUE);
 	init_unit_plan(newunit);
     } else {
 	newunit->act = NULL;
 	newunit->plan = NULL;
     }
+    if (newunit->extras != NULL)
+      init_unit_extras(newunit);
     newunit->occupant = NULL;
     newunit->nexthere = NULL;
     /* Glue this unit into the list of independent units. */
@@ -200,7 +215,6 @@ int type, makebrains;
     unitlist = newunit;
     /* Init more random slots. */
     newunit->prevx = newunit->prevy = -1;
-    newunit->hook = lispnil;
     newunit->aihook = NULL;
     /* Add to the global unit counts. */
     ++numunits;
@@ -217,13 +231,28 @@ Unit *unit;
 }
 
 void
-init_unit_opinions(unit)
+init_unit_opinions(unit, nsides)
 Unit *unit;
+int nsides;
 {
+    int i;
+    short *temp;
+
+    temp = NULL;
     if (u_has_opinions(unit->type)) {
+	if (unit->opinions != NULL && nsides > numsides) {
+	    temp = unit->opinions;
+	    unit->opinions = NULL;
+	}
 	if (unit->opinions == NULL)
-	  unit->opinions = (short *) xmalloc(numsides * sizeof(short));
+	  unit->opinions = (short *) xmalloc(nsides * sizeof(short));
 	/* Opinions are now all neutral. */
+	if (temp != NULL) {
+	    /* Copy over old opinions. */
+	    for (i = 0; i < numsides; ++i)
+	      unit->opinions[i] = temp[i];
+	    free(temp);
+	}
     } else {
 	if (unit->opinions != NULL)
 	  free(unit->opinions);
@@ -234,15 +263,20 @@ Unit *unit;
 /* Alter the actorstate object to be correct for this unit. */
 
 void
-init_unit_actorstate(unit)
+init_unit_actorstate(unit, flagacp)
 Unit *unit;
+int flagacp;
 {
     if (u_acp(unit->type) > 0 && unit->cp > 0) {
-	/* Might already have an actorstate, don't realloc if so. */
+	/* Might already have an actorstate, don't realloc if so;
+	   but do clear an existing actorstate, since might be reusing. */
 	if (unit->act == NULL)
 	  unit->act = (ActorState *) xmalloc(sizeof(ActorState));
+	else
+	  memset((char *) unit->act, 0, sizeof(ActorState));
 	/* Indicate that the action points have not been set. */
-	unit->act->acp = u_acp_min(unit->type) - 1;
+	if (flagacp)
+	  unit->act->acp = u_acp_min(unit->type) - 1;
 	/* Flag the action as undefined. */
 	unit->act->nextaction.type = ACTION_NONE;
     } else {
@@ -252,36 +286,19 @@ Unit *unit;
     }
 }
 
-/* Every unit that can act needs a plan object, types that can't act
-   should have it cleared out. */
-
 void
-init_unit_plan(unit)
+init_unit_extras(unit)
 Unit *unit;
 {
-    if (u_acp(unit->type) > 0 && unit->cp > 0) {
-	/* Might already have a plan, so don't always realloc. */
-	if (unit->plan == NULL) {
-	    unit->plan = create_plan();
-	}
-	/* Put the plan into a default state, side will work it up later. */
-	unit->plan->type = PLAN_NONE;
-	unit->plan->creation_turn = g_turn();
-	unit->plan->asleep = FALSE;
-	unit->plan->reserve = FALSE;
-	unit->plan->delayed = FALSE;
-	clear_task_agenda(unit->plan);
-	/* Allow AIs to make this unit do things. */
-	unit->plan->aicontrol = TRUE;
-	/* Enable supply alarms by default. */
-	unit->plan->supply_alarm = TRUE;
-    } else {
-	/* Brainless units don't need anything, can free up plan. */
-	if (unit->plan != NULL) {
-	    free_plan(unit->plan);
-	}
-	unit->plan = NULL;
+    if (unit->extras == NULL) {
+	unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
     }
+    unit->extras->point_value = -1;
+    unit->extras->appear = -1;
+    unit->extras->appearx = -1;
+    unit->extras->appeary = -1;
+    unit->extras->disappear = -1;
+    unit->extras->hook = lispnil;
 }
 
 /* Changing a unit's type has many effects. */
@@ -293,13 +310,21 @@ int newtype, reason;
 {
     int oldtype = unit->type, oldhp = unit->hp;
     PastUnit *pastunit;
+    Side *oldside;
 
     /* Don't do anything if we're "changing" to the same type. */
     if (oldtype == newtype)
       return;
+    oldside = unit->side;
     pastunit = change_unit_to_past_unit(unit);
     if (reason >= 0)
       record_event(reason, ALLSIDES, pastunit->id, unit->id);
+    /* Decrement viewing coverage of our old type. */
+    if (completed(unit)
+        && (unit->transport == NULL
+	   || uu_occ_can_see(unit->type, unit->transport->type))) {
+	cover_area(unit->side, unit, unit->x, unit->y, -1, -1);
+    }
     /* Do the actual change. */
     unit->type = newtype;
     /* Set the new hp to the same ratio of max as the unit had before.
@@ -308,6 +333,10 @@ int newtype, reason;
     /* Need to guarantee a positive value though. */
     if (unit->hp < 1)
       unit->hp = 1;
+    unit->hp2 = unit->hp;
+    /* Invalidate the side's point value cache. */
+    if (unit->side)
+      unit->side->point_value_valid = FALSE;
     /* We might have to change sides as a result. */
     /* (should modify per-side counts) */
     if (!type_allowed_on_side(newtype, unit->side)) {
@@ -320,10 +349,24 @@ int newtype, reason;
     }
     /* Unit will always need a new number. */
     assign_unit_number(unit);
-    init_unit_opinions(unit);
-    init_unit_actorstate(unit);
+    init_unit_opinions(unit, numsides);
+    /* This clears the unit's acp - desirable? */
+    init_unit_actorstate(unit, FALSE);
     init_unit_plan(unit);
     unit->aihook = NULL;
+    /* Increment viewing coverage, if unit is complete and can see out
+       of its transport. */
+    if (completed(unit)
+    	&& (unit->transport == NULL
+	    || uu_occ_can_see(unit->type, unit->transport->type))) {
+	cover_area(unit->side, unit, -1, -1, unit->x, unit->y);
+	/* If vision range is 0, allow glimpses of adjacent cell terrain.
+	   This applies to terrain only, adjacent units cannot be seen. */
+	if (u_vision_range(unit->type) == 0)
+	  glimpse_adjacent_terrain(unit);
+    }
+    count_loss(oldside, oldtype, (reason == H_UNIT_WRECKED ? combat_loss : other_loss));
+    count_gain(unit->side, newtype, other_gain);
     /* Update global counts. */
     --(numlivebytype[oldtype]);
     ++(numlivebytype[newtype]);
@@ -352,8 +395,8 @@ enter_cell(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    register int u = unit->type, ustack, u2stack;
-    Unit *unit2, *topunit, *prevunit = NULL, *nextunit = NULL;
+    register int u = unit->type;
+    Unit *prevunit = NULL, *nextunit = NULL;
 
 #ifdef DEBUGGING
     /* Not necessarily an error, but indicative of bugs elsewhere. */
@@ -375,28 +418,7 @@ int x, y;
 	/* Let the unit remain off-world. */
 	return FALSE;
     }
-    topunit = unit_at(x, y);
-    if (topunit) {
-    	/* Insert the entering unit into the stack at its correct position. */
-    	ustack = u_stack_order(u);
-    	for_all_stack(x, y, unit2) {
-	    u2stack = u_stack_order(unit2->type);
-	    if (ustack > u2stack
-		|| (ustack == u2stack && unit->id < unit2->id)) {
-		nextunit = unit2;
-		if (unit2 == topunit)
-		  topunit = unit;
-		break;
-	    }
-	    prevunit = unit2;
-    	}
-    	if (prevunit != NULL)
-	  prevunit->nexthere = unit;
-    } else {
-    	topunit = unit;
-    }
-    unit->nexthere = nextunit;
-    set_unit_at(x, y, topunit);
+    add_unit_to_stack(unit, x, y);
     /* Set the location slots now. */
     enter_cell_aux(unit, x, y);
     /* Inevitable side-effect of appearing in the new location. */
@@ -546,6 +568,51 @@ Unit *unit3;
     return (fullness + ut_size(u, t) <= tcap);
 }
 
+/* Test whether the given position has a connection that the unit may use. */
+
+int
+can_occupy_conn(unit, nx, ny, nz)
+Unit *unit;
+int nx, ny, nz;
+{
+    int c, dir, numhere;
+    Unit *unit2;
+
+    if (numconntypes == 0)
+      return FALSE;
+    if (nz & 1 == 1) {
+	c = nz / 2;
+    } else {
+	for_all_terrain_types(c) {
+	    if (t_is_connection(c) && aux_terrain_defined(c))
+	      break;
+	}
+    }
+    if (!t_is_connection(c) || !aux_terrain_defined(c))
+    	run_warning("%s is on an invalid connection type %d?",
+		    unit_desig(unit), c);
+    if (unit->x == nx && unit->y == ny) {
+	for_all_directions(dir) {
+	    if (connection_at(unit->x, unit->y, dir, c))
+	      break;
+	}
+    } else {
+	dir = closest_dir(nx - unit->x, ny - unit->y);
+    }
+    if (!connection_at(unit->x, unit->y, dir, c))
+      return FALSE;
+    numhere = 0;
+    for_all_stack(nx, ny, unit2) {
+	if (unit->z == nz) {
+	    ++numhere;
+	}
+    }
+    if (numhere + 1 <= ut_capacity_x(unit->type, c))
+      return FALSE;
+    /* (should also calc general fullness) */
+    return FALSE;
+}
+
 /* Recursive helper to update everybody's position.  This should be one of
    two routines that modify actual unit positions (leaving is the other). */
 
@@ -554,7 +621,7 @@ enter_cell_aux(unit, x, y)
 Unit *unit;
 int x, y;
 {
-    int u = unit->type, dir, x1, y1, tmpz;
+    int u = unit->type;
     Unit *occ;
     Side *side = unit->side;
 
@@ -568,41 +635,17 @@ int x, y;
     if (!in_area(x, y))
       run_error("trying to enter cell outside world");
     /* Actually set the unit position. */
-    unit->x = x;  unit->y = y;
-    /* Adjust the altitude if nonzero. */
-    if (unit->z != 0) {
-	if (unit->z & 1 == 0) {
-	    tmpz = unit->z / 2;
-	    tmpz = min(tmpz, ut_alt_max(u, terrain_at(x, y)));
-	    tmpz = max(tmpz, ut_alt_min(u, terrain_at(x, y)));
-	    unit->z = tmpz * 2;
-	} else {
-	    /* (still on the same connection?) */
-	}
-    }
+    set_unit_position(unit, x, y, unit->z);
     /* Increment viewing coverage, if unit is complete and can see out
        of its transport. */
     if (completed(unit)
     	&& (unit->transport == NULL
 	    || uu_occ_can_see(u, unit->transport->type))) {
-	cover_area(unit->side, unit, x, y, 1);
+	cover_area(unit->side, unit, -1, -1, x, y);
 	/* If vision range is 0, allow glimpses of adjacent cell terrain.
 	   This applies to terrain only, adjacent units cannot be seen. */
-	if (u_vision_range(u) == 0
-	    && unit->transport == NULL
-	    && !g_see_all()
-	    && !g_terrain_seen()
-	    && side != NULL) {
-	    for_all_directions(dir) {
-		if (point_in_dir(x, y, dir, &x1, &y1)) {
-		    if (terrain_view(side, x1, y1) == UNSEEN) {
-			set_terrain_view(side, x1, y1,
-					 buildtview(terrain_at(x1, y1)));
-			update_cell_display(side, x1, y1, TRUE);
-		    }
-		}
-	    }
-	}
+	if (u_vision_range(u) == 0)
+	  glimpse_adjacent_terrain(unit);
     }
     /* Do for all the occupants too, recursively. */
     for_all_occupants(unit, occ) {
@@ -803,7 +846,7 @@ leave_cell(unit)
 Unit *unit;
 {
     int ux = unit->x, uy = unit->y;
-    Unit *transport = unit->transport, *other;
+    Unit *transport = unit->transport;
 
     if (ux < 0 || uy < 0) {
 	/* Sometimes called twice */
@@ -814,19 +857,7 @@ Unit *unit;
 	/* not all_see_cell here because can't see inside transports */
 	update_unit_display(transport->side, transport, TRUE);
     } else {
-	/* Unsplice ourselves from the list of units in this cell. */
-	if (unit == unit_at(ux, uy)) {
-	    set_unit_at(ux, uy, unit->nexthere);
-	} else {
-	    for_all_stack(ux, uy, other) {
-		if (unit == other->nexthere) {
-		    other->nexthere = other->nexthere->nexthere;
-		    break;
-		}
-	   } 
-	}
-	/* Bash this now-spurious link. */
-	unit->nexthere = NULL;
+	remove_unit_from_stack(unit);
 	/* Now bash the coords. */
 	leave_cell_aux(unit);
 	/* Now let everybody observe that the unit is gone. */
@@ -851,10 +882,10 @@ Unit *unit;
     if (completed(unit)
         && (unit->transport == NULL
 	   || uu_occ_can_see(unit->type, unit->transport->type))) {
-	cover_area(unit->side, unit, unit->x, unit->y, -1);
+	cover_area(unit->side, unit, unit->x, unit->y, -1, -1);
     }
     /* Stash the old coords. */
-    unit->prevx = unit->x; unit->prevy = unit->y;
+    unit->prevx = unit->x;  unit->prevy = unit->y;
     /* Set to a recognizable value. */
     unit->x = -1;  unit->y = -1;
     /* Make any occupants leave too. */
@@ -889,6 +920,180 @@ Unit *unit;
     unit->transport = NULL;
 }
 
+int
+change_cell(unit, x, y)
+Unit *unit;
+int x, y;
+{
+    register int u = unit->type;
+    int ux = unit->x, uy = unit->y;
+    Unit *transport = unit->transport;
+    Side *side = unit->side;
+
+    /* Always check this one, but not necessarily fatal. */
+    if (!inside_area(x, y)) {
+	run_warning("No cell at %d,%d, %s can't enter it",
+		    x, y, unit_desig(unit));
+	/* Let the unit remain off-world. */
+	return FALSE;
+    }
+    if (!can_occupy_cell(unit, x, y)) {
+	run_warning("Cell at %d,%d is too full for %s",
+		    x, y, unit_desig(unit));
+	/* Let the unit remain off-world. */
+	return FALSE;
+    }
+    if (transport != NULL) {
+	leave_transport(unit);
+	update_unit_display(transport->side, transport, TRUE);
+    } else {
+	remove_unit_from_stack(unit);
+    }
+    add_unit_to_stack(unit, x, y);
+    change_cell_aux(unit, ux, uy, x, y);
+    all_see_leave(unit, ux, uy, (transport == NULL));
+    /* Inevitable side-effect of appearing in the new location. */
+    all_see_occupy(unit, x, y, TRUE);
+    return TRUE;
+}
+
+void
+set_unit_position(unit, x, y, z)
+Unit *unit;
+int x, y;
+{
+    int u, t, tmpz;
+
+    /* Actually set the unit position. */
+    unit->x = x;  unit->y = y;  unit->z = z;
+    /* Constrain the altitude according to terrain if nonzero. */
+    if (unit->z != 0) {
+	if (unit->z & 1 == 0) {
+	    u = unit->type;
+	    t = terrain_at(x, y);
+	    tmpz = unit->z / 2;
+	    tmpz = min(tmpz, ut_alt_max(u, t));
+	    tmpz = max(tmpz, ut_alt_min(u, t));
+	    unit->z = tmpz * 2;
+	} else {
+	    /* (should adjust connection type?) */
+	}
+    }
+}
+
+static void
+change_cell_aux(unit, x0, y0, x, y)
+Unit *unit;
+int x0, y0, x, y;
+{
+    int u = unit->type;
+    Unit *occ;
+
+    /* Stash the old coords. */
+    unit->prevx = x0;  unit->prevy = y0;
+    set_unit_position(unit, x, y, unit->z);
+    /* Change viewing coverage, if unit is complete and can see out
+       of its transport. */
+    if (completed(unit)
+    	&& (unit->transport == NULL
+	    || uu_occ_can_see(u, unit->transport->type))) {
+	cover_area(unit->side, unit, x0, y0, x, y);
+	/* If vision range is 0, allow glimpses of adjacent cell terrain.
+	   This applies to terrain only, adjacent units cannot be seen. */
+	if (u_vision_range(u) == 0)
+	  glimpse_adjacent_terrain(unit);
+    }
+    /* Do for all the occupants too, recursively. */
+    for_all_occupants(unit, occ) {
+	change_cell_aux(occ, x0, y0, x, y);
+    }
+}
+
+/* Put the given unit into the cell and/or unit stack at the
+   given location.  Do not modify the unit's actual xyz position,
+   just the unit layer and links to other units. */
+
+void
+add_unit_to_stack(unit, x, y)
+Unit *unit;
+int x, y;
+{
+    int u = unit->type, ustack, u2stack;
+    Unit *topunit, *unit2, *prevunit = NULL, *nextunit = NULL;
+
+    topunit = unit_at(x, y);
+    if (topunit) {
+    	/* Insert the entering unit into the stack at its correct position. */
+    	ustack = u_stack_order(u);
+    	for_all_stack(x, y, unit2) {
+	    u2stack = u_stack_order(unit2->type);
+	    if (ustack > u2stack
+		|| (ustack == u2stack && unit->id < unit2->id)) {
+		nextunit = unit2;
+		if (unit2 == topunit)
+		  topunit = unit;
+		break;
+	    }
+	    prevunit = unit2;
+    	}
+    	if (prevunit != NULL)
+	  prevunit->nexthere = unit;
+    } else {
+    	topunit = unit;
+    }
+    unit->nexthere = nextunit;
+    set_unit_at(x, y, topunit);
+}
+
+/* Remove the given unit from the stack at its current
+   location. */
+
+void
+remove_unit_from_stack(unit)
+Unit *unit;
+{
+    int ux = unit->x, uy = unit->y;
+    Unit *other;
+
+    /* Unsplice ourselves from the list of units in this cell. */
+    if (unit == unit_at(ux, uy)) {
+	set_unit_at(ux, uy, unit->nexthere);
+    } else {
+	for_all_stack(ux, uy, other) {
+	    if (unit == other->nexthere) {
+		other->nexthere = other->nexthere->nexthere;
+		break;
+	    }
+	} 
+    }
+    /* Bash this now-spurious link. */
+    unit->nexthere = NULL;
+}
+
+void
+glimpse_adjacent_terrain(unit)
+Unit *unit;
+{
+    int x = unit->x, y = unit->y, dir, x1, y1;
+    Side *side = unit->side;
+
+    if (u_vision_range(unit->type) == 0
+	&& unit->transport == NULL
+	&& !all_see_all
+	&& !g_terrain_seen()
+	&& side != NULL) {
+	for_all_directions(dir) {
+		if (point_in_dir(x, y, dir, &x1, &y1)) {
+		    if (terrain_view(side, x1, y1) == UNSEEN) {
+			set_terrain_view(side, x1, y1,
+					 buildtview(terrain_at(x1, y1)));
+			update_cell_display(side, x1, y1, TRUE);
+		    }
+		}
+	}
+    }
+}
+
 /* Given an overfull unit, spew out occupants until back within limits. */
 
 void
@@ -1017,11 +1222,11 @@ int reason;
        viewing coverage gets left on or off inadvertantly. */
     if (alive(unit) && inside_area(ux, uy)) {
 	/* Uncover the current viewed area. */
-	cover_area(unit->side, unit, ux, uy, -1);
+	cover_area(unit->side, unit, ux, uy, -1, -1);
 	/* Actually set the side slot of the unit here. */
 	set_unit_side(unit, newside);
 	/* Cover it for the new side now. */
-	cover_area(unit->side, unit, ux, uy, 1);
+	cover_area(unit->side, unit, -1, -1, ux, uy);
 	/* A freebie for the unit's previous side. */
 	see_exact(oldside, ux, uy);
     }
@@ -1043,10 +1248,30 @@ unit_allowed_on_side(unit, side)
 Unit *unit;
 Side *side;
 {
+    int u;
+
     if (unit == NULL)
       return FALSE;
-    /* (allow for tests on individual units also?) */
-    return type_allowed_on_side(unit->type, side);
+    u = unit->type;
+    /* Test general limitations on the type. */
+    if (!type_allowed_on_side(u, side))
+      return FALSE;
+    /* Test specific game limits. */
+    if (u_type_per_side_max(u) >= 0) {
+	if (side->numunits[u] >= u_type_per_side_max(u))
+	  return FALSE;
+    }
+    if (g_units_per_side_max() >= 0) {
+	int u2, sum;
+
+	sum = 0;
+	for_all_unit_types(u2) {
+	    sum += side->numunits[u2];
+	}
+	if (sum >= g_units_per_side_max())
+	  return FALSE;
+    }
+    return TRUE;
 }
 
 int
@@ -1057,10 +1282,10 @@ Obj *leaf;
 
     if (stringp(leaf)) {
 	sclass = c_string(leaf);
-	if (tmpside != NULL) {
-	    if (empty_string(tmpside->sideclass))
+	if (tmpside_classtest != NULL && tmpside_classtest != indepside) {
+	    if (empty_string(tmpside_classtest->sideclass))
 	      return FALSE;
-	    return (strcmp(sclass, tmpside->sideclass) == 0);
+	    return (strcmp(sclass, tmpside_classtest->sideclass) == 0);
 	} else {
 	    return (strcmp(sclass, "independent") == 0);
 	}
@@ -1076,12 +1301,19 @@ type_allowed_on_side(u, side)
 int u;
 Side *side;
 {
-    Obj *sclass = u_possible_sides(u);
+    if (side == NULL)
+      side = indepside;
+    if (side->uavail == NULL) {
+	int u2;
 
-    /* (could optimize by precomputing bit vectors) */
-    tmputype = u;
-    tmpside = side;
-    return eval_boolean_expression(sclass, test_class_membership, TRUE);
+	side->uavail = (short *) xmalloc(numutypes * sizeof(short));
+	for_all_unit_types(u2) {
+	    tmpside_classtest = side;
+	    side->uavail[u2] =
+	      eval_boolean_expression(u_possible_sides(u2), test_class_membership, TRUE);
+	}
+    }
+    return side->uavail[u];
 }
 
 int
@@ -1132,6 +1364,9 @@ Side *side;
 	  ++(newside->numunits[unit->type]);
 	if (newside->numlive && in_play(unit))
 	  ++(newside->numlive[unit->type]);
+	/* Invalidate both sides' point value caches. */
+	oldside->point_value_valid = FALSE;
+	newside->point_value_valid = FALSE;
     }
     return TRUE;
 }
@@ -1148,96 +1383,6 @@ Side *side;
     return TRUE;
 }
 
-void
-set_unit_plan_type(side, unit, type)
-Side *side;
-Unit *unit;
-int type;
-{
-    int oldtype;
-
-    if (unit->plan) {
-	oldtype = unit->plan->type;
-	if (type != oldtype) {
-	    unit->plan->type = type;
-	    if (side != NULL)
-	      update_unit_display(side, unit, TRUE);
-	}
-    }
-}
-
-void
-set_unit_asleep(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
-{
-    int oldflag;
-    Unit *occ;
-
-    if (unit->plan) {
-	oldflag = unit->plan->asleep;
-	if (flag != oldflag) {
-	    unit->plan->asleep = flag;
-	    if (side != NULL)
-	      update_unit_display(side, unit, TRUE);
-	}
-    }
-    if (recurse) {
-    	for_all_occupants(unit, occ) {
-	    set_unit_asleep(side, occ, flag, recurse);
-    	}
-    }
-}
-
-void
-set_unit_reserve(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
-{
-    int oldflag;
-    Unit *occ;
-
-    if (unit->plan) {
-	oldflag = unit->plan->reserve;
-	if (flag != oldflag) {
-	    unit->plan->reserve = flag;
-	    if (side != NULL)
-	      update_unit_display(side, unit, TRUE);
-	}
-    }
-    if (recurse) {
-    	for_all_occupants(unit, occ) {
-	    set_unit_reserve(side, occ, flag, recurse);
-    	}
-    }
-}
-
-void
-set_unit_ai_control(side, unit, flag, recurse)
-Side *side;
-Unit *unit;
-int flag, recurse;
-{
-    int oldflag;
-    Unit *occ;
-
-    if (unit->plan) {
-	oldflag = unit->plan->aicontrol;
-	if (flag != oldflag) {
-	    unit->plan->aicontrol = flag;
-	    if (side != NULL)
-	      update_unit_display(side, unit, TRUE);
-	}
-    }
-    if (recurse) {
-    	for_all_occupants(unit, occ) {
-	    set_unit_ai_control(side, occ, flag, recurse);
-    	}
-    }
-}
-
 void
 set_unit_name(side, unit, newname)
 Side *side;
@@ -1338,6 +1483,8 @@ int reason;
       record_unit_death(unit, reason);
     remove_unit_from_vector((unit->side ? unit->side : indepside)->actionvector, unit, -1);
     if (side != NULL) {
+	/* Invalidate the side's point value cache. */
+	side->point_value_valid = FALSE;
 	if (side_has_ai(side)) {
 	    ai_react_to_unit_loss(side, unit);
 	}
@@ -1353,90 +1500,51 @@ int reason;
     --(numlivebytype[u]);
 }
 
-/* Do something about the plan object, possibly passing to a deputy unit
-   (not implemented yet), or maybe just destroying it entirely. */
-
-void
-dispose_of_plan(unit)
-Unit *unit;
-{
-    /* Might not be anything to dispose of. */
-    if (unit->plan == NULL)
-      return;
-    free_plan(unit->plan);
-    unit->plan = NULL;
-}
-
-/* Get rid of all dead units at once.  It's important to get rid of all
-   of the dead units, so they don't come back and haunt us.
+/* Get rid of all dead units at once.
    (This routine is basically a garbage collector, and should not be called
    during a unit list traversal.) The process starts by finding the first
    live unit, making it the head, then linking around all in the middle.
    Dead units stay on the dead unit list for each side until that side has had
    a chance to move.  Then they are finally flushed in a permanent fashion. */ 
 
+static void flush_one_unit PARAMS ((Unit *unit));
+
 void
 flush_dead_units()
 {
-#if 0 /* def REUSE_DEAD */
-    Unit *unit, *prevunit;
-    Side *side;
+    Unit *unit, *prevunit, *nextunit;
 
-    /* This never ends up flushing independent units, no big loss. */
-    if (!recent_dead_flushed) {
-	recent_dead_flushed = TRUE;
-	for_all_units(unit) {
-	    if (!alive(unit)) {
-		prevunit = unit->prev;
-		delete_unit(unit);
-		put_unit_on_dead_list(unit);
-		unit = prevunit;
-	    }
+    if (unitlist == NULL)
+      return;
+    unit = unitlist;
+    while (!alive(unit)) {
+	nextunit = unit->unext;
+	delete_unit(unit);
+	flush_one_unit(unit);
+	unit = nextunit;
+    }
+    unitlist = unit;
+    /* Since the first unit of unitlist is guaranteed live now,
+       we know that prevunit will always be set correctly;
+       but mollify insufficiently intelligent compilers. */
+    prevunit = NULL;
+    for_all_units(unit) {
+	if (!alive(unit)) {
+	    nextunit = unit->unext;
+	    prevunit->unext = unit->unext;
+	    delete_unit(unit);
+	    flush_one_unit(unit);
+	    unit = prevunit;
+	} else {
+	    prevunit = unit;
 	}
     }
-#endif
-}
-
-/* Put dead units on sides dead unit lists. */
-
-void
-put_unit_on_dead_list(unit)
-Unit *unit;
-{
-#ifdef REUSE_DEAD
-    if (unit->side == NULL) {
-	flush_one_unit(unit);
-    } else {
-	unit->next = unit->side->deadunits;
-	unit->side->deadunits = unit;
-    }
-    --numunits;
-#endif
-}
-
-/* Clean up dead units and put them back on free list. */
-
-void
-flush_side_dead(side)
-Side *side;
-{
-#ifdef REUSE_DEAD
-    Unit *unit, *next;
- 
-    next = side->deadunits;
-    while (next != NULL) {
-	unit = next;
-	next = unit->next;
-	flush_one_unit(unit);
-    }
-    side->deadunits = NULL;
-#endif
 }
 
 /* Keep it clean - hit all links to other places.  Some might not be
    strictly necessary, but this is not an area to take chances with. */
 
-void
+static void
 flush_one_unit(unit)
 Unit *unit;
 {
@@ -1444,7 +1552,9 @@ Unit *unit;
     unit->occupant = NULL;
     unit->transport = NULL;
     unit->nexthere = NULL;
-    /* Add it on the list of available units. */
+    unit->prev = NULL;
+    unit->unext = NULL;
+    /* Add it on the front of the list of available units. */
     unit->next = freeunits;
     freeunits = unit;
 }
@@ -1534,13 +1644,18 @@ char *
 unit_desig(unit)
 Unit *unit;
 {
+    int i;
     char *shortbuf;
 
+    /* Allocate if not yet done so. */
+    for (i = 0; i < NUMSHORTBUFS; ++i) {
+	if (shortbufs[i] == NULL)
+	  shortbufs[i] = xmalloc(BUFSIZE);
+    }
+    /* Note that we test here, so that unit_desig(NULL) can be used
+       to allocate any space that this routine might need later. */
     if (unit == NULL)
       return "no unit";
-    /* Allocate if not yet done so. */
-    if (shortbufs[curshortbuf] == NULL)
-      shortbufs[curshortbuf] = xmalloc(BUFSIZE);
     shortbuf = shortbufs[curshortbuf];
     curshortbuf = (curshortbuf + 1) % NUMSHORTBUFS;
     if (unit->id == -1) {
@@ -1877,6 +1992,9 @@ int flag;
     int i;
     UnitVector *newvec;
 
+    /* Can't add to something that doesn't exist! */
+    if (vec == NULL)
+      run_error("No actionvector!");
     /* (should search to see if already present) */
     if (vec->numunits >= vec->size) {
 	newvec = make_unit_vector((3 * vec->size) / 2);
@@ -1900,6 +2018,10 @@ int pos;
 {
     int j;
 
+    /* It's probably a bug that the vector is null sometimes,
+       but don't flip out over it. */
+    if (vec == NULL)
+      return;
     /* Search for unit in vector. */
     if (pos < 0) {
 	for (j = 0; j < vec->numunits; ++j) {
@@ -2024,7 +2146,7 @@ int subkey;
 {
     Obj *lis, *bdg;
 
-    for (lis = unit->hook; lis != lispnil; lis = cdr(lis)) {
+    for_all_list(unit_hook(unit), lis) {
 	bdg = car(lis);
 	if (symbolp(car(bdg)) && keyword_code(c_string(car(bdg))) == subkey)
 	  return cdr(bdg);
@@ -2039,7 +2161,7 @@ char *str;
 {
     Obj *lis, *bdg;
 
-    for (lis = unit->hook; lis != lispnil; lis = cdr(lis)) {
+    for_all_list(unit_hook(unit), lis) {
 	bdg = car(lis);
 	if (symbolp(car(bdg)) && strcmp(c_string(car(bdg)), str) == 0)
 	  return cdr(bdg);
@@ -2109,6 +2231,22 @@ int x, y;
     }
 }
 
+int
+designer_change_side(unit, side)
+Unit *unit;
+Side *side;
+{
+    Side *side2;
+
+    change_unit_side(unit, side, -1, NULL);
+    for_all_sides(side2) {
+	if (1 /* side2 should see change */) {
+	    update_unit_display(side2, unit, TRUE);
+	}
+    }
+    return TRUE;
+}
+
 int
 designer_disband(unit)
 Unit *unit;
