@@ -1,5 +1,5 @@
 /* Worlds and areas in Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -98,6 +98,8 @@ int circum, warn;
 {
     /* All world circumferences are valid, no checks necessary. */
     world.circumference = circum;
+    world.daylight_width = ((50 * world.circumference) / 100) / 2;
+    world.twilight_width = ((60 * world.circumference) / 100) / 2;
     area.xwrap = (world.circumference == area.width);
     return TRUE;
 }
@@ -109,8 +111,12 @@ int width, height, warn;
     if (!valid_area_shape(width, height, warn))
       return FALSE;
     area.width = width;  area.height = height;
+    area.halfheight = area.height / 2;
     area.maxdim = max(area.width, area.height);
     area.xwrap = (world.circumference == area.width);
+    area.numcells = area.width * area.height;
+    if (!area.xwrap)
+      area.numcells = (area.numcells * 3) / 4;
     return TRUE;
 }
 
@@ -219,6 +225,40 @@ count_terrain_subtypes()
     }
 }
 
+void
+final_init_world()
+{
+    int x, y, el, count, sum, rowcount, rowsum, rowavg;
+
+    area.minelev = area.maxelev = area.avgelev = 0;
+    if (elevations_defined()) {
+	area.maxelev = area.minelev = elev_at(area.width / 2, area.height / 2);
+	/* In order to avoid overflowing by summing all elevations at
+	   once, we average along a row, then average the row averages. */
+	count = sum = 0;
+	for (y = 0; y < area.height; y++) {
+	    rowcount = rowsum = 0;
+	    for (x = 0; x < area.width; x++) {
+		if (in_area(x, y)) {
+		    el = elev_at(x, y);
+		    if (el < area.minelev)
+		      area.minelev = el;
+		    if (el > area.maxelev)
+		      area.maxelev = el;
+		    ++rowcount;
+		    rowsum += el;
+		}
+	    }
+	    rowavg = rowsum / rowcount;
+	    ++count;
+	    sum += rowavg;
+	}
+	area.avgelev = sum / count;
+    }
+    /* Calculate extent and position of geographical features. */
+    compute_all_feature_centroids();
+}
+
 /* Make space for an area's terrain, and for the unit cache. */
 
 /* This can be postponed until it is actually needed. */
@@ -271,6 +311,9 @@ int n;
     if (n >= 3 && !area.tmp3) {
 	area.tmp3 = malloc_area_layer(short);
     }
+    if (n >= 4) {
+	run_error("can't allocate more than 3 scratch layers");
+    }
 }
 
 /* Should have something to free scratch layers up also maybe. */
@@ -304,14 +347,11 @@ allocate_area_temperatures()
 void
 allocate_area_people_sides()
 {
-    int x, y;
-
     if (!people_sides_defined()) {
 	check_area_shape();
 	area.peopleside = malloc_area_layer(char);
-	for_all_cells(x, y) {
-	    set_people_side_at(x, y, NOBODY);
-	}
+	/* NOBODY != 0, so need to blast it over the layer. */
+	memset(area.peopleside, NOBODY, area.width * area.height);
     }
 }
 
@@ -541,6 +581,83 @@ int x, y, val;
     set_raw_cloud_height_at(x, y, val);
 }
 
+/* Change the terrain of the cell at the given location to the given
+   type of terrain, and update everything affected by the change. */
+
+void
+change_terrain_type(x, y, t2)
+int x, y, t2;
+{
+    int t, curview, newview, update, u2, nu2;
+    HistEventType hevttype;
+    Unit *unit2;
+    Side *side;
+
+    t = terrain_at(x, y);
+    set_terrain_at(x, y, t2);
+    /* Let everybody see this change. */
+    for_all_sides(side) {
+	update = FALSE;
+	if (side->see_all) {
+	    update = TRUE;
+	} else if (cover(side, x, y) > 0) {
+	    /* Always update our knowledge of the terrain. */
+	    curview = terrain_view(side, x, y);
+	    newview = buildtview(t2);
+	    set_terrain_view(side, x, y, newview);
+	    if (!g_see_terrain_always())
+	      set_terrain_view_date(side, x, y, g_turn());
+	    if (newview != curview)
+	      update = TRUE;
+	}
+	if (update) {
+	    update_cell_display(side, x, y, TRUE);
+	}
+    }
+    /* There might be catastrophic consequences for any units here. */
+    for_all_stack(x, y, unit2) {
+	u2 = unit2->type;
+	if (ut_vanishes_on(u2, t2)
+	    && !can_occupy_conn(unit2, unit2->x, unit2->y, unit2->z)) {
+	    hevttype = H_UNIT_VANISHED;
+	    kill_unit(unit2, hevttype);
+	} else if (ut_wrecks_on(u2, t2)
+		   && !can_occupy_conn(unit2, unit2->x, unit2->y, unit2->z)) {
+	    if (u_wrecked_type(u2) == NONUTYPE) {
+		/* Occupants always die if the wrecked unit disappears. */
+		hevttype = H_UNIT_WRECKED;
+		kill_unit(unit2, hevttype);
+	    } else {
+		/* Change the unit's type. */
+		hevttype = H_UNIT_WRECKED;
+		change_unit_type(unit2, u_wrecked_type(u2), hevttype);
+		nu2 = unit2->type;
+		/* Restore to default hp for the new type. */
+		unit2->hp = unit2->hp2 = u_hp(nu2);
+		/* Get rid of occupants if now overfull. */
+		eject_excess_occupants(unit2);
+		/* Maybe make it go away, taking remaining unlucky
+                   occupants with. */
+		if (ut_vanishes_on(nu2, t2)) {
+		    hevttype = H_UNIT_VANISHED;
+		    kill_unit(unit2, hevttype);
+		}
+	    }
+	}
+	if (active_display(unit2->side)) {
+	    if (hevttype == H_UNIT_VANISHED) {
+		notify(unit2->side, "Terrain change from %s to %s causes %s to vanish",
+		       t_type_name(t), t_type_name(t2),
+		       unit_handle(unit2->side, unit2));
+	    } else if (hevttype == H_UNIT_WRECKED) {
+		notify(unit2->side, "Terrain change from %s to %s wrecks %s",
+		       t_type_name(t), t_type_name(t2),
+		       unit_handle(unit2->side, unit2));
+	    }
+	}
+    }
+}
+
 /* Generalized area search routine.  It starts in the immediately adjacent
    cells and expands outwards.  The basic structure is to examine successive
    "rings" out to the max distance; within each ring, we must scan each of
@@ -588,10 +705,9 @@ int x0, y0, maxdist, (*pred) PARAMS ((int, int)), *rxp, *ryp, incr;
 
 int
 search_and_apply(x0, y0, maxdist, pred, rxp, ryp, incr, fn, num)
-int x0, y0, maxdist, *rxp, *ryp, incr;
+int x0, y0, maxdist, *rxp, *ryp, incr, num;
 int (*pred) PARAMS ((int, int));
 void (*fn) PARAMS ((int, int));
-long num;
 {
     int clockwise, dist, x0w, dd, d, dir, x1, y1, i, dir2, x, y, xw;
 
@@ -747,14 +863,16 @@ void (*fn) PARAMS ((int, int));
 /* Apply a function all along a path. */
 
 void
-apply_to_path(fx, fy, tx, ty, dirfn, fn, shortest)
+apply_to_path(fx, fy, tx, ty, dirtest, dirsort, fn, shortest)
 int fx, fy, tx, ty, shortest;
-int (*dirfn) PARAMS ((int, int, int *, int));
-int (*fn) PARAMS ((int, int, int, int, int));
+int (*dirtest) PARAMS ((int x, int y, int dir));
+int (*dirsort) PARAMS ((int x, int y, int *dirchoices, int numchoices));
+int (*fn) PARAMS ((int x, int y, int dir, int j, int numchoices));
 {
-    int i = 500, j, x = fx, y = fy;
-    int dx, dxa, dy, d[NUMDIRS], axis, hextant, tmp, sig;
-    int numchoices;
+    int i = 500, j, x = fx, y = fy, moved;
+    int dx, dxa, dy, dirchoices[NUMDIRS], axis, hextant, tmp, sig;
+    int d1, d2, d3, d4;
+    int numchoices, shortestnumchoices;
 
     while (!(x == tx && y == ty) && i-- > 0 /* safety */) {
 	dx = tx - x;  dy = ty - y;
@@ -783,50 +901,74 @@ int (*fn) PARAMS ((int, int, int, int, int));
 	    hextant = (dy < 0 ? WEST :
 		       (ABS(dx) > ABS(dy) ? NORTHWEST : NORTHEAST));
 	}
+	numchoices = 0;
 	/* On an axis, there's no choice. */
 	if (axis >= 0) {
-	    d[0] = d[1] = axis;
-	    numchoices = (shortest ? 1 : 3);
+	    d1 = d2 = axis;
+	    if (dirtest == NULL || (*dirtest)(x, y, d1))
+	      dirchoices[numchoices++] = axis;
 	}
 	/* Two choices in the middle of a hextant. */
 	if (hextant >= 0) {
-	    d[0] = left_dir(hextant);
-	    d[1] = hextant;
-	    numchoices = (shortest ? 2 : 4);
-	}
-	/* If we don't have to pick a shortest path, we have two more
-	   directions to try. */
-	if (!shortest) {
-	    d[2] = left_dir(d[0]);
-	    d[3] = right_dir(d[1]);
-	}
-	if (dirfn != NULL) {
-	    if ((numchoices = (*dirfn)(x, y, d, numchoices)) <= 0)
-	      return;
-	} else {
-	    /* Be a little random in our choice of directions to try first. */
-	    if (flip_coin()) {
-		tmp = d[0];  d[0] = d[1];  d[1] = tmp;
-		tmp = d[2];  d[2] = d[3];  d[3] = tmp;
-	    }
+	    d1 = left_dir(hextant);
+	    d2 = hextant;
+	    if (dirtest == NULL || (*dirtest)(x, y, d1))
+	      dirchoices[numchoices++] = d1;
+	    if (dirtest == NULL || (*dirtest)(x, y, d2))
+	      dirchoices[numchoices++] = d2;
 	}
+	/* Sort the choices if requested. */
+	if (numchoices > 1 && dirsort != NULL)
+	  numchoices = (*dirsort)(x, y, dirchoices, numchoices);
 	/* Try each of the directions. */
 	if (!inside_area(x, y))
 	  return;
+	moved = FALSE;
 	for (j = 0; j < numchoices; ++j) {
-	    sig = (*fn)(x, y, d[j], j, numchoices);
+	    sig = (*fn)(x, y, dirchoices[j], j, numchoices);
 	    if (sig > 0) {
 		/* It's cool - go with this dir. */
 		/* Jump along to the new spot on the path. */
-		x += dirx[d[j]];  y += diry[d[j]];
+		x += dirx[dirchoices[j]];  y += diry[dirchoices[j]];
+		moved = TRUE;
 		/* Out of the loop. */
 		break;
 	    } else if (sig < 0) {
+		/* Stop applying to the path right now. */
 		return;
 	    } else {
 		/* Try another. */
 	    }
 	}
+	/* If we don't have to pick a shortest path, we have two more
+	   directions to try if we need to. */
+	if (!moved && !shortest) {
+	    shortestnumchoices = numchoices;
+	    d3 = left_dir(d1);
+	    d4 = right_dir(d2);
+	    if (dirtest == NULL || (*dirtest)(x, y, d3))
+	      dirchoices[numchoices++] = d3;
+	    if (dirtest == NULL || (*dirtest)(x, y, d4))
+	      dirchoices[numchoices++] = d4;
+	    if (numchoices > shortestnumchoices + 1 && dirsort != NULL)
+	      (*dirsort)(x, y, dirchoices + shortestnumchoices, numchoices - shortestnumchoices);
+	    /* Try each of the directions. */
+	    for (j = shortestnumchoices; j < numchoices; ++j) {
+		sig = (*fn)(x, y, dirchoices[j], j, numchoices - shortestnumchoices);
+		if (sig > 0) {
+		    /* It's cool - go with this dir. */
+		    /* Jump along to the new spot on the path. */
+		    x += dirx[dirchoices[j]];  y += diry[dirchoices[j]];
+		    /* Out of the loop. */
+		    break;
+		} else if (sig < 0) {
+		    /* Stop applying to the path right now. */
+		    return;
+		} else {
+		    /* Try another. */
+		}
+	    }
+	}
     }
 }
 
@@ -868,44 +1010,7 @@ Waypoint *waypoints;
 }
 #endif
 
-/* Test whether x,y is a valid position anywhere in the current area. */
-
-int
-in_area(x, y)
-int x, y;
-{
-    return (between(0, y, area.height-1) &&
-	    (area.xwrap ? TRUE : (between(0, x, area.width-1) &&
-				   between(area.height/2+0,
-					   x+y,
-					   area.width+area.height/2-1))));
-}
-
-/* This is true if the given x, y position is a valid position for units. */
-
-/* Does x testing work right for even/odd heights? */
-
-int
-inside_area(x, y)
-int x, y;
-{
-    return (between(1, y, area.height-2) &&
-	    (area.xwrap ? TRUE : (between(1, x, area.width-2) &&
-				   between(area.height/2+1,
-					   x+y,
-					   area.width+area.height/2-2))));
-}
-
-int
-area_cells()
-{
-    int rslt = area.width * area.height;
-
-    if (!area.xwrap)
-      rslt = (rslt * 3) / 4;
-    return rslt;
-}
-	    
+#if 0	    
 /* Find the type of the border on the given side of the given hex. */
 
 int
@@ -921,6 +1026,7 @@ int x, y, dir, t;
     bord = aux_terrain_at(x, y, t);
     return (bord & (1 << dir));
 }
+#endif
 
 /* For now, set a bit on both sides of a border. */
 
@@ -931,20 +1037,24 @@ int x, y, dir, t, onoff;
     int ox, oy, bord, obord;
     int odir = opposite_dir(dir);
 
-    if (!inside_area(x, y) || !t_is_border(t))
+    if (!t_is_border(t))
       return;
-    onoff = (onoff ? 1 : 0);  /* make sure it's one bit */
+    if (!point_in_dir(x, y, dir, &ox, &oy)) {
+	run_warning("border on outside of world at %d,%d, can't set", x, y);
+	return;
+    }
     allocate_area_aux_terrain(t);
+    onoff = (onoff ? 1 : 0);  /* make sure it's one bit */
     bord = aux_terrain_at(x, y, t);
     bord = ((onoff << dir) | (bord & ~(1 << dir)));
     set_aux_terrain_at(x, y, t, bord);
     /* Go to the other cell and tweak its border bits. */
-    point_in_dir(x, y, dir, &ox, &oy);
     obord = aux_terrain_at(ox, oy, t);
     obord = ((onoff << odir) | (obord & ~(1 << odir)));
     set_aux_terrain_at(ox, oy, t, obord);
 }
 
+#if 0
 /* Find the type of the connection on the given side of the given cell. */
 
 int
@@ -960,6 +1070,7 @@ int x, y, dir, t;
     conn = aux_terrain_at(x, y, t);
     return (conn & (1 << dir));
 }
+#endif
 
 /* For now, set a bit on both sides of a connection. */
 
@@ -970,15 +1081,18 @@ int x, y, dir, t, onoff;
     int ox, oy, conn, oconn;
     int odir = opposite_dir(dir);
 
-    if (!inside_area(x, y) || !t_is_connection(t))
+    if (!t_is_connection(t))
       return;
+    if (!point_in_dir(x, y, dir, &ox, &oy)) {
+	run_warning("connection to outside of world at %d,%d, can't set", x, y);
+	return;
+    }
     allocate_area_aux_terrain(t);
     onoff = (onoff ? 1 : 0);  /* make sure it's one bit */
     conn = aux_terrain_at(x, y, t);
     conn = ((onoff << dir) | (conn & ~(1 << dir)));
     set_aux_terrain_at(x, y, t, conn);
     /* Go to the other cell and tweak its connection bits. */
-    point_in_dir(x, y, dir, &ox, &oy);
     oconn = aux_terrain_at(ox, oy, t);
     oconn = ((onoff << odir) | (oconn & ~(1 << odir)));
     set_aux_terrain_at(ox, oy, t, oconn);
@@ -992,7 +1106,7 @@ void
 patch_linear_terrain(t)
 int t;
 {
-    int x, y, dir;
+    int x, y, dir, x1, y1;
 	
     if (t_is_border(t)) {
 	for_all_cells(x, y) {
@@ -1001,7 +1115,9 @@ int t;
 	       will be fixed up later on, when an adjacent cell is patched. */
 	    if (aux_terrain_at(x, y, t) != 0) {
 		for_all_directions(dir) {
-		    if (border_at(x, y, dir, t))
+		    if (border_at(x, y, dir, t)
+			&& point_in_dir(x, y, dir, &x1, &y1)
+			&& !border_at(x1, y1, opposite_dir(dir), t))
 		      set_border_at(x, y, dir, t, TRUE);
 		}
 	    }
@@ -1010,7 +1126,9 @@ int t;
 	for_all_cells(x, y) {
 	    if (aux_terrain_at(x, y, t) != 0) {
 		for_all_directions(dir) {
-		    if (connection_at(x, y, dir, t))
+		    if (connection_at(x, y, dir, t)
+			&& point_in_dir(x, y, dir, &x1, &y1)
+			&& !connection_at(x1, y1, opposite_dir(dir), t))
 		      set_connection_at(x, y, dir, t, TRUE);
 		}
 	    }
@@ -1269,6 +1387,64 @@ int *xp, *yp;
     return FALSE;
 }
 
+/* Return a random point guaranteed on the edge of the area. */
+
+int
+random_edge_point(xp, yp)
+int *xp, *yp;
+{
+    int tries = 500, ratio, val;
+
+    while (tries-- > 0) {
+	if (area.xwrap) {
+	    *xp = xrandom(area.width);
+	    *yp = (flip_coin() ? 0 : area.height - 1);
+	} else {
+	    ratio = ((area.width - area.halfheight) * 100) / (area.width + area.halfheight);
+	    if (probability(ratio)) {
+		/* Pick along top or bottom edge. */
+		if (flip_coin()) {
+		    *xp = xrandom(area.width - area.halfheight) + area.halfheight;
+		    *yp = 0;
+		} else {
+		    *xp = xrandom(area.width - area.halfheight);
+		    *yp = area.height - 1;
+		}
+	    } else {
+		/* Pick along right or left side. */
+		if (flip_coin()) {
+		    /* Pick along right side. */
+		    if (flip_coin()) {
+			/* Pick along upper right side. */
+			val = xrandom(area.halfheight);
+			*xp = area.width - val;
+			*yp = area.halfheight + val;
+		    } else {
+			/* Pick along lower right side. */
+			*xp = area.width;
+			*yp = xrandom(area.halfheight);
+		    }
+		} else {
+		    /* Pick along left side. */
+		    if (flip_coin()) {
+			/* Pick along upper left side. */
+			*xp = 0;
+			*yp = area.halfheight + xrandom(area.halfheight);
+		    } else {
+			/* Pick along lower left side. */
+			val = xrandom(area.halfheight);
+			*xp = val;
+			*yp = area.halfheight - val;
+		    }
+		}
+	    }
+	}
+	if (in_area(*xp, *yp) && !inside_area(*xp, *yp))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
 /* Return a random point guaranteed to be within a given radius of
    a given point. */
 
@@ -1397,6 +1573,34 @@ int x1, y1, x2, y2;
     }
 }
 
+int
+world_distance(x1, y1, x2, y2)
+int x1, y1, x2, y2;
+{
+    int dx = x2 - x1, dy = y2 - y1;
+
+    /* Choose the shortest way around the world. */
+    dx = (dx < 0 ? (dx < 0 - world.circumference / 2 ? world.circumference + dx : dx)
+	         : (dx > world.circumference / 2 ? dx - world.circumference : dx));
+    if (dx >= 0) {
+	if (dy >= 0) {
+	    return (dx + dy);
+	} else if ((0 - dy) <= dx) {
+	    return dx;
+	} else {
+	    return (0 - dy);
+	}
+    } else {
+	if (dy <= 0) {
+	    return (0 - (dx + dy));
+	} else if (dy <= (0 - dx)) {
+	    return (0 - dx);
+	} else {
+	    return dy;
+	}
+    }
+}
+
 /* Find the direction matching the given x and y, return -1 if no
    match.  Callers should be careful to test for this! */
 
@@ -1453,8 +1657,12 @@ int x, y, dir, t, mode;
     oldbord = border_at(x, y, dir, t);
     set_border_at(x, y, dir, t, (mode < 0 ? !oldbord : mode));
     if (oldbord != border_at(x, y, dir, t)) {
+        /* We do both a see_exact and update because see_exact doesn't
+           account for linear terrain changes. */
 	see_exact(side, x, y);
+	update_cell_display(side, x, y, TRUE);
 	see_exact(side, x+dirx[dir], y+diry[dir]);
+	update_cell_display(side, x+dirx[dir], y+diry[dir], TRUE);
     }
 }
 
@@ -1471,8 +1679,12 @@ int x, y, dir, t, mode;
     oldconn = connection_at(x, y, dir, t);
     set_connection_at(x, y, dir, t, (mode < 0 ? !oldconn : mode));
     if (oldconn != connection_at(x, y, dir, t)) {
+        /* We do both a see_exact and update because see_exact doesn't
+           account for linear terrain changes. */
 	see_exact(side, x, y);
+	update_cell_display(side, x, y, TRUE);
 	see_exact(side, x+dirx[dir], y+diry[dir]);
+	update_cell_display(side, x+dirx[dir], y+diry[dir], TRUE);
     }
 }
 
