@@ -118,7 +118,7 @@ static int cpu_share_rot_cnt;				/* When this reaches _maccur_io_sleep_dvsr its
 
 static void maccur_init_toolbox(void)
 {
-	InitGraf(&thePort);
+	InitGraf(&QD(thePort));
 	InitFonts();
 	FlushEvents(everyEvent, 0);
 	InitWindows();
@@ -519,7 +519,7 @@ EventRecord *event_ptr		/* -R- The event details. */
 
 		case inDrag:
 			DragWindow(wndo_ptr, event_ptr->where,
-								&screenBits.bounds);
+								&QD(screenBits).bounds);
 			break;
 	}
 }
@@ -560,6 +560,11 @@ int until_flags
 	{
 		if (WaitNextEvent(everyEvent, &event, sleep, nil))
 		{
+
+#ifdef __MWERKS__
+		  if (!SIOUXHandleOneEvent(&event)) {
+#endif
+
 			switch (event.what)
 			{
 				case kHighLevelEvent:
@@ -628,6 +633,9 @@ int until_flags
 					break;
 
 			}
+#ifdef __MWERKS__
+		  }
+#endif
 		}
 		else if (TST_BIT(until_flags, MACCUR_RETURN_ON_IDLE_BIT))		/* If just letting CPU run, this is the cue to split. */
 			return MACCUR_RETURN_ON_IDLE_BIT;
