@@ -1,5 +1,5 @@
 /* Commands for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,16 +7,26 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int disband_unit PARAMS ((Side *side, Unit *unit));
+extern void update_debugging PARAMS ((void));
+extern void toggle_debugging PARAMS ((int *flagp));
+extern int can_build_or_help PARAMS ((Unit *unit));
+extern int can_produce PARAMS ((Unit *unit));
+extern Task *create_produce_task PARAMS ((int m, int n));
+extern void push_produce_task PARAMS ((Unit *unit, int m, int n));
+/* Only defined if ansi */
+extern void vnotify(Side *side, char *fmt, va_list ap);
 #include "print.h"
 #include "macconq.h"
+extern void enable_command(void);
+extern void set_construction_run_length(int len);
+
+/* Define in order to get profiling controls. */
+#undef PROFILING
 
 #ifdef THINK_C
 #include <profile.h>
 #endif
 
-extern int Profile;
-
 #ifdef PROFILING
 extern int _trace;
 #endif
@@ -56,6 +66,7 @@ extern char *cursavename;
 
 static int do_one_add_terrain(Unit *unit);
 static int do_one_ai_control(Unit *unit);
+static void unit_do_build_2(Unit *unit);
 extern int do_one_clear_plan(Unit *unit);
 static int do_one_delay(Unit *unit);
 extern int do_one_detach(Unit *unit);
@@ -63,6 +74,8 @@ extern int do_one_detonate(Unit *unit);
 extern int do_one_dir_move(Unit *unit);
 extern int do_one_dir_multiple_move(Unit *unit);
 extern int do_one_disband(Unit *unit);
+extern int do_one_disembark(Unit *unit);
+extern int do_one_embark(Unit *unit);
 extern int do_one_give(Unit *unit);
 extern int do_one_give_unit(Unit *unit);
 static int do_one_occupant(Unit *unit);
@@ -176,6 +189,7 @@ execute_named_command(char *cmdstr)
 
     parse_long_name_command(cmdstr, &cmdname, &cmdargstr, copy_string(cmdstr)); 
     if (empty_string(cmdname)) {
+		notify(dside, "No command name.");
 		beep();
 		return;
 	}
@@ -194,6 +208,7 @@ execute_named_command(char *cmdstr)
 			return;
 		}
 	}
+	notify(dside, "Command \"%s\" not recognized.", cmdname);
 	beep();
 }
 
@@ -256,7 +271,7 @@ get_a_unit(Map **mapp, Unit **unitp)
 void
 do_add_player()
 {
-	cmd_error(dside, "add-player not yet available");
+    request_additional_side(cmdargstr);
 }
 
 void
@@ -282,22 +297,23 @@ do_one_add_terrain(Unit *unit)
     numtypes = 0;
 	for_all_terrain_types(t) {
 		if (ut_acp_to_add_terrain(u, t) > 0) {
-			tfirst = t;
-			++numtypes;
-			possibles[t] = TRUE;
-		} else {
-			possibles[t] = FALSE;
+			possibles[numtypes++] = t;
 		}
 	}
 	if (numtypes == 0) {
-	  return FALSE;
+		return FALSE;
 	} else if (numtypes == 1) {
-		t1 = tfirst;
+		t1 = possibles[0];
 	} else {
 		/* should have better way to choose type */
-		if (prefixarg >= 0)
-		  t1 = prefixarg;
+		if (between(0, prefixarg, numtypes - 1))
+		  t1 = possibles[prefixarg];
+		else {
+			notify(dside, "Prefix should be 0 to %d", numtypes - 1); 
+			return FALSE;
+		}
 	}
+	/* (should put following in ui.c?) */
 	switch (t_subtype(t1)) {
 		case cellsubtype:
 			if (valid(check_alter_cell_action(unit, unit, tmpcmdx, tmpcmdy, t1))) {
@@ -366,72 +382,64 @@ do_one_ai_control(Unit *unit)
 	return TRUE;
 }
 
-int favored_type(Unit *unit);
-
 void
 do_build()
 {
 	int i;
 	Map *map;  List *list;  UnitCloseup *closeup;
 	Unit *unit;
-	
-	if ((map = map_from_window(FrontWindow())) != NULL) {
+
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
 		for (i = 0; i < map->numselections; ++i) {
-			if ((unit = map->selections[i]) != NULL) {
-				if (can_build(unit)) {  /* and completers and researchers too? */
-					modal_construction = TRUE;
-					window_behind_construction = FrontWindow();
-					enable_construction();
-					select_unit_in_construction_window(unit);
-					select_type_in_construction_window(favored_type(unit));
-					return;
-				}
+			unit = map->selections[i];
+			if (unit != NULL && can_build_or_help(unit)) {
+				unit_do_build_2(unit);
+				return;
 			}
 		}
-	} else if ((list = list_from_window(FrontWindow())) != NULL) {
-		if ((unit = (Unit *) selected_unit_in_list(list)) != NULL) {
-				if (can_build(unit)) {  /* and completers and researchers too? */
-					modal_construction = TRUE;
-					enable_construction();
-					select_unit_in_construction_window(unit);
-					select_type_in_construction_window(favored_type(unit));
-					return;
-				}
-		}
-	} else if ((closeup = unit_closeup_from_window(FrontWindow())) != NULL) {
-		if ((unit = closeup->unit) != NULL) {
-				if (can_build(unit)) {  /* and completers and researchers too? */
-					modal_construction = TRUE;
-					enable_construction();
-					select_unit_in_construction_window(unit);
-					select_type_in_construction_window(favored_type(unit));
-					return;
-				}
-		}
+		return;
 	}
+	list = list_from_window(FrontWindow());
+	if (list != NULL) {
+		unit = selected_unit_in_list(list);
+		if (unit != NULL && can_build_or_help(unit)) {
+			unit_do_build_2(unit);
+			return;
+		}
+		return;
+	}
+	closeup = unit_closeup_from_window(FrontWindow());
+	if (closeup != NULL) {
+		unit = closeup->unit;
+		if (unit != NULL && can_build_or_help(unit)) {
+			unit_do_build_2(unit);
+			return;
+		}
+		return;
+	}
+	/* No way to figure out the unit to be building with, so complain. */
+	beep();
 }
 
-/* (should be in kernel?) */
-
-int
-favored_type(Unit *unit)
+void
+unit_do_build_2(Unit *unit)
 {
-	int u;
+    extern int editedrunlength;
 
-	if (unit == NULL)
-	  return NONUTYPE;
-	if (unit->plan
-		&& unit->plan->tasks
-		&& unit->plan->tasks->type == TASK_BUILD)
-	  return unit->plan->tasks->args[0];
-	for_all_unit_types(u) {
-		if (uu_acp_to_create(unit->type, u) > 0)
-		  return u;
-	}
-	return NONUTYPE;
+	modal_construction = TRUE;
+	window_behind_construction = FrontWindow();
+	if (prefixarg > 0)
+	  set_construction_run_length(prefixarg);
+	if (prefixarg > 0)
+	  editedrunlength = prefixarg;
+	else
+	  editedrunlength = -1;
+	enable_construction();
+	select_unit_in_construction_window(unit);
+	select_type_in_construction_window(favored_type(unit));
 }
 
-
 /* Create and/or bring up the construction planning window. */
 
 void
@@ -467,8 +475,7 @@ do_clear_plan()
 void
 do_copying()
 {
-	/* (should be a help node?) */
-	beep();
+	help_dialog(copying_help_node);
 }
 
 void
@@ -536,18 +543,15 @@ do_one_dir_move(Unit *unit)
 void
 do_dir()
 {
-	char *rawdir;
+	int ndirs, dir1, dir2, modif;
 	Map *map;
 
-	if ((map = map_from_window(FrontWindow())) != NULL) {
-		rawdir = strchr(dirchars, tmpkey);
-		if (rawdir)
-		  tmpdir = rawdir - dirchars;
-		else if (tmpkey == 'k')
-		  tmpdir = (flip_coin() ? NORTHEAST : NORTHWEST);
-		else if (tmpkey == 'j')
-		  tmpdir = (flip_coin() ? SOUTHEAST : SOUTHWEST);
-		else {
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
+		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
+		if (ndirs >= 1) {
+			tmpdir = dir1;
+		} else {
 			beep();
 			return;
 		}
@@ -563,7 +567,7 @@ int
 do_one_dir_multiple_move(Unit *unit)
 {
 	if (mobile(unit->type)) {
-		set_movedir_task(unit, tmpdir, 9999);
+		set_move_dir_task(unit, tmpdir, (prefixarg <= 0 ? 9999 : prefixarg));
 		return TRUE;
 	}
 	return FALSE;
@@ -574,18 +578,15 @@ do_one_dir_multiple_move(Unit *unit)
 void
 do_dir_multiple()
 {
-	char *rawdir;
+	int ndirs, dir1, dir2, modif;
 	Map *map;
 
-	if ((map = map_from_window(FrontWindow())) != NULL) {
-		rawdir = strchr(dirchars, tolower(tmpkey));
-		if (rawdir)
-		  tmpdir = rawdir - dirchars;
-		else if (tmpkey == 'k')
-		  tmpdir = (flip_coin() ? NORTHEAST : NORTHWEST);
-		else if (tmpkey == 'j')
-		  tmpdir = (flip_coin() ? SOUTHEAST : SOUTHWEST);
-		else {
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
+		ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
+		if (ndirs >= 1) {
+			tmpdir = dir1;
+		} else {
 			beep();
 			return;
 		}
@@ -605,10 +606,43 @@ do_one_disband(Unit *unit)
     return disband_unit(dside, unit);
 }
 
+void
+do_disembark()
+{
+	apply_to_all_selected(do_one_disembark, TRUE);
+}
+
+int
+do_one_disembark(Unit *unit)
+{
+	Unit *transport = unit->transport;
+
+	if (transport == NULL)
+	  return FALSE;
+	/* Try moving into the transport's transport, if there is one. */
+	if (transport->transport != NULL
+		&& can_occupy(unit, transport->transport)) {
+		prep_enter_action(unit, unit, transport->transport);
+		/* (should be able to set up task if can't do action immediately) */
+		return TRUE;
+	}
+	/* Try moving into the open in the cell. */
+	if (!inside_area(unit->x, unit->y))
+	  return FALSE;
+	if (can_occupy_cell(unit, unit->x, unit->y)
+		|| can_occupy_conn(unit, unit->x, unit->y, unit->z)) {
+		prep_move_action(unit, unit, unit->x, unit->y, unit->z);
+		/* (should be able to set up task if can't do action immediately) */
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void
 do_distance()
 {
-	cmd_error(dside, "distance command not yet available");
+    notify(dside, "Click, then drag to location to which you want the distance.");
+	query_position_modally(DISTANCE_MODAL);
 }
 
 void
@@ -637,7 +671,65 @@ do_draw_willingness()
 void
 do_embark()
 {
-	beep();
+	apply_to_all_selected(do_one_embark, TRUE);
+}
+
+int
+do_one_embark(Unit *unit)
+{
+	int x = unit->x, y = unit->y;
+	Unit *transport = unit->transport, *unit2, *unit3;
+
+	/* Note that occupation tests check for unit == transport case,
+	   so not necessary in the code below. */
+	if (transport == NULL) {
+		/* Unit is in the open. */
+		for_all_stack(x, y, unit2) {
+			if (unit2 != transport && can_occupy(unit, unit2)) {
+				prep_enter_action(unit, unit, unit2);
+				return TRUE;
+			}
+		}
+		for_all_stack(x, y, unit2) {
+			for_all_occupants(unit2, unit3) {
+				if (unit3 != transport && can_occupy(unit, unit3)) {
+					prep_enter_action(unit, unit, unit3);
+					return TRUE;
+				}
+			}
+		}
+	} else {
+		for_all_occupants(transport, unit2) {
+			if (unit2 != transport && can_occupy(unit, unit2)) {
+				prep_enter_action(unit, unit, unit2);
+				return TRUE;
+			}
+		}
+		if (transport->transport == NULL) {
+			for_all_stack(x, y, unit2) {
+				if (unit2 != transport && can_occupy(unit, unit2)) {
+					prep_enter_action(unit, unit, unit2);
+					return TRUE;
+				}
+			}
+			for_all_stack(x, y, unit2) {
+				for_all_occupants(unit2, unit3) {
+					if (unit3 != transport && can_occupy(unit, unit3)) {
+						prep_enter_action(unit, unit, unit3);
+						return TRUE;
+					}
+				}
+			}
+		} else {
+			for_all_occupants(transport->transport, unit2) {
+				if (unit2 != transport && can_occupy(unit, unit2)) {
+					prep_enter_action(unit, unit, unit2);
+					return TRUE;
+				}
+			}
+		}
+	}
+	return FALSE;
 }
 
 void
@@ -680,6 +772,12 @@ do_follow_action()
 	}
 }
 
+void
+do_force_global_replan()
+{
+	force_global_replan(dside);
+}
+
 void
 do_give()
 {
@@ -700,7 +798,9 @@ do_give_unit()
 void
 do_help()
 {
-	help_dialog();
+    /* Bring up the help window at wherever it was, if already
+       created, otherwise will start at topics node. */
+	help_dialog(NULL);
 }
 
 void
@@ -748,7 +848,37 @@ message_dialog()
 void
 do_move_to()
 {
-	beep();
+    notify(dside, "Click on location to which you want to move.");
+	query_position_modally(MOVE_TO_MODAL);
+}
+
+extern int do_one_move_to(Unit *unit);
+
+int
+do_move_to_command()
+{
+	int x, y, dir;
+	Map *map;
+
+	if (get_a_position(&map, &x, &y, &dir)) {
+		tmpcmdx = x;  tmpcmdy = y;
+		apply_to_all_selected(do_one_move_to, TRUE);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+int
+do_one_move_to(Unit *unit)
+{
+#ifdef DESIGNERS
+	if (dside->designer) {
+		designer_teleport(unit, tmpcmdx, tmpcmdy, NULL);
+		return TRUE;
+	}
+#endif /* DESIGNERS */
+	set_move_to_task(unit, tmpcmdx, tmpcmdy);
+	return TRUE;
 }
 
 int
@@ -771,7 +901,8 @@ do_one_occupant(Unit *unit)
 
 	nextocc = find_next_occupant(unit);
 	if (nextocc != unit) {
-		if ((map = map_from_window(FrontWindow())) != NULL) {
+		map = map_from_window(FrontWindow());
+		if (map != NULL) {
 			unselect_unit_on_map(map, unit);
 			select_unit_on_map(map, nextocc);
 			/* Overkill, but draw_row alone not as desirable? */
@@ -793,7 +924,7 @@ do_other()
 	/* Don't allow recursion with this command. */
 	if (!doingother) {
 		doingother = TRUE;
-		command_dialog();
+		enable_command();
 		doingother = FALSE;
 	} else {
 		beep();
@@ -802,35 +933,115 @@ do_other()
 
 /* Dialog for the input of a textual command. */
 
-void
-command_dialog()
-{
-	short done = FALSE, ditem;
-	char *cmd = NULL;
-	DialogPtr win;
-	short itemtype;  Handle itemhandle;  Rect itemrect;
+WindowPtr commandwin;
 
-	win = GetNewDialog(dCommand, NULL, (DialogPtr) -1L);
-	ShowWindow(win);
-	while (!done) {
-		draw_default_button(win, diCommandOK);
-		SetCursor(&QD(arrow));
-		ModalDialog(NULL, &ditem);
-		switch (ditem) {
-			case diCommandOK:
-				GetDItem(win, diCommandText, &itemtype, &itemhandle, &itemrect);
-				cmd = get_string_from_item(itemhandle);
-				/* Fall into next case. */
-			case diCommandCancel:
-				done = TRUE;
-				break;
-		}
+ControlHandle dobutton;
+
+TEHandle command_text = nil;
+
+Rect commandtextrect;
+
+void create_command_dialog(void);
+
+static void get_command_and_do(void);
+
+void
+enable_command()
+{
+	if (commandwin == nil) {
+		create_command_dialog();
 	}
-	/* Close down the dialog (*before* executing any command). */
-	DisposDialog(win);
-	/* Now do the command (if it wasn't cancelled) */
-	if (cmd != NULL) {
-		execute_named_command(cmd);
+	if (commandwin != nil) {
+		ShowWindow(commandwin);
+		SelectWindow(commandwin);
+	}
+}
+
+void
+create_command_dialog()
+{
+	Rect tmprect;
+
+	if (hasColorQD) {
+		commandwin = GetNewCWindow(wCommand, NULL, (WindowPtr) -1L);
+	} else {
+		commandwin = GetNewWindow(wCommand, NULL, (WindowPtr) -1L);
+	}
+	dobutton = GetNewControl(cCommandDoButton, commandwin);
+	SetPort(commandwin);
+	tmprect = commandwin->portRect;
+	commandtextrect = tmprect;
+	commandtextrect.bottom -= 30;
+	InsetRect(&commandtextrect, 5, 5);
+	/* Create and clear the TextEdit record. */
+	command_text = TENew(&commandtextrect, &commandtextrect);
+	TESetSelect(0, 32767, command_text);
+	TEDelete(command_text);
+	ShowWindow(commandwin);
+}
+
+void
+draw_command()
+{
+	Rect tmprect;
+
+	TEUpdate(&(commandwin->portRect), command_text);
+	tmprect = commandtextrect;
+	InsetRect(&tmprect, -1, -1);
+	FrameRect(&tmprect);
+}
+
+void
+activate_command(int activate)
+{
+	if (activate)
+	  TEActivate(command_text);
+	else
+	  TEDeactivate(command_text);
+}
+
+int
+do_key_down_command(key)
+int key;
+{
+	if (key == 13 || key == 3) {
+		get_command_and_do();
+	} else {
+		TEKey(key, command_text);
+	}
+	return TRUE;
+}
+
+void
+do_mouse_down_command(Point mouse, int mods)
+{
+	short part;
+	ControlHandle control;
+
+	part = FindControl(mouse, commandwin, &control);
+	if (control == dobutton) {
+		get_command_and_do();
+	} else if (PtInRect(mouse, &commandtextrect)) {
+		TEClick(mouse, mods, command_text);
+	}
+}
+
+static void
+get_command_and_do()
+{
+	int len;
+	char buffer[BUFSIZE];
+	CharsHandle text;
+
+	text = TEGetText(command_text);
+	len = min((*command_text)->teLength, BUFSIZE);
+	strncpy(buffer, *text, len);
+	buffer[len] = '\0';
+	if (!empty_string(buffer)) {
+		execute_named_command(buffer);
+	} else {
+		notify(dside, "No command.");
+		beep();
 	}
 }
 
@@ -840,12 +1051,65 @@ do_print_view()
 	dump_ps_view(dside, NULL, "View PS");
 }
 
+static void unit_do_produce_2(Unit *unit);
+
 void
 do_produce()
 {
+	int i;
+	Map *map;  List *list;  UnitCloseup *closeup;
+	Unit *unit;
+
+	/* Try different alternatives to find a unit. */
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
+		for (i = 0; i < map->numselections; ++i) {
+			unit = map->selections[i];
+			if (unit != NULL && can_produce(unit)) {
+				unit_do_produce_2(unit);
+				return;
+			}
+		}
+		return;
+	}
+	list = list_from_window(FrontWindow());
+	if (list != NULL) {
+		unit = selected_unit_in_list(list);
+		if (unit != NULL && can_produce(unit)) {
+			unit_do_produce_2(unit);
+			return;
+		}
+		return;
+	}
+	closeup = unit_closeup_from_window(FrontWindow());
+	if (closeup != NULL) {
+		unit = closeup->unit;
+		if (unit != NULL && can_produce(unit)) {
+			unit_do_produce_2(unit);
+			return;
+		}
+		return;
+	}
+	/* No way to figure out the unit to be producing with, so complain. */
 	beep();
 }
 
+static void
+unit_do_produce_2(Unit *unit)
+{
+	int m, n;
+
+    n = 9999;
+    if (prefixarg > 0)
+	  n = prefixarg;
+	for_all_material_types(m) {
+		if (um_acp_to_produce(unit->type, m) > 0) {
+			push_produce_task(unit, m, n);
+			return;
+		}
+	}
+}
+
 void
 do_quit()
 {
@@ -970,7 +1234,7 @@ do_one_reserve(Unit *unit)
 }
 
 void
-do_reserve_command(int value, int recurse)
+do_reserve_command(int value, int radius, int recurse)
 {
 	tmpcmdarg = value;
 	tmprecurse = recurse;
@@ -980,14 +1244,14 @@ do_reserve_command(int value, int recurse)
 void
 do_reserve()
 {
-	do_reserve_command(TRUE, FALSE);
+	do_reserve_command(TRUE, prefixarg, FALSE);
 }
 
 static int
 do_one_return(Unit *unit)
 {
 	if (1 /* has a place to return to */) {
-		set_resupply_task(unit);
+		set_resupply_task(unit, NONMTYPE);
 		return TRUE;
 	}
 	return FALSE;
@@ -1002,7 +1266,15 @@ do_return()
 void
 do_resign()
 {
-	beep();
+	if (endofgame) {
+		notify(dside, "Game is already over.");
+		beep();
+	} else if (!dside->ingame) {
+		notify(dside, "You are already out of the game.");
+		beep();
+	} else if (CautionAlert(aConfirmResign, nil) == aiConfirmResignResign) {
+		resign_game(dside, NULL);
+	}
 }
 
 void
@@ -1064,7 +1336,7 @@ do_one_asleep(Unit *unit)
 }
 
 void
-do_sleep_command(int value, int recurse)
+do_sleep_command(int value, int radius, int recurse)
 {
 	tmpcmdarg = value;
 	tmprecurse = recurse;
@@ -1074,7 +1346,20 @@ do_sleep_command(int value, int recurse)
 void
 do_sleep()
 {
-	do_sleep_command(TRUE, FALSE);
+	do_sleep_command(TRUE, prefixarg, FALSE);
+}
+
+void
+do_standing_orders()
+{
+	int rslt;
+
+	if (cmdargstr) {
+		rslt = parse_standing_order(dside, cmdargstr);
+		if (rslt < 0)
+		  beep();
+	} else
+	  beep();
 }
 
 void
@@ -1131,22 +1416,21 @@ do_version()
 void
 do_wake()
 {
-	do_sleep_command(FALSE, FALSE);
-	do_reserve_command(FALSE, FALSE);
+	do_sleep_command(FALSE, prefixarg, FALSE);
+	do_reserve_command(FALSE, prefixarg, FALSE);
 }
 
 void
 do_wake_all()
 {
-	do_sleep_command(FALSE, TRUE);
-	do_reserve_command(FALSE, TRUE);
+	do_sleep_command(FALSE, prefixarg, TRUE);
+	do_reserve_command(FALSE, prefixarg, TRUE);
 }
 
 void
 do_warranty()
 {
-	/* (should be a help node?) */
-	beep();
+	help_dialog(warranty_help_node);
 }
 
 #ifdef DESIGNERS
@@ -1170,6 +1454,7 @@ do_gdl()
 	  interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL));
 	else
 	  beep();
+	end_printing_forms();
 }
 
 #endif
@@ -1180,30 +1465,33 @@ void
 do_debug()
 {
 	toggle_debugging(&Debug);
+	draw_game();
 }
 
 void
 do_debugg()
 {
 	toggle_debugging(&DebugG);
+	draw_game();
 }
 
 void
 do_debugm()
 {
 	toggle_debugging(&DebugM);
+	draw_game();
 }
 
 void
 do_profile()
 {
-	toggle_debugging(&Profile);
+	toggle_profiling();
 }
 
 void
 do_trace()
 {
-	toggle_debugging(&Profile);
+	toggle_profiling();
 #ifdef PROFILING
 	_trace = 1;
 #endif
@@ -1216,23 +1504,16 @@ do_trace()
 /* Mac-specific command functions. */
 
 void
-do_force_global_replan()
-{
-	force_global_replan(dside);
-}
-
-void
-do_escape()
+do_mac_escape()
 {
 	map_modal = NO_MODAL;
 }
 
 void
-do_set_map_angle()
+do_mac_set_map_angle()
 {
 	int angle;
 	Map *map;
-	extern int vertexagg;
 
 	map = map_from_window(FrontWindow());
 	if (map != NULL) {
@@ -1243,114 +1524,62 @@ do_set_map_angle()
 		  angle = 15;
 		else
 		  angle = 90;
-		vertexagg = (prefixarg < 1 ? 1 : prefixarg);
+		map->vp->vertscale = (prefixarg < 1 ? 1 : prefixarg);
 		set_view_angle(map->vp, angle);
 		force_map_update(map);
 	}
 }
 
-#ifdef DEBUGGING
-
-/* Junk associated with debug output. */
-
-/* This is all fairly elaborate because we need to be able to collect
-   detailed logs of AI activity over different periods of time, and just
-   dumping to stdout doesn't work in a window system. */
-
-#ifdef USE_CONSOLE
-#ifdef THINK_C
-#include <console.h>
-#endif
-#endif
-
-extern FILE *pfp;
-
-FILE *ffp = NULL;
-
-int firstdebug = TRUE;
-
 void
-update_debugging()
+do_mac_zoom_in()
 {
-	if (Debug || DebugG || DebugM || Profile) {
-		/* Always close the file if open, forces to desirable state. */
-		if (ffp != NULL) {
-			fclose(ffp);
-			ffp = NULL;
-		}
-		/* Reopen the file. */
-		if (ffp == NULL) {
-			ffp = fopen("Xconq.DebugOut", "a");
-		}
-		if (ffp != NULL) {
-			if (Debug)
-			  dfp = ffp;
-			if (DebugG)
-			  dgfp = ffp;
-			if (DebugM)
-			  dmfp = ffp;
-			if (Profile)
-			  pfp = ffp;
-		}
+	Map *map;
+
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
+		magnify_map(map, 1);
+	} else {
+		beep();
 	}
 }
 
-/* Debug output goes to a file. */
 
 void
-toggle_debugging(int *flagp)
+do_mac_zoom_out()
+{
+	Map *map;
+
+	map = map_from_window(FrontWindow());
+	if (map != NULL) {
+		magnify_map(map, -1);
+	} else {
+		beep();
+	}
+}
+
+#ifdef DEBUGGING
+
+void
+toggle_profiling()
 {
 #ifdef PROFILING
 	extern int _profile;
 #endif
 
-	/* Always close the file if open, forces to desirable state. */
-	if (ffp != NULL) {
-		fclose(ffp);
-		ffp = NULL;
-	}
-	/* Flip the state of the debugging flag, if supplied. */ 
-	if (flagp != NULL) {
-		*flagp = ! *flagp;
-	}
-	/* (Re-)open the debugging transcript file. */
-	if (ffp == NULL) {
-		ffp = fopen("Xconq.DebugOut", (firstdebug ? "w" : "a"));
-		firstdebug = FALSE;
-	}
-	if (flagp != NULL) {
-		/* Indicate which flags are now on. */
-		fprintf(ffp, "\n\n*********** %s %s %s %s **********\n\n",
-				(Debug ? "Debug" : ""), (DebugM ? "DebugM" : ""),
-				(DebugG ? "DebugG" : ""), (Profile ? "Profile" : ""));
-		/* Indicate this in a window also. */
-		draw_game();
-	}
-	/* Set specific debug file pointers to be the same as the
-	   pointer to the file. */
-	if (ffp != NULL) {
-		if (Debug) dfp = ffp;
-		if (DebugG) dgfp = ffp;
-		if (DebugM) dmfp = ffp;
-		if (Profile) pfp = ffp;
-	}
+	Profile = !Profile;
+
 #ifdef PROFILING
 #ifdef THINK_C
 	if (Profile && !_profile) {
 		InitProfile(1000, 100);
+		freopen("Xconq.ProfileOut", "w", stdout);
 	}
 	if (!Profile && _profile) {
-		EndProfile();
+		DumpProfile();
+		fflush(stdout);
 	}
 #endif
 #endif
-	/* If all debugging flags have been turned off, close the file too. */
-	if (!Debug && !DebugG && !DebugM && !Profile) {
-		if (ffp != NULL) {
-			fclose(ffp);
-			ffp = NULL;
-		}
-	}
 }
 
 #endif /* DEBUGGING */
