@@ -1,5 +1,5 @@
-/* Main program of the Mac interface to Xconq.z
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+/* Main program of the Mac interface to Xconq.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,31 +7,29 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+extern int checkpointinterval;
 extern int statistics_wanted;
+extern int numsoundplays;
+extern int numremotewaiting;
+extern void low_send(int id, char *buf);
+extern int low_receive(int *id, char *buf, int maxchars, int timeout);
+extern void update_debugging PARAMS ((void));
+extern void toggle_debugging PARAMS ((int *flagp));
 #include "macconq.h"
 
-#include <stdarg.h>
 #include <errors.h>
 #include <time.h>
 
-/* The console is a useful debugging hack.  When not in use, anything that uses stdout
-   should be disabled. */
-
-#ifdef USE_CONSOLE
-#ifdef THINK_C
-#include <console.h>
-#include <Folders.h>
-#else
-#ifndef printf
-printf() {}
+#ifdef __MWERKS__
+#include <Sound.h>
 #endif
-#endif
-#endif /* USE_CONSOLE */
 
 #define keyReplyErr 'errn'
 
 /* Function prototypes. */
 
+static void handle_event(EventRecord *event);
+
 static pascal OSErr do_ae_open_application(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_open_documents(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_print_documents(AppleEvent *message, AppleEvent *reply, long refcon);
@@ -167,6 +165,18 @@ int numscheduled;
 
 struct a_movie movies[10];
 
+int connection_method;
+
+int hosting;
+
+ModalFilterUPP filter_warning_alert_proc;
+
+AEEventHandlerUPP do_ae_open_application_proc;
+AEEventHandlerUPP do_ae_open_documents_proc;
+AEEventHandlerUPP do_ae_print_documents_proc;
+AEEventHandlerUPP do_ae_quit_application_proc;
+AEEventHandlerUPP do_ae_join_game_proc;
+
 /* The main Mac program. */
 
 int
@@ -179,16 +189,11 @@ main()
 	init_icons();
 	init_menus();
 	init_rects();
-	init_ae();
-#ifdef USE_CONSOLE
-	/* Make sure the console window is up before anybody tries to write to it. */
-	freopenc(NULL, stdout);
-	freopenc(stdout, stderr);
-#endif /* USE_CONSOLE */
 	/* Put the Xconq kernel into a known state. */
 	clear_game_modules();
 	init_data_structures();
 	init_library_path(NULL);
+	init_ae();
 	/* Acquire Mac-specific files (preferences and resources). */
 	get_files();
 	/* A hack to ensure some memory available for error handling. */
@@ -219,6 +224,7 @@ splash_dialog()
 			open_game_dialog();
 			break;
 		case diSplashConnect:
+			hosting = FALSE;
 			connect_game_dialog();
 			break;
 		case diSplashQuit:
@@ -227,18 +233,6 @@ splash_dialog()
 	return -1;
 }
 
-/* (just a place holder for now) */
-
-void
-connect_game_dialog()
-{
-	LocationNameRec foo;
-	PortInfoRec bar;
-	
-	PPCBrowser("\pChoose an Xconq game to join:",
-			   "\pXconq", 0, &foo, &bar, NULL, "\p?");
-}
-
 /* Do the usual Mac setup calls. */
 
 void
@@ -257,6 +251,10 @@ init_toolbox()
 	InitDialogs(NULL);
 	InitCursor();
 
+	/* We need to set this one up early, since may be used in error
+	   reporting. */
+    filter_warning_alert_proc = NewModalFilterProc(filter_warning_alert);
+
 	SysEnvirons(2, &se);
 	hasColorQD = se.hasColorQD;
 	DGprintf("%s Color QuickDraw\n", (hasColorQD ? "Using" : "Not using"));
@@ -313,40 +311,41 @@ init_ae()
 	hasAppleEvents = (Gestalt(gestaltAppleEventsAttr, &rslt) ? false : (rslt != 0));
 
 	if (hasAppleEvents) {
+		do_ae_open_application_proc = NewAEEventHandlerProc(do_ae_open_application);
+		do_ae_open_documents_proc = NewAEEventHandlerProc(do_ae_open_documents);
+		do_ae_print_documents_proc = NewAEEventHandlerProc(do_ae_print_documents);
+		do_ae_quit_application_proc = NewAEEventHandlerProc(do_ae_quit_application);
+		do_ae_join_game_proc = NewAEEventHandlerProc(do_ae_join_game);
+
 		err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
-									(ProcPtr) do_ae_open_application, 0L, false);
+									do_ae_open_application_proc, 0L, false);
 		if (err) {
-/*				Alert(rErrorAlert, nil);  */
-				beep();
-				return;
+			init_warning("AppleEvent handler could not be installed, skipping others");
+			return;
 		}
 		err = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,
-									(ProcPtr) do_ae_open_documents, 0L, false);
+									do_ae_open_documents_proc, 0L, false);
 		if (err) {
-/*				Alert(rErrorAlert, nil);  */
-				beep();
-				return;
+			init_warning("AppleEvent handler could not be installed, skipping others");
+			return;
 		}
 		err = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments,
-									(ProcPtr) do_ae_print_documents, 0L, false);
+									do_ae_print_documents_proc, 0L, false);
 		if (err) {
-/*				Alert(rErrorAlert, nil);  */
-				beep();
-				return;
+			init_warning("AppleEvent handler could not be installed, skipping others");
+			return;
 		}
 		err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
-									(ProcPtr) do_ae_quit_application, 0L, false);
+									do_ae_quit_application_proc, 0L, false);
 		if (err) {
-/*				Alert(rErrorAlert, nil);  */
-				beep();
-				return;
+			init_warning("AppleEvent handler could not be installed, skipping others");
+			return;
 		}
 		err = AEInstallEventHandler('xcnq', 'join',
-									(ProcPtr) do_ae_join_game, 0L, false);
+									do_ae_join_game_proc, 0L, false);
 		if (err) {
-/*				Alert(rErrorAlert, nil);  */
-				beep();
-				return;
+			init_warning("AppleEvent handler could not be installed, skipping others");
+			return;
 		}
 	}
 }
@@ -491,7 +490,6 @@ interp_mac_ui_data(Obj *uispec)
 	int numval;
 	char *name;
 	Obj *rest, *bdg;
-	extern int checkpointinterval;
 
 	for (rest = uispec; rest != lispnil; rest = cdr(rest)) {
 		bdg = car(rest);
@@ -558,7 +556,6 @@ set_preferences()
 	short done = FALSE, ditem;
 	DialogPtr win;
 	short itemtype;  Handle itemhandle;  Rect itemrect;
-	extern int checkpointinterval;
 
 	win = GetNewDialog(dPreferences, NULL, (DialogPtr) -1L);
 	/* Set the current preferences into the items. */
@@ -624,7 +621,6 @@ event_loop()
 	RgnHandle cursorRgn;
 	short itemhit;
 	DialogPtr dialog;
-	extern WindowPtr playersetupwin;
 
 	/* Figure out if the WaitNextEvent Trap is available. */
 	useWNE = (NGetTrapAddress(0x60, ToolTrap) != NGetTrapAddress(0x9f, ToolTrap));
@@ -644,12 +640,16 @@ event_loop()
 		/* First decide if the event is for a dialog or is just any old event. */
 		if (FrontWindow() != nil && IsDialogEvent(&event)) {
 			/* Handle all the modeless dialogs here. */
+			/* Process any player setup window events that we want to handle
+			   specially (such as keystrokes). */
 			if (playersetupwin != nil && playersetupwin == FrontWindow())
 			  handle_player_setup_event(&event);
 			if (DialogSelect(&event, &dialog, &itemhit)) {
 				if (dialog == instructionswin) {
 					hit_instructions_dialog(dialog, itemhit, &event);
 				}
+				/* Process player setup window events that the Dialog
+				   Manager handles (clicks on buttons for instance). */
 				if (dialog == playersetupwin) {
 					mouse = event.where;
 					SetPort(playersetupwin);
@@ -668,6 +668,13 @@ event_loop()
 			/* Make sure we have the right cursor before handling the event. */
 			adjust_cursor(event.where, cursorRgn);
 			handle_event(&event);
+		} else if (beforestart && hosting && connection_method > 0 && numremotewaiting > 0) {
+			int gotsome, remoteid;
+
+			gotsome = low_receive(&remoteid, NULL, 0, 0);
+			if (gotsome) {
+				add_remote_player(NULL);
+			}
 		} else if (!beforestart && !endofgame) {
 			/* On null events, give the kernel a chance to run something. */
 			/* DON'T automatically go to a watch cursor, since run_game often
@@ -679,6 +686,10 @@ event_loop()
 			if (endofgame)
 			  set_end_of_game_interaction_modes();
 		}
+		/* Blink TE cursors regularly. */
+		if (commandwin != nil && FrontWindow() == commandwin) {
+			TEIdle(command_text);
+		}
 	}
 }
 
@@ -727,16 +738,16 @@ adjust_cursor(Point mouse, RgnHandle region)
 			&& mouse.v > map->toph && mouse.v < map->window->portRect.bottom - sbarwid) {
 			if (map_modal != NO_MODAL) {
 				switch (map_modal) {
-					case ZOOM_MODAL:
-						SetCursor(*opencrosscursor);
-						break;
 					case FIRE_MODAL:
 					case FIRE_INTO_MODAL:
 						SetCursor(*firecursor);
 						break;
+					case MOVE_TO_MODAL:
 					case SET_FORMATION_MODAL:
 					case ADD_TERRAIN_MODAL:
 					case REMOVE_TERRAIN_MODAL:
+					case DISTANCE_MODAL:
+					case ZOOM_MODAL:
 					case GENERIC_MODAL:
 						SetCursor(*opencrosscursor);
 						break;
@@ -771,7 +782,7 @@ adjust_cursor(Point mouse, RgnHandle region)
 			/* This isn't really "cursor adjustment", but this is the right place
 			   to do it - change the topline of the map to indicate what the cursor
 			   is over. */
-			if (map->toph > 0 && !EqualPt(mouse, lastmouse)) {
+			if (map->toplineh > 0 && !EqualPt(mouse, lastmouse)) {
 				oneliner(map, mouse.h, mouse.v);
 				if (strcmp(tmpbuf, mouseoverbuf) != 0) {
 					strcpy(mouseoverbuf, tmpbuf);
@@ -781,7 +792,7 @@ adjust_cursor(Point mouse, RgnHandle region)
 				lastmouse = mouse;
 			}
 		} else {
-			if (map->toph > 0) {
+			if (map->toplineh > 0) {
 				if (mouseover != NULL) {
 					mouseover = NULL;
 					draw_top_line(map);
@@ -802,10 +813,10 @@ adjust_cursor(Point mouse, RgnHandle region)
 
 /* Decipher an event. */
 
-int
+void
 handle_event(EventRecord *event)
 {
-	short part, err, rslt = 0;
+	short part, err;
 	WindowPtr win;
 	char key;
 	Point pnt;
@@ -847,7 +858,8 @@ handle_event(EventRecord *event)
 					zoom_window(win, event->where, part);
 					break;
 				case inGoAway:
-					close_window(win);
+					if (TrackGoAway(win, event->where))
+					  close_window(win);
 					break;
 			}
 			break;
@@ -867,6 +879,8 @@ handle_event(EventRecord *event)
 					/* Random keypress, interpret it. */
 					if (FrontWindow() == constructionwin)
 					  handled = do_key_down_construction(key);
+					if (FrontWindow() == commandwin)
+					  handled = do_key_down_command(key);
 					if (!handled)
 					  do_keyboard_command(key);
 				}
@@ -900,8 +914,10 @@ handle_event(EventRecord *event)
 		case kHighLevelEvent:
 			AEProcessAppleEvent(event);
 			break;
+#if 0 /* null events don't come through here */
 		case nullEvent:
-			rslt = 1;
+			break;
+#endif
 			break;
 		default:
 			break;
@@ -910,7 +926,6 @@ handle_event(EventRecord *event)
 	/* This just forces output into the file. */
 	update_debugging();
 #endif
-	return rslt;
 }
 
 /* Handle window growing by mindlessly tracking via GrowWindow,
@@ -940,6 +955,8 @@ grow_window(WindowPtr win, Point where)
 			grow_help(winh, winv);
 		} else if (win == noticewin) {
 			grow_notice(winh, winv);
+		} else if (win == scoreswin) {
+			grow_scores(winh, winv);
 		}
 		SetPort(oldport);
 	}
@@ -966,6 +983,8 @@ zoom_window(WindowPtr win, Point where, int part)
 			zoom_help(part);
 		} else if (win == noticewin) {
 			zoom_notice(part);
+		} else if (win == scoreswin) {
+			zoom_scores(part);
 		} else {
 			/* Generic window zooming. */
 			EraseRect(&win->portRect);
@@ -1034,6 +1053,10 @@ do_mouse_down(WindowPtr window, EventRecord *event)
 			do_mouse_down_help(mouse, event->modifiers);
 		} else if (window == noticewin) {
 			do_mouse_down_notice(mouse, event->modifiers);
+		} else if (window == commandwin) {
+			do_mouse_down_command(mouse, event->modifiers);
+		} else if (window == scoreswin) {
+			do_mouse_down_scores(mouse, event->modifiers);
 #ifdef DESIGNERS
 		} else if (window == designwin) {
 			do_mouse_down_design(mouse, event->modifiers);
@@ -1069,6 +1092,10 @@ activate_window(WindowPtr win, int activate)
 		activate_help(activate);
 	} else if (win == noticewin) {
 		activate_notice(activate);
+	} else if (win == commandwin) {
+		activate_command(activate);
+	} else if (win == scoreswin) {
+		activate_scores(activate);
 	} else {
 		DGprintf("%sactivating random window\n", (activate ? "" : "de"));
 	}
@@ -1114,6 +1141,12 @@ update_window(WindowPtr win)
 	} else if (win == noticewin) {
 		draw_notice();
 		growbox = TRUE;
+	} else if (win == commandwin) {
+		draw_command();
+		growbox = FALSE;
+	} else if (win == scoreswin) {
+		draw_scores();
+		growbox = TRUE;
 #ifdef DESIGNERS
 	} else if (win == designwin) {
 		draw_design_window();
@@ -1316,6 +1349,7 @@ do_ae_quit_application(AppleEvent *message, AppleEvent *reply, long refcon)
 	OSErr err = noErr;
 
 	/* Set the global that lets the whole program exit. */
+	/* (should end the game reasonably first?) */
 	eventloopdone = TRUE;
 	AEPutParamPtr(reply, keyReplyErr, typeShortInteger, (Ptr) &err, sizeof(short));
 	return noErr;
@@ -1337,6 +1371,110 @@ do_ae_join_game(AppleEvent *message, AppleEvent *reply, long refcon)
 	return err;
 }
 
+void
+connect_game_dialog()
+{
+	hosting = FALSE;
+	connection_method_dialog();
+	init_connection_method();
+	if (connection_method > 0) {
+		low_send(0, "join");
+	}
+}
+
+void
+connection_method_dialog()
+{
+	int done = FALSE, newmethod;
+	short ditem;
+	WindowPtr win;
+	PicHandle pic;
+	short itemtype;  Handle itemhandle;  Rect itemrect;
+
+	newmethod = 9;
+	win = GetNewDialog(170, NULL, (DialogPtr) -1L);
+	ShowWindow(win);
+	SelectWindow(win);
+	while (!done) {
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+			case 1:
+				connection_method = newmethod;
+				done = TRUE;
+				break;
+			case 2:
+				done = TRUE;
+				break;
+			case 3:
+			case 4:
+			case 5:
+				break;
+			default:
+				break;
+		}
+	}
+	DisposDialog(win);
+}
+
+void init_serial_port(void);
+
+void
+init_connection_method()
+{
+	switch (connection_method) {
+		case 0:
+			break;
+		case 1:
+			init_serial_port();
+			break;
+		case 2:
+			break;
+		default:
+			break;
+	}
+}
+
+int ser_input_refnum;
+int ser_output_refnum;
+
+void
+init_serial_port()
+{
+    OSErr err;
+
+	if (1 /* using modem port */) {
+		/* Open the modem port. */
+		err = OpenDriver("\p.AOut", (short *) &ser_input_refnum);
+		if (err != 0) {
+			return;
+		}
+		err = OpenDriver("\p.AIn", (short *) &ser_output_refnum);
+		if (err != 0) {
+			CloseDriver(ser_input_refnum);
+			return;
+		}
+	} else {
+		/* Open the printer port. */
+		err = OpenDriver("\p.BOut", (short *) &ser_input_refnum);
+		if (err != 0) {
+			return;
+		}
+		err = OpenDriver("\p.BIn", (short *) &ser_output_refnum);
+		if (err != 0) {
+			CloseDriver(ser_input_refnum);
+			return;
+		}
+	}
+}
+
+#if 0
+	LocationNameRec foo;
+	PortInfoRec bar;
+	
+	PPCBrowser("\pChoose an Xconq game to join:",
+			   "\pXconq", 0, &foo, &bar, NULL, "\p?");
+#endif
+
 static pascal Boolean
 filter_warning_alert(DialogPtr dialog, EventRecord *evt, short *itemhit)
 {
@@ -1368,7 +1506,7 @@ low_init_warning(char *str)
 	SetCursor(&QD(arrow));
 	c2p(str, buf);
 	ParamText(buf, "\p", "\p", "\p");
-	switch (CautionAlert(aInitWarning, filter_warning_alert)) {
+	switch (CautionAlert(aInitWarning, filter_warning_alert_proc)) {
 		case 1:
 			/* Just keep going, player considers warning a false alarm. */
 			break;
@@ -1414,7 +1552,7 @@ low_run_warning(char *str)
 	}
 	c2p(str, buf);
 	ParamText(buf, "\p", "\p", "\p");
-	switch (CautionAlert(aRunWarning, filter_warning_alert)) {
+	switch (CautionAlert(aRunWarning, filter_warning_alert_proc)) {
 		case 1:
 			/* Just keep going, player considers warning a false alarm. */
 			break;
@@ -1459,6 +1597,31 @@ low_run_error(char *str)
 	ExitToShell();
 }
 
+static FILE *pffp;
+
+static int first_print = TRUE;
+
+void
+print_form(form)
+Obj *form;
+{
+	if (pffp == NULL) {
+		pffp = fopen("Xconq.PrintOut", (first_print ? "w" : "a"));
+		first_print = FALSE;
+	}
+	print_form_and_value(pffp, form);
+	fflush(pffp);
+}
+
+void
+end_printing_forms()
+{
+	if (pffp != NULL) {
+		fclose(pffp);
+		pffp = NULL;
+	}
+}
+
 /* This is true when a side has a display that may be safely written to. */
 
 int
@@ -1502,12 +1665,21 @@ void
 update_side_display(Side *side, Side *side2, int rightnow)
 {
 	GrafPtr oldport;
+	extern int gamenumsides;
 
 	if (active_display(side) && side2 != NULL) {
 		GetPort(&oldport);
 		if (gamewin != nil && ((WindowPeek) gamewin)->visible) {
-			SetPort(gamewin);
-			draw_side_status(side2);
+			if (gamenumsides == numsides) {
+				SetPort(gamewin);
+				draw_side_status(side2);
+			} else {
+				/* A side must have been added recently; redraw
+				   the entire window this time. */
+				if (eimages[side_number(side2)] == NULL)
+				  init_emblem_images(side2);
+				draw_game();
+			}
 		}
 		SetPort(oldport);
 		if (side2 == dside && !side->ingame && side->status == 0 && wasingame) {
@@ -1542,10 +1714,26 @@ update_turn_display(Side *side, int rightnow)
 			draw_game_date();
 		}
 		for_all_maps(map) {
-			if (map->toph > 0) {
+			if (map->toplineh > 0) {
 				SetPort(map->window);
 				draw_top_line(map);
 			}
+			if (map->topunith > 0) {
+				SetPort(map->window);
+				draw_unit_info(map);
+			}
+		}
+		/* This routine might have been called because the game is over. */
+		if (endofgame && !told_outcome && side == dside) {
+			if (side_won(dside)) {
+		    	won_game_dialog();
+		    } else if (side_lost(dside)) {
+		    	lost_game_dialog();
+		    } else {
+				game_over_dialog();
+			}
+			told_outcome = TRUE;
+		    side->may_set_see_all = TRUE;
 		}
 		SetPort(oldport);
 	}
@@ -1603,44 +1791,16 @@ update_action_result_display(Side *side, Unit *unit, int rslt, int rightnow)
     			}
     			break;
     	}
+    	update_unit_in_maps(unit);
     }
 }
 
 void
 update_event_display(Side *side, HistEvent *hevt, int rightnow)
 {
-    int data0;
-
-    data0 = hevt->data[0];
 	if (active_display(side)) {
-		switch (hevt->type) {
-		  case H_SIDE_LOST:
-		    if (side->id == data0) {
-		    	lost_game_dialog();
-		    	told_outcome = TRUE;
-		    	side->may_set_see_all = TRUE;
-		    }
-		  	break;
-		  case H_SIDE_WON:
-		    if (side->id == data0) {
-		    	won_game_dialog();
-		    	told_outcome = TRUE;
-		    	side->may_set_see_all = TRUE;
-		    }
-		  	break;
-		  case H_GAME_ENDED:
-			/* Don't display the generic game-end dialog if we already
-			   got a lost or won dialog. */
-		  	if (!told_outcome)
-		  	  game_over_dialog();
-		    side->may_set_see_all = TRUE;
-		    break;
-		  default:
-		  	/* No special display for these types of events. */
-		  	break;
-		}
-		/* Also work on the history window if it's up. */
-		if (historywin != nil) {
+		/* Tweak the history window if it's up. */
+		if (historywin != nil && ((WindowPeek) historywin)->visible) {
 			update_history_window(hevt);
 		}
 	}
@@ -1662,8 +1822,10 @@ update_fire_at_display(Side *side, Unit *unit, Unit *unit2, int m, int rightnow)
 		for_all_maps(map) {
 			SetPort(map->window);
 			PenMode(patXor);
-			if (map->vp->hw > 10) PenSize(2, 2);
-			else PenSize(1, 1);
+			if (map->vp->hw > 10)
+			  PenSize(2, 2);
+			else
+			  PenSize(1, 1);
 		}
 		while (TickCount() < startticks + 32) {
 			innerticks = TickCount();
@@ -1674,15 +1836,14 @@ update_fire_at_display(Side *side, Unit *unit, Unit *unit2, int m, int rightnow)
 				}
 				m_xform_unit_self(map, unit, &sx1, &sy1, &sw1, &sh1);
 				m_xform_unit_self(map, unit2, &sx2, &sy2, &sw2, &sh2);
-				/* Offset to draw lines from the middle of the units' images. */
-				sx1 += sw1 / 2;  sy1 += sh1 / 2;
-				sx2 += sw2 / 2;  sy2 += sh2 / 2;
-				/* Draw one segment of a line between the units. */
-				dx = (sx2 - sx1) / 4;  dy = (sy2 - sy1) / 4;
-				xx = sx1 + ((i / 2) % 4) * dx;  yy = sy1 + ((i / 2) % 4) * dy;
+				compute_fire_line_segment(sx1 + sw1 / 2, sy1 + sh1 / 2,
+										  sx2 + sw2 / 2, sy2 + sh2 / 2,
+										  i, 4, &xx, &yy, &dx, &dy);
 				MoveTo(xx, yy);  Line(dx, dy);
 			}
-			while (TickCount() < innerticks + 1); /* 2 here seems a bit slowish */
+			/* 2 here seems a bit slowish */
+			while (TickCount() < innerticks + 1)
+			  ;
 			++i;
 		}
 		/* Restore the pen modes of all the maps. */
@@ -1710,8 +1871,10 @@ update_fire_into_display(Side *side, Unit *unit, int x2, int y2, int z2, int m,
 		for_all_maps(map) {
 			SetPort(map->window);
 			PenMode(patXor);
-			if (map->vp->hw > 10) PenSize(2, 2);
-			else PenSize(1, 1);
+			if (map->vp->hw > 10)
+			  PenSize(2, 2);
+			else
+			  PenSize(1, 1);
 		}
 		while (TickCount() < startticks + 32) {
 			innerticks = TickCount();
@@ -1722,16 +1885,14 @@ update_fire_into_display(Side *side, Unit *unit, int x2, int y2, int z2, int m,
 				}
 				m_xform_unit_self(map, unit, &sx1, &sy1, &sw1, &sh1);
 				xform(map, x2, y2, &sx2, &sy2);
-				sw2 = map->vp->hw;  sh2 = map->vp->hh;
-				/* Offset to draw lines from the middle of the units' images. */
-				sx1 += sw1 / 2;  sy1 += sh1 / 2;
-				sx2 += sw2 / 2;  sy2 += sh2 / 2;
-				/* Draw one segment of a line between the units. */
-				dx = (sx2 - sx1) / 4;  dy = (sy2 - sy1) / 4;
-				xx = sx1 + ((i / 2) % 4) * dx;  yy = sy1 + ((i / 2) % 4) * dy;
+				compute_fire_line_segment(sx1 + sw1 / 2, sy1 + sh1 / 2,
+										  sx2 + map->vp->hw / 2, sy2 + map->vp->hh / 2,
+										  i, 4, &xx, &yy, &dx, &dy);
 				MoveTo(xx, yy);  Line(dx, dy);
 			}
-			while (TickCount() < innerticks + 1); /* 2 here seems a bit slowish */
+			/* 2 here seems a bit slowish */
+			while (TickCount() < innerticks + 1)
+			  ;
 			++i;
 		}
 		/* Restore the pen modes of all the maps. */
@@ -1751,8 +1912,7 @@ update_unit_display(Side *side, Unit *unit, int rightnow)
 	UnitCloseup *unitcloseup;
 
 	if (active_display(side) && unit != NULL) {
-		if (side != unit->side)
-		  update_unit_in_maps(unit);
+		update_unit_in_maps(unit);
 		if (1 /* unit visible to side in any way */ && inside_area(unit->x, unit->y)) {
 			update_cell_display(side, unit->x, unit->y, TRUE);
 		}
@@ -1777,9 +1937,8 @@ update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 	UnitCloseup *unitcloseup;
 
 	if (active_display(side) && unit != NULL) {
+		update_unit_in_maps(unit);
 #if 0  /* maybe add later, maybe not - acp change not usually visible tho */
-		if (side != unit->side)
-		  update_unit_in_maps(unit);
 		if (1 /* unit visible to side in any way */ && inside_area(unit->x, unit->y)) {
 			update_cell_display(side, unit->x, unit->y, TRUE);
 		}
@@ -1797,10 +1956,21 @@ update_unit_in_maps(Unit *unit)
 {
 	Map *map;
 
-	if (side_controls_unit(dside, unit))
-	  return;
-	for_all_maps(map) {
-		unselect_unit_on_map(map, unit);
+	if (!side_controls_unit(dside, unit) || !alive(unit)) {
+		for_all_maps(map) {
+			unselect_unit_on_map(map, unit);
+			if (unit == map->curunit) {
+				map->curunit = NULL;
+			}
+		}
+		return;
+	}
+	if (side_controls_unit(dside, unit)) {
+		for_all_maps(map) {
+			if (map->topunith > 0 && map->numselections == 1 && unit == map->selections[0]) {
+				draw_unit_info(map);
+			}
+		}
 	}
 }
 
@@ -1826,7 +1996,7 @@ update_clock_display(Side *side, int rightnow)
 		}
 #if 0  /* no clock display in the topline yet */
 		for_all_maps(map) {
-			if (map->toph > 0) {
+			if (map->toplineh > 0) {
 				SetPort(map->window);
 				draw_top_line(map);
 			}
@@ -2034,17 +2204,14 @@ schedule_movie(Side *side, enum movie_type movie, ...)
 void
 play_movies(SideMask sidemask)
 {
-	int i, j, unitid;
-	int sx, sy, sw, sh;
-	int startticks, innerticks;
-	Str255 sndnamebuf;
+	int i, j, unitid, startticks, innerticks;
 	Map *map;
 	Unit *unit;
 	GrafPtr oldport, curport = NULL;
-	Handle sound;
-	extern int numsoundplays;
 	long startcount;
 
+    if (!side_in_set(dside, sidemask))
+      return;
 	if (1 /* anything visible */) {
 		GetPort(&oldport);
 		startticks = TickCount();
@@ -2053,31 +2220,14 @@ play_movies(SideMask sidemask)
 		for_all_maps(map) {
 			SetPort(map->window);
 			PenMode(patXor);
-			if (map->vp->hw > 10) PenSize(2, 2);
-			else PenSize(1, 1);
+			if (map->vp->hw > 10)
+			  PenSize(2, 2);
+			else
+			  PenSize(1, 1);
 		}
 		while (TickCount() < startticks + 32) {
 			innerticks = TickCount();
 			for (j = 0; j < numscheduled; ++j) {
-				if ((movies[j].type == movie_miss
-				     || movies[j].type == movie_hit
-				     || movies[j].type == movie_death
-				     || movies[j].type == movie_extra_0)
-				    && playsounds
-				    && numsoundplays < 3) {
-
-					if (movies[j].type == movie_extra_0) {
-						c2p((char *) movies[j].args[0], sndnamebuf);
-					    sound = GetNamedResource('snd ', sndnamebuf);
-					} else {
-						sound = GetNamedResource('snd ', "\pcrunch");
-					}
-				    if (sound != nil) {
-				        SndPlay(nil, sound, false);
-				        ReleaseResource(sound);
-				    }
-				    ++numsoundplays;
-				}
 				switch (movies[j].type) {
 					case movie_null:
 						break;
@@ -2091,19 +2241,19 @@ play_movies(SideMask sidemask)
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							draw_blast_image(map->window, sx, sy, sw, sh, 0);
+							draw_unit_blast(map, unit, 0);
 						}
 						startcount = TickCount();
+						play_sound("crunch");
 						/* Delay for part of a second (should relinquish cpu tho) */
-						while ((TickCount() - startcount) < 10);
+						while ((TickCount() - startcount) < 10)
+						  ;
 						for_all_maps(map) {
 							if (curport != map->window) {
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							clear_blast_image(map->window, sx, sy, sw, sh, 2);
+							clear_unit_blast(map, unit, 0);
 						}
 						break;
 					case movie_hit:
@@ -2116,27 +2266,20 @@ play_movies(SideMask sidemask)
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							draw_blast_image(map->window, sx, sy, sw, sh, 1);
+							draw_unit_blast(map, unit, 1);
 						}
 						startcount = TickCount();
-						sound = nil;
-						if (playsounds) {
-						    sound = GetNamedResource('snd ', "\pboom");
-						}
-						if (sound != nil) {
-					        SndPlay(nil, sound, false);
-					        ReleaseResource(sound);
-					    }
+						play_sound("crunch");
+						play_sound("boom");
 						/* Delay for part of a second (should relinquish cpu tho) */
-						while ((TickCount() - startcount) < 10);
+						while ((TickCount() - startcount) < 10)
+						  ;
 						for_all_maps(map) {
 							if (curport != map->window) {
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							clear_blast_image(map->window, sx, sy, sw, sh, 2);
+							clear_unit_blast(map, unit, 1);
 						}
 						break;
 					case movie_death:
@@ -2149,36 +2292,35 @@ play_movies(SideMask sidemask)
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							draw_blast_image(map->window, sx, sy, sw, sh, 2);
+							draw_unit_blast(map, unit, 2);
 						}
 						startcount = TickCount();
-						sound = nil;
-						if (playsounds) {
-						    sound = GetNamedResource('snd ', "\pboom");
-						}
-						if (sound != nil) {
-					        SndPlay(nil, sound, false);
-					        ReleaseResource(sound);
-					    }
+						play_sound("crunch");
+						play_sound("boom");
 						/* Delay for part of a second (should relinquish cpu tho) */
-						while ((TickCount() - startcount) < 10);
+						while ((TickCount() - startcount) < 10)
+						  ;
 						for_all_maps(map) {
 							if (curport != map->window) {
 								SetPort(map->window);
 								curport = map->window;
 							}
-							m_xform_unit(map, unit, &sx, &sy, &sw, &sh);
-							clear_blast_image(map->window, sx, sy, sw, sh, 2);
+							clear_unit_blast(map, unit, 2);
 						}
 						break;
 					case movie_nuke:
+						/* ??? */
+						break;
+					case movie_extra_0:
+						/* Just play the given sound. */
+						play_sound((char *) movies[j].args[0]);
 						break;
 					default:
 						break;
 				}
 			}
-			while (TickCount() < innerticks + 32);
+			while (TickCount() < innerticks + 32)
+			  ;
 			++i;
 		}
 		/* Restore the pen modes of all the maps. */
@@ -2191,6 +2333,30 @@ play_movies(SideMask sidemask)
 	numscheduled = 0;
 }
 
+void
+play_sound(char *soundname)
+{
+	Str255 sndnamebuf;
+	Handle sound;
+
+    if (!playsounds || empty_string(soundname) || numsoundplays >= 5)
+      return;
+	c2p(soundname, sndnamebuf);
+	sound = GetNamedResource('snd ', sndnamebuf);
+	if (sound != nil) {
+		HLock(sound);
+		/* We play the sound synchronously, because the time delay is needed to
+		   make sure the graphics are visible. */
+        SndPlay(nil, (SndListHandle) sound, false);
+        HUnlock(sound);
+        ReleaseResource(sound);
+        ++numsoundplays;
+    } else {
+		run_warning("No sound named \"%s\" available!", soundname);
+		/* (should only complain about each sound once) */
+    }
+}
+
 /* Move the window to a position staggered from the given last position. */
 
 void
@@ -2231,19 +2397,26 @@ stagger_window(WindowPtr win, int *lasthp, int *lastvp)
 	}
 }
 
+/* Return true if the given position is visible on a screen somewhere. */
+
 int
 position_on_screen(int h, int v)
 {
 	Point pnt;
-	GDHandle screen = GetDeviceList();
-	
+	GDHandle screen;
+
 	pnt.h = h;  pnt.v = v;
-	for (screen = GetDeviceList(); screen != nil; screen = GetNextDevice(screen)) {
-		if (TestDeviceAttribute(screen, screenDevice)
-			&& TestDeviceAttribute(screen, screenActive)) {
-			if (PtInRect(pnt, &((*screen)->gdRect)))
-			  return TRUE;
+	if (hasColorQD) {	
+		for (screen = GetDeviceList(); screen != nil; screen = GetNextDevice(screen)) {
+			if (TestDeviceAttribute(screen, screenDevice)
+				&& TestDeviceAttribute(screen, screenActive)) {
+				if (PtInRect(pnt, &((*screen)->gdRect)))
+				  return TRUE;
+			}
 		}
+	} else {
+		if (PtInRect(pnt, &(QD(screenBits).bounds)))
+		  return TRUE;
 	}
 	return FALSE;
 }
@@ -2273,6 +2446,7 @@ position_already_used(int h, int v)
 	return FALSE;
 }
 
+/* (This may only be called if Color Quickdraw is present.) */
 
 GDHandle
 best_zoom_screen(Rect *rectptr)
@@ -2303,14 +2477,11 @@ set_standard_state(WindowPtr win, int fullw, int fullh)
 	Rect winrect, gdrect, zoomrect;
 	GDHandle bestscreen;
 
-	if (!hasColorQD) {
-		zoomrect = QD(screenBits).bounds;
-		InsetRect(&zoomrect, 4, 4);
-	} else {
-		winrect = win->portRect;
-		LocalToGlobal((Point *) &(winrect.top));
-		LocalToGlobal((Point *) &(winrect.bottom));
-		wintitlehgt = winrect.top - 1 - (*(((WindowPeek) win)->strucRgn))->rgnBBox.top;
+	winrect = win->portRect;
+	LocalToGlobal((Point *) &(winrect.top));
+	LocalToGlobal((Point *) &(winrect.bottom));
+	wintitlehgt = winrect.top - 1 - (*(((WindowPeek) win)->strucRgn))->rgnBBox.top;
+	if (hasColorQD) {
 		/* Get the best screen to zoom on. */
 		bestscreen = best_zoom_screen(&winrect);
 		gdrect = (*bestscreen)->gdRect;
@@ -2333,10 +2504,35 @@ set_standard_state(WindowPtr win, int fullw, int fullh)
 		} else {
 			zoomrect = gdrect;
 		}
+	} else {
+		zoomrect = QD(screenBits).bounds;
+		zoomrect.top += GetMBarHeight();
+		InsetRect(&zoomrect, 4, 4);
+		zoomrect.top += wintitlehgt;
 	}
 	((WStateDataPtr) *(((WindowPeek) win)->dataHandle))->stdState = zoomrect;
 }
 
+void
+get_main_screen_size(int *widp, int *hgtp)
+{
+	Rect rect;
+	GrafPtr mainport;
+	GDHandle mainscreen;
+
+	if (hasColorQD) {
+		mainscreen = GetMainDevice();
+		rect = (*mainscreen)->gdRect;
+	} else {
+		GetWMgrPort(&mainport);
+		rect = mainport->portRect;
+	}
+	if (widp)
+	  *widp = rect.right - rect.left;
+	if (hgtp)
+	  *hgtp = rect.bottom - rect.top;
+}
+
 /* General routine to outline the given item of a given dialog. */
 
 void
@@ -2394,6 +2590,8 @@ force_overall_update()
 	force_update(constructionwin);
 	force_update(helpwin);
 	force_update(noticewin);
+	force_update(commandwin);
+	force_update(scoreswin);
 	for_all_maps(map) {
 		force_update(map->window);
 	}
@@ -2439,3 +2637,91 @@ set_game_file_type(char *name)
 		PBSetFInfoSync((ParmBlkPtr) &pb);
 	}
 }
+
+/* Low-level transmission to another program. */
+
+void
+low_send(int id, char *buf)
+{
+	Dprintf("Sent: %d \"%s\"\n", id, (buf ? buf : "(null)"));
+    switch (connection_method) {
+		case 0:
+			break;
+		case 1:
+			break;
+		case 9:
+			break;
+		default:
+			break;
+    }
+}
+
+int
+low_receive(int *id, char *buf, int maxchars, int timeout)
+{
+    int rslt = FALSE;
+
+    switch (connection_method) {
+		case 0:
+			break;
+		case 1:
+			break;
+		case 9:
+			*id = 2;
+			if (maxchars > 0 && buf != NULL) {
+				strcpy(buf, "OK");
+			}
+			rslt = TRUE;
+			break;
+		default:
+			break;
+    }
+    if (rslt)
+      Dprintf("Rcvd: %d \"%s\"\n", *id, (buf ? buf : "(null)"));
+    return rslt;
+}
+
+#ifdef __MWERKS__
+
+/* Empty definitions for Metrowerks' SIOUX console library. */
+
+#ifndef __CONSOLE__
+#include <console.h>
+#endif
+
+short
+InstallConsole(short fd)
+{
+#pragma unused (fd)
+	return 0;
+}
+
+void
+RemoveConsole(void)
+{
+}
+
+long
+WriteCharsToConsole(char *buf, long n)
+{
+#pragma unused (buf, n)
+	return 0;
+}
+
+long ReadCharsFromConsole(char *buf, long n)
+{
+#pragma unused (buf, n)
+	return 0;
+}
+
+extern char *
+__ttyname(long fd)
+{
+	static char *__devicename = "null device";
+
+	if (fd >= 0 && fd <= 2)
+	  return (__devicename);
+	return NULL;
+}
+
+#endif /* __MWERKS__ */
