@@ -1,13 +1,17 @@
 /* Definitions for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-#undef USE_CMDLINE
-#undef USE_CONSOLE
+/* All Mac compilers are close enough to ANSI to include these,
+   even though some don't define __STDC__. */
+
+#include <stddef.h>
+#include <stdarg.h>
+#include <stdlib.h>
 
 #ifdef THINK_C
 #include <MacHeaders>
@@ -15,7 +19,9 @@ any later version.  See the file COPYING.  */
 #endif /* THINK_C */
 
 #ifdef MPW
+#ifndef __MWERKS__ /* actually, "old headers" */
 #include <Values.h>
+#endif
 #include <Types.h>
 #include <Resources.h>
 #include <QuickDraw.h>
@@ -26,6 +32,7 @@ any later version.  See the file COPYING.  */
 #include <TextEdit.h>
 #include <Dialogs.h>
 #include <Desk.h>
+#include <Scrap.h>
 #include <ToolUtils.h>
 #include <Memory.h>
 #include <SegLoad.h>
@@ -39,19 +46,24 @@ any later version.  See the file COPYING.  */
 #include <Lists.h>
 #include <StandardFile.h>
 #include <Sound.h>
+#include <Devices.h>
 #endif /* MPW */
 
 #include <GestaltEqu.h>
 #include <PPCToolbox.h>
 #include <AppleEvents.h>
 
+/* Bring in image- and interface-related definitions. */
+
 #include "imf.h"
 #include "macimf.h"
 #include "ui.h"
 
+/* Bring in the definitions of resources. */
+
 #include "macdefs.h"
 
-#ifdef MPW
+#ifdef MPW_C
 #define QD(whatever) (qd.##whatever)
 #define QDPat(whatever) (&(qd.##whatever))
 #endif
@@ -59,11 +71,28 @@ any later version.  See the file COPYING.  */
 #define QD(whatever) (whatever)
 #define QDPat(whatever) (whatever)
 #endif
-#ifdef MWC
+#ifdef __MWERKS__
 #define QD(whatever) (qd.##whatever)
 #define QDPat(whatever) (&(qd.##whatever))
 #endif
 
+#ifdef MPW_C
+/* dangerous way */
+#define SET_PAT_ELT(pattern,i,val) ((pattern)[(i)] = (val))
+#define PAT_ELT(pattern,i) ((pattern)[(i)])
+#endif
+
+#ifdef THINK_C
+/* dangerous way */
+#define SET_PAT_ELT(pattern,i,val) ((pattern)[(i)] = (val))
+#define PAT_ELT(pattern,i) ((pattern)[(i)])
+#endif
+
+#ifdef __MWERKS__
+#define SET_PAT_ELT(pattern,i,val) ((pattern).pat[(i)] = (val))
+#define PAT_ELT(pattern,i) ((pattern).pat[(i)])
+#endif
+
 #ifdef THINK_C
 /* From MPW, to replace missing definitions in Think C includes. */
 enum {
@@ -95,6 +124,33 @@ pascal OSErr FindFolder(short vRefNum,OSType folderType,Boolean createFolder,
  = {0x7000,0xA823}; 
 #endif
 
+/* This is a way - perhaps not the best way - to distinguish "old headers"
+   from "universal headers". */
+
+#ifndef __CONDITIONALMACROS__
+#define SetMenuItemText SetItem
+#define InsertMenuItem InsMenuItem
+#define SndListHandle Handle
+#define AEEventHandlerUPP EventHandlerProcPtr
+#define ModalFilterUPP ModalFilterProcPtr
+typedef pascal void (*UserItemProcPtr)(WindowPtr theWindow, short itemNo);
+#define UserItemUPP UserItemProcPtr
+#define ControlActionUPP ProcPtr
+#endif
+
+#ifndef NewAEEventHandlerProc
+#define NewAEEventHandlerProc(fn) ((EventHandlerProcPtr) (fn))
+#endif
+#ifndef NewModalFilterProc
+#define NewModalFilterProc(fn) ((ModalFilterProcPtr) (fn))
+#endif
+#ifndef NewUserItemProc
+#define NewUserItemProc(fn) ((UserItemProcPtr) (fn))
+#endif
+#ifndef NewControlActionProc
+#define NewControlActionProc(fn) ((ProcPtr) (fn))
+#endif
+
 enum grays {
 	blackgray,
 	darkgray,
@@ -143,6 +199,8 @@ typedef struct a_map {
 	ControlHandle vscrollbar;
 	/* How to draw the map. */
 	short conw;
+	short toplineh;
+	short topunith;
 	short toph;
 	short drawterrain;		/* Display solid color terrain on the map? */
 	short auxterraintypes[MAXTTYPES];
@@ -192,11 +250,7 @@ typedef struct a_map {
 #define MAXINLIST 500
 
 typedef struct a_list {
-#if (MAXSIDES < 32)
-	long sides;                      /* mask of sides to display the units of */
-#else
-	short sides[MAXSIDES];
-#endif
+    SideMask sides;
 	int mainsortmi;
 	int listglimpsed;              /* list glimpsed units also? */
 	enum sortkeys sortkeys[MAXSORTKEYS];  /* attributes to sort list elements on */
@@ -269,12 +323,14 @@ enum {
 
 enum {
 	NO_MODAL,
-	ZOOM_MODAL,
+	MOVE_TO_MODAL,
 	FIRE_MODAL,
 	FIRE_INTO_MODAL,
 	SET_FORMATION_MODAL,
 	ADD_TERRAIN_MODAL,
 	REMOVE_TERRAIN_MODAL,
+	DISTANCE_MODAL,
+	ZOOM_MODAL,
 	GENERIC_MODAL
 };
 
@@ -350,8 +406,6 @@ extern WindowPtr gamewin;
 extern ControlHandle gamevscrollbar;
 
 extern WindowPtr constructionwin;
-extern ListHandle constructionunitlist;
-extern ListHandle constructiontypelist;
 
 extern WindowPtr designwin;
 extern enum tooltype tooltype;
@@ -366,6 +420,9 @@ extern WindowPtr historywin;
 extern WindowPtr noticewin;
 extern TEHandle noticetext;
 
+extern WindowPtr scoreswin;
+extern TEHandle scorestext;
+
 extern MenuHandle sidemenu;
 extern MenuHandle utypemenu;
 extern MenuHandle mtypemenu;
@@ -445,7 +502,8 @@ extern RGBColor gridcolor;
 extern RGBColor unseencolor;
 extern RGBColor blackcolor;
 
-extern int conwid, tophgt;
+extern int conwid;
+extern int tophgt;
 
 extern int first_windows;
 
@@ -455,8 +513,21 @@ extern int grid_matches_unseen;
 extern BitMap *bordbitmaps;
 extern BitMap *connbitmaps;
 
+extern Pattern *animation_patterns;
+
 extern int animation_pattern_state;
 
+extern WindowPtr playersetupwin;
+
+extern int connection_method;
+extern int hosting;
+extern Rect dragrect;
+extern int foundimagesfile;
+
+extern int Profile;
+
+extern TEHandle command_text;
+
 /* Function declarations. */
 
 /* Note that although the Xconq kernel must conditionalize its prototypes,
@@ -478,7 +549,6 @@ extern void ui_update_state(void);
 extern void event_loop(void);
 extern void get_global_mouse(Point *mouse);
 extern void adjust_cursor(Point mouse, RgnHandle region);
-extern int handle_event(EventRecord *event);
 extern void grow_window(WindowPtr win, Point where);
 extern void zoom_window(WindowPtr win, Point where, int part);
 extern void close_window(WindowPtr win);
@@ -498,11 +568,12 @@ extern int position_on_screen(int h, int v);
 extern int position_already_used(int h, int v);
 extern GDHandle best_zoom_screen(Rect *rectptr);
 
-extern GDHandle best_zoom_screen(Rect *rectptr);
-
 extern void force_update(WindowPtr win);
 extern void force_overall_update(void);
 
+extern void set_standard_state(WindowPtr win, int fullw, int fullh);
+extern void get_main_screen_size(int *widp, int *hgtp);
+
 extern void draw_default_button(DialogPtr dialog, short ditem);
 
 extern void stagger_window(WindowPtr win, int *lasthp, int *lastvp);
@@ -532,8 +603,6 @@ extern Map *map_from_window(WindowPtr window);
 extern void grow_map(Map *map, int w, int h);
 extern void zoom_map(Map *map, int part);
 
-extern void set_standard_state(WindowPtr win, int fullw, int fullh);
-
 extern void adjust_map_decor(Map *map);
 
 extern int m_nearest_cell(Map *map, int sx, int sy, int *xp, int *yp);
@@ -552,13 +621,6 @@ extern void draw_other_map(Map *map, Map *map2);
 extern void draw_row(Map *map, int x0, int y0, int len, int clearit);
 extern int cell_update(Map *map, int x, int y);
 extern void draw_terrain_row(Map *map, int x0, int y0, int len);
-extern void draw_borders(Map *map, int x, int y, int b);
-extern void draw_connections(Map *map, int x, int y, int c);
-extern void draw_units(Map *map, int x, int y);
-extern void draw_unit_and_occs(Map *map, Unit *unit, int sx, int sy, int sw, int sh);
-extern void draw_people_row(Map *map, int x0, int y0, int len);
-extern void draw_materials(Map *map, int x, int y);
-extern void draw_theater(Map *map, int x, int y);
 extern void draw_legend(Map *map, int x, int y);
 
 extern void draw_selections(Map *map);
@@ -584,16 +646,16 @@ extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask
 extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle);
 extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t);
 extern void draw_country_borders(WindowPtr win, int sx, int sy, int bitmask, int power, int shade, int angle);
-extern void draw_theater_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
+extern void draw_ai_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
 extern int draw_elevation_here(int x, int y);
 extern void draw_elevation(int sx, int sy, int power, int elev);
 extern int draw_temperature_here(int x, int y);
 extern void draw_temperature(int sx, int sy, int power, int temp);
 extern int draw_winds_here(int x, int y);
-extern void draw_winds(int sx, int sy, int power, int wdir, int wforce);
+extern void draw_winds(int sx, int sy, int power, int rawwind);
 extern int draw_clouds_here(int x, int y);
 extern void draw_clouds(int sx, int sy, int power, int cloudtype);
-extern void draw_coverage(int sx, int sy, int power, int cov);
+extern void draw_coverage(int sx, int sy, int power, int cov, int altcov);
 extern void draw_unit_name(Unit *unit, int sx, int sy, int sw, int sh);
 extern void draw_legend_text(int sx, int sy, int sh, char *legend, int just);
 extern void draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype);
@@ -602,8 +664,9 @@ extern int picture_width(PicHandle pichandle);
 extern int picture_height(PicHandle pichandle);
 extern void plot_sicn(WindowPtr win, int sx, int sy, Handle sicnhandle, int n, int erase, int mode);
 extern void gray_out_rect(Rect *rectptr);
+extern void draw_unit_blast(Map *map, Unit *unit, int blast);
+extern void clear_unit_blast(Map *map, Unit *unit, int blast);
 
-extern pascal void map_scroll_proc(ControlHandle control, short code);
 extern void do_mouse_down_map(Map *map, Point mouse, int mods);
 extern void do_mouse_down_map_control_panel(Map *map, int h, int v, int mods);
 extern void toggle_survey(Map *map);
@@ -611,6 +674,7 @@ extern void magnify_map(Map *map, int inout);
 extern void set_map_mag(Map *map, int newpower);
 extern void toggle_map_grid(Map *map);
 extern void toggle_map_topline(Map *map);
+extern void toggle_map_topunit(Map *map);
 extern void toggle_map_other_maps(Map *map);
 extern void toggle_map_lighting(Map *map);
 extern void toggle_map_coverage(Map *map);
@@ -664,8 +728,6 @@ extern void draw_unit_list_entry(List *list, int n, int clearfirst);
 extern void grow_list(List *list, int w, int h);
 extern void zoom_list(List *list, int part);
 extern void adjust_list_decor(List *list);
-extern pascal void list_vscroll_proc(ControlHandle control, short code);
-extern pascal void list_hscroll_proc(ControlHandle control, short code);
 extern void do_mouse_down_list(List *list, Point mouse, int mods);
 extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
 extern void toggle_list_large_icons(List *list);
@@ -685,7 +747,6 @@ extern void scroll_best_map_to_unit(Unit *unit);
 extern void init_patterns(void);
 extern void init_icons(void);
 extern void init_cursors(void);
-pascal Boolean filter_splash(DialogPtr dialog, EventRecord *evt, short *itemhit);
 extern int do_splash_box(void);
 extern void new_game_dialog(void);
 extern int start_new_game(void);
@@ -700,7 +761,6 @@ extern int hit_player_setup_dialog(int itemhit, Point mouse);
 extern void launch_game_2(void);
 extern void add_remote_player(char *name);
 extern void check_for_missing_images(void);
-extern pascal void draw_progress(WindowPtr win, short ditem);
 extern void open_progress_dialog(void);
 extern void close_progress_dialog(void);
 extern void init_display(void);
@@ -732,10 +792,11 @@ extern void set_preferences(void);
 extern void maybe_init_print(void);
 extern void quit_the_game(void);
 extern void apply_to_all_selected(int (*fn)(), int beepfailure);
-extern void do_reserve_command(int value, int recurse);
-extern void do_sleep_command(int value, int recurse);
+extern void do_reserve_command(int value, int radius, int recurse);
+extern void do_sleep_command(int value, int radius, int recurse);
 extern void do_attack_command(void);
 extern void enable_construction(void);
+extern int do_move_to_command(void);
 extern int do_fire_command(void);
 extern int do_fire_into_command(void);
 extern void do_profile(void);
@@ -752,12 +813,10 @@ extern void execute_named_command(char *cmdstr);
 extern void describe_commands(int arg, char *key, char *buf);
 struct cmdtab;
 extern void describe_command_table(int arg, char *key, char *buf, struct cmdtab *cmdtab);
-extern void update_debugging(void);
-extern void toggle_debugging(int *flagp);
+extern void toggle_profiling(void);
 
 extern char *get_string_from_item(Handle itemhandle);
 
-extern pascal void draw_instructions_text(WindowPtr win, short ditem);
 extern void instructions_dialog(void);
 extern void create_instructions_dialog(void);
 extern int hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt);
@@ -793,6 +852,7 @@ extern void update_type_list_for_unit(Unit *unit);
 extern void adjust_construction_controls(void);
 extern void grow_construction(int h, int v);
 extern void zoom_construction(int part);
+extern int do_key_down_construction(int key);
 
 extern void side_rename_dialog(Side *side);
 
@@ -813,7 +873,6 @@ extern void set_history_scrollbar(void);
 extern void draw_history(void);
 extern void draw_historical_event(HistEvent *hevt, int y);
 extern void draw_historical_date(HistEvent *hevt, int y);
-extern pascal void history_scroll_proc(ControlHandle control, short code);
 extern void do_mouse_down_history(Point mouse, int mods);
 extern void move_history_scrollbar(int h, int v);
 extern void grow_history(int h, int v);
@@ -826,22 +885,36 @@ extern void append_notice(char *str);
 extern void draw_notice(void);
 extern void adjust_notice_scrollbar(void);
 extern void activate_notice(int activate);
-extern pascal void notice_vscroll_proc(ControlHandle control, short code);
 extern void do_mouse_down_notice(Point mouse, int mods);
 extern void grow_notice(int h, int v);
 extern void zoom_notice(int part);
 
-extern void help_dialog(void);
+extern void scores_dialog(void);
+extern void create_scores_window(void);
+extern void append_scores(char *str);
+extern void draw_scores(void);
+extern void adjust_scores_scrollbar(void);
+extern void activate_scores(int activate);
+extern void do_mouse_down_scores(Point mouse, int mods);
+extern void grow_scores(int h, int v);
+extern void zoom_scores(int part);
+
+extern WindowPtr commandwin;
+extern void draw_command(void);
+extern void activate_command(int activate);
+extern int do_key_down_command(int key);
+extern void do_mouse_down_command(Point mouse, int mods);
+
+extern void help_dialog(HelpNode *helpnode);
 extern void describe_menus(int arg, char *key, char *buf);
 extern void describe_mouse(int arg, char *key, char *buf);
 extern void describe_keyboard(int arg, char *key, char *buf);
 extern void describe_help(int arg, char *key, char *buf);
 extern void create_help_window(void);
-extern void set_help_content(HelpNode *curnode);
+extern void set_help_content(HelpNode *helpnode);
 extern void draw_help(void);
 extern void adjust_help_scrollbar(void);
 extern void activate_help(int activate);
-extern pascal void help_vscroll_proc(ControlHandle control, short code);
 extern void do_mouse_down_help(Point mouse, int mods);
 extern void grow_help(int h, int v);
 extern void zoom_help(int part);
@@ -872,6 +945,18 @@ extern void set_game_file_type(char *name);
 extern int get_a_position(Map **map, int *xp, int *yp, int *dirp);
 extern int get_a_unit(Map **mapp, Unit **unitp);
 
-extern int do_key_down_construction(int);
-
 extern void interp_mac_ui_data(Obj *uispec);
+
+extern void draw_unit_info(Map *map);
+extern void connection_method_dialog(void);
+extern void init_connection_method(void);
+
+extern void play_sound(char *soundname);
+
+extern void query_position_modally(int mode);
+extern void set_position_modally(void);
+
+extern int gameinited;
+extern int position_set_modally;
+extern Point modal_point;
+extern Map *modal_map;
