@@ -1,5 +1,5 @@
 /* Game designer handling for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,6 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING. */
 
 #include "conq.h"
+#define make_raw_wind(d,f) (((f) << 3) | (d))
 #include "macconq.h"
 
 #ifdef DESIGNERS
@@ -198,7 +199,7 @@ create_design_window()
 	enabledtooltype[temperaturetool] = any_temp_variation;
 	enabledtooltype[cloudstool] = any_clouds;
 	enabledtooltype[windstool] = any_wind_variation;
-	enabledtooltype[viewtool] = !g_see_all();
+	enabledtooltype[viewtool] = !all_see_all;
 	ShowWindow(designwin);
 }
 
@@ -363,7 +364,7 @@ draw_design_window_tool(enum tooltype tool)
 		case windstool:
 			if (enabledtooltype[windstool]) {
 				InsetRect(&imagerect, (dtoolh - hws[4]) / 2, (dtoolh - hhs[4]) / 2);
-				draw_winds(imagerect.left, imagerect.top, 4, curwinddir, curwindforce);
+				draw_winds(imagerect.left, imagerect.top, 4, make_raw_wind(curwinddir, curwindforce));
 				if (curwindforce > 0) {
 					sprintf(spbuf, "Winds %s, %d", dirnames[curwinddir], curwindforce);
 					toolname = spbuf;
@@ -437,11 +438,11 @@ draw_design_window_tool(enum tooltype tool)
    values, with direction controlled by the shift key.  If the limit is 0,
    then the cycling part is not done. */
 
-#define OPTION_CYCLE(var, lo, hi, mods)  \
+#define OPTION_CYCLE(var, lo, hi, n, mods)  \
   if ((hi) - (lo) > 0) {  \
-    (var) = (((var) + ((mods) & shiftKey ? -1 : 1) - (lo) + ((hi) - (lo))) % ((hi) - (lo))) + (lo);  \
+    (var) = (((var) + ((mods) & shiftKey ? -(n) : (n)) - (lo) + ((hi) - (lo))) % ((hi) - (lo))) + (lo);  \
   } else {  \
-    (var) = ((var) + ((mods) & shiftKey ? -1 : 1));  \
+    (var) = ((var) + ((mods) & shiftKey ? -(n) : (n)));  \
   }
 
 void
@@ -461,13 +462,15 @@ do_mouse_down_design(Point mouse, int mods)
 	if (control == ttypepopup) {
 		TrackControl(control, mouse, (void *) -1);
 		choice = LoWord(GetCtlValue(control));
-		if (choice > 0) curttype = choice - 1;
+		if (choice > 0)
+		  curttype = choice - 1;
 		poppedtool = terraintool;
 	} else if (control == utypepopup) {
 		mark_allowed_unit_types();
 		TrackControl(control, mouse, (void *) -1);
 		choice = LoWord(GetCtlValue(control));
-		if (choice > 0) curutype = choice - 1;
+		if (choice > 0)
+		  curutype = choice - 1;
 		poppedtool = unittool;
 	} else if (control == sidepopup) {
 		mark_allowed_sides();
@@ -475,7 +478,8 @@ do_mouse_down_design(Point mouse, int mods)
 		choice = LoWord(GetCtlValue(control));
 		if (choice > 0) {
 			cursidenumber = choice;
-			if (cursidenumber > numsides) cursidenumber = 0;;
+			if (cursidenumber > numsides)
+			  cursidenumber = 0;
 		}
 		poppedtool = peopletool;
 	} else if (control == mtypepopup) {
@@ -520,13 +524,13 @@ do_mouse_down_design(Point mouse, int mods)
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click cycle through all
 					   the "foreground" terrain types. */
-					OPTION_CYCLE(curttype, 0, numttypes, mods);
+					OPTION_CYCLE(curttype, 0, numttypes, 1, mods);
 				} else if ((mods & cmdKey) && t_is_cell(curttype)) {
 					/* Cmd-click and Cmd-Shift-click cycle through all
 					   the "background" terrain types. */
 					newbgttype = curbgttype;
 					do {
-						OPTION_CYCLE(newbgttype, 0, numttypes, mods);
+						OPTION_CYCLE(newbgttype, 0, numttypes, 1, mods);
 						if (newbgttype == curbgttype)
 						  break;
 					} while (!t_is_cell(newbgttype));
@@ -539,7 +543,7 @@ do_mouse_down_design(Point mouse, int mods)
 					   the types allowed for the current side. */
 					newutype = curutype;
 					do {
-						OPTION_CYCLE(newutype, 0, numutypes, mods);
+						OPTION_CYCLE(newutype, 0, numutypes, 1, mods);
 						if (newutype == curutype) break;
 					} while (!type_allowed_on_side(newutype, side_n(cursidenumber)));
 					curutype = newutype;
@@ -548,53 +552,62 @@ do_mouse_down_design(Point mouse, int mods)
 			case peopletool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click cycle around all the sides. */
-					OPTION_CYCLE(cursidenumber, 0, numsides + 1, mods);
+					OPTION_CYCLE(cursidenumber, 0, numsides + 1, 1, mods);
 				}
 				break;
 			case featuretool:
 				if (mods & optionKey && nextfid > 1) {
 					/* Option-click and Option-Shift-click cycle around the features. */
-					OPTION_CYCLE(curfid, 0, nextfid, mods);
+					OPTION_CYCLE(curfid, 0, nextfid, 1, mods);
 					curfeature = find_feature(curfid);
 				}
 				break;
 			case brushsizetool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click cycle through brush sizes. */
-					OPTION_CYCLE(curbrushradius, 0, 99, mods);
+					OPTION_CYCLE(curbrushradius, 0, 100, 1, mods);
 				}
 				break;
 			case materialtool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click cycle around amounts. */
-					OPTION_CYCLE(curmamount, 0, 99, mods);
+					OPTION_CYCLE(curmamount, 0, 10000, 1, mods);
+				} else if (mods & cmdKey) {
+					/* Command-click and Command-Shift-click adjust by 10. */
+					OPTION_CYCLE(curmamount, 0, 10000, 10, mods);
 				}
 				break;
 			case elevationtool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click adjust the elevation. */
-					OPTION_CYCLE(curelevation, minelev, maxelev + 1, mods);
+					OPTION_CYCLE(curelevation, minelev, maxelev + 1, 1, mods);
+				} else if (mods & cmdKey) {
+					/* Command-click and Command-Shift-click adjust by 10. */
+					OPTION_CYCLE(curelevation, minelev, maxelev + 1, 10, mods);
 				}
 				break;
 			case temperaturetool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click adjust the temp. */
-					OPTION_CYCLE(curtemperature, mintemp, maxtemp + 1, mods);
+					OPTION_CYCLE(curtemperature, mintemp, maxtemp + 1, 1, mods);
+				} else if (mods & cmdKey) {
+					/* Command-click and Command-Shift-click adjust by 10. */
+					OPTION_CYCLE(curtemperature, mintemp, maxtemp + 1, 10, mods);
 				}
 				break;
 			case cloudstool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click adjust the cloud type. */
-					OPTION_CYCLE(curcloudtype, 0, 4, mods);
+					OPTION_CYCLE(curcloudtype, 0, 4, 1, mods);
 				}
 				break;
 			case windstool:
 				if (mods & optionKey) {
 					/* Option-click and Option-Shift-click adjust the wind force. */
-					OPTION_CYCLE(curwindforce, minwindforce, maxwindforce, mods);
+					OPTION_CYCLE(curwindforce, minwindforce, maxwindforce, 1, mods);
 				} else if (mods & cmdKey) {
 					/* Cmd-click and Cmd-Shift-click adjust the direction. */
-					OPTION_CYCLE(curwinddir, 0, NUMDIRS, mods);
+					OPTION_CYCLE(curwinddir, 0, NUMDIRS, 1, mods);
 				}
 				break;
 			case viewtool:
@@ -612,7 +625,8 @@ do_mouse_down_design(Point mouse, int mods)
 		draw_design_window_tool(poppedtool);
 	}
 	draw_design_window_tool(tooltype);
-	if (toolchoice != tooltype) draw_design_window_tool(toolchoice);
+	if (toolchoice != tooltype)
+	  draw_design_window_tool(toolchoice);
 	/* As a special case, redraw the unit tool if the side tool was touched. */
 	if (tooltype == peopletool || poppedtool == peopletool) {
 		draw_design_window_tool(unittool);
@@ -662,12 +676,14 @@ feature_rename_dialog(Feature *feature)
 	win = GetNewDialog(dFeatureRename, NULL, (DialogPtr) -1L);
 	/* Seed the text items with the original names. */
 	newtypename = feature->typename;
-	if (newtypename == NULL) newtypename = "";
+	if (newtypename == NULL)
+	  newtypename = "";
 	GetDItem(win, diFeatureRenameType, &itemtype, &itemhandle, &itemrect);
 	c2p(newtypename, tmpstr);
 	SetIText(itemhandle, tmpstr);
 	newname = feature->name;
-	if (newname == NULL) newname = "";
+	if (newname == NULL)
+	  newname = "";
 	GetDItem(win, diFeatureRenameName, &itemtype, &itemhandle, &itemrect);
 	c2p(newname, tmpstr);
 	SetIText(itemhandle, tmpstr);
@@ -968,7 +984,7 @@ designer_save_dialog()
 				SetPt(&pnt, 100, 100);
 				sprintf(spbuf, "%s.g", namebuf);
 				c2p(spbuf, tmpstr);
-				SFPutFile(pnt, "\p", tmpstr, (DlgHookProcPtr) nil, &reply);
+				SFPutFile(pnt, "\p", tmpstr, /*(DlgHookProcPtr)*/ nil, &reply);
 				if (reply.good) {
 					/* Make the location of the file be the current volume. */
 					SetVol(reply.fName, reply.vRefNum);
