@@ -1,5 +1,5 @@
 /* Low-level drawing for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,6 +12,8 @@ any later version.  See the file COPYING.  */
    get pretty wedged. */
 
 #include "conq.h"
+#define wind_dir(w) ((w) & 0x07)
+#define wind_force(w) ((w) >> 3)
 #include "macconq.h"
 
 PolyHandle polygons[NUMPOWERS];
@@ -36,6 +38,10 @@ int numwindsicns = 0;
 
 Handle windsicnhandle[5];
 
+int numblastsicns = 0;
+
+Handle blastsicnhandle[3];
+
 static Image **best_terrain_images = NULL;
 
 static Image **best_unseen_images = NULL;
@@ -312,10 +318,10 @@ draw_cell_block(int sx, int sy, int n, int power, int t, int t2, int angle)
 				oldcolor.red = oldcolor.green = oldcolor.blue = 0;
 				RGBForeColor(&oldcolor);
 			} else if (mactimg->patdefined) {
-				FillRect(&rect, (unsigned char *) &(mactimg->monopat));
+				FillRect(&rect, IMG_PAT(mactimg));
 			}
 		} else {
-			FillRect(&rect, (unsigned char *) &(mactimg->monopat));
+			FillRect(&rect, IMG_PAT(mactimg));
 		}
 	} else {
 		/* No image; use a color instead. */
@@ -405,10 +411,10 @@ draw_hex_region(int sx, int sy, int power, int dogrid, int t, int t2, int angle)
 				RGBForeColor(&oldcolor);
 			} else if (mactimg->patdefined) {
 				/* Fall back on the b/w pattern. */
-				FillRgn(rgn, (unsigned char *) &(mactimg->monopat));
+				FillRgn(rgn, IMG_PAT(mactimg));
 			}
 		} else {
-			FillRgn(rgn, (unsigned char *) &(mactimg->monopat));
+			FillRgn(rgn, IMG_PAT(mactimg));
 		}
 	} else {
 		if (hasColorQD) {
@@ -482,10 +488,10 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 				RGBForeColor(&cellcolor);
 				tweakedcolor = TRUE;
 			} else {
-				PenPat((unsigned char *) &(mactimg->monopat));
+				PenPat(IMG_PAT(mactimg));
 			}
 		} else {
-			PenPat((unsigned char *) &(mactimg->monopat));
+			PenPat(IMG_PAT(mactimg));
 		}
 	} else {
 		if (hasColorQD) {
@@ -523,7 +529,7 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 		}
 	}
 	PenNormal();
-	if (tweakedcolor) {
+	if (hasColorQD && tweakedcolor) {
 		/* Restore the previous color. */
 		oldcolor.red = oldcolor.green = oldcolor.blue = 0;
 		RGBForeColor(&oldcolor);
@@ -535,7 +541,7 @@ draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power,
 void
 draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle)
 {
-	int dir, wid = cwid[power], ly, tweakedcolor = FALSE;
+	int dir, wid = cwid[power], lh, ly, xoff, yoff, tweakedcolor = FALSE;
 	Image *timg;
 	MacImage *mactimg;
 	RGBColor cellcolor, oldcolor;				
@@ -567,10 +573,10 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 				RGBForeColor(&cellcolor);
 				tweakedcolor = TRUE;
 			} else {
-				PenPat((unsigned char *) &(mactimg->monopat));
+				PenPat(IMG_PAT(mactimg));
 			}
 		} else {
-			PenPat((unsigned char *) &(mactimg->monopat));
+			PenPat(IMG_PAT(mactimg));
 		}
 	} else {
 		if (hasColorQD) {
@@ -588,9 +594,18 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 			PenPat(QDPat(gray));
 		}
 	}
+	lh = hhs[power];
+	if (angle == 30)
+	  lh /= 4;
+	else if (angle == 15)
+	  lh /= 8;
+	else
+	  lh /= 2;
+	xoff = hws[power] / 2 - wid / 2;
+	yoff = lh - wid / 2;
 	for_all_directions(dir) {
 		if (bitmask & (1 << dir)) {
-			MoveTo(sx + hws[power] / 2 - wid / 2, sy + hhs[power] / (angle == 30 ? 4 : 2) - wid / 2);
+			MoveTo(sx + xoff, sy + yoff);
 			ly = lsy[power][dir];
 			if (angle == 30)
 			  ly /= 2;
@@ -600,7 +615,7 @@ draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int po
 		}
 	}
 	PenNormal();
-	if (tweakedcolor) {
+	if (hasColorQD && tweakedcolor) {
 		/* Restore the previous color. */
 		oldcolor.red = oldcolor.green = oldcolor.blue = 0;
 		RGBForeColor(&oldcolor);
@@ -666,7 +681,7 @@ draw_country_borders(WindowPtr win, int sx, int sy, int bitmask, int power, int
 /* Draw a set of theater borders at the given position. */
 
 void
-draw_theater_borders(WindowPtr win, int sx, int sy, int bitmask, int power)
+draw_ai_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power)
 {
 	int wid, wid2, dir;
 	Rect tmprect;
@@ -714,7 +729,6 @@ draw_elevation(int sx, int sy, int power, int elev)
 		sprintf(spbuf, "%d", elev);
 		draw_legend_text(sx, sy, hhs[power] / 2, spbuf, 0);
 	}
-	/* (Also draw contour lines eventually) */
 }
 
 /* Don't draw the temperature in every cell, only do ones with even coords or
@@ -723,19 +737,20 @@ draw_elevation(int sx, int sy, int power, int elev)
 int
 draw_temperature_here(int x, int y)
 {
-	int dir, x1, y1, temphere = temperature_at(x, y);
+	int dir, x1, y1, temphere = temperature_view(dside, x, y);
 
 #ifdef DESIGNERS
 	/* Designers should see temperature in every cell. */
 	if (dside->designer)
 	  return TRUE;
 #endif /* DESIGNERS */
-	if (!g_see_all() && cover(dside, x, y) == 0)
-	  return FALSE;
 	for_all_directions(dir) {
-		point_in_dir(x, y, dir, &x1, &y1);
-		if (temphere != temperature_at(x1, y1)) {
-			return TRUE;
+		if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
+			if (temphere != temperature_view(dside, x1, y1))
+			  return TRUE;
+			/* Always show temperature around edge of known area. */
+			if (terrain_view(dside, x1, y1) == UNSEEN)
+			  return TRUE;
 		}
 	}
 	return (x % 2 == 0 && y % 2 == 0);
@@ -747,13 +762,37 @@ void
 draw_temperature(int sx, int sy, int power, int temp)
 {
 	sx += hws[power] / 2;  sy += hhs[power] / 2;
-	if (1 /* temp != 0 */) {
+	if (1 /* draw as text */) {
 		sprintf(spbuf, "%d", temp);  /* (should do char more portably) */
 		draw_legend_text(sx, sy, hhs[power] / 2, spbuf, 0);
 	}
 	/* (Also draw isotherms eventually) */
 }
 
+int
+draw_clouds_here(int x, int y)
+{
+	/* Draw clouds in every cell always. */
+	return TRUE;
+}
+
+void
+draw_clouds(int sx, int sy, int power, int cloudtype)
+{
+	Rect tmprect;
+
+	/* Don't draw clear sky. */
+	if (cloudtype == 0)
+	  return;
+	sx += hws[power] / 2;  sy += hhs[power] / 2;
+	SetRect(&tmprect, sx - hws[power] / 2, sy - hhs[power] / 2, sx + hws[power] / 2, sy + hhs[power] / 2);
+	/* Should use pat# 130 patterns for this instead. */
+	PenPat(cloudtype >= 3 ? QDPat(dkGray) : (cloudtype == 2 ? QDPat(gray) : QDPat(ltGray)));
+	PenMode(patBic);
+	PaintOval(&tmprect);
+	PenNormal();
+}
+
 /* Don't draw the winds in every cell, only do ones with odd coords or
    ones where the wind in any adjacent cell is different. */
 /* (generic routine?) */
@@ -761,31 +800,31 @@ draw_temperature(int sx, int sy, int power, int temp)
 int
 draw_winds_here(int x, int y)
 {
-	int dir, x1, y1, windhere = raw_wind_at(x, y);
+	int dir, x1, y1, windhere = wind_view(dside, x, y);
 
 #ifdef DESIGNERS
 	/* Designers should see wind in every cell. */
 	if (dside->designer)
 	  return TRUE;
 #endif /* DESIGNERS */
-	if (!g_see_all() && cover(dside, x, y) == 0)
-	  return FALSE;
 	for_all_directions(dir) {
 		if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-			if (windhere != raw_wind_at(x1, y1)) {
-				return TRUE;
-			}
+			if (windhere != wind_view(dside, x1, y1))
+			  return TRUE;
+			/* Always show wind around edge of known area. */
+			if (terrain_view(dside, x1, y1) == UNSEEN)
+			  return TRUE;
 		}
 	}
 	return (x % 2 == 1 && y % 2 == 1);
 }
 
-/* Indicate the winds at the given location. */
+/* Indicate the given wind at the given location. */
 
 void
-draw_winds(int sx, int sy, int power, int wdir, int wforce)
+draw_winds(int sx, int sy, int power, int rawwind)
 {
-	int i;
+	int i, wdir = wind_dir(rawwind), wforce = wind_force(rawwind);
 	GrafPtr curport;
 
 	sx += (hws[power] - 16) / 2;  sy += (hhs[power] - 16) / 2;
@@ -811,46 +850,25 @@ draw_winds(int sx, int sy, int power, int wdir, int wforce)
 	plot_sicn(curport, sx, sy, windsicnhandle[wforce], wdir, FALSE, srcOr);
 }
 
-int
-draw_clouds_here(int x, int y)
-{
-#ifdef DESIGNERS
-	/* Designers should see clouds in every cell. */
-	if (dside->designer)
-	  return TRUE;
-#endif /* DESIGNERS */
-	if (!g_see_all() && cover(dside, x, y) == 0)
-	  return FALSE;
-	return TRUE;
-}
+/* Draw the view coverage of a cell (for debugging). */
 
 void
-draw_clouds(int sx, int sy, int power, int cloudtype)
+draw_coverage(int sx, int sy, int power, int cov, int altcov)
 {
-	Rect tmprect;
+	char buf[40];
 
-	if (cloudtype == 0)
-	  return;
-	sx += hws[power] / 2;  sy += hhs[power] / 2;
-	SetRect(&tmprect, sx - hws[power] / 2, sy - hhs[power] / 2, sx + hws[power] / 2, sy + hhs[power] / 2);
-	/* Should use pat# 130 patterns for this instead. */
-	PenPat(cloudtype >= 3 ? QDPat(dkGray) : (cloudtype == 2 ? QDPat(gray) : QDPat(ltGray)));
-	PenMode(patBic);
-	PaintOval(&tmprect);
-	PenNormal();
-}
-
-/* Draw the number of units observing a cell (for debugging). */
-
-void
-draw_coverage(int sx, int sy, int power, int cov)
-{
-	/* Adjust to the lower left corner of the cell. */
-	sx += 2;  sy += hcs[power] - 2;
+	buf[0] = '\0';
 	if (cov > 0) {
-		sprintf(spbuf, ":%d:", cov);
+		tprintf(buf, ":%d:", cov);
+	}
+	if (altcov > -1) {
+		tprintf(buf, ",%d:", altcov);
+	}
+	if (buf[0] != '\0') {
+		/* Adjust to the lower left corner of the cell. */
+		sx += 2;  sy += hcs[power] - 2;
+		draw_legend_text(sx, sy, hhs[power] / 2, buf, -1);
 	}
-	draw_legend_text(sx, sy, hhs[power] / 2, spbuf, -1);
 }
 
 /* Draw a unit's name, if it has one. */
@@ -889,11 +907,11 @@ draw_legend_text(int sx, int sy, int sh, char *legend, int just)
 		/* This makes a big white box for name, less attractive but easier to read. */
 		GetFontInfo(&fontinfo);
 		maskrect.top = sy - fontinfo.ascent;
-		/* Ensure a one pixel of white all along the top. */
+		/* Ensure a one pixel strip of white all along the top. */
 		maskrect.top += 1;
 		maskrect.left = strleft - 1;
 		maskrect.bottom = sy + fontinfo.descent;
-		/* Ensure a one pixel of white all along the bottom, even below descenders. */
+		/* Ensure a one pixel strip of white all along the bottom, even below descenders. */
 		maskrect.bottom += 1;
 		maskrect.right = maskrect.left + strwid + 1;
 		FillRect(&maskrect, QDPat(white));
@@ -901,15 +919,52 @@ draw_legend_text(int sx, int sy, int sh, char *legend, int just)
 	DrawText(legend, 0, strlen(legend));
 }
 
+
 void
 draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 {
+	int i;
 	Rect tmprect;
+	Rect srcrect, imagerect;
+	BitMap bm, *winbits;
+	GrafPtr curport;
 
+	/* Collect sicns if not already in. */
+	if (numblastsicns == 0) {
+		for (i = 0; i <= 2; ++i) {
+			blastsicnhandle[i] = GetResource('SICN', sicnMiss + i);
+			++numblastsicns;
+		}
+	}
 	SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
 	if (sw >= 16) {
-		/* Instead should save image under here, then draw blast */
-		InvertRect(&tmprect);
+		winbits = &(((GrafPtr) win)->portBits);
+		imagerect.top = sy;  imagerect.left = sx;
+		imagerect.bottom = imagerect.top + sh;  imagerect.right = imagerect.left + sw;
+		/* should save image under here, then draw blast */
+		GetPort(&curport);
+		SetRect(&srcrect, 0, 0, 16, 16);
+		bm.rowBytes = 2;
+		bm.bounds = srcrect;
+		bm.baseAddr = *(blastsicnhandle[blasttype]);
+		if (hasColorQD) {
+			RGBColor tmpcolor;
+
+			OffsetRect(&imagerect, 1, 1);
+			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
+			tmpcolor.red = 60000;  tmpcolor.green = tmpcolor.blue = 0;
+			RGBForeColor(&tmpcolor);
+			OffsetRect(&imagerect, -1, -1);
+			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
+			/* Restore the previous color. */
+			tmpcolor.red = tmpcolor.green = tmpcolor.blue = 0;
+			RGBForeColor(&tmpcolor);
+		} else {
+			OffsetRect(&imagerect, 1, 1);
+			CopyBits(&bm, winbits, &srcrect, &imagerect, srcBic, nil);
+			OffsetRect(&imagerect, -1, -1);
+			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
+		}
 	} else {
 		InvertRect(&tmprect);
 	}
@@ -922,8 +977,7 @@ clear_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype)
 
 	SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
 	if (sw >= 16) {
-		/* Instead should restore image under here */
-		InvertRect(&tmprect);
+		/* should restore image under here */
 	} else {
 		InvertRect(&tmprect);
     }
@@ -957,7 +1011,8 @@ plot_sicn(WindowPtr win, int sx, int sy, Handle sicnhandle, int n, int erase, in
 	SetRect(&srcrect, 0, 0, 16, 16);
 	bm.rowBytes = 2;
 	bm.bounds = srcrect;
-	if (erase) EraseRect(&imagerect);
+	if (erase)
+	  EraseRect(&imagerect);
 	bm.baseAddr = *(sicnhandle) + 32 * n;
 	CopyBits(&bm, winbits, &srcrect, &imagerect, mode, nil);
 }
