@@ -1,5 +1,5 @@
 /* Help for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,12 +9,17 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
+static pascal void draw_instructions_text(WindowPtr win, short ditem);
+static pascal void help_vscroll_fn(ControlHandle control, short code);
+
 /* Globals for the instructions window. */
 
 WindowPtr instructionswin = nil;
 
 TEHandle instructions_text = nil;
 
+UserItemUPP draw_instructions_text_proc;
+
 /* Globals for the help window. */
 
 DialogPtr helpwin = nil;
@@ -25,6 +30,8 @@ TEHandle helptext = nil;
 
 ControlHandle helpvscrollbar;
 
+ControlActionUPP help_vscroll_proc;
+
 ControlHandle topicsbutton;
 ControlHandle helpbutton;
 ControlHandle prevbutton;
@@ -51,7 +58,7 @@ int nodestackpos;
 
 /* The instructions window. */
 
-pascal void
+static pascal void
 draw_instructions_text(WindowPtr win, short ditem)
 {
 	GrafPtr oldport;
@@ -72,20 +79,18 @@ draw_instructions_text(WindowPtr win, short ditem)
 void
 instructions_dialog()
 {
+	int mainwidth, mainheight;
+	Rect tmprect;
+
 	if (instructionswin == nil) {
 		create_instructions_dialog();
 	}
 	if (first_windows) {
-		GDHandle main_screen = GetMainDevice();
-		int main_screen_width, main_screen_height;
-		Rect tmprect = instructionswin->portRect;
-		Rect gdrect = (*main_screen)->gdRect;
-
-		main_screen_width = gdrect.right - gdrect.left;
-		main_screen_height = gdrect.bottom - gdrect.top;
+		get_main_screen_size(&mainwidth, &mainheight);
+		tmprect = instructionswin->portRect;
 		MoveWindow(instructionswin,
-				   main_screen_width - (tmprect.right - tmprect.left) - 3,
-				   main_screen_height - (tmprect.bottom - tmprect.top) - 3,
+				   mainwidth - (tmprect.right - tmprect.left) - 3,
+				   mainheight - (tmprect.bottom - tmprect.top) - 3,
 				   FALSE);
 	}
 	ShowWindow(instructionswin);
@@ -102,12 +107,15 @@ create_instructions_dialog()
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 	Scorekeeper *sk;
 
+	if (draw_instructions_text_proc == NULL)
+	  draw_instructions_text_proc = NewUserItemProc(draw_instructions_text);
+
 	instructionswin = GetNewDialog(dInstructions, nil, (DialogPtr) -1L);
 	GetDItem(instructionswin, diInstructionsTitle, &itemtype, &itemhandle, &itemrect);
 	c2p(((mainmodule && mainmodule->title) ? mainmodule->title : ""), tmpstr);
 	SetIText(itemhandle, tmpstr);
 	GetDItem(instructionswin, diInstructionsText, &itemtype, &itemhandle, &itemrect);
-	SetDItem(instructionswin, diInstructionsText, itemtype, (Handle) draw_instructions_text, &itemrect);
+	SetDItem(instructionswin, diInstructionsText, itemtype, (Handle) draw_instructions_text_proc, &itemrect);
 	destrect = itemrect;
 	viewrect = itemrect;
 	SetPort(instructionswin);
@@ -187,7 +195,7 @@ hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
 	switch (itemhit) {
 		case diInstructionsHelp:
 			/* Just jump to the help dialog. */
-			help_dialog();
+			help_dialog(NULL);
 			break;
 	}
 	return TRUE;
@@ -199,11 +207,15 @@ hit_instructions_dialog(DialogPtr dialog, int itemhit, EventRecord *evt)
    anywhere, anytime. */
 
 void
-help_dialog()
+help_dialog(HelpNode *helpnode)
 {
 	if (helpwin == nil) {
 		create_help_window();
 	}
+	if (helpnode != NULL) {
+		cur_help_node = helpnode;
+		set_help_content(helpnode);
+	}
 	ShowWindow(helpwin);
 	SelectWindow(helpwin);
 }
@@ -332,11 +344,11 @@ create_help_window()
 	/* Add the Mac-specific help nodes. */
 	/* (Note that these will appear in *reverse* order from here, because each
 	   is being glued right after the first help node. */
-	add_help_node("menus", describe_menus, 0, firsthelpnode);
-	add_help_node("mouse", describe_mouse, 0, firsthelpnode);
-	add_help_node("keyboard", describe_keyboard, 0, firsthelpnode);
-	help_help_node = add_help_node("help", describe_help, 0, firsthelpnode);
-	topics_help_node = add_help_node("topics", describe_topics, 0, firsthelpnode);
+	add_help_node("menus", describe_menus, 0, first_help_node);
+	add_help_node("mouse", describe_mouse, 0, first_help_node);
+	add_help_node("keyboard", describe_keyboard, 0, first_help_node);
+	help_help_node = add_help_node("help", describe_help, 0, first_help_node);
+	topics_help_node = add_help_node("topics", describe_topics, 0, first_help_node);
 	cur_help_node = topics_help_node;
 	set_help_content(cur_help_node);
 	if (nodestack == NULL)
@@ -361,9 +373,9 @@ set_help_content(HelpNode *curnode)
 #ifdef THINK_C
 	/* Hack up newlines so that TextEdit recognizes them. */
 	{
-		int i;
+		int i, len = strlen(str);
 	
-		for (i = 0; i < strlen(str); ++i) {
+		for (i = 0; i < len; ++i) {
 			if (str[i] == '\n')
 			  str[i] = '\r';
 		}
@@ -426,8 +438,8 @@ activate_help(int activate)
 	HiliteControl(helpvscrollbar, (activate ? 0 : 255));
 }
 
-pascal void
-help_vscroll_proc(ControlHandle control, short code)
+static pascal void
+help_vscroll_fn(ControlHandle control, short code)
 {
 	int oldvalue, curvalue, minvalue, maxvalue, pagesize, jump;
 
@@ -473,6 +485,9 @@ do_mouse_down_help(Point mouse, int mods)
 	int i;
 	HelpNode *prevhelpnode = cur_help_node, *helpnode;
 
+	if (help_vscroll_proc == NULL)
+	  help_vscroll_proc = NewControlActionProc(help_vscroll_fn);
+
 	part = FindControl(mouse, helpwin, &control);
 	if (control == topicsbutton) {
 		cur_help_node = topics_help_node;
@@ -493,7 +508,7 @@ do_mouse_down_help(Point mouse, int mods)
 				case inDownButton:
 				case inPageUp:
 				case inUpButton:
-					value = TrackControl(control, mouse, (ProcPtr) help_vscroll_proc);
+					value = TrackControl(control, mouse, (ControlActionUPP) help_vscroll_proc);
 					break;
 				case inThumb:
 					value = GetCtlValue(control);
@@ -578,19 +593,8 @@ zoom_help(int part)
 
 	EraseRect(&helpwin->portRect);
 	if (part == inZoomOut) {
-		if (!hasColorQD) {
-			/* If no Color QD, then there is only one screen. */
-			zoomrect = QD(screenBits).bounds;
-			InsetRect(&zoomrect, 4, 4);
-		} else {
-			for (gd = GetDeviceList(); gd != nil; gd = GetNextDevice(gd)) {
-				if (TestDeviceAttribute(gd, screenDevice)) {
-					if (TestDeviceAttribute(gd, screenActive)) {
-						zoomgd = gd;
-						break;
-					}
-				}
-			}
+		if (hasColorQD) {
+			zoomgd = best_zoom_screen(&helpwin->portRect);
 			zoomrect = (*zoomgd)->gdRect;
 			if (zoomgd == GetMainDevice()) {
 				zoomrect.top += GetMBarHeight();
@@ -598,6 +602,13 @@ zoom_help(int part)
 			titleh = 20; /* (should calc) */
 			zoomrect.top += titleh;
 			InsetRect(&zoomrect, 3, 3);
+		} else {
+			/* If no Color QD, then there is only the one screen. */
+			zoomrect = QD(screenBits).bounds;
+			zoomrect.top += GetMBarHeight();
+			titleh = 20; /* (should calc) */
+			zoomrect.top += titleh;
+			InsetRect(&zoomrect, 4, 4);
 		}
 		(*((WStateDataHandle) ((WindowPeek) helpwin)->dataHandle))->stdState = zoomrect;
 	}
