@@ -73,10 +73,11 @@ ImageFamily *imf;
     Handle imfhandle, pathandle, sicnhandle, iconhandle, maskhandle;
     PicHandle pichandle;
     PixPatHandle ppathandle;
+	PixMapHandle pmhandle;
     CIconHandle cicnhandle;
     PixMap pmap;
     Rect bounds;
-    Str255 namestr, maskstr, resname;
+    Str255 namestr, maskstr, im16x16str, resname;
 	short resid;
 	ResType restype;
 
@@ -87,6 +88,9 @@ ImageFamily *imf;
 	/* The name of the mask is always formed by appending " mask". */
 	sprintf(tmpstrbuf, "%s mask", imf->name);
 	c2p(tmpstrbuf, maskstr);
+	/* The name of the 16x16 cicn is always formed by appending " 16x16". */
+	sprintf(tmpstrbuf, "%s 16x16", imf->name);
+	c2p(tmpstrbuf, im16x16str);
 	/* Look for and load the image family specification resource first. */
 	imfhandle = (Handle) GetNamedResource('XCif', namestr);
 	if (imfhandle != nil) {
@@ -133,8 +137,27 @@ ImageFamily *imf;
 			}
 			HUnlock((Handle) cicnhandle);
 		}
+		cicnhandle = (CIconHandle) GetNamedResource('cicn', im16x16str);
+		if (cicnhandle != nil) {
+			HLock((Handle) cicnhandle);
+			/* Need to get id so we can use special trap. */
+			GetResInfo((Handle) cicnhandle, &resid, &restype, resname);
+			cicnhandle = GetCIcon(resid);
+			pmap = (*cicnhandle)->iconPMap;
+			bounds = pmap.bounds;
+			w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
+			img = get_img(imf, w, h);
+			if (img != NULL) {
+				img->minw = img->minh = 8;
+				img->maxw = img->maxh = 128;
+				macimg = get_mac_image(img);
+				macimg->colricon = (Handle) cicnhandle;
+				/* Mask is built in, don't need to load separately. */
+			}
+			HUnlock((Handle) cicnhandle);
+		}
 	} else {
-		/* (should at least get the mono part, without using CQD traps) */
+		/* (should at least try to get the mono part, without using CQD traps) */
 	}
 	/* Pick up ICONs. */
 	iconhandle = (Handle) GetNamedResource('ICON', namestr);
@@ -188,8 +211,18 @@ ImageFamily *imf;
 			/* Need to get the id of the ppat so we can use special trap. */
 			GetResInfo((Handle) ppathandle, &resid, &restype, resname);
 			ppathandle = GetPixPat(resid);
-			/* (should get by actual size?) */
-			img = get_img(imf, 8, 8);
+			pmhandle = (*ppathandle)->patMap;
+			switch ((*ppathandle)->patType) {
+				case 0:
+					/* (should put something in monodata?) */
+					w = h = 8;
+					break;
+				case 1:
+					/* Compute the actual size of the pattern. */
+					bounds = (*pmhandle)->bounds;
+					w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
+			}
+			img = get_img(imf, w, h);
 			if (img != NULL) {
 				/* Indicate that we can use this pattern for any size area. */
 				img->minw = img->minh = 1;
@@ -197,10 +230,28 @@ ImageFamily *imf;
 				img->istile = TRUE;
 				macimg = get_mac_image(img);
 				macimg->colrpat = ppathandle;
-				/* Set the monopat. */
-				for (i = 0; i < 8; ++i)
-				  SET_IMG_PAT(macimg, i, ((char *) (*ppathandle)->pat1Data)[i]);
-				macimg->patdefined = 1;
+				if (w == 8 && h == 8) {
+					/* Set the monopat. */
+					for (i = 0; i < 8; ++i)
+					  SET_IMG_PAT(macimg, i, QD_PAT_ADDR((*ppathandle)->pat1Data)[i]);
+					macimg->patdefined = 1;
+				}
+			}
+			/* If the pattern is larger than 8x8, we need to make an 8x8 image for
+			   the mono pattern. */
+			if (!(w == 8 && h == 8)) {
+				img = get_img(imf, 8, 8);
+				if (img != NULL) {
+					/* Indicate that we can use this pattern for any size area. */
+					img->minw = img->minh = 1;
+					img->maxw = img->maxh = 9999;
+					img->istile = TRUE;
+					macimg = get_mac_image(img);
+					/* Set the monopat. */
+					for (i = 0; i < 8; ++i)
+					  SET_IMG_PAT(macimg, i, QD_PAT_ADDR((*ppathandle)->pat1Data)[i]);
+					macimg->patdefined = 1;
+				}
 			}
 		}
 	}
@@ -215,7 +266,27 @@ ImageFamily *imf;
 			img->istile = TRUE;
 			macimg = get_mac_image(img);
 			if (macimg->patdefined) {
-				/* (should compare and warn if different) */
+				int allzeros = TRUE;
+
+				for (i = 0; i < 8; ++i) {
+					if (((char *) &(macimg->monopat))[i] != 0) {
+						allzeros = FALSE;
+						break;
+					}
+				}
+				/* A mono pattern of all zeros is a default pat from a ppat; overwrite
+				   it silently. */
+				if (!allzeros) {
+					for (i = 0; i < 8; ++i) {
+						if (((char *) &(macimg->monopat))[i] != ((char *) *pathandle)[i]) {
+							run_warning("ppat/PAT mismatch for \"%s\", overwriting ppat",
+										imf->name);
+							break;
+						}
+					}
+				}
+				for (i = 0; i < 8; ++i)
+				  SET_IMG_PAT(macimg, i, ((char *) *pathandle)[i]);
 			} else {
 				/* Set the monopat. */
 				for (i = 0; i < 8; ++i)
@@ -305,28 +376,28 @@ ImageFamily *imf;
 		if (!monodone && img->monodata != lispnil) {
 			if (rowbytes < 1)
 			  rowbytes = ((actualw + 15) / 16) * 2;
-			numbytes = actualh * rowbytes;
+			numbytes = actualh * (actualw <= 8 ? 2 : rowbytes);
 			monohandle = NewHandle(numbytes);
 			memset(*maskhandle, 0, numbytes);
 			/* Read as many bytes as directed. */
-			mac_interp_bytes(img->monodata, numbytes, monohandle, (actualw == 8 ? 1 : 0));
+			mac_interp_bytes(img->monodata, numbytes, monohandle, (actualw <= 8 ? 1 : 0));
 			monodone = TRUE;
 		}
 		if (!maskdone && img->maskdata != lispnil) {
 			if (rowbytes < 1)
 			  rowbytes = ((actualw + 15) / 16) * 2;
-			numbytes = actualh * rowbytes;
+			numbytes = actualh * (actualw <= 8 ? 2 : rowbytes);
 			maskhandle = NewHandle(numbytes);
 			memset(*maskhandle, 0, numbytes);
 			/* Read as many bytes as directed. */
-			mac_interp_bytes(img->maskdata, numbytes, maskhandle, (actualw == 8 ? 1 : 0));
+			mac_interp_bytes(img->maskdata, numbytes, maskhandle, (actualw <= 8 ? 1 : 0));
 			maskdone = TRUE;
 		}
 		/* Limit bits per pixel to reasonable values. */
 		if (pixelsize < 1 || pixelsize > 8)
 		  pixelsize = 8;
 		/* A sufficiently small tiling color image can be a color pattern. */
-		if (actualw <= 64 && actualh <= 64 && img->istile && img->colrdata != lispnil) {
+		if (hasColorQD && actualw <= 64 && actualh <= 64 && img->istile && img->colrdata != lispnil) {
 			/* Make a color pattern. */
 			ppathandle = NewPixPat();
 			pmhandle = (*ppathandle)->patMap;
@@ -344,7 +415,7 @@ ImageFamily *imf;
 			macimg->colrpat = ppathandle;
 			colrdone = maskdone = TRUE;
 		}
-		if (actualw <= 64 && actualh <= 64
+		if (hasColorQD && actualw <= 64 && actualh <= 64
 		    && !img->istile
 		    && (img->colrdata != lispnil
 		        || (macimg->monosicn == nil
@@ -394,11 +465,11 @@ ImageFamily *imf;
 			if (img->colrdata != lispnil) {
 				mac_interp_bytes(img->colrdata, numbytes, datahandle, 0);
 			} else if (img->monodata != lispnil) {
-				mac_interp_bytes(img->monodata, numbytes, datahandle, (actualw == 8 ? 1 : 0));
+				mac_interp_bytes(img->monodata, numbytes, datahandle, (actualw <= 8 ? 1 : 0));
 				/* We need an ersatz color table. */
 				(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 			} else if (img->maskdata != lispnil) {
-				mac_interp_bytes(img->maskdata, numbytes, datahandle, (actualw == 8 ? 1 : 0));
+				mac_interp_bytes(img->maskdata, numbytes, datahandle, (actualw <= 8 ? 1 : 0));
 				/* We need an ersatz color table. */
 				(*cicnhandle)->iconPMap.pmTable = synth_ctab();
 			}
@@ -410,6 +481,14 @@ ImageFamily *imf;
 				(*cicnhandle)->iconBMap.rowBytes = bitrowbytes;
 				HUnlock(macimg->monoicon);
 			}
+			/* If a mono sicn is already set up, use it as the icon's bitmap. */
+			if (macimg->monosicn != nil) {
+				HLock(macimg->monosicn);
+				memcpy(((char *) (*cicnhandle)->iconMaskData) + actualh * bitrowbytes,
+					   *(macimg->monosicn), actualh * bitrowbytes);
+				(*cicnhandle)->iconBMap.rowBytes = bitrowbytes;
+				HUnlock(macimg->monosicn);
+			}
 			/* ...or just use the monohandle that we built earlier. */
 			if (monohandle != nil) {
 				HLock(monohandle);
@@ -425,6 +504,14 @@ ImageFamily *imf;
 					   *(macimg->maskicon), actualh * bitrowbytes);
 				HUnlock(macimg->maskicon);
 			}
+			/* If a mask sicn is already set up, use it as the icon's mask. */
+			if (macimg->masksicn != nil) {
+				HLock(macimg->masksicn);
+				memcpy((char *) (*cicnhandle)->iconMaskData,
+					   *(macimg->masksicn), actualh * bitrowbytes);
+				HUnlock(macimg->masksicn);
+			}
+			/* ...or just use the maskhandle that we built earlier. */
 			if (maskhandle != nil) {
 				HLock(maskhandle);
 				memcpy((char *) (*cicnhandle)->iconMaskData,
@@ -687,7 +774,9 @@ convert_ctab(CTabHandle ctabhandle)
 					   			 	  lispnil))));
 			/* Splice the new entry into the list so that all are sorted by index. */
 			restprev = lispnil;
-			for (rest = rslt; cspec.value > c_number(car(car(rest))); rest = cdr(rest)) {
+			for (rest = rslt;
+				 cspec.value > (numberp(car(car(rest))) ? c_number(car(car(rest))) : 1000000);
+				 rest = cdr(rest)) {
 				restprev = rest;
 			}
 			tmp2 = cons(tmp, rest);
