@@ -1,5 +1,5 @@
 /* Unit lists for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,6 +9,9 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "macconq.h"
 
+static pascal void list_vscroll_fn(ControlHandle control, short code);
+static pascal void list_hscroll_fn(ControlHandle control, short code);
+
 /* (should be adjusted for expected range of values) */
 
 int curactcolw = 10;
@@ -45,6 +48,9 @@ int listnum = 1;
 
 int lastlisth = -1, lastlistv = -1;
 
+ControlActionUPP list_vscroll_proc;
+ControlActionUPP list_hscroll_proc;
+
 /* Create a new list of units. */
 
 void
@@ -423,18 +429,23 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 		DrawText(spbuf, 0, strlen(spbuf));
 	}
 	x += hpcolw;
-	if (side_sees_unit(dside, unit)) {
+	if (side_sees_unit(dside, unit) && inside_area(unit->x, unit->y)) {
 		MoveTo(x, texty);
 		acp_desc(spbuf, unit, FALSE);
-		if (strlen(spbuf) == 0) strcpy(spbuf, "-");
+		if (strlen(spbuf) == 0)
+		  strcpy(spbuf, "-");
 		DrawText(spbuf, 0, strlen(spbuf));
 	}
 	/* Draw the location. */
 	x += acpcolw;
 	MoveTo(x, texty);
-	sprintf(spbuf, "%d,%d", unit->x, unit->y);
-	if (unit->z != 0) {
-		tprintf(spbuf, ",%d", unit->z);
+	if (inside_area(unit->x, unit->y)) {
+		sprintf(spbuf, "%d,%d", unit->x, unit->y);
+		if (unit->z != 0) {
+			tprintf(spbuf, ",%d", unit->z);
+		}
+	} else {
+		strcpy(spbuf, "  -  ");
 	}
 	DrawText(spbuf, 0, strlen(spbuf));
 	/* Draw the state of all the supplies. */
@@ -447,7 +458,17 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 			x += supcolw;
 		}
 	} else {
-		x += nummtypes * supcolw; /* actually not necessary since no more cols */
+		x += nummtypes * supcolw;
+	}
+	spbuf[0] = '\0';
+	if (side_sees_unit(dside, unit)
+		&& !inside_area(unit->x, unit->y)
+		&& unit_appear_turn(unit) >= 0) {
+		tprintf(spbuf, "appear %s", absolute_date_string(unit_appear_turn(unit)));
+	}
+	if (strlen(spbuf) > 0) {
+		MoveTo(x, texty);
+		DrawText(spbuf, 0, strlen(spbuf));
 	}
 	/* Fix the clipping. */
 	cliprect = list->window->portRect;
@@ -507,8 +528,8 @@ adjust_list_decor(List *list)
 
 List *curlist;
 
-pascal void
-list_vscroll_proc(ControlHandle control, short code)
+static pascal void
+list_vscroll_fn(ControlHandle control, short code)
 {
 	int curvalue, minvalue, maxvalue, oldvalue, pagesize, jump;
 	RgnHandle tmprgn;
@@ -553,8 +574,8 @@ list_vscroll_proc(ControlHandle control, short code)
 	DisposeRgn(tmprgn);
 }
 
-pascal void
-list_hscroll_proc(ControlHandle control, short code)
+static pascal void
+list_hscroll_fn(ControlHandle control, short code)
 {
 	int curvalue, minvalue, maxvalue, oldvalue, pagesize, jump;
 	RgnHandle tmprgn;
@@ -611,6 +632,11 @@ do_mouse_down_list(List *list, Point mouse, int mods)
 	int n, tmp;
 	WindowPtr window = list->window;
 
+	if (list_vscroll_proc == NULL)
+	  list_vscroll_proc = NewControlActionProc(list_vscroll_fn);
+	if (list_hscroll_proc == NULL)
+	  list_hscroll_proc = NewControlActionProc(list_hscroll_fn);
+
 	part = FindControl(mouse, window, &control);
 	if (control == list->vscrollbar) {
 		switch (part) {
@@ -623,7 +649,7 @@ do_mouse_down_list(List *list, Point mouse, int mods)
 				break;
 			default:
 				curlist = list;
-				part = TrackControl(control, mouse, (ProcPtr) list_vscroll_proc);
+				part = TrackControl(control, mouse, list_vscroll_proc);
 				list->firstvisible = GetCtlValue(control);
 				break;
 		}
@@ -638,7 +664,7 @@ do_mouse_down_list(List *list, Point mouse, int mods)
 				break;
 			default:
 				curlist = list;
-				part = TrackControl(control, mouse, (ProcPtr) list_hscroll_proc);
+				part = TrackControl(control, mouse, list_hscroll_proc);
 				list->firstvisfield = GetCtlValue(control);
 				break;
 		}
