@@ -1,5 +1,5 @@
 /* Map interaction for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,14 +13,20 @@ extern void do_remove_terrain(void);
 extern void do_set_formation(void);
 extern void set_position_modally(void);
 
+static pascal void map_scroll_fn(ControlHandle control, short code);
+
+int topunithgt = 60;
+
 /* This is a temporary used by the map scroll proc. */
 
 Map *curmap;
 
+ControlActionUPP map_scroll_proc;
+
 /* This scroll proc is shared by both the horizontal and vertical scrollbars. */
 
-pascal void
-map_scroll_proc(ControlHandle control, short code)
+static pascal void
+map_scroll_fn(ControlHandle control, short code)
 {
 	int curvalue, pagesize, jump;
 	int oldsx, oldsy;
@@ -82,11 +88,14 @@ do_mouse_down_map(Map *map, Point mouse, int mods)
 {
 	ControlHandle control;
 	short part;
-	long oldsx = map->vp->sx, oldsy = map->vp->sy;
+	int oldsx = map->vp->sx, oldsy = map->vp->sy;
 	WindowPtr window = map->window;
 	RgnHandle tmprgn;
 	Rect tmprect;
 
+	if (map_scroll_proc == NULL)
+	  map_scroll_proc = NewControlActionProc(map_scroll_fn);
+
 	part = FindControl(mouse, window, &control);
 	if (control == map->hscrollbar) {
 		/* Handle the horizontal scrollbar. */
@@ -111,7 +120,7 @@ do_mouse_down_map(Map *map, Point mouse, int mods)
 				break;
 			default:
 				curmap = map;
-				part = TrackControl(control, mouse, (ProcPtr) map_scroll_proc);
+				part = TrackControl(control, mouse, map_scroll_proc);
 				break;
 		}
 	} else if (control == map->vscrollbar) {
@@ -137,7 +146,7 @@ do_mouse_down_map(Map *map, Point mouse, int mods)
 				break;
 			default:
 				curmap = map;
-				part = TrackControl(control, mouse, (ProcPtr) map_scroll_proc);
+				part = TrackControl(control, mouse, map_scroll_proc);
 				break;
 		}
 	} else if (mouse.h <= conwid) {
@@ -289,26 +298,58 @@ toggle_map_grid(Map *map)
 void
 toggle_map_topline(Map *map)
 {
-#if 0
+	int oldtoph;
 	Rect tmprect;
 	RgnHandle tmprgn;
-#endif
 	GrafPtr oldport;
 
-	map->toph = (map->toph ? 0 : tophgt);
+	oldtoph = map->toph;
+	map->toplineh = (map->toplineh ? 0 : tophgt);
+	map->toph = map->toplineh + map->topunith;
 	set_content_rect(map);
-	if (map->toph) {
+	if (map->toplineh) {
  		GetPort(&oldport);
 		SetPort(map->window);
-#if 0 /* this would be better than redrawing everything */
+		tmprgn = NewRgn();
 		tmprect = map->window->portRect;
 		tmprect.left += conwid;
 		tmprect.right -= sbarwid;  tmprect.bottom -= sbarwid;
-/*		ScrollRect(&tmprect, 0, map->toph, tmprgn); */
-		/* (should draw the topline here now) */
-#endif
+		ScrollRect(&tmprect, 0, map->toplineh, tmprgn);
+		draw_top_line(map);
+		InvalRgn(tmprgn);
+		update_window(map->window);
+		DisposeRgn(tmprgn);
 		SetPort(oldport);
+	} else {
 		force_map_update(map);
+	}
+}
+
+void
+toggle_map_topunit(Map *map)
+{
+	int oldtoph;
+	Rect tmprect;
+	RgnHandle tmprgn;
+	GrafPtr oldport;
+
+	oldtoph = map->toph;
+	map->topunith = (map->topunith ? 0 : topunithgt);
+	map->toph = map->toplineh + map->topunith;
+	set_content_rect(map);
+	if (map->topunith) {
+ 		GetPort(&oldport);
+		SetPort(map->window);
+		tmprect = map->window->portRect;
+		tmprect.left += conwid;  tmprect.top += oldtoph;
+		tmprect.right -= sbarwid;  tmprect.bottom -= sbarwid;
+		tmprgn = NewRgn();
+		ScrollRect(&tmprect, 0, map->topunith, tmprgn);
+		InvalRgn(tmprgn);
+		update_window(map->window);
+		DisposeRgn(tmprgn);
+		draw_unit_info(map);
+		SetPort(oldport);
 	} else {
 		force_map_update(map);
 	}
@@ -437,6 +478,8 @@ toggle_map_ai(Map *map)
 static int selrect;
 static int downx, downy, downdir;
 
+void drag_for_distance(Map *map, int h0, int v0);
+
 void
 do_mouse_down_map_content(Map *map, int h, int v, int mods)
 {
@@ -451,9 +494,10 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 	}
 	if (map_modal > 0) {
 		switch (map_modal) {
-			case ZOOM_MODAL:
-				select_area_and_zoom(map, h, v, mods);
-				/* Done being modal. */
+			case MOVE_TO_MODAL:
+				do_move_to_command();
+				/* Reset modality whether or not the command succeeded, otherwise
+				   the player can get caught here if no fire commands can succeed. */
 				map_modal = NO_MODAL;
 				break;
 			case FIRE_MODAL:
@@ -471,19 +515,29 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 			case SET_FORMATION_MODAL:
 				do_set_formation();
 				/* Reset modality whether or not the command succeeded, otherwise
-				   the player can get caught here if no fire commands can succeed. */
+				   the player can get caught here if no commands can succeed. */
 				map_modal = NO_MODAL;
 				break;
 			case ADD_TERRAIN_MODAL:
 				do_add_terrain();
 				/* Reset modality whether or not the command succeeded, otherwise
-				   the player can get caught here if no fire commands can succeed. */
+				   the player can get caught here if no commands can succeed. */
 				map_modal = NO_MODAL;
 				break;
 			case REMOVE_TERRAIN_MODAL:
 				do_remove_terrain();
 				/* Reset modality whether or not the command succeeded, otherwise
-				   the player can get caught here if no fire commands can succeed. */
+				   the player can get caught here if no commands can succeed. */
+				map_modal = NO_MODAL;
+				break;
+			case DISTANCE_MODAL:
+				drag_for_distance(map, h, v);
+				/* Done being modal. */
+				map_modal = NO_MODAL;
+				break;
+			case ZOOM_MODAL:
+				select_area_and_zoom(map, h, v, mods);
+				/* Done being modal. */
 				map_modal = NO_MODAL;
 				break;
 			case GENERIC_MODAL:
@@ -691,6 +745,41 @@ move_on_drag(Map *map, Unit *unit, int mods)
 	}
 }
 
+void
+drag_for_distance(Map *map, int h0, int v0)
+{
+	Point pt0, pt1, newmouse;
+	int drawn = FALSE, x, y;
+
+	SetPt(&pt0, h0, v0);
+	SetPt(&pt1, h0, v0);
+	/* (should be a generic subr) */
+	PenMode(patXor);
+	PenPat(QDPat(gray));
+	while (WaitMouseUp()) {
+		GetMouse(&newmouse);
+		/* should scroll, then abort if we drag outside the window */
+		if (0 /* PtInRect(newmouse, &(map->window->portRect)) */) {
+		}
+		if (!EqualPt(pt1, newmouse)) {
+			if (drawn) {
+				MoveTo(h0, v0);  LineTo(pt1.h, pt1.v);
+			}
+			pt1 = newmouse;
+			MoveTo(h0, v0);  LineTo(pt1.h, pt1.v);
+			drawn = TRUE;
+		}
+	}
+	/* Erase the last drawn line. */
+	if (drawn) {
+		MoveTo(h0, v0);  LineTo(pt1.h, pt1.v);
+	}
+	PenNormal();
+	m_nearest_cell(map, pt1.h, pt1.v, &x, &y);
+	notify(dside, "Distance from %d,%d to %d,%d is %d",
+		   downx, downy, x, y, distance(downx, downy, x, y));
+}
+
 void
 unselect_all(Map *map)
 {
