@@ -1,5 +1,5 @@
 /* Menus and commands for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,16 +9,9 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "print.h"
 #include "macconq.h"
-extern WindowPtr playersetupwin;
-extern int gameinited;
-extern int position_set_modally;
-extern Point modal_point;
-extern Map *modal_map;
-
-extern int get_a_position(Map **map, int *xp, int *yp, int *dirp);
-extern int get_a_unit(Map **mapp, Unit **unitp);
-
-void query_position_modally(int mode);
+extern void enable_command(void);
+extern TEHandle command_text;
+extern TEHandle run_length_text;
 
 void
 query_position_modally(int mode)
@@ -31,8 +24,6 @@ query_position_modally(int mode)
 	}
 }
 
-void set_position_modally(void);
-
 void
 set_position_modally()
 {
@@ -62,10 +53,14 @@ extern int do_one_dir_move(Unit *unit);
 extern int do_one_dir_multiple_move(Unit *unit);
 extern int do_one_set_name(Unit *unit);
 
-static void sanitize_for_menu(char *str, char *outstr);
+static void sanitize_for_menu(char *str, Str255 outstr);
 
 static void resign_the_game(int forced);
 
+static void te_cut(TEHandle hdl);
+static void te_copy(TEHandle hdl);
+static void te_paste(TEHandle hdl);
+
 static void do_page_setup_mi(void);
 static void do_print_mi(void);
 static void do_select_all_mi(void);
@@ -110,9 +105,6 @@ extern Feature *featurelist;
 
 int Profile = FALSE;
 
-#define SetMenuItemText SetItem
-#define InsertMenuItem InsMenuItem
-
 #ifdef DESIGNERS
 #define side_may_select(unit) (in_play(unit) && ((unit)->side == dside || dside->designer))
 
@@ -202,6 +194,9 @@ init_menus()
 	optterrmenu = GetMenu(mOptTerrainTypes);
 	if (optterrmenu != nil)
 	  InsertMenu(optterrmenu, -1);
+	menu = GetMenu(mViewAngles);
+	if (menu != nil)
+	  InsertMenu(menu, -1);
 	/* Init the support for the Windows menu. */
 	winmenuwins = (WindowPtr *) xmalloc(MAXWINDOWS * sizeof(WindowPtr));
 	for (i = 0; i < MAXWINDOWS; ++i)
@@ -445,7 +440,7 @@ update_feature_menu(Feature *feature)
    menus; innocuous because this affects only the appearance in the menu. */
 
 static void
-sanitize_for_menu(char *str, char *outstr)
+sanitize_for_menu(char *str, Str255 outstr)
 {
 	int i = 0;
 
@@ -496,7 +491,7 @@ do_menu_command(long which)
 					do_about_box();
 					break;
 				case miAppleHelp:
-					help_dialog();
+					help_dialog(NULL);
 					break;
 				case miAppleInstructions:
 					instructions_dialog();
@@ -535,7 +530,7 @@ do_menu_command(long which)
 					do_print_mi();
 					break;
 				case miFileResign:
-					resign_the_game(FALSE);
+					do_resign();
 					break;
 				case miFileQuit:
 					quit_the_game();
@@ -543,11 +538,36 @@ do_menu_command(long which)
 			}
 			break;
 		case mEdit:
-			/* handledbyda = SystemEdit(menuitem-1); */
+			if (SystemEdit(menuitem - 1))
+			  break;
 			switch (menuitem)  {
 				case miEditCut:
-					/* disband_selection(); */
-					/* (should depend on whether this is a map or a text field) */
+					if (FrontWindow() == commandwin) {
+						te_cut(command_text);
+					} else if(FrontWindow() == constructionwin) {
+						te_cut(run_length_text);
+					}
+					break;
+				case miEditCopy:
+					if (FrontWindow() == commandwin) {
+						te_copy(command_text);
+					} else if(FrontWindow() == constructionwin) {
+						te_copy(run_length_text);
+					}
+					break;
+				case miEditPaste:
+					if (FrontWindow() == commandwin) {
+						te_paste(command_text);
+					} else if (FrontWindow() == constructionwin) {
+						te_paste(run_length_text);
+					}
+					break;
+				case miEditClear:
+					if (FrontWindow() == commandwin) {
+						TEDelete(command_text);						
+					} else if (FrontWindow() == constructionwin) {
+						TEDelete(run_length_text);						
+					}
 					break;
 				case miEditSelectAll:
 					do_select_all_mi();
@@ -584,7 +604,7 @@ do_menu_command(long which)
 					do_closeup_mi();
 					break;
 				case miPlayMove:
-					do_move_to();
+					query_position_modally(MOVE_TO_MODAL);
 					break;
 				case miPlaySetFormation:
 					query_position_modally(SET_FORMATION_MODAL);
@@ -703,7 +723,13 @@ do_menu_command(long which)
 				case miWindowsConstruction:
 					enable_construction();
 					break;
+				case miWindowsCommand:
+					enable_command();
+					break;
 				/* should have agreements list etc handling here also */
+				case miWindowsScores:
+					scores_dialog();
+					break;
 				case miWindowsNewList:
 					create_list();
 					break;
@@ -764,6 +790,9 @@ do_menu_command(long which)
 				case miViewTopline:
 					toggle_map_topline(map);
 					break;
+				case miViewTopunit:
+					toggle_map_topunit(map);
+					break;
 				case miViewOtherMaps:
 					toggle_map_other_maps(map);
 					break;
@@ -786,6 +815,23 @@ do_menu_command(long which)
 					break;
 			}
 			break;
+		case mViewAngles:
+			if (map == NULL) break;
+			switch (menuitem)  {
+				case miAngle15:
+					set_view_angle(map->vp, 15);
+					force_map_update(map);
+					break;
+				case miAngle30:
+					set_view_angle(map->vp, 30);
+					force_map_update(map);
+					break;
+				case miAngle90:
+					set_view_angle(map->vp, 90);
+					force_map_update(map);
+					break;
+			}
+			break;
 		case mViewList:
 			switch (menuitem)  {
 				case miViewByType:
@@ -897,7 +943,7 @@ save_the_game(int askname, int quitting)
 		/* Collect the file and path to save to. */
 		SetPt(&pnt, 100, 100);
 		c2p(cursavename, tmpstr);
-		SFPutFile(pnt, "\p", tmpstr, (DlgHookProcPtr) nil, &reply);
+		SFPutFile(pnt, "\p", tmpstr, /*(DlgHookProcPtr)*/ nil, &reply);
 		if (!reply.good)
 		  return;
 		/* Make the location of the file be the current volume. */
@@ -977,29 +1023,29 @@ quit_the_game()
 {
 	int confirmresign = FALSE;
 
+#ifdef DESIGNERS
+	/* If we're designing, need only confirm the save before quitting;
+	   no resignation requirement. */
+	if (dside->designer) {
+		switch (CautionAlert(aDesignerQuitGame, nil)) {
+			case aiDesignerQuitGameOK:
+				SetCursor(*watchcursor);
+				save_the_game(FALSE, TRUE);
+				SetCursor(&QD(arrow));
+				/* Fall through to exit. */
+			case aiDesignerQuitGameDontSave:
+				ExitToShell();
+				break;
+			case aiDesignerQuitGameCancel:
+				break;
+		}
+		return;
+	}
+#endif /* DESIGNERS */
 	if (endofgame) {
 		/* Game is already over, nothing to save, just leave. */
 		ExitToShell();
 	} else if (!gamestatesafe || !interfacestatesafe) {
-#ifdef DESIGNERS
-		/* If we're designing, need only confirm the save before quitting;
-		   no resignation requirement. */
-		if (dside->designer) {
-			switch (CautionAlert(aDesignerQuitGame, nil)) {
-				case aiDesignerQuitGameOK:
-					SetCursor(*watchcursor);
-					save_the_game(FALSE, TRUE);
-					SetCursor(&QD(arrow));
-					/* Fall through to exit. */
-				case aiDesignerQuitGameDontSave:
-					ExitToShell();
-					break;
-				case aiDesignerQuitGameCancel:
-					break;
-			}
-			return;
-		}
-#endif /* DESIGNERS */
 		switch (CautionAlert(aQuitGame, nil)) {
 			case aiQuitGameOK:
 				if (all_others_willing_to_save(dside)) {
@@ -1014,7 +1060,7 @@ quit_the_game()
 				break;
 			case aiQuitGameDontSave:
 				if (all_others_willing_to_quit(dside)) {
-					ExitToShell();
+					set_willing_to_draw(dside, TRUE);
 				} else {
 					/* Can only get out by resigning. */
 					confirmresign = TRUE;
@@ -1032,13 +1078,45 @@ quit_the_game()
 	}
 }
 
+static void
+te_cut(TEHandle hdl)
+{
+	if (ZeroScrap() == noErr) {
+		TECut(hdl);
+		if (TEToScrap() != noErr)
+		  ZeroScrap();
+	}
+}
+
+static void
+te_copy(TEHandle hdl)
+{
+	if (ZeroScrap() == noErr) {
+		TECopy(hdl);
+		if (TEToScrap() != noErr)
+		  ZeroScrap();
+	}
+}
+
+static void
+te_paste(TEHandle hdl)
+{
+	if (TEFromScrap() == noErr) {
+		TEPaste(hdl);
+	}
+}
+
 static void
 do_select_all_mi()
 {
 	Map *map;
 	Unit *unit;
-	
-	if ((map = map_from_window(FrontWindow())) != NULL) {
+
+	if (FrontWindow() == commandwin) {
+		/* (should add this) */
+	} else if (FrontWindow() == constructionwin) {
+		/* (should add this) */
+	} else if ((map = map_from_window(FrontWindow())) != NULL) {
 		for_all_units(unit) {
 			if (side_may_select(unit)) {
 				select_unit_on_map(map, unit);
@@ -1046,6 +1124,7 @@ do_select_all_mi()
 		}
 		/* Do all the drawing at once. */
 		draw_selections(map);
+	} else if (0 /* is list */) {
 	}
 }
 
@@ -1430,13 +1509,11 @@ do_one_give_unit(Unit *unit)
 	  return FALSE;
 #ifdef DESIGNERS
 	if (dside->designer) {
-		change_unit_side(unit, tmpcmdside, -1, NULL);
-		update_unit_display(dside, unit, TRUE);
-	} else
-#endif /* DESIGNERS */
-	{
-		prep_change_side_action(unit, unit, tmpcmdside);
+		designer_change_side(unit, tmpcmdside);
+		return TRUE;
 	}
+#endif /* DESIGNERS */
+	prep_change_side_action(unit, unit, tmpcmdside);
 	return TRUE;
 }
 
@@ -1503,7 +1580,7 @@ adjust_menu_item(MenuHandle menu, int item, int value)
 void
 adjust_menus()
 {
-	int i, m, t, numwins, menus_useful, numitems;
+	int i, m, t, numwins, menus_useful, numitems, has_text, offset;
 	MenuHandle menu;
 	Map *frontmap;
 	List *frontlist;
@@ -1553,9 +1630,19 @@ adjust_menus()
 	}
 	menu = GetMHandle(mEdit);
 	if (menu != nil) {
-		/* Edit menu is always available, but most items may be disabled. */
-		/* (should do the other items also) */
-		adjust_menu_item(menu, miEditSelectAll, (frontmap != NULL || frontlist != NULL));
+		/* Edit menu is always available, but individual items may be disabled. */
+		adjust_menu_item(menu, miEditUndo, FALSE);
+		has_text = FALSE;
+		if (frontwin == commandwin) {
+			has_text = ((*command_text)->selStart < (*command_text)->selEnd);
+		} else if (frontwin == constructionwin) {
+			has_text = ((*run_length_text)->selStart < (*run_length_text)->selEnd);
+		}
+		adjust_menu_item(menu, miEditCut, has_text);
+		adjust_menu_item(menu, miEditCopy, has_text);
+		adjust_menu_item(menu, miEditPaste, (GetScrap(nil, 'TEXT', (long *) &offset) > 0));
+		adjust_menu_item(menu, miEditClear, has_text);
+		adjust_menu_item(menu, miEditSelectAll, (frontmap != NULL || frontlist != NULL || has_text));
 		if (menus_useful) {
 #ifdef DESIGNERS
 			CheckItem(menu, miEditDesign, dside->designer);
@@ -1658,6 +1745,8 @@ adjust_menus()
 				(noticewin && ((WindowPeek) noticewin)->visible));
 			CheckItem(menu, miWindowsHistory,
 				(historywin && ((WindowPeek) historywin)->visible));
+			CheckItem(menu, miWindowsCommand,
+				(commandwin && ((WindowPeek) commandwin)->visible));
 			if (any_construction_possible()) {
 				EnableItem(menu, miWindowsConstruction);
 				CheckItem(menu, miWindowsConstruction,
@@ -1698,7 +1787,9 @@ adjust_menus()
 			EnableItem(menu, miViewGrid);
 			CheckItem(menu, miViewGrid, frontmap->drawgrid);
 			EnableItem(menu, miViewTopline);
-			CheckItem(menu, miViewTopline, (frontmap->toph > 0));
+			CheckItem(menu, miViewTopline, (frontmap->toplineh > 0));
+			EnableItem(menu, miViewTopunit);
+			CheckItem(menu, miViewTopunit, (frontmap->topunith > 0));
 			EnableItem(menu, miViewNames);
 			CheckItem(menu, miViewNames, frontmap->drawnames);
 			if (people_sides_defined()) {
@@ -1834,41 +1925,40 @@ adjust_menus()
 		/* Always on, if any material types defined. */
 		if (nummtypes > 0) {
 			EnableItem(menu, 0);
-		} else {
-			DisableItem(menu, 0);
-		}
-		if (frontmap != NULL) {
-			for_all_material_types(m) {
-				if (any_cell_materials_defined() && cell_material_defined(m)) {
-					EnableItem(menu, m + 1);
-					CheckItem(menu, m + 1, frontmap->drawmaterials[m]);
-				} else {
-				    CheckItem(menu, i, FALSE);
-					DisableItem(menu, m + 1);
+			if (frontmap != NULL) {
+				for_all_material_types(m) {
+					if (any_cell_materials_defined() && cell_material_defined(m)) {
+						EnableItem(menu, m + 1);
+						CheckItem(menu, m + 1, frontmap->drawmaterials[m]);
+					} else {
+					    CheckItem(menu, i, FALSE);
+						DisableItem(menu, m + 1);
+					}
 				}
 			}
+		} else {
+			DisableItem(menu, 0);
 		}
 	}
 	menu = GetMHandle(mOptTerrainTypes);
 	if (menu != nil) {
-		/* Always on, if any material types defined. */
-		if (1) {
+		if (any_aux_terrain_defined()) {
 			EnableItem(menu, 0);
-		} else {
-			DisableItem(menu, 0);
-		}
-		if (frontmap != NULL) {
-			numitems = CountMItems(optterrmenu);
-			for (i = 1; i <= numitems; ++i) {
-				t = frontmap->auxterraintypes[i];
-				if (any_aux_terrain_defined() && aux_terrain_defined(t)) {
-					EnableItem(menu, i);
-				    CheckItem(menu, i, frontmap->drawauxterrain[t]);
-				} else {
-				    CheckItem(menu, i, FALSE);
-					DisableItem(menu, i);
+			if (frontmap != NULL) {
+				numitems = CountMItems(menu);
+				for (i = 1; i <= numitems; ++i) {
+					t = frontmap->auxterraintypes[i];
+					if (any_aux_terrain_defined() && aux_terrain_defined(t)) {
+						EnableItem(menu, i);
+					    CheckItem(menu, i, frontmap->drawauxterrain[t]);
+					} else {
+					    CheckItem(menu, i, FALSE);
+						DisableItem(menu, i);
+					}
 				}
 			}
+		} else {
+			DisableItem(menu, 0);
 		}
 	}
 	menu = GetMHandle(mAITypes);
@@ -1938,7 +2028,8 @@ enable_commands_for_unit(MenuHandle menu, Unit *unit)
 		EnableItem(menu, miPlayDisband);
 	}
 #if 0
-	if (can_change_type(dside, unit)) {
+	/* change-type menu item not yet defined */ 
+	if (can_change_type(unit)) {
 		EnableItem(menu, miPlayChangeType);
 	}
 #endif
