@@ -6,10 +6,6 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-#ifndef USE_CONSOLE
-#define USE_CONSOLE
-#endif
-
 #include "config.h"
 #include "misc.h"
 #include "lisp.h"
