@@ -1,5 +1,5 @@
 /* Definitions for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -128,7 +128,9 @@ enum tooltypes {
   unitshoottool,
   unitbuildtool,
 #ifdef DESIGNERS
-  terrainpainttool,
+  cellpainttool,
+  bordpainttool,
+  connpainttool,
   unitaddtool,
   peoplepainttool,
   featurepainttool,
@@ -284,6 +286,27 @@ typedef struct a_ui {
     Font flegendfids[6];
     Legend *legends;
     int numfeatures;
+    /* Side closeup list */
+    struct a_side_closeup *sidecloseuplist;
+    /* Unit list */
+    struct a_unit_list *unitlistlist;
+    /* Unit closeup list */
+    struct a_unit_closeup *unitcloseuplist;
+    /* Unit closeup summary */
+    struct a_closeup_summary *closeupsummary;
+    /* Orders interface */
+    Widget orders_shell;
+    Widget orders_label;
+    Widget orders_form;
+    Widget orders_help_shell;
+    StandingOrder *sorder_edit;
+    char *sorder_types_edit;
+    Task *sorder_task_edit;
+    struct a_order_interface *ordi_edit;
+    struct a_order_interface *orderlist;
+    int *grok_p1;
+    int *grok_p2;
+    int grok_size;
     /* Random stuff */
     Pixmap bombpics[4];
     Pixmap hitpics[3];
@@ -296,6 +319,60 @@ typedef struct a_ui {
     struct a_movie movies[10];
 } UI;
 
+/* A closer look at a side. */
+typedef struct a_side_closeup {
+    struct a_map *map;
+    struct a_side *side;
+    Widget shell, info, *filter, list_units;
+    struct a_side_closeup *next;
+} SideCloseup;
+
+/* A list of units. */
+typedef struct a_unit_list {
+    struct a_map *map;
+    int number;
+    Unit **units;
+    char **labels;
+    Widget shell, label, list, close;
+    struct a_unit_list *next;
+} UnitList;
+
+/* A closer look at a unit. */
+typedef struct a_unit_closeup {
+    struct a_map *map;
+    struct a_unit *unit;
+    Widget shell, info;
+    struct a_unit_closeup *next;
+} UnitCloseup;
+
+/* List of unit closeups */
+typedef struct a_closeup_summary {
+    int number;
+    UnitCloseup **unitcloseups;
+    char **labels;
+    Widget shell, label, list;
+} CloseupSummary;
+ 
+/* Orders interface */
+typedef struct a_order_interface {
+    StandingOrder *sorder;
+    Widget form, toggle, types, etype, eparms, task, tparms;
+    Pixel form_bg, form_fg;
+    struct a_order_interface *next;
+} OrderInterface;
+
+/* Iteration over all of a side's unit closeups. */
+#define for_all_unit_closeups(c,s)  \
+  for ((c) = (s)->ui->unitcloseuplist; (c) != NULL; (c) = (c)->next)
+
+/* Iteration over all of a side's unit lists. */
+#define for_all_unit_lists(l,s)  \
+  for ((l) = (s)->ui->unitlistlist; (l) != NULL; (l) = (l)->next)
+
+/* Iteration over all of a side's side closeups. */
+#define for_all_side_closeups(c,s)  \
+  for ((c) = (s)->ui->sidecloseuplist; (c) != NULL; (c) = (c)->next)
+
 /* Each side can open up any number and shape of maps. */
 
 /* All the maps will share colors and bitmaps though. */
@@ -334,6 +411,8 @@ typedef struct a_map {
     Window infowin;		/* info about a unit and/or cell */
     Pixmap viewwin;		/* actual graphical display of an area */
     Window sideswin;		/* list of sides */
+    /* panner pixmap */
+    Pixmap panner_pix;
     /* How to draw the map. */
     short seeall;		/* True if viewing world data directly */
     short drawterrain;		/* Display solid color terrain on the map? */
@@ -661,3 +740,16 @@ extern void handle_designer_map_click PARAMS ((Side *side, Map *map,
 					      int sx, int sy));
 extern void popdown_design PARAMS ((Side *side));
 #endif /* DESIGNERS */
+
+extern UnitCloseup *find_unit_closeup PARAMS ((Unit *unit, Side *side));
+extern UnitCloseup *find_unit_closeup_via_button PARAMS ((Widget w, Side *side));
+extern UnitCloseup *create_unit_closeup PARAMS ((Unit *unit, Side *side, Map *map));
+extern void draw_unit_closeup PARAMS ((UnitCloseup *unitcloseup, Side *side));
+extern void destroy_unit_closeup PARAMS ((UnitCloseup *unitcloseup, Side *side));
+extern SideCloseup *find_side_closeup PARAMS ((Side *side1, Side *side));
+extern SideCloseup *find_side_closeup_via_button PARAMS ((Widget w, Side *side));
+extern SideCloseup *create_side_closeup PARAMS ((Side *side1, Side *side, Map *map));
+extern void draw_side_closeup PARAMS ((SideCloseup *sidecloseup, Side *side));
+extern void destroy_side_closeup PARAMS ((SideCloseup *sidecloseup, Side *side));
+extern Pixmap get_unit_picture PARAMS ((int utype, Side *side));
+extern Pixmap get_side_picture PARAMS ((Side *side, Side *side1));
