@@ -1,5 +1,5 @@
 /* Designer tools for the X11 interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -116,7 +116,7 @@ Side *side;
 			      XtNlabel, "Terr",
 			      NULL);
     XtAddCallback(side->ui->terrain_button, XtNcallback,
-		  design_tool_callback, (XtPointer) terrainpainttool);
+		  design_tool_callback, (XtPointer) cellpainttool);
     side->ui->terrain_label =
       XtVaCreateManagedWidget("Terrain", labelWidgetClass, pane,
 			      XtNborderWidth, 0,
@@ -359,7 +359,7 @@ int tool;
     XtSetArg(tmpargs[nargs], XtNstate, (Boolean) (tool == looktool));  nargs++;
     XtSetValues (side->ui->normal_button, tmpargs, nargs);
     nargs = 0;
-    XtSetArg(tmpargs[nargs], XtNstate, (Boolean) (tool == terrainpainttool));  nargs++;
+    XtSetArg(tmpargs[nargs], XtNstate, (Boolean) (tool == cellpainttool));  nargs++;
     XtSetValues (side->ui->terrain_button, tmpargs, nargs);
     nargs = 0;
     XtSetArg(tmpargs[nargs], XtNstate, (Boolean) (tool == unitaddtool));  nargs++;
@@ -370,6 +370,12 @@ int tool;
     nargs = 0;
     XtSetArg(tmpargs[nargs], XtNstate, (Boolean) (tool == featurepainttool));  nargs++;
     XtSetValues (side->ui->feature_button, tmpargs, nargs);
+    /* As a special case, set the actual terrain tool based on terrain
+       subtype. */
+    if (tool == cellpainttool && t_is_border(side->ui->curttype))
+      side->ui->curdesigntool = bordpainttool;
+    if (tool == cellpainttool && t_is_connection(side->ui->curttype))
+      side->ui->curdesigntool = connpainttool;
     for_all_maps(side, map) {
 	map->curtool = side->ui->curdesigntool;
     }
@@ -394,12 +400,21 @@ XtPointer call_data;
 {
     int which = (int) client_data;
     Side *side;
+    Map *map;
 
     if (!find_side_via_widget(w, &side))
       return;
 
     OPTION_CYCLE(side->ui->curttype, 0, numttypes, which);
     update_curttype(side);
+    /* Do this also, because the terrain type change might necessitate
+       a tool cursor change.  If the type should not be
+       "cellpainttool", this will change it to the appropriate one. */
+    set_design_tool(side, cellpainttool);
+    for_all_maps(side, map) {
+	update_controls(side, map);
+	set_tool_cursor(side, map);
+    }
 }
 
 static void
@@ -410,6 +425,7 @@ XtPointer call_data;
 {
     int which = (int) client_data;
     Side *side;
+    Map *map;
 
     if (!find_side_via_widget(w, &side))
       return;
@@ -418,6 +434,11 @@ XtPointer call_data;
     side->ui->curbgttype =
       (side->ui->curbgttype + which + numttypes) % numttypes;
     update_curttype(side);
+    set_design_tool(side, cellpainttool);
+    for_all_maps(side, map) {
+	update_controls(side, map);
+	set_tool_cursor(side, map);
+    }
 }
 
 static void
@@ -728,10 +749,21 @@ int sx, sy;
 	return;
     }
     switch (map->curtool) {
-      case terrainpainttool:
-	/* (should be able to do conn/bord types also?) */
+      case cellpainttool:
 	side->ui->curttype = terrain_at(x, y);
 	update_curttype(side);
+	/* Do this also, because the terrain type change might necessitate
+	   a tool cursor change.  If the type should not be
+	   "cellpainttool", this will change it to the appropriate one. */
+	set_design_tool(side, cellpainttool);
+	break;
+      case bordpainttool:
+	/* (should make this work) */
+	beep(side);
+	break;
+      case connpainttool:
+	/* (should make this work) */
+	beep(side);
 	break;
       case unitaddtool:
 	/* (should be unit exactly under sx,sy) */
@@ -782,7 +814,9 @@ int sx, sy;
 	return;
     }
     switch (map->curtool) {
-      case terrainpainttool:
+      case cellpainttool:
+      case bordpainttool:
+      case connpainttool:
 	x = ax;  y = ay;
 	switch (t_subtype(side->ui->curttype)) {
 	  case cellsubtype:
