@@ -1,5 +1,5 @@
 /* Lower-level drawing routines for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -15,6 +15,7 @@ any later version.  See the file COPYING.  */
    the unit info, the map view itself, and the list of sides. */
 
 #include "conq.h"
+extern void location_desc PARAMS ((char *buf, Side *side, Unit *unit, int u, int x, int y));
 #include "xconq.h"
 
 int tmpdrawlighting;
@@ -433,7 +434,9 @@ Map *map;
 	    if (x1 + y < halfheight)
 	      x1 = halfheight - y;
 	}
-	draw_row(side, map, x1, y, x2 - x1, FALSE);
+	/* I don't understand the wrapx below, but Massimo says it
+	   fixes bugs. -sts */
+	draw_row(side, map, wrapx(x1), y, x2 - x1, FALSE);
 	/* (should test for input events here, would respond better) */
     }
     draw_current(side, map);
@@ -830,6 +833,7 @@ int x, y;
 	    sx += (map->vp->hw - uw) / 2;  sy += (map->vp->hh - uh) / 2;
 	    if (unit != NULL) {
 		if (unit->occupant != NULL
+		    && uw >= 8
 	    	    && (side_controls_unit(side, unit)
 			|| map->seeall
 			|| u_see_occupants(unit->type))) {
@@ -838,7 +842,7 @@ int x, y;
 		    XSetClipMask(dpy, gc, None);
 		    XSetForeground(dpy, gc, side->ui->whitecolor);
 		    XFillRectangle(dpy, map->viewwin, gc,
-		       		   sx + 1, sy + 1, uw - 2, uh - 2);
+				   sx + 1, sy + 1, uw - 2, uh - 2);
 		    /* Put a black border around it, for better contrast. */
 		    XSetForeground(dpy, gc, side->ui->blackcolor);
 		    XDrawRectangle(dpy, map->viewwin, gc,
@@ -885,7 +889,7 @@ int sx, sy, sw, sh, drawoccs;
        will also be visible. */
     if (unit->transport && unit->side == unit->transport->side)
       s = -1;
-    if (unit->occupant == NULL) {
+    if (unit->occupant == NULL || sw <= 8) {
 	draw_unit_image(side, map->viewwin, sx, sy, sw, sh, u, s,
 			-1, -1);
 	if (map->drawnames)
@@ -897,11 +901,11 @@ int sx, sy, sw, sh, drawoccs;
 	XSetClipMask(dpy, gc, None);
 	XSetForeground(dpy, gc, side->ui->whitecolor);
 	XFillRectangle(dpy, map->viewwin, gc,
-		       sx+1, sy+1, sw-2, sh-2);
+		       sx + 1, sy + 1, sw - 2, sh - 2);
 	/* Put a black border around it, for better contrast. */
 	XSetForeground(dpy, gc, side->ui->blackcolor);
 	XDrawRectangle(dpy, map->viewwin, gc,
-		       sx+1, sy+1, sw-2, sh-2);
+		       sx + 1, sy + 1, sw - 2, sh - 2);
 	/* Draw the transport's image. */
 	draw_unit_image(side, map->viewwin, sx2, sy2, sw2, sh2, u, s,
 			-1, -1);
@@ -1739,24 +1743,8 @@ Map *map;
 		sprintf(infobuf, "%s %s",
 			       side_adjective(side2), u_type_name(u));
 		draw_info_text(side, map, 0, 0, 40, infobuf);
-		sprintf(infobuf, "In %s", t_type_name(t));
-	    } else if (terrain_visible(side, x, y)) {
-		sprintf(infobuf, "Empty %s", t_type_name(t));
-	    } else {
-		sprintf(infobuf, "Unknown");
 	    }
-	    if (terrain_visible(side, x, y)) {
-		linear_desc(infobuf, x, y);
-		featurename = feature_name_at(x, y);
-		if (!empty_string(featurename))
-		  tprintf(infobuf, " (%s)", featurename);
-		if (temperatures_defined())
-		  tprintf(infobuf, " (T %d)", temperature_at(x, y));
-		if (elevations_defined())
-		  tprintf(infobuf, " (El %d)", elev_at(x, y));
-		/* (should list local weather also) */
-	    }
-	    tprintf(infobuf, " at %d,%d", x, y);
+	    location_desc(infobuf, side, NULL, u, x, y);
 	    draw_info_text(side, map, 0, 1, 40, infobuf);
 	} else {
 	    /* (should never happen?) */
@@ -1767,22 +1755,7 @@ Map *map;
     /* Say which unit this is. */
     sprintf(infobuf, "%s", unit_handle(side, unit));
     draw_info_text(side, map, 0, 0, 40, infobuf);
-    /* Mention its transport (or underlying terrain) if there is one. */
-    if (unit->transport != NULL) {
-	sprintf(infobuf, "In %s", short_unit_handle(unit->transport));
-    } else {
-	sprintf(infobuf, "In %s", t_type_name(terrain_at(x, y)));
-	linear_desc(infobuf, x, y);
-	tmpbuf[0] = '\0';
-	featurename = feature_name_at(x, y);
-	if (!empty_string(featurename))
-	  sprintf(tmpbuf, " (%s)", featurename);
-	if (temperatures_defined())
-	  tprintf(tmpbuf, " (T %d)", temperature_at(x, y));
-	if (elevations_defined())
-	  tprintf(tmpbuf, " (El %d)", elev_at(x, y));
-    }
-    tprintf(infobuf, " at %d,%d", x, y);
+    location_desc(infobuf, side, unit, u, x, y);
     draw_info_text(side, map, 0, 1, 40, infobuf);
     /* Very briefly list the numbers and types of the occupants. */
     infobuf[0] = '\0';
