@@ -1,5 +1,5 @@
 /* Initialization for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -135,10 +135,6 @@ int min_h_for_unit_image = 16;
 
 /* various bitmap definitions. */
 
-#if 0
-#include "bitmaps/snowy.b"
-#endif
-
 #include "bitmaps/lookglass.b"
 #include "bitmaps/lookmask.b"
 #include "bitmaps/movecurs.b"
@@ -176,18 +172,10 @@ int min_h_for_unit_image = 16;
 #ifdef DESIGNERS
 #include "bitmaps/hexcurs.b"
 #include "bitmaps/hexcursmask.b"
-#if 0
 #include "bitmaps/bordcurs.b"
 #include "bitmaps/conncurs.b"
-#endif
 #include "bitmaps/addunits.b"
 #include "bitmaps/people.b"
-#if 0
-#include "bitmaps/elev.b"
-#include "bitmaps/therm.b"
-#include "bitmaps/units.b"
-#include "bitmaps/resource.b"
-#endif
 #include "bitmaps/feature.b"
 #include "bitmaps/featuremask.b"
 #endif /* DESIGNERS */
@@ -228,8 +216,7 @@ check_player_displays()
     Player *player;
     Display *dpy;
 
-    /* (should be a for_all_players macro) */
-    for (player = playerlist ; player != NULL; player = player->next) {
+    for_all_players(player) {
 	/* Try to open the display. */
 	dpyname = player->displayname;
 	if (dpyname != NULL) {
@@ -717,9 +704,15 @@ Side *side;
       make_cursor(dpy, win, buildcursor_bits, buildmask_bits,
 		  fg, bg, buildcursor_x_hot, buildcursor_y_hot);
 #ifdef DESIGNERS
-    side->ui->toolcursors[terrainpainttool] =
+    side->ui->toolcursors[cellpainttool] =
       make_cursor(dpy, win, hexcursor_bits, hexcursormask_bits,
 		  fg, bg, hexcursor_x_hot, hexcursor_y_hot);
+    side->ui->toolcursors[bordpainttool] =
+      make_cursor(dpy, win, bordcursor_bits, bordcursor_bits, /* should have separate mask */
+		  fg, bg, bordcursor_x_hot, bordcursor_y_hot);
+    side->ui->toolcursors[connpainttool] =
+      make_cursor(dpy, win, conncursor_bits, conncursor_bits, /* should have separate mask */
+		  fg, bg, conncursor_x_hot, conncursor_y_hot);
     side->ui->toolcursors[unitaddtool] =
       make_cursor(dpy, win, addunits_bits, addunits_bits,
 		  fg, bg, addunits_x_hot, addunits_y_hot);
@@ -1036,16 +1029,29 @@ popup_game_dialog ()
 			      NULL);
     label =
       XtVaCreateManagedWidget("label", labelWidgetClass, outform,
+			      XtNtop,    XawChainTop, 
+			      XtNbottom, XawChainTop, 
+			      XtNleft,   XawChainLeft, 
+			      XtNright,  XawChainLeft,
 			      NULL);
     quit =
       XtVaCreateManagedWidget("quit", commandWidgetClass, outform,
 			      XtNfromHoriz, label,
+			      XtNtop,    XawChainTop, 
+			      XtNbottom, XawChainTop, 
+			      XtNleft,   XawChainLeft, 
+			      XtNright,  XawChainLeft,
 			      NULL);
     XtAddCallback (quit, XtNcallback, quit_dialog, NULL);
+
     viewp =
       XtVaCreateManagedWidget("viewport", viewportWidgetClass,  outform,
 			      XtNfromVert,  label,
 			      XtNallowVert, True,
+			      XtNtop,    XawChainTop, 
+			      XtNbottom, XawChainBottom, 
+			      XtNleft,   XawChainLeft, 
+			      XtNright,  XawChainRight,
 			      NULL);
     inform =
       XtVaCreateManagedWidget("inForm", formWidgetClass, viewp,
@@ -1056,10 +1062,18 @@ popup_game_dialog ()
 	games[i].form =
 	  XtVaCreateManagedWidget(module->name, formWidgetClass, inform, 
 				  XtNfromVert, i ? games[i-1].form : NULL,
+				  XtNtop,    XawChainTop, 
+				  XtNbottom, XawChainTop, 
+				  XtNleft,   XawChainLeft, 
+				  XtNright,  XawChainLeft,
 				  NULL);
 	games[i].name =
 	  XtVaCreateManagedWidget("name", commandWidgetClass, games[i].form,
 				  XtNlabel, module->name,
+				  XtNtop,    XawChainTop, 
+				  XtNbottom, XawChainTop, 
+				  XtNleft,   XawChainLeft, 
+				  XtNright,  XawChainLeft,
 				  NULL);
 	XtAddCallback (games[i].name, XtNcallback, game_callback, NULL);
 	if (module->title && module->title[0]) {
@@ -1067,6 +1081,10 @@ popup_game_dialog ()
 	      XtVaCreateManagedWidget("title", labelWidgetClass, games[i].form,
 				      XtNlabel, module->title,
 				      XtNfromHoriz, games[i].name,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	} else {
 	    games[i].title = NULL;
@@ -1077,6 +1095,10 @@ popup_game_dialog ()
 				      XtNlabel, module->blurb,
 				      XtNfromHoriz,
 				      games[i].title ? games[i].title : games[i].name,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	} else {
 	    games[i].blurb = NULL;
@@ -1110,6 +1132,7 @@ int g, flag;
     Side *side;
     Player *player;
     static char *player_type[] = { "none", "human", "machine" };
+    Widget pane;
 
     if (!pop || flag) {
 	if (!pop) {
@@ -1128,6 +1151,10 @@ int g, flag;
 	up = left = pop->name =
 	  XtVaCreateManagedWidget("name", labelWidgetClass, pop->form,
 				  XtNlabel, module->name,
+				  XtNtop,    XawChainTop, 
+				  XtNbottom, XawChainTop, 
+				  XtNleft,   XawChainLeft, 
+				  XtNright,  XawChainLeft,
 				  NULL);
 	if (module->basemodulename && module->basemodulename[0]) {
 	    sprintf(buf1, "base module: %s", module->basemodulename);
@@ -1138,6 +1165,10 @@ int g, flag;
 	  XtVaCreateManagedWidget("basemodule", labelWidgetClass, pop->form,
 				  XtNlabel, buf1,
 				  XtNfromHoriz, left,
+				  XtNtop,    XawChainTop, 
+				  XtNbottom, XawChainTop, 
+				  XtNleft,   XawChainLeft, 
+				  XtNright,  XawChainLeft,
 				  NULL);
 	if (module->version && module->version[0]) {
 	    sprintf(buf1, "version: %s", module->version);
@@ -1145,6 +1176,10 @@ int g, flag;
 	      XtVaCreateManagedWidget("version", labelWidgetClass, pop->form,
 				      XtNlabel, module->version,
 				      XtNfromHoriz, left,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	}
 	if (module->title && module->title[0]) {
@@ -1152,6 +1187,10 @@ int g, flag;
 	      XtVaCreateManagedWidget("title", labelWidgetClass, pop->form,
 				      XtNlabel, module->title,
 				      XtNfromVert, pop->name,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	}
 	if (module->blurb && module->blurb[0]) {
@@ -1160,6 +1199,10 @@ int g, flag;
 				      XtNlabel, module->blurb,
 				      XtNfromHoriz, pop->title,
 				      XtNfromVert, pop->name,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	}
 
@@ -1178,6 +1221,10 @@ int g, flag;
 	      XtVaCreateManagedWidget("variant", labelWidgetClass, pop->form,
 				      XtNlabel, "Variants:",
 				      XtNfromVert, up,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainTop, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 
 	    for (i = 0; i < pop->numvariants; ++i) {
@@ -1193,6 +1240,10 @@ int g, flag;
 					  XtNlabel, variant_name(var),
 					  XtNvalue, varw->value,
 					  XtNfromVert, up,
+					  XtNtop,    XawChainTop, 
+					  XtNbottom, XawChainTop, 
+					  XtNleft,   XawChainLeft, 
+					  XtNright,  XawChainLeft,
 					  NULL);
 		horizontalDialog(varw->dialog);
 		if (varw->mess && varw->mess[0]) {
@@ -1202,23 +1253,42 @@ int g, flag;
 					      pop->form,
 					      XtNlabel, varw->mess,
 					      XtNfromVert, up,
+					      XtNtop,    XawChainTop, 
+					      XtNbottom, XawChainTop, 
+					      XtNleft,   XawChainLeft, 
+					      XtNright,  XawChainLeft,
 					      NULL);
 		}
 	    }
 	}
 
-	pop->instructions = createTextWidget("instructions", pop->form,
-					     up, module->instructions);
-	if (pop->instructions)
-	  up = pop->instructions;
-	pop->notes = createTextWidget("notes", pop->form, up, module->notes);
-	if (pop->notes)
-	  up = pop->notes;
-
+	if (module->instructions != lispnil ||
+	    module->notes != lispnil) {
+	    up = pane =
+	      XtVaCreateManagedWidget("pane", panedWidgetClass, pop->form,
+				      XtNfromVert, up,
+				      XtNorientation, XtorientVertical,
+				      XtNtop,    XawChainTop, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainRight,
+				      NULL);
+	    pop->instructions =
+	      createTextWidget("instructions", pane,
+			       NULL, module->instructions);
+	    pop->notes =
+	      createTextWidget("notes", pane,
+			       pop->instructions, module->notes);
+	    
+	}
 	if (flag) {
 	    up = XtVaCreateManagedWidget("assign", labelWidgetClass, pop->form,
 					 XtNlabel, "Assignments:",
 					 XtNfromVert, up,
+					 XtNtop,    XawChainBottom, 
+					 XtNbottom, XawChainBottom, 
+					 XtNleft,   XawChainLeft, 
+					 XtNright,  XawChainLeft,
 					 NULL);
 	    if (flag == 1) {
 		pop->assign = (Assignw *) xmalloc(MAXSIDES*sizeof(Assignw));
@@ -1239,6 +1309,10 @@ int g, flag;
 		  XtVaCreateManagedWidget("side", labelWidgetClass, pop->form,
 					  XtNlabel, buf1,
 					  XtNfromVert, up,
+					  XtNtop,    XawChainBottom, 
+					  XtNbottom, XawChainBottom, 
+					  XtNleft,   XawChainLeft, 
+					  XtNright,  XawChainLeft,
 					  NULL);
 		left = pop->assign[i].pname =
 		  XtVaCreateManagedWidget("pname", dialogWidgetClass,
@@ -1247,6 +1321,10 @@ int g, flag;
 					  XtNvalue, pop->assign[i].vname,
 					  XtNfromHoriz, left,
 					  XtNfromVert, up,
+					  XtNtop,    XawChainBottom, 
+					  XtNbottom, XawChainBottom, 
+					  XtNleft,   XawChainLeft, 
+					  XtNright,  XawChainLeft,
 					  NULL);
 		horizontalDialog(pop->assign[i].pname);
 		up = pop->assign[i].pid =
@@ -1255,6 +1333,10 @@ int g, flag;
 					  XtNvalue, pop->assign[i].vid,
 					  XtNfromHoriz, left,
 					  XtNfromVert, up,
+					  XtNtop,    XawChainBottom, 
+					  XtNbottom, XawChainBottom, 
+					  XtNleft,   XawChainLeft, 
+					  XtNright,  XawChainLeft,
 					  NULL);
 		horizontalDialog(pop->assign[i].pid);
 		if (pop->assign[i].messname && pop->assign[i].messname[0]) {
@@ -1264,6 +1346,10 @@ int g, flag;
 						 XtNlabel, pop->assign[i].messname,
 						 XtNfromHoriz, save,
 						 XtNfromVert, up,
+						 XtNtop,    XawChainBottom, 
+						 XtNbottom, XawChainBottom, 
+						 XtNleft,   XawChainLeft, 
+						 XtNright,  XawChainLeft,
 						 NULL);
 		}
 		if (pop->assign[i].messid && pop->assign[i].messid[0]) {
@@ -1273,42 +1359,71 @@ int g, flag;
 						 XtNlabel, pop->assign[i].messid,
 						 XtNfromHoriz, save,
 						 XtNfromVert, up,
+						 XtNtop,    XawChainBottom, 
+						 XtNbottom, XawChainBottom, 
+						 XtNleft,   XawChainLeft, 
+						 XtNright,  XawChainLeft,
 						 NULL);
 		}
 	    }
 	    left = pop->add =
 	      XtVaCreateManagedWidget("add", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	    XtAddCallback (pop->add, XtNcallback, add_player_dialog, NULL);
 	    left = pop->verify =
 	      XtVaCreateManagedWidget("verify", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
 				      XtNfromHoriz, left,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	    XtAddCallback (pop->verify, XtNcallback, verify_dialog, NULL);
 	    left = pop->ok =
 	      XtVaCreateManagedWidget("go", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
 				      XtNfromHoriz, left,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	    XtAddCallback (pop->ok, XtNcallback, go_dialog, NULL);
 	    pop->cancel =
 	      XtVaCreateManagedWidget("quit", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
 				      XtNfromHoriz, left,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	    XtAddCallback (pop->cancel, XtNcallback, quit_dialog, NULL);
 	} else {
 	    left = pop->ok =
 	      XtVaCreateManagedWidget("ok", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
 				      NULL);
 	    XtAddCallback (pop->ok, XtNcallback, ok_dialog, NULL);
 	    pop->cancel =
 	      XtVaCreateManagedWidget("cancel", commandWidgetClass, pop->form,
 				      XtNfromVert, up,
-				      XtNfromHoriz, left,  NULL);
+				      XtNfromHoriz, left,
+				      XtNtop,    XawChainBottom, 
+				      XtNbottom, XawChainBottom, 
+				      XtNleft,   XawChainLeft, 
+				      XtNright,  XawChainLeft,
+				      NULL);
 	    XtAddCallback (pop->cancel, XtNcallback, cancel_dialog, NULL);
 	}
     }
@@ -1322,21 +1437,33 @@ char *name;
 Widget parent, up;
 Obj *obj;
 {
-    Widget w;
+    Widget w, form;
 
     if (obj == lispnil)
       return NULL;
 
+    form =
+      XtVaCreateManagedWidget("form", formWidgetClass, parent,
+			      NULL);
+
     sprintf(buf1, "%s_label", name);
     w =
-      XtVaCreateManagedWidget(buf1, labelWidgetClass, parent,
+      XtVaCreateManagedWidget(buf1, labelWidgetClass, form,
 			      XtNlabel, name,
-			      XtNfromVert, up,
+			      XtNtop,    XawChainTop, 
+			      XtNbottom, XawChainTop, 
+			      XtNleft,   XawChainLeft, 
+			      XtNright,  XawChainLeft,
 			      NULL);
     w =
-      XtVaCreateManagedWidget(name, asciiTextWidgetClass, parent,
+      XtVaCreateManagedWidget(name, asciiTextWidgetClass, form,
 			      XtNfromVert, w,
 			      XtNvertDistance, -1,
+			      XtNdisplayCaret, False,
+			      XtNtop,    XawChainTop, 
+			      XtNbottom, XawChainBottom, 
+			      XtNleft,   XawChainLeft, 
+			      XtNright,  XawChainRight,
 			      NULL);
     splat_obj_in_textw(w, obj);
     return w;
