@@ -1,5 +1,5 @@
 /* Map handling for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
+   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
    Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -260,7 +260,7 @@ char *geospec;
       map->drawresources[m] = FALSE;
     map->drawtemp = FALSE;
     map->drawweather = FALSE;
-    map->seeall = g_see_all();
+    map->seeall = all_see_all;
     /* Start the map off in generic move mode normally. */
     map->curtool = movetool;
     map->inpch = '\0';
@@ -538,6 +538,9 @@ char *geospec;
     map->viewwin = XtWindow(map->portlabel);
     map->sideswin = XtWindow(map->sides);
 
+    /* initialize panner pixmap */
+    map->panner_pix = None;
+
     draw_map(side, map);
 
     flush_output(side);
@@ -695,7 +698,7 @@ Map *map;
       XtVaCreateManagedWidget("all", toggleWidgetClass, map->controlform,
 			      XtNlabel, "All",
 			      XtNresize, False,
-			      XtNsensitive, !g_see_all(),
+			      XtNsensitive, !all_see_all,
 			      XtNwidth, wid,
 			      XtNfromVert, map->controls[ZOOM_OUT],
 			      XtNvertDistance, 10,
@@ -1602,7 +1605,7 @@ Map *map;
 char *prompt;
 void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
 {
-    sprintf(map->prompt, "%s [click to set]", prompt);
+    strcpy(map->prompt, prompt);
     map->answer[0] = '\0';
     draw_prompt(side, map);
     map->modalhandler = handler;
@@ -1970,7 +1973,7 @@ XtPointer call_data;
 	map->drawpeople = !map->drawpeople;
 	break;
       case SHOW_ALL:
-	if (!g_see_all())
+	if (!all_see_all)
 	  map->seeall = !map->seeall;
 	else
 	  beep(side);
@@ -2189,6 +2192,16 @@ int sx, sy;
 	return;
     }
 #endif /* DESIGNERS */
+    if (map->modalhandler) {
+       void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
+
+       fn = map->modalhandler;
+       map->modalhandler = NULL;
+       move_look(side, map, sx, sy);
+       (*fn)(side, map, 0);
+       return;
+    }
+
     switch (map->curtool) {
       case looktool:
 	move_look(side, map, sx, sy);
@@ -2263,7 +2276,7 @@ int sx, sy;
 	    if (distance(unit->x, unit->y, x, y) > 1) {
 		DGprintf("Ordering %s to move to %d,%d\n",
 			 unit_desig(unit), x, y);
-		set_moveto_task(unit, x, y);
+		set_move_to_task(unit, x, y);
 		return;
 	    }
 
@@ -2294,7 +2307,7 @@ int sx, sy;
 		    /* (not quite right, move should happen after other unit
 		       is actually inside, in case it fills dest) */
 		    prep_enter_action(other, other, unit);
-		    set_moveto_task(unit, x, y);
+		    set_move_to_task(unit, x, y);
 		} else if (can_occupy_cell(unit, x, y)) {
 		    prep_move_action(unit, unit, x, y, 0);
 		} else {
@@ -2372,7 +2385,6 @@ draw_view_in_panner(side, map)
 Side *side;
 Map *map;
 {
-    static Pixmap pix = None;
     int x, y, l, t, mx, my, depth, w, h, b;
     Display *dpy = side->ui->dpy;
     Pixel pixel, bg=0, unknown;
@@ -2381,11 +2393,6 @@ Map *map;
     GC gc;
     extern int numdisplays;
 
-    /* This is a hack that works around a bug where Xconq will crash soon
-       after opening up displays on several different screens. */
-    if (numdisplays > 1)
-      return;
-
     if (map->panner == NULL)
       return;
 
@@ -2470,34 +2477,28 @@ Map *map;
     XtVaSetValues(map->panner, XtNbackgroundPixmap, None, NULL);
 #endif
 
-    /* (this is maybe premature - how to guarantee that server has it?) */
-    if (pix != None)
-      XFreePixmap(dpy, pix);
-    pix = XCreatePixmap(dpy, side->ui->rootwin, w, h, depth);
-    if (!pix || pix == None) {
+    if (map->panner_pix != None)
+      XFreePixmap(dpy, map->panner_pix);
+    map->panner_pix = XCreatePixmap(dpy, side->ui->rootwin, w, h, depth);
+    if (!map->panner_pix || map->panner_pix == None) {
 	XDestroyImage(img);
 	/* XDestroyImage also frees data */
 	return;
     }
 
-    gc = XCreateGC(dpy, pix, 0, NULL);
-    XPutImage(dpy, pix, gc, img, 0, 0, 0, 0, w, h);
+    gc = XCreateGC(dpy, map->panner_pix, 0, NULL);
+    XPutImage(dpy, map->panner_pix, gc, img, 0, 0, 0, 0, w, h);
     XFreeGC(dpy, gc);
     XDestroyImage(img);
     /* XDestroyImage also frees data */
 
-    if (pix != None) {
-#ifdef NO_PANNER_BGPIXMAP
-	XSetWindowBackgroundPixmap(dpy, XtWindow(map->panner), pix);
-#else
-	XtVaSetValues(map->panner, XtNbackgroundPixmap, pix, NULL);
-#endif
-    }
-
 #ifdef NO_PANNER_BGPIXMAP
+    XSetWindowBackgroundPixmap(dpy, XtWindow(map->panner), map->panner_pix);
     /* at this point we force a redraw of the panner */
     XClearWindow(dpy, XtWindow(map->panner));
-#endif /* NO_PANNER_BGPIXMAP */
+#else
+    XtVaSetValues(map->panner, XtNbackgroundPixmap, map->panner_pix, NULL);
+#endif
 }
 
 /* Popup control panel for maps, written by Massimo Campostrini. */
@@ -2590,7 +2591,7 @@ Map *map;
 			      NULL);
     map->ctrlpanel_buttons[SHOW_ALL] =
       XtVaCreateManagedWidget("See All", toggleWidgetClass, map->ctrlpanel_form,
-			      XtNsensitive, !g_see_all(),
+			      XtNsensitive, !all_see_all,
 			      NULL);
 
     label =
