@@ -1,5 +1,5 @@
 /* A no-longer simple X program to display and edit xconq image families.
-   Copyright (C) 1994, 1995 Massimo Campostrini & Stanley T. Shebs.
+   Copyright (C) 1994, 1995, 1996 Massimo Campostrini & Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -10,10 +10,6 @@ any later version.  See the file COPYING.  */
    by the resource "maskColor" (see XShowimf-color.ad) or by the 
    command-line argument "-mc" */
 
-#ifndef USE_CONSOLE
-#define USE_CONSOLE
-#endif
-
 #include <stdio.h>
 #include <X11/Intrinsic.h>
 #include <X11/Xresource.h>
@@ -28,8 +24,6 @@ any later version.  See the file COPYING.  */
 #include <X11/cursorfont.h>
 #include <bitmaps/check.b>
 
-char *strdup();
-
 #include "config.h"
 #include "misc.h"
 #include "lisp.h"
@@ -230,7 +224,7 @@ Image *find_image PARAMS ((ImageFamily *imf, int w, int h));
 void destroy_family PARAMS ((int i));
 void delete_family PARAMS ((int i));
 int empty_family PARAMS ((int i)); 
-void mark_changed PARAMS ((ImageFamily *imf));
+void mark_changed PARAMS ((ImageFamily *imf, int dummy));
 
 #ifdef HAVE_SELFILE
 extern FILE *XsraSelFile();
@@ -1018,8 +1012,9 @@ Widget w;
 }
 
 void
-mark_changed(imf)
+mark_changed(imf, dummy)
 ImageFamily *imf;
+int dummy;
 {
   int i;
 
@@ -1624,7 +1619,7 @@ int flag;
 	    }
 	} else {
 	    if (read_xbm_file(shortbuf, imf, mark_changed)) {
-		mark_changed(imf);
+		mark_changed(imf, 0);
 		ok = 1;
 	    }
 	}
@@ -1637,7 +1632,7 @@ int flag;
 	    }
 	} else {
 	    if (read_xbm_file(shortbuf, imf, mark_changed)) {
-		mark_changed(imf);
+		mark_changed(imf, 0);
 		ok = 1;
 	    }
 	}
@@ -1648,7 +1643,7 @@ int flag;
 	    ok = 1;
 	} else {
 	    if (read_xpm_file(shortbuf, imf, mark_changed)) {
-		mark_changed(imf);
+		mark_changed(imf, 0);
 		ok = 1;
 	    }
 	}
