@@ -6,8 +6,6 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-typedef void (*readimf_hook) PARAMS ((ImageFamily *));
-
 extern void close_displays PARAMS ((void));
 extern int write_entire_game_state PARAMS ((char *fname));
 extern void low_init_error PARAMS ((char *str));
@@ -24,8 +22,6 @@ extern void write_xpm_file PARAMS ((FILE *fp, char *name, Image *img));
 extern void write_x11_bitmaps PARAMS ((ImageFamily *imf, int mkfiles));
 extern void write_xbm_file PARAMS ((FILE *fp, char *name, int cols, int rows,
 				   char *data)); 
-extern int read_imf_file PARAMS ((char *filename, readimf_hook hook));
-extern void interp_form_imfonly PARAMS ((Obj *form, readimf_hook hook));
 extern char *find_imf_name PARAMS ((char *rawname));
 extern int read_any_file PARAMS ((char *filename, readimf_hook hook));
 extern void reverse_rawdata PARAMS ((ImageFamily *imf));
